/* screen.c */

#include <stdio.h>
#include "sys/types.h"
#include "LibGTE.h"
#include "LibGPU.h"
#include "LibCD.h"
//#include <libds.h>
#include "LibEtc.h"
#include "LibMath.h"
#include "LibPress.h"
#include "LibSN.h"
#include "LibSnd.h"
#include "LibSPU.h"
#include "ourtypes.h"
#include "main.h"
#include "mathbits.h"
#include "sys/types.h"
#include "profile.h"
#include "cd.h"
#include "tracks.h"
#include "screen.h"
#include "frontend.h"

/* variable definitions */

texturedescriptor	textureids[32];
textureid			tids[10];
uint8				*snowtexture;
uint8				*skytexture;
uint16				textpage;
uint16				clutpage;
uint16				geomx,geomy;
int32				geomscreen;
uint32				fntid1;
uint32				fntid2;
uint32				fntid3;
int32				z,d;		/* temp */
POLY_G4  			clearsquare[4];
MATRIX				MyLocalLight;
MATRIX				MyColourMatrix;
MATRIX	   			RotationMatrix[2];
SVECTOR				RotationVector[2];
CVECTOR	    		PrimaryColourVector;
CVECTOR				fc,bc;		/* temp */
DRAWENV				drawenv[2][6];
DISPENV				dispenv[2][6];
RECT				extrarect={896,384,128,128};

/* texture VRAM file and textured material file load */

long*	timsizeptr;
char* 	tpageptr;
char*	fileptr;
long	timsize;
uint16	filecharindex;
int32	notpages;
int32	notmats;
uint8	cdextraname;


char *ptrfindchar( char a, char *p )
{
	long found;

	found=0;
	while (found==0)
	{
		if (*p==a)
		{
			found=1;
		}
		else
		{
			p++;
		}
	}
	return p;
}

long eoltexttoint( char *fileptr )
{
	char *storeptr;
	char thischar[2]={0,0};
	long tenscount,intans;

	storeptr=fileptr;
	tenscount=1;
	intans=0;
	fileptr++;
	while (*fileptr!=0x0d)
	{
 		tenscount*=10;
 		fileptr++;
	}
	while (storeptr!=fileptr)
	{
		thischar[0]=*storeptr;
		intans+=atoi(&thischar[0])*tenscount;
		tenscount/=10;
		storeptr++;
	}
	return intans;
}

long ptrtexttoint( char *fp )
{
	char *storeptr;
	char thischar[2]={0,0};
	long tenscount,intans;

	storeptr=fp;
	tenscount=1;
	intans=0;
	fp++;
	while (*fp!=0x20)
	{
 		tenscount*=10;
 		fp++;
	}
	while (storeptr!=fp)
	{
		thischar[0]=*storeptr;
		intans+=atoi(&thischar[0])*tenscount;
		tenscount/=10;
		storeptr++;
	}
	return intans;
}

void MakeFogCluts(long *clutptr)
{
	uint32 temp1,temp2,writeaddr,*writeptr,*tempptr;
	uint8  i,j;
	int32  r1inc,b1inc,g1inc,r1,g1,b1;
	int32  r2inc,b2inc,g2inc,r2,g2,b2;
	long   stp1,stp2;

	tempptr=clutptr;
	cdbuffer=(char *)&Ib2[0];
	writeaddr=cdbuffer+200000;
	for (j=0;j<128;j++)
	{
		temp1=*(tempptr+j);
		for (i=0;i<4;i++)
		{
			writeptr=(writeaddr+(512*i)+(j*4));
			if (i==0)
			{
				r1=temp1&0x01f;
				g1=(temp1>>5)&0x01f;
				b1=(temp1>>10)&0x01f;
				stp1=temp1&0x8000;
				r2=(temp1>>16)&0x01f;
				g2=(temp1>>21)&0x01f;
				b2=(temp1>>26)&0x01f;
				stp2=(temp1>>16)&0x8000;
				stp2<<=16;
				r1inc=((28-r1)*1000)/4;
				g1inc=((28-g1)*1000)/4;
				b1inc=((28-b1)*1000)/4;
				r2inc=((28-r2)*1000)/4;
				g2inc=((28-g2)*1000)/4;
				b2inc=((28-b2)*1000)/4;
			}
			temp1=r1+((r1inc*(i+1))/1000);
			temp2=g1+((g1inc*(i+1))/1000);
			temp2<<=5;
			temp1+=temp2;
			temp2=b1+((b1inc*(i+1))/1000);
			temp2<<=10;
			temp1+=temp2;
			temp1+=stp1;
			temp2=r2+((r2inc*(i+1))/1000);
			temp2<<=16;
			temp1+=temp2;
			temp2=g2+((g2inc*(i+1))/1000);
			temp2<<=21;
			temp1+=temp2;
			temp2=b2+((b2inc*(i+1))/1000);
			temp2<<=26;
			temp1+=temp2;
			temp1+=stp2;
			if ((r1==0)&&(b1==0)&&(g1==0))
			{
				temp1&=0x0ffff8000;
			}
			if ((r2==0)&&(b2==0)&&(g2==0))
			{
				temp1&=0x08000ffff;
			}
			*writeptr=temp1;
		}
	}
}

void LoadTextures(void	)
{
	uint32		fin,count1,count2;
	long		filesize,x,y;

	TIM_IMAGE	timheader;
	char		cdres[2];

	notpages=7;
	cdbuffer=(char *)&Ib2[0];
	for (count2=0;count2<notpages;count2++)
	{
		/* read texture into VRAM */
#ifndef CD
		fin	   		=	PCopen( &Texturenames[Mountain[thistrack].tpagenos[count2]], 0, 0 );
		filesize	=	PClseek( fin, 0, 2 );
		PClseek( fin, 0, 0 );
		fileptr		=	cdbuffer;
		PCread( fin, (char *)fileptr, filesize );
		PCclose( fin );
#else
		fileptr=cdbuffer;
		load_fpcd(&Texturenames[Mountain[thistrack].tpagenos[count2]],(char *)fileptr);
do
 {

 }
 while (CdReadSync(1,0) > 0 );
#endif
		timsizeptr=cdbuffer;
		timsizeptr+=2;
		timsize=*timsizeptr;
		timsize/=4;
		timsizeptr+=timsize;
		timsizeptr+=2;
		timsize=*timsizeptr;
		y=((timsize&0xffff0000)>>16)-1;
		x=((timsize&0xffff)*2)-1;
		tids[count2].x=x;
		tids[count2].y=y;
		OpenTIM((char *)fileptr);
		ReadTIM(&timheader);
		if (count2<5)
		{
			LoadTPage(timheader.paddr,1,3,(320+(count2*128)),0,x+1,y+1);  /* 96 */
			tids[count2].textpage=GetTPage(1,3,(320+(count2*128)),0);
		}
		else
		{
			LoadTPage(timheader.paddr,1,3,(320+((count2-5)*128)),256,x+1,y+1);  /* 96 */
			tids[count2].textpage=GetTPage(1,3,(320+((count2-5)*128)),256);
		}
		MakeFogCluts(timheader.caddr);
		tids[count2].clutline[0]=LoadClut(timheader.caddr,768,256+(count2*5));
		tids[count2].clutline[1]=LoadClut(cdbuffer+200000,768,257+(count2*5));
		tids[count2].clutline[2]=LoadClut(cdbuffer+200000+512,768,258+(count2*5));
		tids[count2].clutline[3]=LoadClut(cdbuffer+200000+1024,768,259+(count2*5));
		tids[count2].clutline[4]=LoadClut(cdbuffer+200000+1536,768,260+(count2*5));
//		tids[count2].clutline=256+(count2*5);
	}
	switch (language)
	{
#ifndef CD
		default:
		case (0):
		{
			fin	=	PCopen( "textures/ingame.tim", 0, 0 );
			break;
		}
		case (1):
		{
			fin	=	PCopen( "textures/ingamef.tim", 0, 0 );
			break;
		}
		case (2):
		{
			fin	=	PCopen( "textures/ingamei.tim", 0, 0 );
			break;
		}
		case (3):
		{
			fin	=	PCopen( "textures/ingames.tim", 0, 0 );
			break;
		}
		case (4):
		{
			fin	=	PCopen( "textures/ingameg.tim", 0, 0 );
			break;
		}
#else
		default:
		case (0):
		{
			load_cd("\\TEXTURES\\INGAME.TIM;1",cdbuffer+180000);
			break;
		}
		case (1):
		{
			load_cd("\\TEXTURES\\INGAMEF.TIM;1",cdbuffer+180000);
			break;
		}
		case (2):
		{
			load_cd("\\TEXTURES\\INGAMEI.TIM;1",cdbuffer+180000);
			break;
		}
		case (3):
		{
			load_cd("\\TEXTURES\\INGAMES.TIM;1",cdbuffer+180000);
			break;
		}
		case (4):
		{
			load_cd("\\TEXTURES\\INGAMEG.TIM;1",cdbuffer+180000);
			break;
		}
#endif
	}

#ifndef CD
	filesize	=	PClseek( fin, 0, 2 );
	PClseek( fin, 0, 0 );
	fileptr	=	cdbuffer+180000;
	PCread( fin, (char *)fileptr, filesize );
	PCclose( fin );
#else
	do
 	{

 	}
 	while (CdReadSync(1,0) > 0 );
#endif
	OpenTIM(cdbuffer+180000);
	ReadTIM(&timheader);
	LoadImage(&extrarect,timheader.paddr);
	tids[9].textpage=GetTPage(1,3,896,256);  /* 96 */
	tids[9].clutline[0]=LoadClut(timheader.caddr,768,297);
}

void InitialisePSX(void)
{
	uint8 count1;
	long flushrand;

//        PadInit(0);
    Pre_Init();     // Routine to replace Pad_Init(0)


	ResetGraph(0);
	SetGraphDebug(0);
	InitGeom();
	srand(1);
	for (count1=0;count1<50;count1++)
	{
		flushrand=rand();
	}
#ifdef PROFILE
	ProfileInit(2);
#endif
//	LoadTextures();
#ifdef NTSC
	SetDefDrawEnv(&drawenv[0][0],0,0,320,240);
	SetDefDispEnv(&dispenv[0][0],0,240,320,240);
	SetDefDrawEnv(&drawenv[0][1],0,240,320,240);
	SetDefDispEnv(&dispenv[0][1],0,0,320,240);
	SetDefDispEnv(&dispenv[0][2],0,240,320,240);




	SetDefDispEnv(&dispenv[0][3],0,0,320,240);
	SetDefDispEnv(&dispenv[0][4],0,240,320,240);


	SetDefDrawEnv(&drawenv[0][2],0,360,320,120);
	SetDefDrawEnv(&drawenv[0][3],0,240,320,120);
	SetDefDrawEnv(&drawenv[0][4],0,120,320,120);
	SetDefDrawEnv(&drawenv[0][5],0,0,320,120);

	SetDefDispEnv(&dispenv[0][5],0,0,320,240);

	SetDefDrawEnv(&drawenv[1][0],0,0,320,240);
	SetDefDispEnv(&dispenv[1][0],0,240,320,240);
	SetDefDrawEnv(&drawenv[1][1],0,240,320,240);
	SetDefDispEnv(&dispenv[1][1],0,0,320,240);


	SetDefDispEnv(&dispenv[1][3],0,0,320,240);
	SetDefDispEnv(&dispenv[1][4],0,240,320,240);

	SetDefDrawEnv(&drawenv[1][2],0,240,160,240);
	SetDefDrawEnv(&drawenv[1][3],160,240,160,240);
	SetDefDrawEnv(&drawenv[1][4],0,0,160,240);
	SetDefDrawEnv(&drawenv[1][5],160,0,160,240);



	SetDefDispEnv(&dispenv[1][2],0,240,320,240);
	SetDefDispEnv(&dispenv[1][5],0,0,320,240);
#else
	SetDefDrawEnv(&drawenv[0][0],0,0,320,256);
	SetDefDispEnv(&dispenv[0][0],0,256,320,256);
	SetDefDrawEnv(&drawenv[0][1],0,256,320,256);
	SetDefDispEnv(&dispenv[0][1],0,0,320,256);
	SetDefDrawEnv(&drawenv[0][2],0,256,320,128);
	SetDefDispEnv(&dispenv[0][2],0,256,320,256);
	SetDefDrawEnv(&drawenv[0][3],0,384,320,128);
	SetDefDispEnv(&dispenv[0][3],0,0,320,256);
	SetDefDrawEnv(&drawenv[0][4],0,0,320,128);
	SetDefDispEnv(&dispenv[0][4],0,256,320,256);
	SetDefDrawEnv(&drawenv[0][5],0,128,320,128);
	SetDefDispEnv(&dispenv[0][5],0,0,320,256);

	SetDefDrawEnv(&drawenv[1][0],0,0,320,256);
	SetDefDispEnv(&dispenv[1][0],0,256,320,256);
	SetDefDrawEnv(&drawenv[1][1],0,256,320,256);
	SetDefDispEnv(&dispenv[1][1],0,0,320,256);

	SetDefDrawEnv(&drawenv[1][2],160,256,160,256);
	SetDefDispEnv(&dispenv[1][2],0,256,320,256);
	SetDefDrawEnv(&drawenv[1][3],160,256,160,256);
	SetDefDispEnv(&dispenv[1][3],0,0,320,256);

	SetDefDrawEnv(&drawenv[1][4],0,0,160,256);
	SetDefDispEnv(&dispenv[1][4],0,256,320,256);
	SetDefDrawEnv(&drawenv[1][5],0,0,160,256);
	SetDefDispEnv(&dispenv[1][5],0,0,320,256);

#endif
	PutDrawEnv(&drawenv[0][0]);
	PutDispEnv(&dispenv[0][0]);

//	ResetGraph(0);

//	FntLoad(896,0);
//	fntid1 = FntOpen(45,85,250,8,0,50);
//	SetDumpFnt(fntid1);
//	fntid2 = FntOpen(45,94,250,8,0,50);
//	SetDumpFnt(fntid2);
//	fntid3 = FntOpen(45,103,250,9,0,50);
//	SetDumpFnt(fntid3);
	geomx=120;
	geomy=160;
	SetGeomOffset(geomx,geomy);      /* 160,120 */
	geomscreen=200;
	SetGeomScreen( geomscreen );        /* 512 */
	fc.r=230;
	fc.g=230;
	fc.b=230;
	z=8000;
//	z=9000;
	d=500;
	bc.r=255;
	bc.g=255;
//	bc.r=188;
//	bc.g=188;
	bc.b=255;
	SetFarColor(fc.r,fc.g,fc.b);
//	SetFarColor(0,0,0);

	SetFogNear(z,d);
	SetBackColor(bc.r,bc.g,bc.b);
}

void InitialiseGame(void	)
{
	uint32	Count1,Count2;

	gameexit=0;
	display_id=0;
	otid=0;
	rotid=0;
	TopScreenDrawn=0;
	LoadTextures();
	for (Count1=0;Count1<4;Count1++)
	{
		clearsquare[Count1].x0=0;
		clearsquare[Count1].x1=320;
		clearsquare[Count1].y0=0;
		clearsquare[Count1].y1=0;
		clearsquare[Count1].x2=0;
		clearsquare[Count1].x3=320;
		clearsquare[Count1].y2=240;
		clearsquare[Count1].y3=240;
		clearsquare[Count1].r0=153;           /* 0 */
		clearsquare[Count1].g0=153;           /* 0 */
		clearsquare[Count1].b0=255;
		clearsquare[Count1].r1=153;
		clearsquare[Count1].g1=153;
		clearsquare[Count1].b1=255;
		clearsquare[Count1].r3=153;
		clearsquare[Count1].g3=153;
		clearsquare[Count1].b3=255;
		clearsquare[Count1].r2=153;
		clearsquare[Count1].g2=153;
		clearsquare[Count1].b2=255;

	}


	for (Count1=0;Count1<3;Count1++)
	{
		for (Count2=0;Count2<3;Count2++)
		{
			if (Count2==0)
			{
	  			MyColourMatrix.m[Count1][Count2]=0x0;
			}
			else
			{
 				MyColourMatrix.m[Count1][Count2]=0;
			}
			if (Count1==Count2)
			{
				RotationMatrix[0].m[Count1][Count2]=0x1000;
				MyLocalLight.m[Count1][Count2]=0x000;
				RotationMatrix[1].m[Count1][Count2]=0x1000;
			}
			else
			{
				RotationMatrix[0].m[Count1][Count2]=0;
				MyLocalLight.m[Count1][Count2]=0;
				RotationMatrix[1].m[Count1][Count2]=0;
			}
		}
		MyLocalLight.t[Count1]=1000;
		MyColourMatrix.t[Count1]=0;
	}
//	MyLocalLight.m[0][0]=2365;   /* -3768 */
//	MyLocalLight.m[0][1]=-2365;   /* -2548 */
//	MyLocalLight.m[0][2]=2365;    /* 1204 */
//	MyColourMatrix.m[0][0]=4000;
//	MyColourMatrix.m[1][0]=4000;
//	MyColourMatrix.m[2][0]=4000;
	SetLightMatrix(&MyLocalLight);
	SetColorMatrix(&MyColourMatrix);
	PrimaryColourVector.r=178;  /* 168,168,198 */
	PrimaryColourVector.g=178;
	PrimaryColourVector.b=178;

}


