// LOGIC.H

#ifndef _LOGIC_H_
#define _LOGIC_H_

#define	STY_NORMAL			0	// Going straight
#define STY_TURNING			1	// Turning
#define STY_PRE_TURNING		2	// Tween frames for straight to turn
#define STY_DOLMIO			3	// Remove later - it's just a test mode
#define STY_PRE_JUMP		4	// Hold crouch frame, wait for button release
#define STY_PREP_JUMP		5	// Prepare to jump (go to the crouch frame)
#define STY_SJUMPM			6	// Straight medium jump
#define STY_SJUMPPREPLANDM	7	// Straight medium jump, landing frames
#define STY_SJUMPL			8	// Straight little jump
#define STY_AUTO_JUMP		9	// Automatic, uncontrolled jump
#define STY_FRONTSPIN		10	// Front spin
#define STY_FSLAND			11	// Front spin natural landing
#define STY_BACKSPIN		12	// Back spin
#define STY_BSLAND			13	// Back spin natural landing
#define STY_RECOVERY		14	// Recover from some airborne capers
#define STY_RECOVERED		15	// Recovery successfull
#define STY_RECLAND1		16	// Recovery Landing 1
#define STY_RECLAND2		17	// Recovery Landing 2
#define STY_SJUMPM_AIR		18	// Straight medium jump, airborne part
#define STY_LANDONHEAD		19	// Landing on head
#define STY_LANDONLEFT		20	// Landing on left side
#define	STY_LANDONRIGHT		21	// Landing on right side
#define STY_TUCK			22	// Going into the tuck
#define STY_TREESOFT		23	// Soft tree collision
#define STY_TREEBUMP		24	// Medium tree collision
#define STY_TREESMACK		25	// Hard tree smack collision
#define STY_BRAKE			26	// Braking
#define STY_ENDBRAKE		27	// Coming out of brake
#define STY_LATEGRAB		28	// Late landing on the grab
#define STY_TWISTR			29	// Twist right
#define STY_TWISTRLAND		30	// Twist right landing
#define STY_TWISTL			31	// Twist left
#define STY_TWISTANDSPIN	32	// Twist and Spin combo
#define STY_STUMPTRIP		33	// Stump trip
#define STY_BONK			34	// Bonk
#define STY_TWISTLB			35	// Twist land backwards (heelside first)
#define STY_TWISTLF			36	// Twist land forwards (toeside first)
#define STY_BONK1			37	// End of the bonk
#define STY_RAILSLIDE		38	// RailSlide
#define STY_ENDRAILSLIDE	39	// End RailSlide
#define STY_ST_ROLL			40	// Spin and Twist roll crash
#define STY_END_ST_ROLL		41	// End of Spin and Twist roll crash
#define STY_ST_RECOVERY		42	// Spin and Twist recovery
#define STY_ST_RECOVERY2	43	// Spin and Twist recovery 2
#define STY_OFFRAIL			44	// Falling off the railslide
#define STY_CHILLTRICK		45	// Chill trick
#define STY_OLLIE			46	// Ollie



#define PARA_STY_AIR	0		// player style when parachute open
#define PARA_STY_LANDED	1		// obvious really
#define PARA_STY_FLY	2		// player style when free falling



#define MD_NOGROUNDORIENTBOARD	1	// Don't orient the board with the ground
#define MD_RECOVERY				2	// Allowed to enter recovery from this mode
#define MD_ALLOWGRAB			4	// Allowed to enter a grab mode from this mode
#define MD_ALLOWTUCK			8	// Allowed to enter the tuck from this frame
#define MD_NOCOLLISION			16	// Ignore collision detection
#define MD_ALLOWAUTOJUMP		32	// Allow automatic jumping
#define MD_ALLOWBRAKE			64	// Allow braking
#define MD_NOVEL				128	// No velocity calculations
#define MD_NOBONK				256	// No bonking
#define MD_AIRTURNS				512 // Allow turning in the air

void BoardModel( uint8 PlayerNumber );

/* Individual board/device movement modes */
void CalcRotXYZ( player *Plyr );

void FallOff_RailSlide( player *Plyr, int32 PlayerNumber );

void InitialiseSpaceHopper( uint8 PlayerNumber );
void SpaceHopper( uint8 PlayerNumber );	// Serves as a test mode, but could be included as a secret mode

void InitialiseTestCam( uint8 PlayerNumber );

void InitialiseSnowboard( uint8 PlayerNumber );	// The actual boarder logic
void Snowboard( uint8 PlayerNumber );

// Animation assistant routines
void Initialise_Railslide( player *Plyr, int32 PlayerNumber );
void TwistLandOrientation( player *Plyr, int32 PlayerNumber );
uint8 OrientedWithGround( player *Plyr );
void HalfAzimuth( player *Plyr );
void Grab( player *Plyr, int32 PlayerNumber );
void CalculateCrouch( player *Plyr );
void LerpToNormalStance( player *Plyr, int32 PlayerNumber );
void LerpToFakieStance( player *Plyr, int32 PlayerNumber );

void InitLerpHomeSpinRotY( player *Plyr,  uint8 PlayerNumber, uint16 Lerps, chfloat DestA );
void LerpHomeSpinRotY( player *Plyr, uint8 PlayerNumber );
uint8 NotYLerping( uint8 PlayerNumber );

void Analog_Swing( player *Plyr, chfloat Swing, chfloat Desired );

void GrabScoring( player *Plyr, int32 PlayerNumber );
void Spin_Scoring( player *Plyr, int32 PlayerNumber );

#define RAIL_FRONT	0
#define RAIL_LEFT	1
#define RAIL_BACK	2
#define RAIL_RIGHT	3

/*
#define	NONE				0
#define	SIDES				1
#define	TOP					2
#define	TREESCRAPE			3
#define ROCK				4
#define TRACKEDGE			5

TREESMACK		// Hard smack into the tree
TREEBUMP		// Medium
TREESCRAPE		// Same as tree scrape
*/

/*
Left to do to the logic
-----------------------
Iron out the rotation problems in the various animations
Analog everything
*/

#endif

