/* pads.h */

#ifndef _PADS_H_
#define _PADS_H_

#define MAX_PADS	2

typedef uint16 standard;
typedef struct anastick
{
	uint16	Buttons;
	uint8	Rx, Ry;
	uint8	Lx, Ly;
}anastick;

typedef struct negcon
{
	uint16	Buttons;
	int8 Twist;

	int8 ButtonI;
	int8 ButtonII;
	int8 TopLeft;
	
}negcon;

typedef struct padbuffer
{
	int8 Status;
	uint8 ID;

	union
	{
		standard	Standard;
		anastick	AnaStick;
		negcon		Negcon;
	}Data;

}padbuffer;


typedef struct gunpos
{
	uint8 y;
	uint8 x;

}gunpos;

typedef struct gunbuffer
{
	int8 Status;
	uint8 Count;

	gunpos Pos[ 20 ];

}gunbuffer;


typedef struct padsendbuffer
{
	int8	Status;
	uint8	ID;

	uint8	Data;
}padsendbuffer;


/* All PAD_ #define's are Chill mandatory */

#define	PAD_UP		0x0010
#define	PAD_DOWN	0x0040
#define PAD_LEFT	0x0080
#define	PAD_RIGHT	0x0020

#define	PAD_B1		0x8000
#define	PAD_B2		0x4000
#define	PAD_B3		0x2000

#define PAD_STOP	0x0001
#define PAD_SEL		0x0001
#define PAD_SELECT	0x0001

#define PAD_START	0x0008	
//PADstart

#define	PAD_TOPLEFT		0x0400
#define	PAD_TOPRIGHT	0x0800

#define	PAD_BOTTOMLEFT	0x0100			/* See note 1 */
#define	PAD_BOTTOMRIGHT	0x0200			/* See note 1 */

#define PAD_TL			PAD_TOPLEFT
#define PAD_TR			PAD_TOPRIGHT

#define PAD_L1			PAD_TOPLEFT
#define PAD_R1			PAD_TOPRIGHT

#define PAD_ACTION	PAD_B2	/* PSX action button is X */




/*
Note 1	:	These pad buttons will not appear on the M2 or Saturn version and have been put in just for compatability with
			the other versions during development.  They should be equated to similar or combinations of buttons.
*/

#define PAD_BL			PAD_BOTTOMLEFT
#define PAD_BR			PAD_BOTTOMRIGHT
#define PAD_L2			PAD_BOTTOMLEFT
#define PAD_R2			PAD_BOTTOMRIGHT
#define	PAD_B4		0x1000


#define PAD_PSXANALOGLEFT	0x0002
#define PAD_PSXANALOGRIGHT	0x0004


/* Types of pad chill uses */

#define PAD_DIGITAL				1
#define PAD_ANALOG				2
#define PAD_ANALOG_CONTROLLER	3
#define PAD_NEGCON				4
#define PAD_GUN					5


/* M2 Specific routines */

//void InitialiseM2Pads( void );
//void KillM2Pads( void );
//void Pad_Update( void );

void Pre_Init( void );

/* Optional routine */

void Pad_Update( void ); /* Advised implementation to Poll pad hardware */


/* Chill mandatory routines */

uint16 Pad_State( uint8 PadNumber );	/* Return current state of pad ( 0 is the first pad )*/
uint16 Pad_Debounced( uint8 PadNumber );	/* Return current debounced state of pad */
uint16 Pad_RealState( uint8 PadNumber );	/* Return current state of pad ( 0 is the first pad )*/
uint16 Pad_RealDebounced( uint8 PadNumber );	/* Return current debounced state of pad */
uint16 Pad_Type( uint8 PadNumber );	/* Analog or Digital, etc */

uint8 Pad_Analog_RX( uint8 PadNumber );


uint8 Pad_Negcon_Button1( uint8 PadNumber );
uint8 Pad_Negcon_Button2( uint8 PadNumber );

uint8 Config_GRAB1( uint8 PadNumber );	// State of the user configurated GRAB 1 button
uint8 Config_GRAB2( uint8 PadNumber );	// State of the user configurated GRAB 2 button
uint8 Config_BRAKE( uint8 PadNumber );	// State of the user configurated BRAKE button
uint8 Config_TUCK( uint8 PadNumber );	// State of the used configurated TUCK button
uint8 Config_JUMP( uint8 PadNumber );	// State of the used configurated JUMP button
uint8 Config_FRONTSPIN( uint8 PadNumber );	// State of the used configurated FRONTSPIN button
uint8 Config_BACKSPIN( uint8 PadNumber );	// State of the used configurated BACKSPIN button
uint8 Config_LEFTTWIST( uint8 PadNumber );	// State of the used configurated LEFTTWIST button
uint8 Config_RIGHTTWIST( uint8 PadNumber );	// State of the used configurated RIGHTTWIST button
uint8 Config_CHILLTRICK( uint8 PadNumber );	// State of CHILLTRICK key combo signal

uint8 Analog_Config_Steer( uint8 PadNumber );


extern uint16 Paddys[ MAX_PADS ];


#endif




