/* players.h */

#ifndef _PLAYERS_H_
#define _PLAYERS_H_

#define MAX_PLAYERS	2

/* ConsoleRemoteGhost field possibilities */

#define CONSOLE_PLAYER	1
#define REMOTE_PLAYER	2
#define GHOSTED_PLAYER	3

/* Transport field possibilities */

#define	TESTCAM		0
#define	SNOWBOARD	1
#define SPACEHOPPER 2
#define	SKIS		3	// Probably not be used
#define	BIGFEET		4	// Probably not be used
#define PARACHUTE 128


/* Character field possibilities */

#define NONE 0
#define BONE 1
#define TRIX 2
#define GOAT 3
#define YOYO 4
#define YETI 5
#define CUBE 6	/* This is a test character skin */

typedef struct player
{
	// Player information for save game
	char		Name[ 8 ];

	uint8		Transport;
	uint8		PlayerNumber;

	uint8		PadNumber;

	uint32		Credibility;
	uint8		Lives, Etc;
	uint32		Score;
	uint16		Junction;

	uint8		Character;		// Which character the player is
	uint8		Board;

	// The information after this point should not be saved

	uint16 HiLo, Suit, SuitLoaded;

	camera		Cam;

	vec3		Pos;
	chfloat		TerrainY, LastTerrainY;
	vec3		Rot;
	chfloat		PrefRotZ;
	vec3		SpinRot;

	chfloat		JumpY;
	uint8		JumpD, JumpHeld;

	uint8		SpinHeld, TwistHeld;
	chfloat		SpinSpeed, TwistSpeed;

	uint8		GrabHeld, HaveGrabbed;
	uint32		GrabDuration;

	uint8		ConsoleRemoteGhosted, Fakie;

	// Information to point to the characters model
	uint8		Loaded, LimbDepth;
	uint32 		CharNumVerts;
	chfloat 	*CharVerts;
	chfloat 	*CharNorms;
	uint32 		CharNumPolys;
	void 		*CharPolys;
	void 		*SkeletonRoot;

	uint8		*PoseBuffer;
	uint8		PoseLoaded;

	// Movement variables
	vec3		LastPos;
	chfloat		TurnInert;
	chfloat		Vel;
	chfloat		SideSlip;
	chfloat		BoardElevation, LastBoardElevation;
	chfloat		BoardRoll, LastBoardRoll;
	chfloat		BoardAzimuth, LastBoardAzimuth;
	chfloat		MoveAzimuth;
	chfloat		RailY, RailAzimuth;
	chfloat		BoardTwirl;
	uint8		Style, LastStyle, RailType;
	chfloat		TurnX, TurnY;
	chfloat		JumpPower;
	chfloat		Grav;

	chfloat		RailSlip, RailSlipVel;

	uint16 		CharPage, CharClut;

	chfloat		BrakeTurn;

	chfloat		Desired;

	uint32		AnimStage, TurnYStage;
	uint32		AnimTween;
	uint32		AnimFrame;

	uint16		GrabStage;
//	chfloat		GrabRotX, GrabRotZ;

	chfloat		TwistDist, SpinDist;

	chfloat		BonusY;

	uint8		SlowTransition;

	// Jeb
	uint16 NoTaggedPolys;
	uint16 TaggedPolys[ 512 ];

	trackdispbuffer *Track;
	uint8 Buffer;

	vec3 BoardPoint;
}player;



extern uint8	NumberOfPlayers;	/* Total number of players */
extern uint8	ConsolePlayers;		/* This is the number of split screens you should have */
extern uint8	RemotePlayers;
extern uint8	GameMode;
extern player	Player[ MAX_PLAYERS ];

void InitialisePlayers( void );
void KillPlayers( void );

#endif



