// ANIMCTRL.C - Simply the best lerping code ever

#include <stdio.h>
#include <sys/types.h>
#include <LibGTE.h>
#include <LibGPU.h>
#include <LibCD.h>
#include <LibComb.h>
#include <LibEtc.h>
#include <LibGS.h>
#include <LibMath.h>
#include <LibPress.h>
#include <LibSN.h>
#include <LibSnd.h>
#include <LibSPU.h>

#include "ourtypes.h"
#include "mathbits.h"

#include "main.h"
#include "skeleton.h"
#include "camera.h"
#include "tracks.h"
#include "players.h"
#include "pads.h"

#include "screen.h"
#include "who.h"
#include "animctrl.h"
#include "bonebag.h"
#include "trixbag.h"
#include "profile.h"


uint8 PoseBuffer[ POSEBUFFERSIZE ];	// Buffers for all the frames of each players character
vogue Vogues[ MAX_PLAYERS ];				// Current pose of the characters

vogue LerpStart[ MAX_PLAYERS ];				// Lerp src and dest storage and temporary vogue storage
vogue LerpDest[ MAX_PLAYERS ];

uint8 AnimStatus[ MAX_PLAYERS ];
uint16 LerpFrames[ MAX_PLAYERS ];
uint16 LerpProgress[ MAX_PLAYERS ];
uint32 SourceLerp[ MAX_PLAYERS ];
uint32 DestLerp[ MAX_PLAYERS ];


chfloat RDRx[ MAX_PLAYERS ], RDRy[ MAX_PLAYERS ], RDRz[ MAX_PLAYERS ];
uint16 RLerpProgress[ MAX_PLAYERS ];
uint16 RLerpFrames[ MAX_PLAYERS ];
vec3 RSrc[ MAX_PLAYERS ], RDest[ MAX_PLAYERS ];
uint8 RMode[ MAX_PLAYERS ];
vec3 *Dump[ MAX_PLAYERS ];


// ********************************************************************
// void Control_Anim( uint8 PNum )
//
// Animation engine controller.  Called every frame by skeleton drawer.
// ********************************************************************

void Control_Anim( uint8 PNum )
{
	switch( AnimStatus[ PNum ] )
	{
		case AS_STATIC:
			break;

		case AS_LERPING:
			NextLerp( PNum );
			if( RMode[ PNum ] == AS_LERPING ) LerpRots( PNum );
			break;
	}
}





void SetLerpRots( uint8 PNum, vec3 *Src, vec3 *Dest, uint16 Lerps, vec3 *EDest )//, vec3 *Base )
{
		Dump[ PNum ] = EDest;

		RDest[ PNum ].X = Dest->X;//Base->X - Dest->X;
		RDest[ PNum ].Y = Dest->Y;//Base->Y - Dest->Y;
		RDest[ PNum ].Z = Dest->Z;//Base->Z - Dest->Z;
		RSrc[ PNum ].X = Src->X;
		RSrc[ PNum ].Y = Src->Y;
		RSrc[ PNum ].Z = Src->Z;

		RDRx[ PNum ] = RDest[ PNum ].X - RSrc[ PNum ].X;
		RDRy[ PNum ] = RDest[ PNum ].Y - RSrc[ PNum ].Y;
		RDRz[ PNum ] = RDest[ PNum ].Z - RSrc[ PNum ].Z;

		if( RDRx[ PNum ] > 2048 ) RDRx[ PNum ] = RDRx[ PNum ] - 4096;
		else if( RDRx[ PNum ] < -2048 ) RDRx[ PNum ] = RDRx[ PNum ] + 4096;
		if( RDRy[ PNum ] > 2048 ) RDRy[ PNum ] = RDRy[ PNum ] - 4096;
		else if( RDRy[ PNum ] < -2048 ) RDRy[ PNum ] = RDRy[ PNum ] + 4096;
		if( RDRz[ PNum ] > 2048 ) RDRz[ PNum ] = RDRz[ PNum ] - 4096;
		else if( RDRz[ PNum ] < -2048 ) RDRz[ PNum ] = RDRz[ PNum ] + 4096;

		RLerpProgress[ PNum ] = 0;
		RLerpFrames[ PNum ] = Lerps;

		RMode[ PNum ] = AS_LERPING;

		LerpRots( PNum );
}

void LerpRots( uint8 PNum )//, vec3 *Dump )
{
	RLerpProgress[ PNum ]++;
	if( RLerpProgress[ PNum ] < RLerpFrames[ PNum ] )
	{
		Dump[ PNum ]->X = RSrc[ PNum ].X + ( ( RDRx[ PNum ] * RLerpProgress[ PNum ] ) / RLerpFrames[ PNum ] );
		Dump[ PNum ]->Y = RSrc[ PNum ].Y + ( ( RDRy[ PNum ] * RLerpProgress[ PNum ] ) / RLerpFrames[ PNum ] );
		Dump[ PNum ]->Z = RSrc[ PNum ].Z + ( ( RDRz[ PNum ] * RLerpProgress[ PNum ] ) / RLerpFrames[ PNum ] );
	}
	else
	{
		Dump[ PNum ]->X = RDest[ PNum ].X;
		Dump[ PNum ]->Y = RDest[ PNum ].Y;
		Dump[ PNum ]->Z = RDest[ PNum ].Z;
		RMode[ PNum ] = AS_STATIC;

//		printf("Goof %d\n", Player[ PNum ].SpinRot.X );
	}
}





// Namby pamby status routines
uint8 Anim_Status( uint8 PNum )
{
	return AnimStatus[ PNum ];
}

uint32 SourceLerpFrame( uint8 PNum )
{
	return SourceLerp[ PNum ];
}

uint32 DestLerpFrame( uint8 PNum )
{
	return DestLerp[ PNum ];
}

void SetPose( uint8 PNum, uint32 FrameNumber )
{
	uint16 i;
	chfloat *Trans = &PoseBuffer[ FrameNumber ];
	int16 *Rots = &PoseBuffer[ FrameNumber + 12 ];

	DestLerp[ PNum ] = FrameNumber;

	Vogues[ PNum ].Tx = Trans[ 0 ];							// Extract the translations of the Pose
	Vogues[ PNum ].Ty = Trans[ 1 ];
	Vogues[ PNum ].Tz = Trans[ 2 ];

//	printf("Trans = %ld, %ld, %ld\n", Trans[ 0 ], Trans[ 1 ], Trans[ 2 ] );

	for( i = 0; i < Player[ PNum ].LimbDepth; i++ )
	{
		Vogues[ PNum ].Rx[ i ] = Rots[ ( i * 3 ) + 0 ];		// Extract the rotations of the Pose
		Vogues[ PNum ].Ry[ i ] = Rots[ ( i * 3 ) + 1 ];
		Vogues[ PNum ].Rz[ i ] = Rots[ ( i * 3 ) + 2 ];

//		printf("Rots = %ld, %ld, %ld\n", Vogues[ PNum ].Rx[ i ], Vogues[ PNum ].Ry[ i ], Vogues[ PNum ].Rz[ i ] );

	}

	AnimStatus[ PNum ] = AS_STATIC;
}


// PlayerNumber, DestFrame, LerpCount
void SetLerp( uint8 PNum, uint32 FrameNumber, uint16 Lerps )
{
	uint16 i;
	chfloat *Trans = &PoseBuffer[ FrameNumber ];
	int16 *Rots = &PoseBuffer[ FrameNumber + 12 ];

	DestLerp[ PNum ] = FrameNumber;

	// Set the new mode
	AnimStatus[ PNum ] = AS_LERPING;

	// (a) Copy current vogue to lerp start & (b) extract the destination pose
	// (combined together for speed)

	LerpStart[ PNum ].Tx = Vogues[ PNum ].Tx;	//(a)
	LerpStart[ PNum ].Ty = Vogues[ PNum ].Ty;	//(a)
	LerpStart[ PNum ].Tz = Vogues[ PNum ].Tz;	//(a)

	LerpDest[ PNum ].Tx = Trans[ 0 ];	//(b)
	LerpDest[ PNum ].Ty = Trans[ 1 ];	//(b)
	LerpDest[ PNum ].Tz = Trans[ 2 ];	//(b)

//	if( FrameNumber == B_SJUMPM_2 )
//	{
//		printf("Start = %ld, %ld, %ld\n", LerpStart[ PNum ].Tx,LerpStart[ PNum ].Ty,LerpStart[ PNum ].Tz );
//		printf("Dest = %ld, %ld, %ld\n", LerpDest[ PNum ].Tx,LerpDest[ PNum ].Ty,LerpDest[ PNum ].Tz );
//	}

	for( i = 0; i < Player[ PNum ].LimbDepth; i++ )
	{
		LerpStart[ PNum ].Rx[ i ] = Vogues[ PNum ].Rx[ i ];	//(a)
		LerpStart[ PNum ].Ry[ i ] = Vogues[ PNum ].Ry[ i ];	//(a)
		LerpStart[ PNum ].Rz[ i ] = Vogues[ PNum ].Rz[ i ];	//(a)

		LerpDest[ PNum ].Rx[ i ] = Rots[ ( i * 3 ) + 0 ];	//(b)
		LerpDest[ PNum ].Ry[ i ] = Rots[ ( i * 3 ) + 1 ];	//(b)
		LerpDest[ PNum ].Rz[ i ] = Rots[ ( i * 3 ) + 2 ];	//(b)
	}

	// Calculate the first frame and put it into the current vogue
	LerpProgress[ PNum ] = 0;		// We haven't begun yet so 0
	LerpFrames[ PNum ] = Lerps;		// Total number of lerps

	NextLerp( PNum );
}


// *********************************************************************************************************
// void SetLerpToTween( uint8 PNum, uint32 Frame1, uint32 Frame2, uint16 Lerps, uint16 Twerps, int16 Tween )
//
// Lerp to the tween of two frames.  Lerps is the number of lerps.
// Twerps and Tweens is the number of lerps and the numbers of tweens between the two frames.
// *********************************************************************************************************

void SetLerpToTween( uint8 PNum, uint32 Frame1, uint32 Frame2, uint16 Lerps, uint16 Twerps, int16 Tween )
{
	uint16 i, j;
	chfloat *Trans1 = &PoseBuffer[ Frame1 ];
	int16 *Rots1 = &PoseBuffer[ Frame1 + 12 ];
	chfloat *Trans2 = &PoseBuffer[ Frame2 ];
	int16 *Rots2 = &PoseBuffer[ Frame2 + 12 ];

	chfloat DRx, DRy, DRz;

	DestLerp[ PNum ] = 0;	// Are you trying to be funny

	// Set the new mode
	AnimStatus[ PNum ] = AS_LERPING;

	LerpStart[ PNum ].Tx = Vogues[ PNum ].Tx;
	LerpStart[ PNum ].Ty = Vogues[ PNum ].Ty;
	LerpStart[ PNum ].Tz = Vogues[ PNum ].Tz;

	DRx = Trans2[ 0 ] - Trans1[ 0 ];
	DRy = Trans2[ 1 ] - Trans1[ 1 ];
	DRz = Trans2[ 2 ] - Trans1[ 2 ];

	LerpDest[ PNum ].Tx = Trans1[ 0 ] + ( ( DRx * Tween ) / Twerps );
	LerpDest[ PNum ].Ty = Trans1[ 1 ] + ( ( DRy * Tween ) / Twerps );
	LerpDest[ PNum ].Tz = Trans1[ 2 ] + ( ( DRz * Tween ) / Twerps );

	j = 0;
	for( i = 0; i < Player[ PNum ].LimbDepth; i++ )
	{
		LerpStart[ PNum ].Rx[ i ] = Vogues[ PNum ].Rx[ i ];
		LerpStart[ PNum ].Ry[ i ] = Vogues[ PNum ].Ry[ i ];
		LerpStart[ PNum ].Rz[ i ] = Vogues[ PNum ].Rz[ i ];

		DRx = Rots2[ j + 0 ] - Rots1[ j + 0 ];
		DRy = Rots2[ j + 1 ] - Rots1[ j + 1 ];
		DRz = Rots2[ j + 2 ] - Rots1[ j + 2 ];
		if( DRx > 2048 ) DRx = DRx - 4096;
		else if( DRx < -2048 ) DRx = DRx + 4096;
		if( DRy > 2048 ) DRy = DRy - 4096;
		else if( DRy < -2048 ) DRy = DRy + 4096;
		if( DRz > 2048 ) DRz = DRz - 4096;
		else if( DRz < -2048 ) DRz = DRz + 4096;

		LerpDest[ PNum ].Rx[ i ] = Rots1[ j + 0 ] + ( ( DRx * Tween ) / Twerps );
		LerpDest[ PNum ].Ry[ i ] = Rots1[ j + 1 ] + ( ( DRy * Tween ) / Twerps );
		LerpDest[ PNum ].Rz[ i ] = Rots1[ j + 2 ] + ( ( DRz * Tween ) / Twerps );
		j += 3;
	}

	// Calculate the first frame and put it into the current vogue
	LerpProgress[ PNum ] = 0;		// We haven't begun yet so 0
	LerpFrames[ PNum ] = Lerps;		// Total number of lerps

	NextLerp( PNum );
}

void SetLerpToTweenOfTwoTweens( uint8 PNum, uint32 Src1, uint32 Dest1, uint32 Src2, uint32 Dest2, uint16 Lerps1, int16 Tween1, uint16 Lerps2, int16 Tween2, uint16 Lerps, int16 Tween )
{
	uint16 i, j;
	chfloat *Src1Tran = &PoseBuffer[ Src1 ];
	int16 *Src1Rot = &PoseBuffer[ Src1 + 12 ];
	chfloat *Dest1Tran = &PoseBuffer[ Dest1 ];
	int16 *Dest1Rot = &PoseBuffer[ Dest1 + 12 ];
	chfloat *Src2Tran = &PoseBuffer[ Src2 ];
	int16 *Src2Rot = &PoseBuffer[ Src2 + 12 ];
	chfloat *Dest2Tran = &PoseBuffer[ Dest2 ];
	int16 *Dest2Rot = &PoseBuffer[ Dest2 + 12 ];
	chfloat DRx, DRy, DRz;

	DestLerp[ PNum ] = 0;	// Are you trying to be funny

	// Set the new mode
	AnimStatus[ PNum ] = AS_LERPING;

	DRx = Dest1Tran[ 0 ] - Src1Tran[ 0 ];
	DRy = Dest1Tran[ 1 ] - Src1Tran[ 1 ];
	DRz = Dest1Tran[ 2 ] - Src1Tran[ 2 ];
	LerpStart[ PNum ].Tx = Src1Tran[ 0 ] + ( ( DRx * Tween1 ) / Lerps1 );
	LerpStart[ PNum ].Ty = Src1Tran[ 1 ] + ( ( DRy * Tween1 ) / Lerps1 );
	LerpStart[ PNum ].Tz = Src1Tran[ 2 ] + ( ( DRz * Tween1 ) / Lerps1 );
	DRx = Dest2Tran[ 0 ] - Src2Tran[ 0 ];
	DRy = Dest2Tran[ 1 ] - Src2Tran[ 1 ];
	DRz = Dest2Tran[ 2 ] - Src2Tran[ 2 ];
	LerpDest[ PNum ].Tx = Src2Tran[ 0 ] + ( ( DRx * Tween2 ) / Lerps2 );
	LerpDest[ PNum ].Ty = Src2Tran[ 1 ] + ( ( DRy * Tween2 ) / Lerps2 );
	LerpDest[ PNum ].Tz = Src2Tran[ 2 ] + ( ( DRz * Tween2 ) / Lerps2 );
	DRx = LerpDest[ PNum ].Tx - LerpStart[ PNum ].Tx;
	DRy = LerpDest[ PNum ].Ty - LerpStart[ PNum ].Ty;
	DRz = LerpDest[ PNum ].Tz - LerpStart[ PNum ].Tz;
	LerpDest[ PNum ].Tx = LerpStart[ PNum ].Tx + ( ( DRx * Tween ) / Lerps );
	LerpDest[ PNum ].Ty = LerpStart[ PNum ].Ty + ( ( DRy * Tween ) / Lerps );
	LerpDest[ PNum ].Tz = LerpStart[ PNum ].Tz + ( ( DRz * Tween ) / Lerps );
	LerpStart[ PNum ].Tx = Vogues[ PNum ].Tx;
	LerpStart[ PNum ].Ty = Vogues[ PNum ].Ty;
	LerpStart[ PNum ].Tz = Vogues[ PNum ].Tz;

	j = 0;
	for( i = 0; i < Player[ PNum ].LimbDepth; i++ )
	{
		// Get tween1 into vogue LerpStart
		DRx = Dest1Rot[ j + 0 ] - Src1Rot[ j + 0 ];
		DRy = Dest1Rot[ j + 1 ] - Src1Rot[ j + 1 ];
		DRz = Dest1Rot[ j + 2 ] - Src1Rot[ j + 2 ];
		if( DRx > 2048 ) DRx = DRx - 4096;
		else if( DRx < -2048 ) DRx = DRx + 4096;
		if( DRy > 2048 ) DRy = DRy - 4096;
		else if( DRy < -2048 ) DRy = DRy + 4096;
		if( DRz > 2048 ) DRz = DRz - 4096;
		else if( DRz < -2048 ) DRz = DRz + 4096;
		LerpStart[ PNum ].Rx[ i ] = Src1Rot[ j + 0 ] + ( ( DRx * Tween1 ) / Lerps1 );
		LerpStart[ PNum ].Ry[ i ] = Src1Rot[ j + 1 ] + ( ( DRy * Tween1 ) / Lerps1 );
		LerpStart[ PNum ].Rz[ i ] = Src1Rot[ j + 2 ] + ( ( DRz * Tween1 ) / Lerps1 );

		// Get tween2 into vogue LerpDest
		DRx = Dest2Rot[ j + 0 ] - Src2Rot[ j + 0 ];
		DRy = Dest2Rot[ j + 1 ] - Src2Rot[ j + 1 ];
		DRz = Dest2Rot[ j + 2 ] - Src2Rot[ j + 2 ];
		if( DRx > 2048 ) DRx = DRx - 4096;
		else if( DRx < -2048 ) DRx = DRx + 4096;
		if( DRy > 2048 ) DRy = DRy - 4096;
		else if( DRy < -2048 ) DRy = DRy + 4096;
		if( DRz > 2048 ) DRz = DRz - 4096;
		else if( DRz < -2048 ) DRz = DRz + 4096;
		LerpDest[ PNum ].Rx[ i ] = Src2Rot[ j + 0 ] + ( ( DRx * Tween2 ) / Lerps2 );
		LerpDest[ PNum ].Ry[ i ] = Src2Rot[ j + 1 ] + ( ( DRy * Tween2 ) / Lerps2 );
		LerpDest[ PNum ].Rz[ i ] = Src2Rot[ j + 2 ] + ( ( DRz * Tween2 ) / Lerps2 );

		// now tween between LerpStart and LerpDest to get the TweenOfTwoTweens
		DRx = LerpDest[ PNum ].Rx[ i ] - LerpStart[ PNum ].Rx[ i ];
		DRy = LerpDest[ PNum ].Ry[ i ] - LerpStart[ PNum ].Ry[ i ];
		DRz = LerpDest[ PNum ].Rz[ i ] - LerpStart[ PNum ].Rz[ i ];
		if( DRx > 2048 ) DRx = DRx - 4096;
		else if( DRx < -2048 ) DRx = DRx + 4096;
		if( DRy > 2048 ) DRy = DRy - 4096;
		else if( DRy < -2048 ) DRy = DRy + 4096;
		if( DRz > 2048 ) DRz = DRz - 4096;
		else if( DRz < -2048 ) DRz = DRz + 4096;
		LerpDest[ PNum ].Rx[ i ] = LerpStart[ PNum ].Rx[ i ] + ( ( DRx * Tween ) / Lerps );
		LerpDest[ PNum ].Ry[ i ] = LerpStart[ PNum ].Ry[ i ] + ( ( DRy * Tween ) / Lerps );
		LerpDest[ PNum ].Rz[ i ] = LerpStart[ PNum ].Rz[ i ] + ( ( DRz * Tween ) / Lerps );

		LerpStart[ PNum ].Rx[ i ] = Vogues[ PNum ].Rx[ i ];
		LerpStart[ PNum ].Ry[ i ] = Vogues[ PNum ].Ry[ i ];
		LerpStart[ PNum ].Rz[ i ] = Vogues[ PNum ].Rz[ i ];

		j += 3;	// Fast indexer
	}
	// Calculate the first frame and put it into the current vogue
	LerpProgress[ PNum ] = 0;		// We haven't begun yet so 0
	LerpFrames[ PNum ] = Lerps;		// Total number of lerps

	NextLerp( PNum );
}

void NextLerp( uint8 PNum )
{
	uint16 i;
	chfloat DRx, DRy, DRz;

	LerpProgress[ PNum ]++;

	DRx = LerpDest[ PNum ].Tx - LerpStart[ PNum ].Tx;
	DRy = LerpDest[ PNum ].Ty - LerpStart[ PNum ].Ty;
	DRz = LerpDest[ PNum ].Tz - LerpStart[ PNum ].Tz;
/*
	if( DestLerp[ PNum ] == B_SJUMPM_2 )
	{
		printf("D=%ld,S=%ld - ", LerpDest[ PNum ].Tx, LerpStart[ PNum ].Tx );

// 		DRx = 0;
//		DRz = 0;

		printf("Trans = %ld, %ld, %ld\n",
		( DRx * LerpProgress[ PNum ] ) / LerpFrames[ PNum ],
		( DRy * LerpProgress[ PNum ] ) / LerpFrames[ PNum ],
		( DRz * LerpProgress[ PNum ] ) / LerpFrames[ PNum ] );

		while( Pad_Debounced( 0 ) != PAD_START )
		{
			VSync( 0 );
			Pad_Update();
			pollhost();
		}
	}
*/
//	DRx *= 100;

	Vogues[ PNum ].Tx = LerpStart[ PNum ].Tx + ( ( DRx * LerpProgress[ PNum ] ) / LerpFrames[ PNum ] );
//	if( DestLerp[ PNum ] == B_SJUMPM_2 )
//	{
//		printf("%ld\n", Vogues[ PNum ].Tx);
//	}


	Vogues[ PNum ].Ty = LerpStart[ PNum ].Ty + ( ( DRy * LerpProgress[ PNum ] ) / LerpFrames[ PNum ] );
	Vogues[ PNum ].Tz = LerpStart[ PNum ].Tz + ( ( DRz * LerpProgress[ PNum ] ) / LerpFrames[ PNum ] );

	for( i = 0; i < Player[ PNum ].LimbDepth; i++ )
	{
		DRx = LerpDest[ PNum ].Rx[ i ] - LerpStart[ PNum ].Rx[ i ];
		DRy = LerpDest[ PNum ].Ry[ i ] - LerpStart[ PNum ].Ry[ i ];
		DRz = LerpDest[ PNum ].Rz[ i ] - LerpStart[ PNum ].Rz[ i ];

		if( DRx > 2048 ) DRx = DRx - 4096;
		else if( DRx < -2048 ) DRx = DRx + 4096;
		if( DRy > 2048 ) DRy = DRy - 4096;
		else if( DRy < -2048 ) DRy = DRy + 4096;
		if( DRz > 2048 ) DRz = DRz - 4096;
		else if( DRz < -2048 ) DRz = DRz + 4096;

		// Set the vogue (current pose)
		Vogues[ PNum ].Rx[ i ] = LerpStart[ PNum ].Rx[ i ] + ( ( DRx * LerpProgress[ PNum ] ) / LerpFrames[ PNum ] );
		Vogues[ PNum ].Ry[ i ] = LerpStart[ PNum ].Ry[ i ] + ( ( DRy * LerpProgress[ PNum ] ) / LerpFrames[ PNum ] );
		Vogues[ PNum ].Rz[ i ] = LerpStart[ PNum ].Rz[ i ] + ( ( DRz * LerpProgress[ PNum ] ) / LerpFrames[ PNum ] );
	}

	if( LerpProgress[ PNum ] == LerpFrames[ PNum ] )
	{
		Vogues[ PNum ].Tx = LerpDest[ PNum ].Tx;
		Vogues[ PNum ].Ty = LerpDest[ PNum ].Ty;
		Vogues[ PNum ].Tz = LerpDest[ PNum ].Tz;

		for( i = 0; i < Player[ PNum ].LimbDepth; i++ )
		{
			// Set the vogue (current pose)
			Vogues[ PNum ].Rx[ i ] = LerpDest[ PNum ].Rx[ i ];
			Vogues[ PNum ].Ry[ i ] = LerpDest[ PNum ].Ry[ i ];
			Vogues[ PNum ].Rz[ i ] = LerpDest[ PNum ].Rz[ i ];
		}

		AnimStatus[ PNum ] = AS_STATIC;	// Signal it has ended
	}
}



// PlayerNumber, Frame1, Frame2, Lerps between F1 and F2, Lerp Number to set to
void SetTween( uint8 PNum, uint32 Frame1, uint32 Frame2, uint16 Lerps, int16 Tween )
{
	uint16 i, j;
	chfloat *SrcTran = &PoseBuffer[ Frame1 ];		// Frame 1 trans
	int16 *SrcRot = &PoseBuffer[ Frame1 + 12 ];		// Frame 1 rots
	chfloat *DestTran = &PoseBuffer[ Frame2 ];		// Frame 2 trans
	int16 *DestRot = &PoseBuffer[ Frame2 + 12 ];	// Frame 2 rots
	chfloat DRx, DRy, DRz;

	DRx = DestTran[ 0 ] - SrcTran[ 0 ];
	DRy = DestTran[ 1 ] - SrcTran[ 1 ];
	DRz = DestTran[ 2 ] - SrcTran[ 2 ];

	Vogues[ PNum ].Tx = SrcTran[ 0 ] + ( ( DRx * Tween ) / Lerps );
	Vogues[ PNum ].Ty = SrcTran[ 1 ] + ( ( DRy * Tween ) / Lerps );
	Vogues[ PNum ].Tz = SrcTran[ 2 ] + ( ( DRz * Tween ) / Lerps );

	j = 0;
	for( i = 0; i < Player[ PNum ].LimbDepth; i++ )
	{
		DRx = DestRot[ j + 0 ] - SrcRot[ j + 0 ];
		DRy = DestRot[ j + 1 ] - SrcRot[ j + 1 ];
		DRz = DestRot[ j + 2 ] - SrcRot[ j + 2 ];

		if( DRx > 2048 ) DRx = DRx - 4096;
		else if( DRx < -2048 ) DRx = DRx + 4096;
		if( DRy > 2048 ) DRy = DRy - 4096;
		else if( DRy < -2048 ) DRy = DRy + 4096;
		if( DRz > 2048 ) DRz = DRz - 4096;
		else if( DRz < -2048 ) DRz = DRz + 4096;

		// Set the vogue (current pose)
		Vogues[ PNum ].Rx[ i ] = SrcRot[ j + 0 ] + ( ( DRx * Tween ) / Lerps );
		Vogues[ PNum ].Ry[ i ] = SrcRot[ j + 1 ] + ( ( DRy * Tween ) / Lerps );
		Vogues[ PNum ].Rz[ i ] = SrcRot[ j + 2 ] + ( ( DRz * Tween ) / Lerps );

		j += 3;	// Fast indexer
	}

	AnimStatus[ PNum ] = AS_STATIC;
}


// ******************************************************************************************************************************************************************************
// void SetTweenOfTwoTweens( uint8 PNum, uint32 Src1, uint32 Dest1, uint32 Src2, uint32 Dest2, uint16 Lerps1, int16 Tween1, uint16 Lerps2, int16 Tween2, uint16 Lerps, int16 Tween )
//
// Complex load of parameters rubbish to work out a tween between two frames (tween1) and a tween
// between two other frames (tween2) and then work out a tween frame between tween1 and tween2
// (tween).
// Each calculated tween has it's own number of lerps (Lerps1, Lerps2, Lerps) between the two
// relevant frames, and the actual tween lerp number to set it to.
// Did you understand any of that ?
// ******************************************************************************************************************************************************************************

void SetTweenOfTwoTweens( uint8 PNum, uint32 Src1, uint32 Dest1, uint32 Src2, uint32 Dest2, uint16 Lerps1, int16 Tween1, uint16 Lerps2, int16 Tween2, uint16 Lerps, int16 Tween )
{
	uint16 i, j;
	chfloat *Src1Tran = &PoseBuffer[ Src1 ];
	int16 *Src1Rot = &PoseBuffer[ Src1 + 12 ];
	chfloat *Dest1Tran = &PoseBuffer[ Dest1 ];
	int16 *Dest1Rot = &PoseBuffer[ Dest1 + 12 ];
	chfloat *Src2Tran = &PoseBuffer[ Src2 ];
	int16 *Src2Rot = &PoseBuffer[ Src2 + 12 ];
	chfloat *Dest2Tran = &PoseBuffer[ Dest2 ];
	int16 *Dest2Rot = &PoseBuffer[ Dest2 + 12 ];
	chfloat DRx, DRy, DRz;

	DRx = Dest1Tran[ 0 ] - Src1Tran[ 0 ];
	DRy = Dest1Tran[ 1 ] - Src1Tran[ 1 ];
	DRz = Dest1Tran[ 2 ] - Src1Tran[ 2 ];
	LerpStart[ PNum ].Tx = Src1Tran[ 0 ] + ( ( DRx * Tween1 ) / Lerps1 );
	LerpStart[ PNum ].Ty = Src1Tran[ 1 ] + ( ( DRy * Tween1 ) / Lerps1 );
	LerpStart[ PNum ].Tz = Src1Tran[ 2 ] + ( ( DRz * Tween1 ) / Lerps1 );
	DRx = Dest2Tran[ 0 ] - Src2Tran[ 0 ];
	DRy = Dest2Tran[ 1 ] - Src2Tran[ 1 ];
	DRz = Dest2Tran[ 2 ] - Src2Tran[ 2 ];
	LerpDest[ PNum ].Tx = Src2Tran[ 0 ] + ( ( DRx * Tween2 ) / Lerps2 );
	LerpDest[ PNum ].Ty = Src2Tran[ 1 ] + ( ( DRy * Tween2 ) / Lerps2 );
	LerpDest[ PNum ].Tz = Src2Tran[ 2 ] + ( ( DRz * Tween2 ) / Lerps2 );
	DRx = LerpDest[ PNum ].Tx - LerpStart[ PNum ].Tx;
	DRy = LerpDest[ PNum ].Ty - LerpStart[ PNum ].Ty;
	DRz = LerpDest[ PNum ].Tz - LerpStart[ PNum ].Tz;
	Vogues[ PNum ].Tx = LerpStart[ PNum ].Tx + ( ( DRx * Tween ) / Lerps );
	Vogues[ PNum ].Ty = LerpStart[ PNum ].Ty + ( ( DRy * Tween ) / Lerps );
	Vogues[ PNum ].Tz = LerpStart[ PNum ].Tz + ( ( DRz * Tween ) / Lerps );

	j = 0;
	for( i = 0; i < Player[ PNum ].LimbDepth; i++ )
	{
		// Get tween1 into vogue LerpStart
		DRx = Dest1Rot[ j + 0 ] - Src1Rot[ j + 0 ];
		DRy = Dest1Rot[ j + 1 ] - Src1Rot[ j + 1 ];
		DRz = Dest1Rot[ j + 2 ] - Src1Rot[ j + 2 ];
		if( DRx > 2048 ) DRx = DRx - 4096;
		else if( DRx < -2048 ) DRx = DRx + 4096;
		if( DRy > 2048 ) DRy = DRy - 4096;
		else if( DRy < -2048 ) DRy = DRy + 4096;
		if( DRz > 2048 ) DRz = DRz - 4096;
		else if( DRz < -2048 ) DRz = DRz + 4096;
		LerpStart[ PNum ].Rx[ i ] = Src1Rot[ j + 0 ] + ( ( DRx * Tween1 ) / Lerps1 );
		LerpStart[ PNum ].Ry[ i ] = Src1Rot[ j + 1 ] + ( ( DRy * Tween1 ) / Lerps1 );
		LerpStart[ PNum ].Rz[ i ] = Src1Rot[ j + 2 ] + ( ( DRz * Tween1 ) / Lerps1 );

		// Get tween2 into vogue LerpDest
		DRx = Dest2Rot[ j + 0 ] - Src2Rot[ j + 0 ];
		DRy = Dest2Rot[ j + 1 ] - Src2Rot[ j + 1 ];
		DRz = Dest2Rot[ j + 2 ] - Src2Rot[ j + 2 ];
		if( DRx > 2048 ) DRx = DRx - 4096;
		else if( DRx < -2048 ) DRx = DRx + 4096;
		if( DRy > 2048 ) DRy = DRy - 4096;
		else if( DRy < -2048 ) DRy = DRy + 4096;
		if( DRz > 2048 ) DRz = DRz - 4096;
		else if( DRz < -2048 ) DRz = DRz + 4096;
		LerpDest[ PNum ].Rx[ i ] = Src2Rot[ j + 0 ] + ( ( DRx * Tween2 ) / Lerps2 );
		LerpDest[ PNum ].Ry[ i ] = Src2Rot[ j + 1 ] + ( ( DRy * Tween2 ) / Lerps2 );
		LerpDest[ PNum ].Rz[ i ] = Src2Rot[ j + 2 ] + ( ( DRz * Tween2 ) / Lerps2 );

		// now tween between LerpStart and LerpDest to get the TweenOfTwoTweens
		DRx = LerpDest[ PNum ].Rx[ i ] - LerpStart[ PNum ].Rx[ i ];
		DRy = LerpDest[ PNum ].Ry[ i ] - LerpStart[ PNum ].Ry[ i ];
		DRz = LerpDest[ PNum ].Rz[ i ] - LerpStart[ PNum ].Rz[ i ];
		if( DRx > 2048 ) DRx = DRx - 4096;
		else if( DRx < -2048 ) DRx = DRx + 4096;
		if( DRy > 2048 ) DRy = DRy - 4096;
		else if( DRy < -2048 ) DRy = DRy + 4096;
		if( DRz > 2048 ) DRz = DRz - 4096;
		else if( DRz < -2048 ) DRz = DRz + 4096;
		Vogues[ PNum ].Rx[ i ] = LerpStart[ PNum ].Rx[ i ] + ( ( DRx * Tween ) / Lerps );
		Vogues[ PNum ].Ry[ i ] = LerpStart[ PNum ].Ry[ i ] + ( ( DRy * Tween ) / Lerps );
		Vogues[ PNum ].Rz[ i ] = LerpStart[ PNum ].Rz[ i ] + ( ( DRz * Tween ) / Lerps );

		j += 3;	// Fast indexer
	}
	// puff puff, pant pant

	AnimStatus[ PNum ] = AS_STATIC;
}






uint32 ConverterTable[][5] = {
//	BONE			TRIX 				GOAT 				YOYO 				YETI

{B_STA1_H1        ,T_STA1_H1        ,B_STA1_H1        ,B_STA1_H1        ,B_STA1_H1        },
{B_STA2_H1        ,T_STA2_H1        ,B_STA2_H1        ,B_STA2_H1        ,B_STA2_H1        },
				   
				   
};				   																			
																								
uint32 ConvertFrame( uint16 Character, uint32 BoneFrame )
{
	return ConverterTable[ BoneFrame / 128 ][ Character ];
}
																								
																									