// Skeleton.c

#include <stdio.h>
#include <sys/types.h>
#include <LibGTE.h>
#include <LibGPU.h>
#include <libcd.h>
#include <LibComb.h>
#include <LibEtc.h>
#include <LibGS.h>
#include <LibMath.h>
#include <LibPress.h>
#include <LibSN.h>
#include <LibSnd.h>
#include <LibSPU.h>
#include <inline_o.h>
#include <gtemac.h>


#include "ourtypes.h"
#include "mathbits.h"

#include "main.h"
#include "skeleton.h"
#include "camera.h"
#include "tracks.h"
#include "players.h"
#include "pads.h"

#include "screen.h"
#include "who.h"
#include "bored.h"
#include "animctrl.h"

#include "bonebag.h"

//#define LIMBS	// Limb manipulator and skeleton debug mode

uint16 Tp;
uint16 Nm;
uint16 Joints;

MATRIX TransMat;
MATRIX RotMat;

// and the verts ( these change all the time )
uint16 NumberModVerts;
SVECTOR ModVerts[ MAX_MODEL_VERTS ];

LINE_F2 TPrims[ 2 ][ 300 ];
POLY_F4 NodeMarker[ 2 ][ 300 ];


player *Plyr;	// This hold the current player for all the routines to access

uint16 LimbDepth;
uint16 Limb;

// Ordering Table for the skeleton
uint32	SkellyOT[4][512];
uint8 SkeletonMode;


// *********************************************************************************
// void Initialise_Skeleton( void )
//
// Called before starting each race.  Initialises everything to do with the skeleton
// and characters of each player.
// *********************************************************************************

void Initialise_Skeleton( void )
{
	static uint8 CalledBefore = 0;
	uint8 i;

	printf("Exec : Initialise_Skeleton");

	if( CalledBefore == 0 )
	{
		// Cold Initialise
		printf(" - Cold");

		CalledBefore = 1;

		for( i = 0; i < MAX_PLAYERS; i++ )
		{
			Player[ i ].Character = BONE;	// Each player initially wants to be BONE
			Player[ i ].Loaded = NONE;		// But no character is loaded
			Player[ i ].PoseBuffer = (uint8 *)&PoseBuffer[ i ];
			Player[ i ].PoseLoaded = NONE;
		}
		SkeletonMode = 0;

	}

	printf("\n");

	// Warm Initialise
	Limb = 0;

	Init_Bored();

	for( i = 0; i < MAX_PLAYERS; i++ )
	{
		if( Player[ i ].Character != Player[ i ].Loaded )
		{
			// This player does not have the required skeleton.  We need to load it.
			char BagName[ 32 ];
			uint32 FileHandle;
			long FileSize;

			#ifndef CD
			sprintf( BagName, "BAGS/BONE.BAG" );
			#else
			sprintf( BagName, "\\BAGS\\BONE.BAG;1" );
			#endif

			switch( Player[ i ].Character )
			{
				case GOAT:
				{
				}
				break;

				case BONE:
				{
					Player[ i ].CharNumVerts = BONE_NUM_VERTS;
					Player[ i ].CharVerts = (chfloat *)&BONE_VERTS;
					Player[ i ].CharNumPolys = BONE_NUM_POLYS;
					Player[ i ].CharPolys = (model_polys *)&BONE_POLYS;
					Player[ i ].SkeletonRoot = (node *)&BONE_ROOT;
					Player[ i ].CharNorms = (chfloat *)&BONE_NORMS;
					Player[ i ].LimbDepth = 19;

					#ifndef CD
					sprintf( BagName, "BAGS/BONE.BAG" );
					#else
					sprintf( BagName, "\\BAGS\\BONE.BAG;1" );
					#endif
				}
				break;

				case TRIX:
				{
					Player[ i ].CharNumVerts = TRIX_NUM_VERTS;
					Player[ i ].CharVerts = (chfloat *)&TRIX_VERTS;
					Player[ i ].CharNumPolys = TRIX_NUM_POLYS;
					Player[ i ].CharPolys = (model_polys *)&TRIX_POLYS;
					Player[ i ].SkeletonRoot = (node *)&TRIX_ROOT;
					Player[ i ].CharNorms = (chfloat *)&TRIX_NORMS;
				}
				break;

				case YOYO:
				{
					Player[ i ].CharNumVerts = YOYO_NUM_VERTS;
					Player[ i ].CharVerts = (chfloat *)&YOYO_VERTS;
					Player[ i ].CharNumPolys = YOYO_NUM_POLYS;
					Player[ i ].CharPolys = (model_polys *)&YOYO_POLYS;
					Player[ i ].SkeletonRoot = (node *)&YOYO_ROOT;
					Player[ i ].CharNorms = (chfloat *)&YOYO_NORMS;
				}
				break;

				case YETI:
				{
				}
				break;
			}

			// Ensure the correct pose bag is loaded
//		   	SetPose( i, 0 );
			if( Player[ i ].PoseLoaded != Player[ i ].Character )
			{
				#ifndef CD
					printf( "PC Loading %s\n", BagName );
					FileHandle = PCopen( (char*)&BagName, 0, 0 );
					FileSize = PClseek( FileHandle, 0, 2 );
					PClseek( FileHandle, 0, 0 );
					PCread( FileHandle, &PoseBuffer[ i ][ 0 ], FileSize );
					PCclose( FileHandle );
				#else
					printf( "CD Loading %s\n", BagName );
					load_cd( &BagName, &PoseBuffer[ i ][ 0 ] );
					while (CdReadSync(1,0) > 0 );
				#endif
			}

			SetPose( i, 0 );	// Set this character to the first frame

			Player[ i ].PoseLoaded = Player[ i ].Character;
			Player[ i ].Loaded = Player[ i ].Character;
		}
	}
}

// ******************************************************************************
// void Draw_Character( uint8 PlayerNumber )
//
// Calculates the required pose, then transforms the monged vertices to that pose
// and finally renders the characters polygons.
// ******************************************************************************
VECTOR ScaleUp = { 4<<12, 4<<12, 4<<12 };
VECTOR ScaleDown = { 1024, 1024, 1024 };
vec3 EmptyVec = { CHFL( 0.0 ), CHFL( 0.0 ), CHFL( 0.0 ) };
chmatrix IDMatrix;

uint8 TestScale( void )
{
	if( ScaleUp.vx != 16384 )
	{
		return 1;
	}
	return 0;
}

int32 TosserX, TosserY, TosserZ;

void Draw_Character( uint8 PlayerNumber )
{
	SVECTOR ObjRot, T1;
	VECTOR T2;
	static MATRIX ObjectRotation, ObjectTrans;
	uint16 j;
	uint32 i, index;

//	PSX_Matrix_Init( &ObjectRotation );
//	PSX_Matrix_Init( &ObjectTrans );

	Plyr = &Player[ PlayerNumber ];

	Control_Anim( PlayerNumber );	// Animation controller

	Matrix_Init( &IDMatrix );

	// inlined Create_Local_Model();
	NumberModVerts = Plyr->CharNumVerts;
	index = 0;
	for( i = 0; i < NumberModVerts; i++ )
	{
		ModVerts[ i ].vx = Plyr->CharVerts[ index + 0 ] >> 12;
		ModVerts[ i ].vy = Plyr->CharVerts[ index + 1 ] >> 12;
		ModVerts[ i ].vz = Plyr->CharVerts[ index + 2 ] >> 12;
		index += 3;
	}

	// This is how I want the GTE
	PushMatrix();
	SetGeomOffset( 120, 160 );

	Prepare_Skeleton( PlayerNumber );

	Traverse_Matrix_Skeleton( PlayerNumber, (node *)Plyr->SkeletonRoot, IDMatrix, EmptyVec, EmptyVec );

	for( j = 0; j < Plyr->CharNumVerts; j++ )
	{
		#ifndef LIMBS
		ModVerts[ j ].vx -= Vogues[ PlayerNumber ].Tx;
		ModVerts[ j ].vy += Vogues[ PlayerNumber ].Ty;
		ModVerts[ j ].vz += Vogues[ PlayerNumber ].Tz;
		#endif
	}

	SetRotMatrix( &Plyr->Cam.ViewMatrix );

	ObjRot.vx = CHSHIFTR( Plyr->Rot.X );
	ObjRot.vy = CHSHIFTR( Plyr->Rot.Y );
	ObjRot.vz = CHSHIFTR( Plyr->Rot.Z );

	T1.vx = CHSHIFTR( -Plyr->Pos.X ) + CHSHIFTR( Plyr->Cam.Pos.X );
	T1.vy = CHSHIFTR( Plyr->Pos.Y ) - CHSHIFTR( Plyr->Cam.Pos.Y );
	T1.vz = CHSHIFTR( Plyr->Pos.Z ) - CHSHIFTR( Plyr->Cam.Pos.Z );

	ApplyRotMatrix( &T1, &T2 );
	ObjectTrans.t[ 1 ] = T2.vx;
	ObjectTrans.t[ 0 ] = T2.vy;
	ObjectTrans.t[ 2 ] = T2.vz;

	RotMatrixYXZ( &ObjRot, &ObjectRotation );

	SetRotMatrix( &Plyr->Cam.ObjectView );
	MulRotMatrix( &ObjectRotation );

	ScaleMatrix( &ObjectRotation, &ScaleDown );	// Scale the x4 man down to normal

	SetRotMatrix( &ObjectRotation );
	SetTransMatrix( &ObjectTrans );

	// Debugging skeleton drawer code
	if( SkeletonMode )
		Draw_Skeleton_3d( (node *)Plyr->SkeletonRoot, CHFL( 0.0 ), CHFL( 0.0 ), CHFL( 0.0 ), CHFL( 0.0 ), CHFL( 0.0 ), CHFL( 0.0 ) );

	// Draw the character
	Draw_Local_Model();

	// Draw the snowboard
	ScaleMatrix( &ObjectRotation, &ScaleUp );	// Reset PSX object matrix to correct scale
	SetRotMatrix( &ObjectRotation );
	Draw_Bored();

	// temporary speed bar
	#if I_AM == JEZ
	{
		static POLY_F4 tSpeedBar[ 2 ];
		static POLY_F4 tSpeedBar2[ 2 ];
		static POLY_F4 tSpeedBar3[ 2 ];

		uint16 Spam = Plyr->Vel / ( CHFL( 30.0 ) / 150 );
		uint16 Spam2 = Plyr->Credibility / ( CHFL( 30.0 ) / 150 );

		setPolyF4( &tSpeedBar[ otid ] );
		setPolyF4( &tSpeedBar2[ otid ] );
		setPolyF4( &tSpeedBar3[ otid ] );

		tSpeedBar[ otid ].r0 = 192;
		tSpeedBar[ otid ].g0 = 0;
		tSpeedBar[ otid ].b0 = 0;
		tSpeedBar2[ otid ].r0 = 0;
		tSpeedBar2[ otid ].g0 = 0;
		tSpeedBar2[ otid ].b0 = 0;
		tSpeedBar3[ otid ].r0 = 0;
		tSpeedBar3[ otid ].g0 = 192;
		tSpeedBar3[ otid ].b0 = 0;
		if( CRUISING )
		{
			tSpeedBar2[ otid ].r0 = 0;
			tSpeedBar2[ otid ].g0 = 0;
			tSpeedBar2[ otid ].b0 = 192;
		}
		tSpeedBar[ otid ].x0 = 16;
		tSpeedBar[ otid ].y0 = 52;
		tSpeedBar[ otid ].x2 = 16;
		tSpeedBar[ otid ].y2 = 60;
		tSpeedBar[ otid ].x1 = 16 + Spam;
		tSpeedBar[ otid ].y1 = 52;
		tSpeedBar[ otid ].x3 = 16 + Spam;
		tSpeedBar[ otid ].y3 = 60;

		tSpeedBar2[ otid ].x0 = 15;
		tSpeedBar2[ otid ].y0 = 51;
		tSpeedBar2[ otid ].x2 = 15;
		tSpeedBar2[ otid ].y2 = 61;
		tSpeedBar2[ otid ].x1 = 17 + 150;
		tSpeedBar2[ otid ].y1 = 51;
		tSpeedBar2[ otid ].x3 = 17 + 150;
		tSpeedBar2[ otid ].y3 = 61;

		tSpeedBar3[ otid ].x0 = 15;
		tSpeedBar3[ otid ].y0 = 51;
		tSpeedBar3[ otid ].x2 = 15;
		tSpeedBar3[ otid ].y2 = 61;
		tSpeedBar3[ otid ].x1 = 17 + Spam2;
		tSpeedBar3[ otid ].y1 = 51;
		tSpeedBar3[ otid ].x3 = 17 + Spam2;
		tSpeedBar3[ otid ].y3 = 61;

		AddPrim( &OrderingTable[ otid ][ 0 ], &tSpeedBar[ otid ] );
		AddPrim( &OrderingTable[ otid ][ 0 ], &tSpeedBar3[ otid ] );
		AddPrim( &OrderingTable[ otid ][ 0 ], &tSpeedBar2[ otid ] );
	}
	#endif

	// Reset corrupted GTE settings for tidy exit
	SetGeomOffset( 160, 120 );

	PopMatrix();
//	SetRotMatrix( &Plyr->Cam.ViewMatrix );
//	SetTransMatrix( &Plyr->Cam.ViewMatrix );
}


// *******************************************************************************************
// void Prepare_Skeleton( uint8 PlayerNumber )
//
// Pre-Draw initialise of the individual appropriate skeleton (routine scheduled to be dumped)
// *******************************************************************************************
void Prepare_Skeleton( uint8 PlayerNumber )
{
	Tp = 0;
	Nm = 0;
	Joints = 0;

	LimbDepth = 0;

	// Limb manipulator
	#ifdef LIMBS
	if( Pad_State( 1 ) == ( PAD_B2 + PAD_UP ) )
	{
		if( Limb > 0 ) Limb--;
	}

	if( Pad_State( 1 ) == ( PAD_B2 + PAD_DOWN ) )
	{
		Limb++;
	}
	#endif

	// Enable skeleton mode
	#if I_AM == JEZ
	if( Pad_Debounced( 1 ) == PAD_START )
	{
		SkeletonMode ^= 1;
	}
	#endif
}



void Traverse_Matrix_Skeleton( uint8 PNum, node *Node, chmatrix Rp, vec3 Cursed, vec3 Unt )
{
	uint16 i;
	chmatrix XRot, Scratch;//, YRot, ZRot, ID;
	vec3 Local, TLocal;
	int16 *Verts = Node->Attached;
	int16 Vn, Vnx3;
	vec3 T, Tn;
	vec3 Pass;

	Pass.X = Node->Tx + Unt.X;
	Pass.Y = Node->Ty + Unt.Y;
	Pass.Z = Node->Tz + Unt.Z;

//	Matrix_Init( &XRot );
//	Matrix_Init( &YRot );
//	Matrix_Init( &ZRot );
//	Matrix_Init( &ID );

	Local.X = Node->Tx;
	Local.Y = Node->Ty;
	Local.Z = Node->Tz;

	Vec3_x_Matrix( &Local, &Rp, &TLocal );

	TLocal.X += Cursed.X;
	TLocal.Y += Cursed.Y;
	TLocal.Z += Cursed.Z;

	if( SkeletonMode )
	{
		Node->X = CHSHIFTR( TLocal.X );	// These bits of code only needed to draw skeleton
		Node->Y = CHSHIFTR( TLocal.Y );
		Node->Z = CHSHIFTR( TLocal.Z );
	}
/*
	#ifndef LIMBS
 	Matrix_MakeRotX( &XRot, Vogues[ PNum ].Rx[ LimbDepth ] );	// Formerly Node->Rx
	Matrix_MakeRotY( &YRot, -Vogues[ PNum ].Ry[ LimbDepth ] );
	Matrix_MakeRotZ( &ZRot, Vogues[ PNum ].Rz[ LimbDepth ] );
	#endif

	#ifdef LIMBS
	Matrix_MakeRotX( &XRot, Node->Rx );							// Formerly Node->Rx
	Matrix_MakeRotY( &YRot, -Node->Ry );
	Matrix_MakeRotZ( &ZRot, Node->Rz );
	#endif

	Matrix_x_Matrix( &ID, &XRot, &Scratch );
	Matrix_x_Matrix( &Scratch, &YRot, &ID );
	Matrix_x_Matrix( &ID, &ZRot, &XRot );
	Matrix_x_Matrix( &XRot, &Rp, &Scratch );
 */

	#ifndef LIMBS
	Matrix_MakeRotXYZ( &XRot, Vogues[ PNum ].Rx[ LimbDepth ], Vogues[ PNum ].Ry[ LimbDepth ], Vogues[ PNum ].Rz[ LimbDepth ] );
	#endif

	#ifdef LIMBS
	Matrix_MakeRotXYZ( &XRot, Node->Rx, Node->Ry, Node->Rz );
	#endif

	Matrix_x_Matrix( &XRot, &Rp, &Scratch );

	if( Node->Type == JOINT )	// There may be an attachment here
	{
		i = 0;

		while( *( Verts + i ) != -1 )
		{
			Vn = *( Verts + i );
			Vnx3 = Vn * 3;

			T.X = Plyr->CharVerts[ Vnx3 + 0 ] - Pass.X;
			T.Y = Plyr->CharVerts[ Vnx3 + 1 ] - Pass.Y;
			T.Z = Plyr->CharVerts[ Vnx3 + 2 ] - Pass.Z;

			Vec3_x_Matrix( &T, &Scratch, &Tn );

			ModVerts[ Vn ].vx = -CHSHIFTR( Tn.X +TLocal.X );
			ModVerts[ Vn ].vy = CHSHIFTR( Tn.Y + TLocal.Y );
			ModVerts[ Vn ].vz = CHSHIFTR( Tn.Z + TLocal.Z );

			i++;
		}
	}

	// Limb manipulator
	#ifdef LIMBS
	if( LimbDepth == Limb )
	{
		if( Pad_State( 1 ) == ( PAD_B2 + PAD_BL ) ) Node->Rx -= CHANG( 5.0 );
		if( Pad_State( 1 ) == ( PAD_B2 + PAD_BR ) ) Node->Rx += CHANG( 5.0 );
		if( Pad_State( 1 ) == ( PAD_B3 + PAD_BL ) ) Node->Ry -= CHANG( 5.0 );
		if( Pad_State( 1 ) == ( PAD_B3 + PAD_BR ) ) Node->Ry += CHANG( 5.0 );
		if( Pad_State( 1 ) == ( PAD_B4 + PAD_BL ) ) Node->Rz -= CHANG( 5.0 );
		if( Pad_State( 1 ) == ( PAD_B4 + PAD_BR ) ) Node->Rz += CHANG( 5.0 );
	}
	#endif

	LimbDepth++;

	for( i = 0; i < Node->NumChildren; i++ )
	{
		if( Node->Child[ i ] != NULL )
		{
			Traverse_Matrix_Skeleton( PNum, (node*)Node->Child[ i ], Scratch, TLocal, Pass );
		}
	}
}



void Draw_Skeleton_3d( node *Node, chfloat X, chfloat Y, chfloat Z, chfloat Rx, chfloat Ry, chfloat Rz )
{
	uint16 i;

	SVECTOR A, B;

	long D1, D2, P;
	long Flag;

	// Transform my start, which has been passed on from my parent's end
	A.vx = -Node->X;
	A.vy = Node->Y;
	A.vz = Node->Z;
//	RotTransPers( &A, &D1, &P, &Flag );
	gte_ldv0(&A);
	gte_rtps();

	OTPtr = &OrderingTable[ otid ][ 0 ];

	gte_stsxy(&D1);
	switch( Node->Type )
	{
		case CHAIN_ROOT:
			setLineF2( &TPrims[ otid ][ Tp ] );
			TPrims[ otid ][ Tp ].r0 = 255;
			TPrims[ otid ][ Tp ].g0 = 0;
			TPrims[ otid ][ Tp ].b0 = 0;
		 	TPrims[ otid ][ Tp ].x0 = -4 + ( D1 >> 16 );
			TPrims[ otid ][ Tp ].y0 = -4 + ( D1 & 0xffff );
		 	TPrims[ otid ][ Tp ].x1 = 4 + ( D1 >> 16 );
			TPrims[ otid ][ Tp ].y1 = 4 + ( D1 & 0xffff );
			AddPrim( OTPtr, &TPrims[ otid ][ Tp ] );
			Tp++;
			setLineF2( &TPrims[ otid ][ Tp ] );
			TPrims[ otid ][ Tp ].r0 = 255;
			TPrims[ otid ][ Tp ].g0 = 0;
			TPrims[ otid ][ Tp ].b0 = 0;
		 	TPrims[ otid ][ Tp ].x0 = +4 + ( D1 >> 16 );
			TPrims[ otid ][ Tp ].y0 = -4 + ( D1 & 0xffff );
		 	TPrims[ otid ][ Tp ].x1 = -4 + ( D1 >> 16 );
			TPrims[ otid ][ Tp ].y1 = 4 + ( D1 & 0xffff );
			AddPrim( OTPtr, &TPrims[ otid ][ Tp ] );
			Tp++;
			break;

		case EFFECTOR:
			setLineF2( &TPrims[ otid ][ Tp ] );
			TPrims[ otid ][ Tp ].r0 = 0;
			TPrims[ otid ][ Tp ].g0 = 0;
			TPrims[ otid ][ Tp ].b0 = 255;
		 	TPrims[ otid ][ Tp ].x0 = -4 + ( D1 >> 16 );
			TPrims[ otid ][ Tp ].y0 = -4 + ( D1 & 0xffff );
		 	TPrims[ otid ][ Tp ].x1 = 4 + ( D1 >> 16 );
			TPrims[ otid ][ Tp ].y1 = 4 + ( D1 & 0xffff );
			AddPrim( OTPtr, &TPrims[ otid ][ Tp ] );
			Tp++;
			setLineF2( &TPrims[ otid ][ Tp ] );
			TPrims[ otid ][ Tp ].r0 = 0;
			TPrims[ otid ][ Tp ].g0 = 0;
			TPrims[ otid ][ Tp ].b0 = 255;
		 	TPrims[ otid ][ Tp ].x0 = +4 + ( D1 >> 16 );
			TPrims[ otid ][ Tp ].y0 = -4 + ( D1 & 0xffff );
		 	TPrims[ otid ][ Tp ].x1 = -4 + ( D1 >> 16 );
			TPrims[ otid ][ Tp ].y1 = 4 + ( D1 & 0xffff );
			AddPrim( OTPtr, &TPrims[ otid ][ Tp ] );
			Tp++;
			break;

		case JOINT:
			if( (node*)Node->Child[ 0 ] == NULL )
			{
				setLineF2( &TPrims[ otid ][ Tp ] );
				TPrims[ otid ][ Tp ].r0 = 0;
				TPrims[ otid ][ Tp ].g0 = 255;
				TPrims[ otid ][ Tp ].b0 = 0;
			 	TPrims[ otid ][ Tp ].x0 = -2 + ( D1 >> 16 );
				TPrims[ otid ][ Tp ].y0 = -2 + ( D1 & 0xffff );
			 	TPrims[ otid ][ Tp ].x1 = 2 + ( D1 >> 16 );
				TPrims[ otid ][ Tp ].y1 = 2 + ( D1 & 0xffff );
				AddPrim( OTPtr, &TPrims[ otid ][ Tp ] );
				Tp++;
				setLineF2( &TPrims[ otid ][ Tp ] );
				TPrims[ otid ][ Tp ].r0 = 0;
				TPrims[ otid ][ Tp ].g0 = 255;
				TPrims[ otid ][ Tp ].b0 = 0;
			 	TPrims[ otid ][ Tp ].x0 = +2 + ( D1 >> 16 );
				TPrims[ otid ][ Tp ].y0 = -2 + ( D1 & 0xffff );
			 	TPrims[ otid ][ Tp ].x1 = -2 + ( D1 >> 16 );
				TPrims[ otid ][ Tp ].y1 = 2 + ( D1 & 0xffff );
				AddPrim( OTPtr, &TPrims[ otid ][ Tp ] );
				Tp++;
			}
			else
			{
				uint8 g;
				for( g = 0; g < Node->NumChildren; g++ )
				{
				B.vx = -((node*)Node->Child[ g ])->X;
				B.vy = ((node*)Node->Child[ g ])->Y;
				B.vz = ((node*)Node->Child[ g ])->Z;
				RotTransPers( &B, &D2, &P, &Flag );

				setLineF2( &TPrims[ otid ][ Tp ] );
				TPrims[ otid ][ Tp ].r0 = 0;
				TPrims[ otid ][ Tp ].g0 = 0;
				TPrims[ otid ][ Tp ].b0 = 0;
			 	TPrims[ otid ][ Tp ].x0 = 0 + ( D1 >> 16 );
				TPrims[ otid ][ Tp ].y0 = 0 + ( D1 & 0xffff );
			 	TPrims[ otid ][ Tp ].x1 = 0 + ( D2 >> 16 );
				TPrims[ otid ][ Tp ].y1 = 0 + ( D2 & 0xffff );
				AddPrim( OTPtr, &TPrims[ otid ][ Tp ] );
				Tp++;
				}
			}
			break;
	}

	// Now loop through my children
	for( i = 0; i < Node->NumChildren; i++ )
	{
		if( Node->Child[ i ] != NULL )
		{
			Draw_Skeleton_3d( (node*)Node->Child[ i ], Node->X, Node->Y, Node->Z, Rx, Ry, Rz );
		}
	}
}









// Separate primitive list (for any primitive type)
int8 AnyType[ 2 ][ MAX_MODEL_POLYS * sizeof( POLY_GT4 ) ];	// (this is only big enough for one character at the moment)
void *Ptr;

// Call these once each loop
void Refresh_Skeleton( void )
{
	Ptr = &AnyType[ otid ][ 0 ];				// Point to the start of the skeleton primitive list
//	ClearOTagR( &OrderingTable[ otid ][ 0 ], 512 );	// Clear the primitive list
}


SVECTOR N0, N1, N2, N3;
CVECTOR C0, C1, C2, C3;

// Each player may be drawn twice you know ( split screen mode )
void Draw_Local_Model( void )
{
	uint32 NumberModPolys =	Plyr->CharNumPolys;	// How many polys this player has
	model_polys *ModPolys = Plyr->CharPolys;	// Point to this players polygon list
	chfloat *Norms = Plyr->CharNorms;			// Point to the players normals

	// Loop through all the polygons in ModPolys
	static long TVert[ 300 ], Z[ 300 ], P, Otz, Flag;
	long Val;
	uint16 i, v0, v1, v2, v3;

	for( i = 0; i < NumberModVerts; i++ )
	{
//		gte_RotTransPers( &ModVerts[ i ], &TVert[ i ], &P, &Flag, &Z[ i ] );
//		Z[ i ] = RotTransPers( &ModVerts[ i ], &TVert[ i ], &P, &Flag );
		gte_ldv0(&ModVerts[i]);
        gte_rtps();
        gte_stsxy(&TVert[i]);
//      gte_stdp(r3);
//      gte_stflg(r4);
		gte_stszotz(&Z[i]);

	}

	for( i = 0; i < NumberModPolys; i++ )
	{
		v0 = ModPolys[ i ].VertNum[ 0 ];
		v1 = ModPolys[ i ].VertNum[ 1 ];
		v2 = ModPolys[ i ].VertNum[ 2 ];

		if( ModPolys[ i ].NumberOfVerts == 3 )
		{
			// It's a triangle
			gte_NormalClip( TVert[ v0 ], TVert[ v1 ], TVert[ v2 ] ,&Val);

			if( Val >= 0 )
			{
				switch( ModPolys[ i ].Type )
				{
						case GOURAUD:
						{

							POLY_G3 *G3 = Ptr;
							Ptr += sizeof( POLY_G3 );

							setPolyG3( G3 );

							gte_NormalColorCol3(	&Norms[ v0 * 3 ], &Norms[ v1 * 3 ], &Norms[ v2 * 3 ],
												&ModPolys[ i ].RGB,
												&C0, &C1, &C2 );

							Otz = AverageZ3( Z[ v0 ], Z[ v1 ], Z[ v2 ] );

							G3->r0 = C0.r;
							G3->g0 = C0.g;
							G3->b0 = C0.b;
							G3->r1 = C1.r;
							G3->g1 = C1.g;
							G3->b1 = C1.b;
							G3->r2 = C2.r;
							G3->g2 = C2.g;
							G3->b2 = C2.b;

							G3->x0 = TVert[ v0 ] >> 16;
							G3->y0 = TVert[ v0 ] & 0xffff;
							G3->x1 = TVert[ v1 ] >> 16;
							G3->y1 = TVert[ v1 ] & 0xffff;
							G3->x2 = TVert[ v2 ] >> 16;
							G3->y2 = TVert[ v2 ] & 0xffff;

							if( Otz < 512 && Otz > 0 )
							{
								Otz+=100;
								Otz=(Cnt*200)+(Otz-((Otz/100)*100));
								AddPrim( &OrderingTable[ otid ][ Otz ], G3 );
							}
						}
				}
			}
		}	// end of triangle render
		else
		{
			// It's a quad
			v3 = ModPolys[ i ].VertNum[ 3 ];

			gte_NormalClip( TVert[ v0 ], TVert[ v1 ], TVert[ v2 ], &Val );

			if( Val >= 0 )
			{
				switch( ModPolys[ i ].Type )
				{
				 	case GOURAUD:
				  	{
				  		POLY_G4 *G4 = Ptr;
				  		Ptr += sizeof( POLY_G4 );
				  		setPolyG4( G4 );

				  		gte_NormalColorCol3(	&Norms[ v0 * 3 ], &Norms[ v1 * 3 ], &Norms[ v2 * 3 ],
				  							&ModPolys[ i ].RGB,
				  							&C0, &C1, &C2 );
				  		gte_NormalColorCol(	&Norms[ v3 * 3 ],
		  								&ModPolys[ i ].RGB,
		  								&C3 );

				  		Otz = AverageZ4( Z[ v0 ], Z[ v1 ], Z[ v2 ], Z[ v3 ] );

				  		G4->r0 = C0.r;
				  		G4->g0 = C0.g;
				  		G4->b0 = C0.b;
				  		G4->r1 = C1.r;
				  		G4->g1 = C1.g;
				  		G4->b1 = C1.b;
				  		G4->r2 = C3.r;
				  		G4->g2 = C3.g;
				  		G4->b2 = C3.b;
				  		G4->r3 = C2.r;
				  		G4->g3 = C2.g;
				  		G4->b3 = C2.b;

				  		G4->x0 = TVert[ v0 ] >> 16;
				  		G4->y0 = TVert[ v0 ] & 0xffff;
				  		G4->x1 = TVert[ v1 ] >> 16;
				  		G4->y1 = TVert[ v1 ] & 0xffff;
			  			G4->x2 = TVert[ v3 ] >> 16;
				  		G4->y2 = TVert[ v3 ] & 0xffff;
			  			G4->x3 = TVert[ v2 ] >> 16;
				  		G4->y3 = TVert[ v2 ] & 0xffff;

				  		if( Otz < 512 && Otz > 0 )
			  			{
							Otz+=100;
							Otz=(Cnt*200)+(Otz-((Otz/100)*100));
							AddPrim( &OrderingTable[ otid ][ Otz ], G4 );
				  		}
				  	}
				  	break;

				  	case TEXTURED:
				  	{
				  		POLY_GT3 *GT3 = Ptr;
				  		printf("We haven't catered for textured polys - yet\n");
				  	}
				  	break;
				}
			}
		}	// end of quad render
	}	// end of all polygon rendering
}











