// GHOSTREP.C

// Handles ghost and replay mode

#include <stdio.h>
#include "sys/types.h"
#include "LibGTE.h"
#include "LibGPU.h"
#include "LibCD.h"
#include "LibComb.h"
#include "LibEtc.h"
#include "LibGS.h"
#include "LibMath.h"
#include "LibPress.h"
#include "LibSN.h"
#include "LibSnd.h"
#include "LibSPU.h"
#include "ourtypes.h"
#include "main.h"
#include "mathbits.h"
#include "camera.h"
#include "pads.h"
#include "tracks.h"
#include "players.h"
#include "logic.h"
#include "animctrl.h"
#include "bonebag.h"
#include "ghostrep.h"
#include "skeleton.h"

// In 1 player
// Buffer 0 = Replay and Record buffer
// Buffer 1 = Stored ghost buffer

// In 2 player
// Buffer 0 = Record and Replay data of player 1
// Buffer 1 = Record and Replay data of player 2

// CONSOLE_PLAYER	- Player under human control being recorded
// REMOTE_PLAYER	- Player following pre-recorded trail (for replay & demo)
// GHOSTED_PLAYER	- Ghost player

// 0 = Keys
// 1 = Collision
// 2 = XYZ
// 3 = Hit the wall


uint8	GhostData = 0;
uint8	GhostBuffer[ 2 ][ RECORDBUFFERSIZE ];
uint8	Validity[ 2 ] = { 0, 0 };
void *Pointers[ MAX_PLAYERS ];

uint32 Count[ 2 ];

void Init_Ghost( void )
{
	Pointers[ 0 ] = &GhostBuffer[ 0 ][ 0 ];
	Pointers[ 1 ] = &GhostBuffer[ 1 ][ 0 ];
	Validity[ 0 ] = 1;
	Validity[ 1 ] = 1;
//	clockon[ 0 ] = 1;

	Count[ 0 ] = 0;
	Count[ 1 ] = 0;
}

void UpdateGhostData( void )
{
	if( Validity[ 0 ] == 1 )
	{
		memcpy( &GhostBuffer[ 1 ][ 0 ], &GhostBuffer[ 0 ][ 0 ], RECORDBUFFERSIZE );
		GhostData = 1;
		printf("Ghost data has been updated\n");
	}
	else
	{
		GhostData = 0;
		printf("Ghost data has not been updated as data was invalid\n");
	}
}

void HandleGhosts( void )
{
	uint8 i;
	for( i = 0; i < NumberOfPlayers; i++ )
	{
		GhostPacket *Pack;
		player *Plyr = &Player[ i ];

		switch( Plyr->ConsoleRemoteGhosted )
		{
			case CONSOLE_PLAYER:	// Record the information in the appropriate buffer
				if( Validity[ i ] == 1 )
				{
					Pack = Pointers[ i ];
					Validity[ i ] = 1;

					Pack->Type = 0;	// Signal it's a normal type of packet
					Pack->PadData = Paddys[ Plyr->PadNumber ];

					Pack++;

					if( Pack >= ( &GhostBuffer[ i ][ 0 ] + RECORDBUFFERSIZE ) )
					{
						Validity[ i ] = 0;
						printf("Error : Ran out of memory to record ghost data\n");
//						clockon[ 0 ] = 0;
					}

					Pack->Type = 15;		// Tag the end marker on
					Pointers[ i ] = Pack;
				}
				else
				{
//					printf("Error : No more memory available for recording of ghost data\n");
				}
				break;

			case REMOTE_PLAYER:
				Pack = Pointers[ i ];	// RemoteBuffer

				if( Pack->Type == 15 )
				{
					// It's the end of the data - return nothing
					Paddys[ Plyr->PadNumber ] = 0;
					break;
				}

				if( Pack->Type != 0 )
				{
					printf("Inconsistency in remote data\n");
				}
				else
					Paddys[ Plyr->PadNumber ] = Pack->PadData;

				Pack++;
				Pointers[ i ] = Pack;
				break;

			case GHOSTED_PLAYER:
				if( GhostData == 1 )
				{
					Pack = Pointers[ 1 ];	// GhostBuffer

					if( Pack->Type == 15 )
					{
						// It's the end of the data - return nothing
						Paddys[ Plyr->PadNumber ] = 0;
						break;
					}

					if( Pack->Type != 0 )
					{
						printf("Inconsistency in ghost data\n");
					}
					else
						Paddys[ Plyr->PadNumber ] = Pack->PadData;

					Pack++;
					Pointers[ 1 ] = Pack;
				}
				break;
		}
	}
}





uint16 RetrieveHitTheWallPacket( uint8 PlayerNumber )
{
	HitTheWallPacket *HitPack = Pointers[ PlayerNumber ];

	if( PermitMove == 1 )
	{

	if( HitPack->Type == 3 )
	{
		Player[ PlayerNumber ].LastPos.X = Player[ PlayerNumber ].Pos.X = HitPack->X;
		Player[ PlayerNumber ].LastPos.Y = Player[ PlayerNumber ].Pos.Y = HitPack->Y;
		Player[ PlayerNumber ].LastPos.Z = Player[ PlayerNumber ].Pos.Z = HitPack->Z;

		HitPack++;
		Pointers[ PlayerNumber ] = HitPack;
		return 1;
	}

	}

	return 0;
}


void InsertHitTheWallPacket( uint8 PlayerNumber )
{
	HitTheWallPacket *HitPack = Pointers[ PlayerNumber ];

	if( PermitMove == 1 )
	{

	printf("Inserting hit the wall into replay\n");

   	HitPack->Type = 3;	// Signal that it is an XYZPacket

	// insert XYZ data into the packet
	HitPack->X = Player[ PlayerNumber ].Pos.X;
	HitPack->Y = Player[ PlayerNumber ].Pos.Y;
	HitPack->Z = Player[ PlayerNumber ].Pos.Z;

	HitPack++;
	Pointers[ PlayerNumber ] = HitPack;

	HitPack->Type = 15;

	}
}







void RetrieveXYZPacket( uint8 PlayerNumber )
{
		XYZPacket *XYZPack = Pointers[ PlayerNumber ];

	if( PermitMove == 1 )
	{
		if( XYZPack->Type == 2 )
		{
			Player[ PlayerNumber ].Pos.X = XYZPack->X;
			Player[ PlayerNumber ].Pos.Y = XYZPack->Y;
			Player[ PlayerNumber ].Pos.Z = XYZPack->Z;

			XYZPack++;
			Pointers[ PlayerNumber ] = XYZPack;
		}
	}
}


void InsertXYZPacket( uint8 PlayerNumber )
{
	if( PermitMove == 1 )
	{
	Count[ PlayerNumber ]++;
	Count[ PlayerNumber ]&=127;
	if( Count[ PlayerNumber ] == 0 )
	{
		XYZPacket *XYZPack = Pointers[ PlayerNumber ];

	   	XYZPack->Type = 2;	// Signal that it is an XYZPacket

		// insert XYZ data into the packet
		XYZPack->X = Player[ PlayerNumber ].Pos.X;
		XYZPack->Y = Player[ PlayerNumber ].Pos.Y;
		XYZPack->Z = Player[ PlayerNumber ].Pos.Z;

		XYZPack++;
		Pointers[ PlayerNumber ] = XYZPack;

		XYZPack->Type = 15;
	}
	}
}



void InsertCollisionPacket( uint8 PlayerNumber, uint8 CollType )
{
	CollisionPacket *CollPack = Pointers[ PlayerNumber ];

	if( PermitMove == 1 )
	{

	CollPack->Type = 1;	// Signal that it is a CollisionPacket

	// insert collision data into the packet
	CollPack->CollisionType = CollType;

	CollPack->CollideAngle = CollideAngle( PlayerNumber );

	CollPack->CollidePoint.X = CollidePoint( PlayerNumber ).X;
	CollPack->CollidePoint.Y = CollidePoint( PlayerNumber ).Y;
	CollPack->CollidePoint.Z = CollidePoint( PlayerNumber ).Z;

	CollPack++;
	Pointers[ PlayerNumber ] = CollPack;

	CollPack->Type = 15;

	}
}

void RetrieveCollisionPacket( uint8 PlayerNumber )
{
	CollisionPacket *CollPack = Pointers[ PlayerNumber ];
	player *P = &Player[ PlayerNumber ];

	if( PermitMove == 1 )
	{
	if( CollPack->Type == 1 )	// Is there a collision packet available
	{
		thiscollision[ P->Buffer ].type = CollPack->CollisionType;

		thiscollision[ P->Buffer ].angle = CollPack->CollideAngle >>12;

		thiscollision[ P->Buffer ].point.X = CollPack->CollidePoint.X;
		thiscollision[ P->Buffer ].point.Y = CollPack->CollidePoint.Y;
		thiscollision[ P->Buffer ].point.Z = CollPack->CollidePoint.Z;

		CollPack++;
		Pointers[ PlayerNumber ] = CollPack;
	}
	}
}