// BORED.C - Bored Bored Bored

// 2:23.88
// 2:23.16
// 2:22.25
// 2:18.00

#include <stdio.h>
#include <sys/types.h>
#include <LibGTE.h>
#include <LibGPU.h>
#include <LibCD.h>
#include <LibComb.h>
#include <LibEtc.h>
#include <LibGS.h>
#include <LibMath.h>
#include <LibPress.h>
#include <LibSN.h>
#include <LibSnd.h>
#include <LibSPU.h>

#include "ourtypes.h"
#include "mathbits.h"

#include "main.h"
#include "skeleton.h"
#include "camera.h"
#include "tracks.h"
#include "players.h"
#include "pads.h"

#include "screen.h"
#include "who.h"
#include "bored.h"

POLY_G3	ShadowTri[ 8 ][ 4 ];
POLY_G4	ShadowQuad[ 8 ][ 4 ];

DR_TPAGE TPagePrim[ 8 ];

//POLY_G4 Shadow[ 2 ][ 5 ];
//POLY_FT4 BoredQ[ 2 ][ 3 ];

SVECTOR Bored_Verts[ 8 ];
SVECTOR Shadow_Verts[ 11 ];

SVECTOR Bored_Norms[ 8 ] = {
{ CHFL( 0.000 ),  CHFL( 0.227 ),  CHFL( -0.974 ), },
{ CHFL( 0.000 ),  CHFL( 0.227 ),  CHFL( -0.974 ), },
{ CHFL( 0.000 ),  CHFL( 0.201 ),  CHFL( 0.980 ), },
{ CHFL( 0.000 ),  CHFL( 0.201 ),  CHFL( 0.980 ), },
{ CHFL( 0.000 ),  CHFL( 0.227 ),  CHFL( -0.974 ), },
{ CHFL( 0.000 ),  CHFL( 0.227 ),  CHFL( -0.974 ), },
{ CHFL( 0.000 ),  CHFL( 0.201 ),  CHFL( 0.980 ), },
{ CHFL( 0.000 ),  CHFL( 0.201 ),  CHFL( 0.980 ), },
};

CVECTOR Bored_Shade[ 8 ];

long Trans[ 12 ], P, Otz, Otz2, Flag, Val;

long ShadTrans[ 11 ];


CVECTOR Tris[] =	{
						{ 0, 3, 4 },
						{ 3, 8, 4 },
						{ 2, 6, 7 },
						{ 6, 10, 7},
					};

uint8 Quads[] = { 0, 1, 4, 5 };

//long Vals[ 3 ];
long Otzs[ 3 ];

void SetThis( SVECTOR *V, chfloat X, chfloat Y, chfloat Z )
{
	V->vx = ( X >> 12 ) * 4;
	V->vy = ( Y >> 12 ) * 4;
	V->vz = ( Z >> 12 ) * 4;
}

#define LENGTHD2	14.5
#define FIN			4.0
#define WIDTHD2		4.0

// 6   2					 0	 5
// |---|---------------------|---|
// |   |					 |	 |
// | 0 |		  1			 | 2 |
// |   |			   		 |	 |
// |---|---------------------|---|
// 7   3					 1	 4

void Init_Bored( void )
{
	SetThis( &Bored_Verts[ 0 ], CHFL( -WIDTHD2 ), CHFL( 0.212 ), CHFL( LENGTHD2 ) );
	SetThis( &Bored_Verts[ 1 ], CHFL( WIDTHD2 ), CHFL( 0.212 ), CHFL( LENGTHD2 ) );
	SetThis( &Bored_Verts[ 2 ], CHFL( -WIDTHD2 ), CHFL( 0.212 ), CHFL( -LENGTHD2 ) );
	SetThis( &Bored_Verts[ 3 ], CHFL( WIDTHD2 ), CHFL( 0.212 ), CHFL( -LENGTHD2 ) );
	SetThis( &Bored_Verts[ 4 ], CHFL( WIDTHD2 ), CHFL( 1.210 ), CHFL( LENGTHD2 + FIN ) );
	SetThis( &Bored_Verts[ 5 ], CHFL( -WIDTHD2 ), CHFL( 1.210 ), CHFL( LENGTHD2 + FIN ) );
	SetThis( &Bored_Verts[ 6 ], CHFL( -WIDTHD2 ), CHFL( 1.319 ), CHFL( -LENGTHD2 - FIN ) );
	SetThis( &Bored_Verts[ 7 ], CHFL( WIDTHD2 ), CHFL( 1.319 ), CHFL( -LENGTHD2 - FIN ) );

	SetThis( &Shadow_Verts[ 0 ], CHFL( WIDTHD2 ), CHFL( 0.0 ), CHFL( - LENGTHD2 - FIN ) );
	SetThis( &Shadow_Verts[ 1 ], CHFL( 0.0 ), CHFL( 0.0 ), CHFL( - LENGTHD2 - FIN ) );
	SetThis( &Shadow_Verts[ 2 ], CHFL( -WIDTHD2 ), CHFL( 0.0 ), CHFL( - LENGTHD2 - FIN ) );

	SetThis( &Shadow_Verts[ 3 ], CHFL( WIDTHD2 + WIDTHD2 ), CHFL( 0.0 ), CHFL( 0.0 ) );
	SetThis( &Shadow_Verts[ 4 ], CHFL( WIDTHD2 ), CHFL( 0.0 ), CHFL( 0.0 ) );
	SetThis( &Shadow_Verts[ 5 ], CHFL( 0.0 ), CHFL( 0.0 ), CHFL( 0.0 ) );
	SetThis( &Shadow_Verts[ 6 ], CHFL( -WIDTHD2 ), CHFL( 0.0 ), CHFL( 0.0 ) );
	SetThis( &Shadow_Verts[ 7 ], CHFL( -WIDTHD2 - WIDTHD2 ), CHFL( 0.0 ), CHFL( 0.0 ) );

	SetThis( &Shadow_Verts[ 8 ], CHFL( WIDTHD2 ), CHFL( 0.0 ), CHFL( LENGTHD2 + FIN ) );
	SetThis( &Shadow_Verts[ 9 ], CHFL( 0.0 ), CHFL( 0.0 ), CHFL( LENGTHD2 + FIN ) );
	SetThis( &Shadow_Verts[ 10 ], CHFL( -WIDTHD2 ), CHFL( 0.0 ), CHFL( LENGTHD2 + FIN ) );
}


long Plot_Bored( void )
{
	Otzs[ 0 ] = RotAverage4(	&Bored_Verts[ 2 ], &Bored_Verts[ 6 ], &Bored_Verts[ 7 ], &Bored_Verts[ 3 ],
					&Trans[ 2 ], &Trans[ 6 ], &Trans[ 7 ], &Trans[ 3 ],
					&P, &Flag );

	Otzs[ 1 ] = RotAverage4(	&Bored_Verts[ 0 ], &Bored_Verts[ 2 ], &Bored_Verts[ 3 ], &Bored_Verts[ 1 ],
					&Trans[ 0 ], &Trans[ 2 ], &Trans[ 3 ], &Trans[ 1 ],
					&P, &Flag );

	Otzs[ 2 ] = RotAverage4(	&Bored_Verts[ 5 ], &Bored_Verts[ 0 ], &Bored_Verts[ 1 ], &Bored_Verts[ 4 ],
					&Trans[ 5 ], &Trans[ 0 ], &Trans[ 1 ], &Trans[ 4 ],
					&P, &Flag );

	return 0;		//Vals[ 1 ];
}


CVECTOR ShadowRGB[] =	{
							{ 128, 128, 128 },
							{ 96, 96, 96 },
							{ 128, 128, 128 },

							{ 128, 128, 128 },
							{ 64, 64, 64 },
							{ 0, 0, 0 },
							{ 64, 64, 64 },
							{ 128, 128, 128 },

							{ 128, 128, 128 },
							{ 96, 96, 96 },
							{ 128, 128, 128 },
						};


void Draw_Shadow( uint8 PlayerNumber, uint8 Other )
{
 	uint16 i, x;
	player *Plyr = &Player[ PlayerNumber ];
	long NewOtid = ( otid * 2 ) + Other;



	if( Plyr->JumpY > CHFL( 0.0 ) )	// If off the ground
	{
		// then draw a shadow

//	  /|\				3
//	 / | \
//  /  |  \
// /   |   \
// ---------		0	4	8
// |   |   |
// ---------		1	5	9
// |   |   |
// ---------		2	6	10
// \   |   /
//	\  |  /
//	 \ | /
//	  \|/				7



		for( i = 0; i < 11; i++ )
			RotTransPers( &Shadow_Verts[ i ], &ShadTrans[ i ], &P, &Flag );

		// Plot the quads
		for( i = 0; i < 4; i++ )
		{
			x = Quads[ i ];
			ShadowQuad[ NewOtid ][ i ].x0 = ShadTrans[ x ] >> 16;
			ShadowQuad[ NewOtid ][ i ].y0 = ShadTrans[ x ] & 0xffff;
			ShadowQuad[ NewOtid ][ i ].r0 = ShadowRGB[ x ].r;
			ShadowQuad[ NewOtid ][ i ].g0 = ShadowRGB[ x ].g;
			ShadowQuad[ NewOtid ][ i ].b0 = ShadowRGB[ x ].b;

			ShadowQuad[ NewOtid ][ i ].x1 = ShadTrans[ x + 4 ] >> 16;
			ShadowQuad[ NewOtid ][ i ].y1 = ShadTrans[ x + 4 ] & 0xffff;
			ShadowQuad[ NewOtid ][ i ].r1 = ShadowRGB[ x + 4 ].r;
			ShadowQuad[ NewOtid ][ i ].g1 = ShadowRGB[ x + 4 ].g;
			ShadowQuad[ NewOtid ][ i ].b1 = ShadowRGB[ x + 4 ].b;

			ShadowQuad[ NewOtid ][ i ].x2 = ShadTrans[ x + 1 ] >> 16;
			ShadowQuad[ NewOtid ][ i ].y2 = ShadTrans[ x + 1 ] & 0xffff;
			ShadowQuad[ NewOtid ][ i ].r2 = ShadowRGB[ x + 1 ].r;
			ShadowQuad[ NewOtid ][ i ].g2 = ShadowRGB[ x + 1 ].g;
			ShadowQuad[ NewOtid ][ i ].b2 = ShadowRGB[ x + 1 ].b;

			ShadowQuad[ NewOtid ][ i ].x3 = ShadTrans[ x + 5 ] >> 16;
			ShadowQuad[ NewOtid ][ i ].y3 = ShadTrans[ x + 5 ] & 0xffff;
			ShadowQuad[ NewOtid ][ i ].r3 = ShadowRGB[ x + 5 ].r;
			ShadowQuad[ NewOtid ][ i ].g3 = ShadowRGB[ x + 5 ].g;
			ShadowQuad[ NewOtid ][ i ].b3 = ShadowRGB[ x + 5 ].b;

			setPolyG4( &ShadowQuad[ NewOtid ][ i ] );
			SetSemiTrans( &ShadowQuad[ NewOtid ][ i ], 1 );
			AddPrim( &OrderingTable[ otid ][ Otzs[ 0 ] ], &ShadowQuad[ NewOtid ][ i ] );
		}

		// Plot the tris
		for( i = 0; i < 4; i++ )
		{
			ShadowTri[ NewOtid ][ i ].x0 = ShadTrans[ Tris[ i ].r ] >> 16;
			ShadowTri[ NewOtid ][ i ].y0 = ShadTrans[ Tris[ i ].r ] & 0xffff;
			ShadowTri[ NewOtid ][ i ].r0 = ShadowRGB[ Tris[ i ].r ].r;
			ShadowTri[ NewOtid ][ i ].g0 = ShadowRGB[ Tris[ i ].r ].g;
			ShadowTri[ NewOtid ][ i ].b0 = ShadowRGB[ Tris[ i ].r ].b;
			ShadowTri[ NewOtid ][ i ].x1 = ShadTrans[ Tris[ i ].g ] >> 16;
			ShadowTri[ NewOtid ][ i ].y1 = ShadTrans[ Tris[ i ].g ] & 0xffff;
			ShadowTri[ NewOtid ][ i ].r1 = ShadowRGB[ Tris[ i ].g ].r;
			ShadowTri[ NewOtid ][ i ].g1 = ShadowRGB[ Tris[ i ].g ].g;
			ShadowTri[ NewOtid ][ i ].b1 = ShadowRGB[ Tris[ i ].g ].b;
			ShadowTri[ NewOtid ][ i ].x2 = ShadTrans[ Tris[ i ].b ] >> 16;
			ShadowTri[ NewOtid ][ i ].y2 = ShadTrans[ Tris[ i ].b ] & 0xffff;
			ShadowTri[ NewOtid ][ i ].r2 = ShadowRGB[ Tris[ i ].b ].r;
			ShadowTri[ NewOtid ][ i ].g2 = ShadowRGB[ Tris[ i ].b ].g;
			ShadowTri[ NewOtid ][ i ].b2 = ShadowRGB[ Tris[ i ].b ].b;

			setPolyG3( &ShadowTri[ NewOtid ][ i ] );
			SetSemiTrans( &ShadowTri[ NewOtid ][ i ], 1 );
			AddPrim( &OrderingTable[ otid ][ Otzs[ 0 ] ], &ShadowTri[ NewOtid ][ i ] );
		}

		SetDrawTPage( &TPagePrim[ NewOtid ], 1, 1, ( int )GetTPage(1,0,0,0) );
		AddPrim( &OrderingTable[ otid ][ Otzs[ 0 ] ], &TPagePrim[ NewOtid ] );

	}
}













