/* camera.c */

/* Machine dependant - you may not be using the chill Matrix routines for your version */

#include <stdio.h>

#include "ourtypes.h"
#include <stdio.h>
#include "sys/types.h"
#include "LibGTE.h"
#include "LibGPU.h"
#include "LibCD.h"
#include "LibComb.h"
#include "LibEtc.h"
#include "LibGS.h"
#include "LibMath.h"
#include "LibPress.h"
#include "LibSN.h"
#include "LibSnd.h"
#include "LibSPU.h"
#include "main.h"
#include "mathbits.h"
#include "pads.h"
#include "camlogic.h"
#include "camera.h"
#include "tracks.h"
#include "players.h"
#include "screen.h"
#include "parachut.h"
#include "logic.h"
#include "rotmesh.h"

long	cam_zrot = 0;
long	cam_zrotspeed = 0;
long	cam_zrotaccel = 24;
long	cam_zrotmaxspeed = 128;
long	cam_zrotmax	= 1024;

VECTOR	CamScale={12288,12288,12288};

void InitialiseCameras( void )
{
	printf("Executing : InitialiseCameras()\n");
}

void KillCameras( void )
{
	printf("Executing : KillCameras()\n");
}


void CameraLogic( uint8 PlayerNumber )
{
	player *Plyr = &Player[ PlayerNumber ];
	static int32 DotP;
	static VECTOR X,Y,Z,T,F,FX,FY,FZ,FN,FR;
	uint8 count1,count2;


	/* Call the appropriate camera movement code */
	switch( Plyr->Cam.Mode )
	{
		case CAMERA_TEST_TARGET:
			Camera_Test_Target( PlayerNumber );
			break;

		case CAMERA_TEST_POSITIONAL:
			Camera_Test_Positional( PlayerNumber );
			break;

		case CAMERA_FIRST_PERSON:
			Camera_First_Person( PlayerNumber );
			break;

		case CAMERA_FOLLOW:
			Camera_Follow( PlayerNumber );
			break;

		case CAMERA_PARACHUTE:
			Camera_Parachute( PlayerNumber );
			break;

	}


	switch( Plyr->Cam.Type )
	{
		case CAMERA_TYPE_TARGET:
		{

				if(onice)
				{
					Plyr->Cam.Up.X = Sine(cam_zrot);
					Plyr->Cam.Up.Y = Cosine(cam_zrot);
					Plyr->Cam.Up.Z = 0;

				}
				else
				{
					Plyr->Cam.Up.X = 0;
					Plyr->Cam.Up.Y = 4096;
					Plyr->Cam.Up.Z = 0;


				}



//				Z.vx = ((-(Plyr->Cam.Targ.X>>12)) - (-(Plyr->Cam.Pos.X>>12)));
//				Z.vy = ((Plyr->Cam.Targ.Y>>12) - (Plyr->Cam.Pos.Y>>12));
//				Z.vz = ((Plyr->Cam.Targ.Z>>12) - (Plyr->Cam.Pos.Z)>>12);


				Z.vx = CHSHIFTR( -Plyr->Cam.Targ.X ) - CHSHIFTR( -Plyr->Cam.Pos.X );
				Z.vy = CHSHIFTR( Plyr->Cam.Targ.Y ) - CHSHIFTR( Plyr->Cam.Pos.Y );
				Z.vz = CHSHIFTR( Plyr->Cam.Targ.Z ) - CHSHIFTR( Plyr->Cam.Pos.Z );



//				Z.vx = ((-Plyr->Cam.Targ.X) - (-Plyr->Cam.Pos.X))>>12;
//				Z.vy = (Plyr->Cam.Targ.Y - Plyr->Cam.Pos.Y)>>12;
//				Z.vz = (Plyr->Cam.Targ.Z - Plyr->Cam.Pos.Z)>>12;

				VectorNormal( &Z, &T );
				Z.vx=T.vx;
				Z.vy=T.vy;
				Z.vz=T.vz;
				B3.vx=T.vx;
				B3.vy=T.vy/2;
				B3.vz=T.vz;

				DotP = PSX_Dot( &Plyr->Cam.Up, &Z )>>12;

				Y.vx = Plyr->Cam.Up.X - ( (DotP * Z.vx)>>12 );
				Y.vy = Plyr->Cam.Up.Y - ( (DotP * Z.vy)>>12 );
				Y.vz = Plyr->Cam.Up.Z - ( (DotP * Z.vz)>>12 );
				VectorNormal( &Y, &T );
				Y.vx=T.vx;
				Y.vy=T.vy;
				Y.vz=T.vz;

				OuterProduct0(&Z, &T, &X);

				B1.vx = Plyr->Cam.Up.X - ( (DotP * B3.vx)>>12 );
				B1.vy = Plyr->Cam.Up.Y - ( (DotP * B3.vy)>>12 );
				B1.vz = Plyr->Cam.Up.Z - ( (DotP * B3.vz)>>12 );
				VectorNormal( &B1, &B2 );
				B1.vx=B2.vx;
				B1.vy=B2.vy;
				B1.vz=B2.vz;
				OuterProduct0(&Z, &B1, &B2);

				mountmatrix[Plyr->Buffer].m[ 1 ][ 0 ]=B2.vx>>12;
				mountmatrix[Plyr->Buffer].m[ 1 ][ 1 ]=B2.vy>>12;
				mountmatrix[Plyr->Buffer].m[ 1 ][ 2 ]=B2.vz>>12;
				mountmatrix[Plyr->Buffer].m[ 0 ][ 0 ]=-B1.vx;
				mountmatrix[Plyr->Buffer].m[ 0 ][ 1 ]=-B1.vy;
				mountmatrix[Plyr->Buffer].m[ 0 ][ 2 ]=-B1.vz;
				mountmatrix[Plyr->Buffer].m[ 2 ][ 0 ]=B3.vx;
				mountmatrix[Plyr->Buffer].m[ 2 ][ 1 ]=B3.vy;
				mountmatrix[Plyr->Buffer].m[ 2 ][ 2 ]=B3.vz;

				Plyr->Cam.ViewMatrix.m[ 0 ][ 0 ]=X.vx>>12;
				Plyr->Cam.ViewMatrix.m[ 0 ][ 1 ]=X.vy>>12;
				Plyr->Cam.ViewMatrix.m[ 0 ][ 2 ]=X.vz>>12;
				Plyr->Cam.ViewMatrix.m[ 1 ][ 0 ]=-Y.vx;
				Plyr->Cam.ViewMatrix.m[ 1 ][ 1 ]=-Y.vy;
				Plyr->Cam.ViewMatrix.m[ 1 ][ 2 ]=-Y.vz;
				Plyr->Cam.ViewMatrix.m[ 2 ][ 0 ]=Z.vx;
				Plyr->Cam.ViewMatrix.m[ 2 ][ 1 ]=Z.vy;
				Plyr->Cam.ViewMatrix.m[ 2 ][ 2 ]=Z.vz;

// view matrix for objects, textured track polys

				Plyr->Cam.ObjectView.m[0][0]=-Plyr->Cam.ViewMatrix.m[1][0];
				Plyr->Cam.ObjectView.m[0][1]=Plyr->Cam.ViewMatrix.m[1][1];
				Plyr->Cam.ObjectView.m[0][2]=Plyr->Cam.ViewMatrix.m[1][2];
				Plyr->Cam.ObjectView.m[1][0]=-Plyr->Cam.ViewMatrix.m[0][0];
				Plyr->Cam.ObjectView.m[1][1]=Plyr->Cam. ViewMatrix.m[0][1];
				Plyr->Cam.ObjectView.m[1][2]=Plyr->Cam.ViewMatrix.m[0][2];
				Plyr->Cam.ObjectView.m[2][0]=-Plyr->Cam.ViewMatrix.m[2][0];
				Plyr->Cam.ObjectView.m[2][1]=Plyr->Cam.ViewMatrix.m[2][1];
				Plyr->Cam.ObjectView.m[2][2]=Plyr->Cam.ViewMatrix.m[2][2];
				ScaleMatrix(&(Plyr->Cam.ObjectView),&CamScale);
				SetRotMatrix(&(Plyr->Cam.ObjectView));

				X.vx=-(Plyr->Cam.Pos.X>>12);
				X.vy=-(Plyr->Cam.Pos.Y>>12);
				X.vz=-(Plyr->Cam.Pos.Z>>12);
				ApplyRotMatrixLV(&X,&Y);

				Plyr->Cam.TransMatrix.t[1]=Y.vx;
				Plyr->Cam.TransMatrix.t[0]=Y.vy;
				Plyr->Cam.TransMatrix.t[2]=Y.vz;

  				ScaleMatrix(&(Plyr->Cam.ViewMatrix),&CamScale);
				SetRotMatrix(&(Plyr->Cam.ViewMatrix));
				SetTransMatrix(&(Plyr->Cam.TransMatrix));
				ScaleMatrix(&mountmatrix[Plyr->Buffer],&CamScale);

			}
			break;

		case CAMERA_TYPE_POSITIONAL:
		{
			RotationVector[rotid].vx= -( Plyr->Cam.Rot.X >> PSX_PREC ) ;
			RotationVector[rotid].vy= +( Plyr->Cam.Rot.Y >> PSX_PREC ) ;
			RotationVector[rotid].vz=0;/* -( Plyr->Cam.Rot.Z >> PSX_PREC ) +2048; */
			X.vx=(Plyr->Cam.Pos.X>>12);
			X.vy=-(Plyr->Cam.Pos.Y>>12);
			X.vz=-(Plyr->Cam.Pos.Z>>12);
			RotMatrix(&RotationVector[rotid],&(Plyr->Cam.ViewMatrix));
			RotMatrixZ(-((Plyr->Cam.Rot.Z >> PSX_PREC)+2048),&Plyr->Cam.ViewMatrix);
			for (count1=0;count1<3;count1++)
			{
				for (count2=0;count2<3;count2++)
				{
					if (count1==count2)
					{
						Plyr->Cam.TransMatrix.m[count1][count2]=0x1000;
					}
					else
					{
						Plyr->Cam.TransMatrix.m[count1][count2]=0;
					}
				}
				Plyr->Cam.TransMatrix.t[count1]=0;
			}
			RotMatrix(&RotationVector[rotid],&(Plyr->Cam.TransMatrix));
			RotMatrixZ(-((Plyr->Cam.Rot.Z >> PSX_PREC)+2048),&Plyr->Cam.TransMatrix);
			ScaleMatrix(&(Plyr->Cam.TransMatrix),&CamScale);
			ApplyMatrixLV(&(Plyr->Cam.TransMatrix),&X,&Y);
			Plyr->Cam.TransMatrix.t[0]=Y.vx;
			Plyr->Cam.TransMatrix.t[1]=Y.vy;
			Plyr->Cam.TransMatrix.t[2]=Y.vz;
			ScaleMatrix(&(Plyr->Cam.ViewMatrix),&CamScale);
			SetRotMatrix(&(Plyr->Cam.ViewMatrix));
			SetTransMatrix(&(Plyr->Cam.TransMatrix));

// work out viewmatrix for objects

			Plyr->Cam.ObjectView.m[0][0]=-Plyr->Cam.ViewMatrix.m[1][0];
			Plyr->Cam.ObjectView.m[0][1]=Plyr->Cam.ViewMatrix.m[1][1];
			Plyr->Cam.ObjectView.m[0][2]=Plyr->Cam.ViewMatrix.m[1][2];
			Plyr->Cam.ObjectView.m[1][0]=-Plyr->Cam.ViewMatrix.m[0][0];
			Plyr->Cam.ObjectView.m[1][1]=Plyr->Cam.ViewMatrix.m[0][1];
			Plyr->Cam.ObjectView.m[1][2]=Plyr->Cam.ViewMatrix.m[0][2];
			Plyr->Cam.ObjectView.m[2][0]=-Plyr->Cam.ViewMatrix.m[2][0];
			Plyr->Cam.ObjectView.m[2][1]=Plyr->Cam.ViewMatrix.m[2][1];
			Plyr->Cam.ObjectView.m[2][2]=Plyr->Cam.ViewMatrix.m[2][2];

		}
		break;
	}
}

