/* pads.c */

#include <stdio.h>
#include "sys/types.h"
#include "LibGTE.h"
#include "LibGPU.h"
#include "LibCD.h"
#include "LibComb.h"
#include "LibEtc.h"
#include "LibGS.h"
#include "LibMath.h"
#include "LibPress.h"
#include "LibSN.h"
#include "LibSnd.h"
#include "LibSPU.h"
//#include "libgun.h"
#include "ourtypes.h"
#include "mathbits.h"
#include "main.h"
#include "pads.h"
#include "ghostrep.h"
#include "skeleton.h"
#include "tracks.h"

uint16 Paddys[ MAX_PADS ], Last[ MAX_PADS ], Deb[ MAX_PADS ];
uint16 RealPaddys[ MAX_PADS ], RealLast[ MAX_PADS ], RealDeb[ MAX_PADS ];
uint8 Type[ MAX_PADS ];
uint8 Config[ MAX_PADS ];

padbuffer PadBuf[ MAX_PADS ];
//padsendbuffer SendBuf[ MAX_PADS ];

//gunbuffer GunBuf[ 2 ];

void Pre_Init( void )
{
//	SendBuf[ 0 ].Status = 0x01;
//	SendBuf[ 0 ].ID = 0x03;
//	SendBuf[ 0 ].Data = 0x00;

	ResetCallback();
	InitPAD( &PadBuf[ 0 ], sizeof( padbuffer ), &PadBuf[ 1 ], sizeof( padbuffer ) );
//	InitGUN( &PadBuf[ 0 ], sizeof( padbuffer ), &PadBuf[ 1 ], sizeof( padbuffer ), &GunBuf[ 0 ], &GunBuf[ 1 ], 20 );


//	SendPAD( &SendBuf[ 0 ], 3, &SendBuf[ 1 ], 3 );


	StartPAD();
//	StartGUN();


	ChangeClearPAD( 0 );

//	SelectGUN( 0, 0 );
//	SelectGUN( 1, 0 );
}

void InitialisePads( void )
{
	uint8 i;

	printf("InitialisePads()\n");
	VSync( 0 );

	for( i = 0; i < MAX_PADS; i++ )
	{
		Type[ i ] = 0;
		Config[ i ] = 0;

		switch( PadBuf[ i ].ID )
		{
			case 0x00:
				printf("Pad %d = NOTHING PLUGGED IN\n");
				break;

			case 0x23:
				printf("Pad %d = NEGCON / MAD CATZ WHEEL\n");
				Type[ i ] = PAD_NEGCON;
				Config[ i ] |= 512;
				break;
/*
			case 0x31:
				printf("Pad %d = LIGHT GUN (not supported)\n", i );
//				Type[ i ] = PAD_GUN;
				break;
*/
			case 0x41:
				printf("Pad %d = STANDARD PAD\n", i );
				Type[ i ] = PAD_DIGITAL;
				break;
/*
			case 0x53:
				printf("Pad %d = ANALOG JOYSTICK\n", i );
				Type[ i ] = PAD_ANALOG;
				Config[ i ] |= 256;
				break;
*/
			case 0x73:
				printf("Pad %d = ANALOG CONTROLLER\n", i );
				Type[ i ] = PAD_ANALOG_CONTROLLER;
				Config[ i ] |= 128;
				break;

			default:
				printf("Pad %d = UNSUPPORTED CONTROLLER (%02x)\n", i, PadBuf[ i ].ID );
				break;
		}
		Paddys[ i ] = Last[ i ] = Deb[ i ] = 0;
	}
}


void Pad_Update( void )
{
	uint8 i;

	for( i = 0; i < MAX_PADS; i++ )
	{
		switch( Type[ i ] )
		{
			case PAD_DIGITAL:
				// If pad has changed then insert a pad change packet
				RealPaddys[ i ] = Paddys[ i ] = PadBuf[ i ].Data.Standard ^ 0xffff;
				break;

//			case PAD_GUN:
//				Paddys[ i ] = PadBuf[ i ].Data.Standard ^ 0xffff;
//				break;

			case PAD_ANALOG:
				RealPaddys[ i ] = Paddys[ i ] = PadBuf[ i ].Data.AnaStick.Buttons ^ 0xffff;
				break;

			case PAD_NEGCON:
				RealPaddys[ i ] = Paddys[ i ] = PadBuf[ i ].Data.AnaStick.Buttons ^ 0xffff;
				break;

			case PAD_ANALOG_CONTROLLER:
				RealPaddys[ i ] = Paddys[ i ] = PadBuf[ i ].Data.AnaStick.Buttons ^ 0xffff;
				break;
		}

		if( PermitMove == 0 && Ingame == 1 )
		{
			Paddys[ i ] &= ( PAD_START + PAD_SELECT );
		}
	}

	// Get the ghost data here (but only if we are in the game)
	if( Ingame == 1 && PermitMove == 1 )
	{
//		Paddys[ i ] &= ( 0xFFFF ^ ( PAD_START | PAD_SELECT ) );
		HandleGhosts();
	}

	for( i = 0; i < MAX_PADS; i++ )
	{
		Deb[ i ] = ( Last[ i ] ^ 0xffffffff ) & Paddys[ i ];
		Last[ i ] = Paddys[ i ];

		RealDeb[ i ] = ( RealLast[ i ] ^ 0xffffffff ) & RealPaddys[ i ];
		RealLast[ i ] = RealPaddys[ i ];
	}
}

uint16	Pad_State( uint8 PadNumber )
{
	return Paddys[ PadNumber ];
}

uint16	Pad_Debounced( uint8 PadNumber )
{
	return Deb[ PadNumber ];
}

uint16	Pad_RealState( uint8 PadNumber )
{
	return RealPaddys[ PadNumber ];
}

uint16	Pad_RealDebounced( uint8 PadNumber )
{
	return RealDeb[ PadNumber ];
}

uint16 Pad_Type( uint8 PadNumber )
{
	return Type[ PadNumber ];
}


uint8 Pad_Analog_RX( uint8 PadNumber )
{
	return PadBuf[ PadNumber ].Data.AnaStick.Rx;
}


uint8 Pad_Negcon_Button1( uint8 PadNumber )
{
	return PadBuf[ PadNumber ].Data.Negcon.ButtonI;
}

uint8 Pad_Negcon_Button2( uint8 PadNumber )
{
	return PadBuf[ PadNumber ].Data.Negcon.ButtonII;
}




uint8 Config_GRAB1( uint8 PadNumber )
{
	switch( Config[ PadNumber ] )
	{
		case 0:
			if( Paddys[ PadNumber ] & PAD_BL ) return 1;
			break;

		case 1:
			if( Paddys[ PadNumber ] & PAD_B1 ) return 1;
			break;
	}

	return 0;	// No, the GRAB 1 key was not pressed
}

uint8 Config_GRAB2( uint8 PadNumber )
{
	switch( Config[ PadNumber ] )
	{
		case 0:
			if( Paddys[ PadNumber ] & PAD_BR ) return 1;
			break;

		case 1:
			if( Paddys[ PadNumber ] & PAD_B3 ) return 1;
			break;
	}

	return 0;	// No, the GRAB 2 key was not pressed
}

uint8 Config_BRAKE( uint8 PadNumber )
{
	switch( Config[ PadNumber ] )
	{
		case 0:
			if( Paddys[ PadNumber ] & PAD_B3 ) return 1;
			break;

		case 1:
			if( Paddys[ PadNumber ] & PAD_BR ) return 1;
			break;
	}

	return 0;	// No, the BRAKE key was not pressed
}

uint8 Config_TUCK( uint8 PadNumber )
{
	switch( Config[ PadNumber ] )
	{
		case 0:
		case 128:
			if( Paddys[ PadNumber ] & PAD_B1 ) return 1;
			break;

		case 1:
			if( Paddys[ PadNumber ] & PAD_BL ) return 1;
			break;
	}

	return 0;	// No, the TUCK key was not pressed
}

uint8 Config_JUMP( uint8 PadNumber )
{
	switch( Config[ PadNumber ] )
	{
		case 0:
		case 128:
			if( Paddys[ PadNumber ] & PAD_B2 ) return 1;
			break;

		case 1:
		case 129:
			break;
	}

	return 0;	// No, the JUMP key was not pressed
}

uint8 Config_FRONTSPIN( uint8 PadNumber )
{
	switch( Config[ PadNumber ] )
	{
		case 0:
		case 128:
			if( Paddys[ PadNumber ] & PAD_UP ) return 1;
			break;

		case 1:
		case 129:
			break;
	}

	return 0;	// No, the FRONTSPIN key was not pressed
}

uint8 Config_BACKSPIN( uint8 PadNumber )
{
	switch( Config[ PadNumber ] )
	{
		case 0:
		case 128:
			if( Paddys[ PadNumber ] & PAD_DOWN ) return 1;
			break;

		case 1:
		case 129:
			break;
	}

	return 0;	// No, the BACKSPIN key was not pressed
}

uint8 Config_LEFTTWIST( uint8 PadNumber )
{
	switch( Config[ PadNumber ] )
	{
		case 0:
		case 128:
			if( Paddys[ PadNumber ] & PAD_TL ) return 1;
			break;

		case 1:
		case 129:
			break;
	}

	return 0;	// No, the TWISTLEFT key was not pressed
}

uint8 Config_RIGHTTWIST( uint8 PadNumber )
{
	switch( Config[ PadNumber ] )
	{
		case 0:
		case 128:
			if( Paddys[ PadNumber ] & PAD_TR ) return 1;
			break;

		case 1:
		case 129:
			break;
	}

	return 0;	// No, the TWISTRIGHT key was not pressed
}

uint8 Config_CHILLTRICK( uint8 PadNumber )
{
	switch( Config[ PadNumber ] )
	{
		case 0:
			if( Paddys[ PadNumber ] & PAD_B4 ) return 1;
			break;


	}

	return 0;
}


uint8 Analog_Config_Steer( uint8 PadNumber )
{
	switch( Type[ PadNumber ] )
	{
		case PAD_DIGITAL:
			return 0;
			break;

		case PAD_ANALOG:
			return PadBuf[ PadNumber ].Data.AnaStick.Lx;
			break;

		case PAD_ANALOG_CONTROLLER:
			return PadBuf[ PadNumber ].Data.AnaStick.Lx;
			break;

		case PAD_NEGCON:
			return PadBuf[ PadNumber ].Data.Negcon.Twist;
			break;
	}

	return 128;
}


