// SCORES.C

#include <stdio.h>
#include "sys/types.h"
#include "LibGTE.h"
#include "LibGPU.h"
#include "LibCD.h"
#include "LibComb.h"
#include "LibEtc.h"
#include "LibGS.h"
#include "LibMath.h"
#include "LibPress.h"
#include "LibSN.h"
#include "LibSnd.h"
#include "LibSPU.h"


#include "ourtypes.h"
#include "mathbits.h"

#include "main.h"
#include "skeleton.h"
#include "camera.h"
#include "tracks.h"
#include "players.h"

#include "scores.h"

extern uint16 lastchillscore;
extern uint32 tempchill;

uint32 InitialValue[ MAX_SCORES ] =
{
	0,			// not defined - ignore
	500,			// MUTE					Grabs							Note : You can multiply the value of a grab the longer you hold it
	500,			// METHOD
	1000,		// TFTAIL				Spin and Twist grab combos		Note :must be 180 degrees to count as a combo
	1000,		// TFMETHOD
	1000,		// TBTAIL
	1000,		// TBMETHOD
	1000,		// FSTAIL
	1000,		// FSMETHOD
	1000,		// BSTAIL
	1000,		// BSMETHOD
	1500,		// MC_TWIST_TAIL
	1500,		// MC_TWIST_METHOD
	1000,		// FS180				FrontSide 180
	2000,		// FS360
	3000,		// FS540
	4000,		// FS720
	5000,		// FS900
	6000,		// FS1080
	7000,		// FS1260
	8000,		// FS1440
	1000,		// BS180				BackSide 180
	2000,		// BS360
	3000,		// BS540
	4000,		// BS720
	5000,		// BS900
	6000,		// BS1080
	7000,		// BS1260
	8000,		// BS1440
	1000,		// FSpin360				FrontSpin 360
	2000,		// FSpin720
	3000,		// FSpin1080
	4000,		// FSpin1440
	2000,		// BSpin360				BackSpin 360
	4000,		// BSpin720
	6000,		// BSpin1080
	8000,		// BSpin1440
	2000,		// FIFTYFIFTY			Rail related tricks				Note : Rail scores not implemented yet as collision code does not work properly
	4000,		// FAKIEFIFTYFIFTY
	2000,		// RAILSLIDE
	4000,		// FAKIERAILSLIDE
	4000,		// RAILSPIN
	2000,		// TREERIDE
	2000,		// BONK													Note : The score for a bonk is never halved, you can bonk as much as you like
	2500,		// CHILLTRICK			leghorn							Note : Only scored for full completion with leg back on board and steady landing
	4000,		// MC_TWIST				Spin and Twist combo			Note : Must go 360 degrees round the two axis to qualify for a score, the further it is the better the multiplier
};

uint32 ScoreValue[ 2 ][ MAX_SCORES ];

uint16 ScoreQueueSize[ 2 ];
trickscore ScoreBuffers[ 32 ];

trickscore *ScoreQueue[ 2 ][ 32 ];

void InitScoringSystem( void )
{
	uint16 i;

	ScoreQueueSize[ 0 ] = 0;
	ScoreQueueSize[ 1 ] = 0;

	for( i = 0; i < 32; i++ )
	{
		ScoreBuffers[ i ].InUse = 0;
	}

	// Copy initial StartingValue to ScoreValue
	for( i = 0; i < MAX_SCORES; i++ )
		ScoreValue[ 0 ][ i ] = ScoreValue[ 1 ][ i ] = InitialValue[ i ];
}

trickscore* GetNextScore( uint8 PlayerNumber )
{
	if( ScoreQueueSize[ PlayerNumber ] == 0 )
	{
		return NULL;
	}

	return ScoreQueue[ PlayerNumber ][ 0 ];
}

void DeleteScore( uint8 PlayerNumber )
{
	uint16 i;

	ScoreQueue[ PlayerNumber ][ 0 ]->InUse = 0;
	ScoreQueueSize[ PlayerNumber ]--;

	for( i = 0; i < ScoreQueueSize[ PlayerNumber ]; i++ )
	{
		ScoreQueue[ PlayerNumber ][ i ] = ScoreQueue[ PlayerNumber ][ i + 1 ];
	}
}

// The defined multiply is used to pass a multiplier equivalent to how long you held the grab for
void AwardTrick( uint16 PlayerNumber, uint16 DefinedScore, uint16 DefinedMultiply )
{
	uint32 i;
	trickscore *Details;

	if( ScoreQueueSize[ PlayerNumber ] == 32 ) return;

	for( i = 0; i < 32; i++ )
	{
		if( ScoreBuffers[ i ].InUse == 0 )
		{
			Details = &ScoreBuffers[ i ];
			break;
		}
	}

	// Set the in use flag
	Details->InUse = 1;
	Details->Trick = DefinedScore;	// Store away which trick the score was for
	Details->Multiply = DefinedMultiply;

	// Set the current score value of this trick on to the .Score
	Details->Score = ScoreValue[ PlayerNumber ][ DefinedScore ];

	// Half the current score value ( so it's not worth repeating the same trick all the time ) unless it's a bonk
	if( DefinedScore != T_BONK )
		ScoreValue[ PlayerNumber ][ DefinedScore ] /= 2;

//	// If the player is not in the trick area then quarter the .Score .and FinalScore
//	if( NotInTrickArea( PlayerNumber ) )
//		Details->Score /= 4;

	// Calculate final score using the multiplier
	Details->FinalScore = Details->Score + ( ( Details->Score * DefinedMultiply ) / 10 );

	// Add it to the end of the players queue and increase the queue size
	ScoreQueue[ PlayerNumber ][ ScoreQueueSize[ PlayerNumber ] ] = Details;
	ScoreQueueSize[ PlayerNumber ]++;
}


/*

Use GetNextScore ( trickscore* GetNextScore( PlayerNumber ) ) to determine
if there is a score in the queue and retrieve a pointer to it.  NULL is
returned if there is no score.

After extracting the information use DeleteScore( PlayerNumber ) to discard
the head of the queue and move on to the next score.


// Example to add just the scores on and not bother with any sort of effects
// or multiplier information
// for player 0 ( 1 )

while( GetNextScore( 0 ) != NULL )
{
	trickscore *Details = GetNextScore( 0 );

	Score += Details.FinalScore;
	DeleteScore( 0 );
}




*/

// Example code for polling and reading the scoring queues - this code was used as test code and works
void TestScoring( uint8 PlayerNumber )
{
	while( GetNextScore( PlayerNumber ) != NULL )
	{
		trickscore *Details = GetNextScore( PlayerNumber );

		printf("Yeah you got a trick !!! ");

		switch( Details->Trick )
		{
			case T_MUTE:
				printf("Mute = %d ", Details->Score );
				// or call StartTailgrabSpecialFX();
				break;

			case T_METHOD:
				printf("Method = %d ", Details->Score );
				// or call StartMethodSpecialFX();
				break;

			case T_FSMUTE:
			case T_BSMUTE:
			case T_TFMUTE:
			case T_TBMUTE:
			case T_MC_TWIST_MUTE:
				printf("Combo Mute = %d ", Details->Score );
				break;

			case T_FSMETHOD:
			case T_BSMETHOD:
			case T_TFMETHOD:
			case T_TBMETHOD:
			case T_MC_TWIST_METHOD:
				printf("Combo Method = %d ", Details->Score );
				break;

			// Twist moves FSdegrees & BSdegrees
			case T_FS180:
			case T_FS360:
			case T_FS540:
			case T_FS720:
			case T_FS900:
			case T_FS1080:
			case T_FS1260:
			case T_FS1440:
				printf("FS%d = %d ", ( Details->Trick - T_FS180 + 1 ) * 180, Details->Score );
				// or call StartFSSpecialFX();
				break;
			case T_BS180:
			case T_BS360:
			case T_BS540:
			case T_BS720:
			case T_BS900:
			case T_BS1080:
			case T_BS1260:
			case T_BS1440:
				printf("BS%d = %d ", ( Details->Trick - T_BS180 + 1 ) * 180, Details->Score );
				// or call StartBSSpecialFX();
				break;

			case T_FSpin360:
			case T_FSpin720:
			case T_FSpin1080:
			case T_FSpin1440:
				printf("FrontSpin%d = %d ", ( Details->Trick - T_FSpin360 + 1  ) * 360, Details->Score );
				// or call StartFSpinSpecialFX();
				break;

			case T_BSpin360:
			case T_BSpin720:
			case T_BSpin1080:
			case T_BSpin1440:
				printf("BackSpin%d = %d ", ( Details->Trick - T_BSpin360 + 1 ) * 360, Details->Score );
				// or call StartFSpinSpecialFX();
				break;

			case T_FIFTYFIFTY:
			case T_FAKIEFIFTYFIFTY:
			case T_RAILSLIDE:
			case T_FAKIERAILSLIDE:
			case T_RAILSPIN:
			case T_TREERIDE:
				printf("To be implemented later when rail collision works 100%\n");
				break;


			case T_BONK:
				printf("Bonk = %d ", Details->Score );
				// or call StartBonkSpecialFX();
				break;

			case T_CHILLTRICK:
				printf("Leghorn = %d ", Details->Score );
				// or call StartLeghornSpecialFX();
				break;

			case T_MC_TWIST:
				printf("McTwist = %d ", Details->Score );
				// or call StartMcTwistSpecialFX();
				break;
		}

		switch( Details->Multiply )
		{
			case ONEPOINTZERO:
				// 1.0 is not really a multiplier so don't print anything up
				break;
			case ONEPOINTONE:
				printf("x 1.1 " );
				// or call Startx11SpecialFX();
				break;
			case ONEPOINTTWO:
				printf("x 1.2 " );
				// or call Startx12SpecialFX();
				break;
			case ONEPOINTTHREE:
				printf("x 1.3 " );
				// or call Startx13SpecialFX();
				break;
			case ONEPOINTFOUR:
				printf("x 1.4 " );
				// or call Startx14SpecialFX();
				break;
			case ONEPOINTFIVE:
				printf("x 1.5 " );
				// or call Startx15SpecialFX();
				break;
			case ONEPOINTSIX:
				printf("x 1.6 " );
				// or call Startx16SpecialFX();
				break;
			case ONEPOINTSEVEN:
				printf("x 1.7 " );
				// or call Startx17SpecialFX();
				break;
			case ONEPOINTEIGHT:
				printf("x 1.8 " );
				// or call Startx18SpecialFX();
				break;
			case ONEPOINTNINE:
				printf("x 1.9 " );
				// or call Startx19SpecialFX();
				break;
			case TWOPOINTZERO:
				printf("x 2.0 " );
				// or call Startx20SpecialFX();
				break;
		}

		printf(" TOTAL = %d", Details->FinalScore );
		printf("\n");
		lastchillscore=Details->FinalScore;
		tempchill+=Details->FinalScore;

		Player[ PlayerNumber ].Score += Details->FinalScore;
		DeleteScore( PlayerNumber );
	}
}








