// Skeleton.c

#include <stdio.h>
#include <sys/types.h>
#include <LibGTE.h>
#include <LibGPU.h>
#include <LibCD.h>
#include <LibComb.h>
#include <LibEtc.h>
#include <LibGS.h>
#include <LibMath.h>
#include <LibPress.h>
#include <LibSN.h>
#include <LibSnd.h>
#include <LibSPU.h>
#include <inline_o.h>
#include <gtemac.h>


#include "ourtypes.h"
#include "mathbits.h"

#include "main.h"
#include "skeleton.h"
#include "camera.h"
#include "tracks.h"
#include "players.h"
#include "pads.h"

#include "screen.h"
#include "who.h"
#include "bored.h"
#include "animctrl.h"

#include "bonebag.h"
#include "trixbag.h"
#include "plume.h"
#include "cd.h"
#include "skins.h"
#include "frontend.h"
#include "parachut.h"
#include "logic.h"

//#define LIMBS	// Limb manipulator and skeleton debug mode

long	playerrotz = 0;

MATRIX TransMat;
MATRIX RotMat;

// and the verts ( these change all the time )
uint16 NumberModVerts[ MAX_PLAYERS ];
SVECTOR ModVerts[ MAX_PLAYERS ][ MAX_MODEL_VERTS ];

uint16 SkeletonModelled[ 2 ];

uint16 Ingame = 0;

LINE_F2 TPrims[ 2 ][ 300 ];
POLY_F4 NodeMarker[ 2 ][ 300 ];

vec3 EmptyVec = { CHFL( 0.0 ), CHFL( 0.0 ), CHFL( 0.0 ) };

player *Plyr;	// This hold the current player for all the routines to access

chmatrix Matrix_Stack[ 40 ];
vec3 Cursed_Stack[ 40 ];


uint16 LimbDepth;
uint16 Limb;

// Ordering Table for the skeleton
uint8 SkeletonMode;

MATRIX ColourMatrix, LightMatrix;

// *********************************************************************************
// void Initialise_Skeleton( void )
//
// Called before starting each race.  Initialises everything to do with the skeleton
// and characters of each player.
// *********************************************************************************

void SetSkeletonMatrices( void )
{
	SVECTOR LightVec = { 3072, 0, 3072 };
	VECTOR Out;

	ColourMatrix.m[ 0 ][ 0 ] = 4096;
	ColourMatrix.m[ 1 ][ 0 ] = 4096;
	ColourMatrix.m[ 2 ][ 0 ] = 4096;


	ApplyRotMatrix( &LightVec, &Out );

	LightMatrix.m[ 0 ][ 0 ] = Out.vx;
	LightMatrix.m[ 0 ][ 1 ] = Out.vy;
	LightMatrix.m[ 0 ][ 2 ] = Out.vz;

	gte_SetBackColor( 127, 127, 127 );
	gte_SetLightMatrix(&LightMatrix);
	gte_SetColorMatrix(&ColourMatrix);
}




void Initialise_Skeleton( void )
{
	static uint16 CalledBefore = 0;
	uint8 i;
	uint16 j;

	printf("Exec : Initialise_Skeleton");

	if( CalledBefore == 0 )
	{
		// Cold Initialise
		printf(" - Cold");

		CalledBefore = 1;

		for( i = 0; i < MAX_PLAYERS; i++ )
		{
			Player[ i ].Character = BONE;	// Each player initially wants to be BONE
			Player[ i ].Loaded = NONE;		// But no character is loaded
			Player[ i ].PoseBuffer = (uint8 *)&PoseBuffer[ 0 ];
			Player[ i ].PoseLoaded = NONE;
			Player[ i ].SuitLoaded = 80;
			Player[ i ].Suit = 0;
		}
		SkeletonMode = 0;
	}

	printf("\n");

	// Warm Initialise
	Limb = 0;

	Init_Bored();

	for( i = 0; i < MAX_PLAYERS; i++ )
	{
//		if( ( Player[ i ].Character != Player[ i ].Loaded ) || ( Player[ i ].Suit != Player[ i ].SuitLoaded ) )
		{
			// This player does not have the required skeleton.  We need to load it.
			char LoadName[ 32 ];
			char BagName[ 32 ];
			uint32 FileHandle;
			long FileSize;

			// Set the default file names - these may be overridden soon though
			sprintf( BagName, "BONE.BAG" );

#if THIS_IS_THE_PSX == 1
			if( Pad_State( i ) == ( PAD_UP + PAD_L1 + PAD_L2 + PAD_B1 + PAD_B3 ) )
				Player[ i ].Character = YETI;
#endif

			if( Ingame == 0 )
			{
				LoadSkinAndTexture( i, 0, Player[ i ].Suit );
			}
			else
			{
				if( NumberOfPlayers == 2 )
					LoadSkinAndTexture( i, 1, Player[ i ].Suit );	// Load lo-res characters
				else
					LoadSkinAndTexture( i, 0, Player[ i ].Suit );
			}

			Player[ i ].CharVerts = (chfloat *)&Skin_Verts[ i ][ 0 ];
			Player[ i ].CharNorms = (chfloat *)&Skin_Norms[ i ][ 0 ];
			Player[ i ].CharPolys = (model_polys *)&Skin_Polys[ i ][ 0 ];
			Player[ i ].LimbDepth = 19;

			// Ensure the correct pose bag is loaded
			if( Player[ i ].PoseLoaded != Player[ i ].Character )
			{
				#ifndef CD
					sprintf( LoadName, "BAGS/%s", BagName );
					printf( "PC Loading %s\n", LoadName );
					FileHandle = PCopen( (char*)&LoadName, 0, 0 );
					FileSize = PClseek( FileHandle, 0, 2 );
					PClseek( FileHandle, 0, 0 );
					PCread( FileHandle, &PoseBuffer[ 0 ], FileSize );
					PCclose( FileHandle );
				#else
					sprintf( LoadName, "\\BAGS\\%s;1", BagName );
					printf( "CD Loading %s\n", LoadName );
					load_cd( &LoadName, &PoseBuffer[ 0 ] );
					while (CdReadSync(1,0) > 0 );
				#endif
			}

			SetPose( i, 0 );	// Set this character to the first frame

			Player[ i ].PoseLoaded = Player[ i ].Character;
		}
	}

//	printf("Calling NEWInitialiseSkeletonVertices\n");
//	NEWInitialiseSkeletonVertices( (node *)Player[ 0 ].SkeletonRoot, EmptyVec );

	Initialise_Skeleton_Traverse( (node*)&BONE_ROOT, EmptyVec );
	Initialise_Skeleton_Traverse( (node*)&GOAT_ROOT, EmptyVec );
	Initialise_Skeleton_Traverse( (node*)&TRIX_ROOT, EmptyVec );
	Initialise_Skeleton_Traverse( (node*)&YOYO_ROOT, EmptyVec );
	Initialise_Skeleton_Traverse( (node*)&YETI_ROOT, EmptyVec );
}






void LoadSkinAndTexture( uint16 i, uint16 HiLo, uint16 Suit )
{
	char LoadName[ 32 ];
	char SkinName[ 32 ];
	char TextureName[ 32 ];
	uint32 FileHandle;
	long FileSize;
	TIM_IMAGE Tim_Header;
	uint32 j;

//	HiLo = 1;

	// default skin name
	sprintf( SkinName, "Y_SKIN.BIN" );

	Player[ i ].HiLo = HiLo;
	Player[ i ].Suit = Suit;

	switch( Player[ i ].Character )
	{
		case BONE:
		{
			sprintf( TextureName, "BONE" );

			Player[ i ].SkeletonRoot = (node *)&BONE_ROOT;
			if( HiLo == 0 )
				sprintf( SkinName, "B_SKIN.BIN" );
			else
				sprintf( SkinName, "B_SKIN2.BIN" );
		}
		break;

		case GOAT:
		{
			sprintf( TextureName, "GOAT" );

			Player[ i ].SkeletonRoot = (node *)&GOAT_ROOT;
			if( HiLo == 0 )
				sprintf( SkinName, "G_SKIN.BIN" );
			else
				sprintf( SkinName, "G_SKIN2.BIN" );
		}
		break;

		case TRIX:
		{
			sprintf( TextureName, "TRIX" );

			Player[ i ].SkeletonRoot = (node *)&TRIX_ROOT;
			if( HiLo == 0 )
				sprintf( SkinName, "T_SKIN.BIN" );
			else
				sprintf( SkinName, "T_SKIN2.BIN" );
		}
		break;

		case YOYO:
		{
			sprintf( TextureName, "YOYO" );

			Player[ i ].SkeletonRoot = (node *)&YOYO_ROOT;
			if( HiLo == 0 )
				sprintf( SkinName, "Y_SKIN.BIN" );
			else
				sprintf( SkinName, "Y_SKIN2.BIN" );
		}
		break;

		case YETI:
		{
			sprintf( TextureName, "YETI" );

			Player[ i ].SkeletonRoot = (node *)&YETI_ROOT;
			if( HiLo == 0 )
				sprintf( SkinName, "M_SKIN.BIN" );
			else
				sprintf( SkinName, "M_SKIN2.BIN" );
		}
		break;
	}

	cdbuffer = (char*)&Ib2[ 0 ];

	// We may only need to load the textures if the skin is already loaded
//	if( Player[ i ].Loaded != Player[ i ].Character )
	{
		#ifndef CD
			sprintf( LoadName, "CHARS/%s", SkinName );
			printf( "PC Loading SKIN %s\n", LoadName );
			FileHandle = PCopen( (char*)&LoadName, 0, 0 );
			FileSize = PClseek( FileHandle, 0, 2 );
			PClseek( FileHandle, 0, 0 );
			PCread( FileHandle, (char*)(cdbuffer + 180000), FileSize );
			PCclose( FileHandle );
		#else
			sprintf( LoadName, "\\CHARS\\%s;1", SkinName );
			printf( "CD Loading SKIN %s\n", LoadName );
			load_cd( &LoadName, (char*)(cdbuffer + 180000) );
			while (CdReadSync(1,0) > 0 );
		#endif


		cdbuffer = (char*)&Ib2[ 0 ];
		// Extract the skin data from the CD buffer into the skins.c structures
//		Player[ i ].CharNumVerts = (uint32) *( cdbuffer + 180000 );
		//(uint32)cdbuffer[ 45000 ];
		//(uint32)*( cdbuffer + 180000 );
//		Player[ i ].CharNumPolys = (uint32) *( cdbuffer + 180004 );
		//(uint32)cdbuffer[ 45001 ];
		//(uint32)*( cdbuffer + 180004 );
		memcpy( &Player[ i ].CharNumVerts, (uint8*)( cdbuffer + 180000 ), 4 );
		memcpy( &Player[ i ].CharNumPolys, (uint8*)( cdbuffer + 180004 ), 4 );

		printf("Number of verts %ld, Number of polys = %ld\n", Player[ i ].CharNumVerts, Player[ i ].CharNumPolys );

		memcpy( &Skin_Verts[ i ][ 0 ], (uint8*)( cdbuffer + 180008 ), ( Player[ i ].CharNumVerts * 12  ) );
		memcpy( &Skin_Norms[ i ][ 0 ], (uint8*)( cdbuffer + 180008 + ( Player[ i ].CharNumVerts * 12  ) ), Player[ i ].CharNumVerts * 12 );
		memcpy( &Skin_Polys[ i ][ 0 ], (uint8*)( cdbuffer + 180008 + ( Player[ i ].CharNumVerts * 12 * 2  ) ), Player[ i ].CharNumPolys * sizeof( model_polys ) );

		Player[ i ].Loaded = Player[ i ].Character;
		Player[ i ].SuitLoaded = -1;

		for( j = 0; j < Player[ i ].CharNumPolys; j++ )
		{
			if(
				Skin_Polys[ i ][ j ].RGB.r == 255 &&
				Skin_Polys[ i ][ j ].RGB.b == 255 &&
				Skin_Polys[ i ][ j ].RGB.g == 255 )

			{
//				printf("Change %d\n", j );
				Skin_Polys[ i ][ j ].Type = TEXTUREDB;

				Skin_Polys[ i ][ j ].V0 = 128 - ( Skin_Polys[ i ][ j ].V0 /= 2 );
				Skin_Polys[ i ][ j ].V1 = 128 - ( Skin_Polys[ i ][ j ].V1 /= 2 );
				Skin_Polys[ i ][ j ].V2 = 128 - ( Skin_Polys[ i ][ j ].V2 /= 2 );
				Skin_Polys[ i ][ j ].V3 = 128 - ( Skin_Polys[ i ][ j ].V3 /= 2 );

				if( i == 1 )
				{
					Skin_Polys[ i ][ j ].V0 += 128;
					Skin_Polys[ i ][ j ].V1 += 128;
					Skin_Polys[ i ][ j ].V2 += 128;
					Skin_Polys[ i ][ j ].V3 += 128;
				}
			}
		}
	}

	if( Player[ i ].SuitLoaded != Player[ i ].Suit )
	{
		// Now load up the correct textures
		#ifndef CD
			sprintf( LoadName, "CHARS/%s%d.TIM", TextureName, Suit + 1 );
			printf( "PC Loading SUIT %s\n", LoadName );
			FileHandle = PCopen( LoadName, 0, 0 );
			FileSize = PClseek( FileHandle, 0, 2 );
			PClseek( FileHandle, 0, 0 );
			PCread( FileHandle, (char*)(cdbuffer + 180000), FileSize );
			PCclose( FileHandle );
		#else
			sprintf( LoadName, "\\CHARS\\%s%d.TIM;1", TextureName, Suit + 1 );
			printf( "CD Loading SUIT %s\n", LoadName );
			load_cd( LoadName, (char*)(cdbuffer + 180000) );
			while( CdReadSync( 1 , 0 ) > 0 );
		#endif

		OpenTIM( (unsigned long *)&cdbuffer[ 180000 ] );
		ReadTIM( &Tim_Header );

		if( i == 0 )
		{
			Player[ i ].CharPage = LoadTPage( Tim_Header.paddr, 1, 3, 576, 256, 128, 256 ); // 576 + 64
			Player[ i ].CharClut = LoadClut( Tim_Header.caddr, 768, 291 ); // 262
		}
		else
		{
			Player[ i ].CharPage = LoadTPage( Tim_Header.paddr, 1, 3, 576 + 64, 256, 128, 256 );
			Player[ i ].CharClut = LoadClut( Tim_Header.caddr, 768, 292 );
		}

		Player[ i ].SuitLoaded = Player[ i ].Suit;
	}
}







// ******************************************************************************
// void Draw_Character( uint8 PlayerNumber )
//
// Calculates the required pose, then transforms the monged vertices to that pose
// and finally renders the characters polygons.
// ******************************************************************************
VECTOR ScaleUp = { 4<<12, 4<<12, 4<<12 };
VECTOR ScaleDown = { 1024, 1024, 1024 };
VECTOR ScaleShadowDown = { 1024, 1024, 1024 };
//chmatrix IDMatrix;

void Draw_Character( uint8 PlayerNumber, uint8 Other )
{
	SVECTOR ObjRot, T1, ShadRot, TRot, LightRot;
	player* Plyr2 = &Player[ ( PlayerNumber + 1 ) & 1 ];
	VECTOR T2;
	static MATRIX ShadowRotation, ObjectRotation, ObjectTrans, BoredTrans, ShadowTrans, LightRotation;
	uint16 j;
	uint32 i, index;
	long Tempvz;
	chfloat t0, t1, t2;

	Plyr = &Player[ PlayerNumber ];

	if( SkeletonModelled[ PlayerNumber ] == 0 )
	{
		Control_Anim( PlayerNumber );	// Animation controller

		Matrix_Init( &Matrix_Stack[ 0 ] );

		Cursed_Stack[ 0 ].X = 0;
		Cursed_Stack[ 0 ].Y = 0;
		Cursed_Stack[ 0 ].Z = 0;

		NumberModVerts[ PlayerNumber ] = Plyr->CharNumVerts;
		LimbDepth = 0;
		Traverse_Matrix_Skeleton( PlayerNumber, (node *)Plyr->SkeletonRoot, &Matrix_Stack[ 0 ], &Cursed_Stack[ 0 ] );//EmptyVec );
		t0 = -Vogues[ PlayerNumber ].Tx;
		t1 = Vogues[ PlayerNumber ].Ty;
		t2 = Vogues[ PlayerNumber ].Tz;
		for( j = 0; j < NumberModVerts[ PlayerNumber ]; j++ )
		{
			#ifndef LIMBS
			ModVerts[ PlayerNumber ][ j ].vx += t0;//Vogues[ PlayerNumber ].Tx;
			ModVerts[ PlayerNumber ][ j ].vy += t1;//Vogues[ PlayerNumber ].Ty;
			ModVerts[ PlayerNumber ][ j ].vz += t2;//Vogues[ PlayerNumber ].Tz;
			#endif
		}

		SkeletonModelled[ PlayerNumber ] = 1;	// Signal that this beast has been modelled
	}

	PushMatrix();

	if( ConsolePlayers == 2 )
	{
		if( Ingame == 1 )
		{
			if( currentsave.options[ SPLIT_SET ] == 0 )
			{
				gte_SetGeomOffset( 60,160 );
			}
			else
			{
				gte_SetGeomOffset( 120, 80 );
			}
		}
		else
		{
			gte_SetGeomOffset( 120,160 );
		}
	}
	else
	{
		gte_SetGeomOffset( 120,160 );
	}

	if( Other == 0 )
		SetRotMatrix( &Plyr->Cam.ViewMatrix );
	else
		SetRotMatrix( &Plyr2->Cam.ViewMatrix );

	ObjRot.vx = CHSHIFTR( Plyr->Rot.X );
	ObjRot.vy = CHSHIFTR( Plyr->Rot.Y );
	//ObjRot.vz = CHSHIFTR( Plyr->Rot.Z );

	if(onice)
	{
		ObjRot.vz = CHSHIFTR(Plyr->BoardRoll) + CHSHIFTR( Plyr->Rot.Z );
		if(abs(ObjRot.vz - cam_zrot) < cam_zrotaccel)
			cam_zrot = ObjRot.vz;
		else
		{
			if(ObjRot.vz > cam_zrot)
			{
				cam_zrot += cam_zrotaccel;
				if(cam_zrot > cam_zrotmax)
					cam_zrot = cam_zrotmax;
				if(cam_zrot < -cam_zrotmax)
					cam_zrot = -cam_zrotmax;
			}
			else
			{
				cam_zrot -= cam_zrotaccel;
				if(cam_zrot > cam_zrotmax)
					cam_zrot = cam_zrotmax;
				if(cam_zrot < -cam_zrotmax)
					cam_zrot = -cam_zrotmax;
			}
		}

	}
	else
		ObjRot.vz = CHSHIFTR( Plyr->Rot.Z );



	ShadRot.vx = CHSHIFTR( Plyr->Rot.X );
	ShadRot.vy = CHSHIFTR( Plyr->Rot.Y );
//	ShadRot.vz = CHSHIFTR( Plyr->Rot.Z );
//	ShadRot.vx = CHFL( 0.0 );
//	ShadRot.vy = CHFL( 0.0 );
	ShadRot.vz = CHFL( 0.0 );

	if( Other == 0 )
	{
		T1.vx = CHSHIFTR( -Plyr->Pos.X ) + CHSHIFTR( Plyr->Cam.Pos.X );
		T1.vy = CHSHIFTR( Plyr->Pos.Y ) - CHSHIFTR( Plyr->Cam.Pos.Y );
		T1.vz = CHSHIFTR( Plyr->Pos.Z ) - CHSHIFTR( Plyr->Cam.Pos.Z );
	}
	else
	{
		T1.vx = CHSHIFTR( -Plyr->Pos.X ) + CHSHIFTR( Plyr2->Cam.Pos.X );
		T1.vy = CHSHIFTR( Plyr->Pos.Y ) - CHSHIFTR( Plyr2->Cam.Pos.Y );
		T1.vz = CHSHIFTR( Plyr->Pos.Z ) - CHSHIFTR( Plyr2->Cam.Pos.Z );
	}

	ApplyRotMatrix( &T1, &T2 );
	ObjectTrans.t[ 1 ] = T2.vx;
	ObjectTrans.t[ 0 ] = T2.vy;
	ObjectTrans.t[ 2 ] = T2.vz;

	ShadowTrans.t[ 1 ] = T2.vx;
	ShadowTrans.t[ 0 ] = T2.vy + ( CHSHIFTR( Plyr->Pos.Y ) - CHSHIFTR( Plyr->TerrainY ) );
	ShadowTrans.t[ 2 ] = T2.vz;

//	Tempvz = ObjRot.vz;
//	ObjRot.vz = 0;
//	RotMatrix( &ObjRot, &ObjectRotation );
//	RotMatrixZ( Tempvz, &ObjectRotation );
	RotMatrixYXZ( &ObjRot, &ObjectRotation );
//	RotMatrix( &ObjRot, &ObjectRotation );



	RotMatrixYXZ( &ShadRot, &ShadowRotation );

	ScaleShadowDown.vx = 1024 - ( CHSHIFTR( Plyr->JumpY ) * 5 );
	if( ScaleShadowDown.vx < 0 ) ScaleShadowDown.vx = 0;
	ScaleShadowDown.vy = ScaleShadowDown.vz = ScaleShadowDown.vx;

/*

	TRot.vx = 0;
	TRot.vy = 0;
	TRot.vz = 0;
	RotMatrixYXZ( &TRot, &ShadowRotation );


//	RotMatrixZ( ShadRot.vz, &ShadowRotation );
	RotMatrixX( ShadRot.vx, &ShadowRotation );
	RotMatrixY( ShadRot.vy, &ShadowRotation );
	ScaleMatrix( &ShadowRotation, &ScaleShadowDown );	// Scale the x4 man down to normal
*/


	if( Other == 0 )
		SetRotMatrix( &Plyr->Cam.ObjectView );
	else
		SetRotMatrix( &Plyr2->Cam.ObjectView );

	MulRotMatrix( &ShadowRotation );
	MulRotMatrix( &ObjectRotation );

	SetRotMatrix( &ObjectRotation );
	SetTransMatrix( &ObjectTrans );

	ScaleMatrix( &ObjectRotation, &ScaleDown );	// Scale the x4 man down to normal
	ScaleMatrix( &ShadowRotation, &ScaleShadowDown );	// Scale the x4 man down to normal
//	SetRotMatrix( &ObjectRotation );

	// Debugging skeleton drawer code
//	if( SkeletonMode )
//		Draw_Skeleton_3d( (node *)Plyr->SkeletonRoot, CHFL( 0.0 ), CHFL( 0.0 ), CHFL( 0.0 ), CHFL( 0.0 ), CHFL( 0.0 ), CHFL( 0.0 ) );



	LightRot.vx = 0;
	LightRot.vy = -CHSHIFTR( Plyr->Rot.Y );
	LightRot.vz = 0;
	RotMatrix( &LightRot, &LightRotation );
	SetRotMatrix( &LightRotation );

	SetSkeletonMatrices();


	// Draw the model
	SetRotMatrix( &ObjectRotation );
	Draw_Local_Model( PlayerNumber );

//	if( Other == 1 )
//	{
		// Plot imaginary snowboard position and draw the plume
		SetTransMatrix( &BoredTrans );
		Plot_Bored();

		// Draw the shadow
		SetRotMatrix( &ShadowRotation );
		SetTransMatrix( &ShadowTrans );
		Draw_Shadow( PlayerNumber, Other );
//	}

	if( Other==0 )
	{
		DrawLeaves(PlayerNumber);
		if(ConsolePlayers == 2)
			nopolyG3s+=DrawPlume( PlayerNumber,&trackprims[otid][nopolyG3s]);
		else
			nopolyG3s+=DrawPlume( PlayerNumber,&trackprims[otid*2][nopolyG3s]);
	}
	// temporary speed bar
/*
	#if I_AM == JEZ
	{
		static POLY_F4 tSpeedBar[ 2 ];
		static POLY_F4 tSpeedBar2[ 2 ];
		static POLY_F4 tSpeedBar3[ 2 ];

		uint16 Spam = Plyr->Vel / ( CHFL( 30.0 ) / 150 );
		uint16 Spam2 = Plyr->Desired / ( CHFL( 30.0 ) / 150 );

		setPolyF4( &tSpeedBar[ otid ] );
		setPolyF4( &tSpeedBar2[ otid ] );
		setPolyF4( &tSpeedBar3[ otid ] );

		tSpeedBar[ otid ].r0 = 192;
		tSpeedBar[ otid ].g0 = 0;
		tSpeedBar[ otid ].b0 = 0;
		tSpeedBar2[ otid ].r0 = 0;
		tSpeedBar2[ otid ].g0 = 0;
		tSpeedBar2[ otid ].b0 = 0;
		tSpeedBar3[ otid ].r0 = 0;
		tSpeedBar3[ otid ].g0 = 192;
		tSpeedBar3[ otid ].b0 = 0;
		tSpeedBar[ otid ].x0 = 16;
		tSpeedBar[ otid ].y0 = 52;
		tSpeedBar[ otid ].x2 = 16;
		tSpeedBar[ otid ].y2 = 60;
		tSpeedBar[ otid ].x1 = 16 + Spam;
		tSpeedBar[ otid ].y1 = 52;
		tSpeedBar[ otid ].x3 = 16 + Spam;
		tSpeedBar[ otid ].y3 = 60;

		tSpeedBar2[ otid ].x0 = 15;
		tSpeedBar2[ otid ].y0 = 51;
		tSpeedBar2[ otid ].x2 = 15;
		tSpeedBar2[ otid ].y2 = 61;
		tSpeedBar2[ otid ].x1 = 17 + 150;
		tSpeedBar2[ otid ].y1 = 51;
		tSpeedBar2[ otid ].x3 = 17 + 150;
		tSpeedBar2[ otid ].y3 = 61;

		tSpeedBar3[ otid ].x0 = 15;
		tSpeedBar3[ otid ].y0 = 51;
		tSpeedBar3[ otid ].x2 = 15;
		tSpeedBar3[ otid ].y2 = 61;
		tSpeedBar3[ otid ].x1 = 17 + Spam2;
		tSpeedBar3[ otid ].y1 = 51;
		tSpeedBar3[ otid ].x3 = 17 + Spam2;
		tSpeedBar3[ otid ].y3 = 61;

		AddPrim( &OrderingTable[ otid ][ 0 ], &tSpeedBar[ otid ] );
		AddPrim( &OrderingTable[ otid ][ 0 ], &tSpeedBar3[ otid ] );
		AddPrim( &OrderingTable[ otid ][ 0 ], &tSpeedBar2[ otid ] );
	}
	#endif
*/
	// Reset corrupted GTE settings for tidy exit
	SetGeomOffset( 160, 120 );

	PopMatrix();

	// Should there be another dude drawn on this screen ?
	if( NumberOfPlayers == 2 && Other == 0 && Ingame == 1 )
	{
		Draw_Character( ( PlayerNumber + 1 ) & 1, 1 );
	}
}


// *******************************************************************************************
// void Prepare_Skeleton( uint8 PlayerNumber )
//
// Pre-Draw initialise of the individual appropriate skeleton (routine scheduled to be dumped)
// *******************************************************************************************


// Some pre-processing which took some rubbish from the real-time animation
void Initialise_Skeleton_Traverse( node *Node, vec3 Unt )
{
	vec3 Pass;

	int32 i;

	Pass.X = Node->X = Node->Tx + Unt.X;
	Pass.Y = Node->Y = Node->Ty + Unt.Y;
	Pass.Z = Node->Z = Node->Tz + Unt.Z;

 	for( i = 0; i < Node->NumChildren; i++ )
	{
		if( Node->Child[ i ] != NULL )
		{
			Initialise_Skeleton_Traverse( (node*)Node->Child[ i ], Pass );
		}
	}
}


void Traverse_Matrix_Skeleton( uint8 PNum, node *Node, chmatrix *Rp, vec3 *Cursed )
{
	static chmatrix Rotation;

	chmatrix *Scratch = Rp + 1;	// Fake stack
	vec3 *TLocal = Cursed + 1;	// Fake stack

	uint16 i;

//	Vec3_x_Matrix( (vec3*)&Node->Tx, Rp, TLocal );
	// inlined version of the above
	TLocal->X	=	CHSHIFTR( ( Node->Tx * Rp->E00 ) ) +
				CHSHIFTR( ( Node->Ty * Rp->E10 ) ) +
				CHSHIFTR( ( Node->Tz * Rp->E20 ) ) +
				Cursed->X;//Rp->E30 +

	TLocal->Y	=	CHSHIFTR( ( Node->Tx * Rp->E01 ) ) +
				CHSHIFTR( ( Node->Ty * Rp->E11 ) ) +
				CHSHIFTR( ( Node->Tz * Rp->E21 ) ) +
				Cursed->Y;//Rp->E31 +

	TLocal->Z	=	CHSHIFTR( ( Node->Tx * Rp->E02 ) ) +
				CHSHIFTR( ( Node->Ty * Rp->E12 ) ) +
				CHSHIFTR( ( Node->Tz * Rp->E22 ) ) +
				Cursed->Z;//Rp->E32 +

//	This has been included in the above inlining
//	TLocal->X += Cursed->X;
//	TLocal->Y += Cursed->Y;
//	TLocal->Z += Cursed->Z;




//	Matrix_MakeRotXYZ( &Rotation, Vogues[ PNum ].Rx[ LimbDepth ], Vogues[ PNum ].Ry[ LimbDepth ], Vogues[ PNum ].Rz[ LimbDepth ] );
//	inlined version of the above
{
	chfloat CX = rcos( Vogues[ PNum ].Rx[ LimbDepth ] & 4095 );
	chfloat CY = rcos( Vogues[ PNum ].Ry[ LimbDepth ] & 4095 );
	chfloat CZ = rcos( Vogues[ PNum ].Rz[ LimbDepth ] & 4095 );
	chfloat SX = rsin( Vogues[ PNum ].Rx[ LimbDepth ] & 4095 );
	chfloat SY = rsin( Vogues[ PNum ].Ry[ LimbDepth ] & 4095 );
	chfloat SZ = rsin( Vogues[ PNum ].Rz[ LimbDepth ] & 4095 );

	Rotation.E00 = CHSHIFTR( CZ * CY );
	Rotation.E01 = CHSHIFTR( SZ * CY );
	Rotation.E02 = -SY;
	Rotation.E10 = CHSHIFTR(CHSHIFTR(CZ*SY)*SX)-CHSHIFTR(SZ*CX);
	Rotation.E11 = CHSHIFTR(CHSHIFTR(SZ*SY)*SX)+CHSHIFTR(CZ*CX);
	Rotation.E12 = CHSHIFTR( CY*SX );
	Rotation.E20 = CHSHIFTR(CHSHIFTR(CZ*SY)*CX)+CHSHIFTR(SZ*SX);
	Rotation.E21 = CHSHIFTR(CHSHIFTR(SZ*SY)*CX)-CHSHIFTR(CZ*SX);
	Rotation.E22 = CHSHIFTR( CY*CX );
	Rotation.E30 = 0;
	Rotation.E31 = 0;
	Rotation.E32 = 0;
}




//	Matrix_x_Matrix( &Rotation, Rp, Scratch );
//	inlined version of the above
	Scratch->E00	=	CHSHIFTR( ( Rotation.E00 * Rp->E00 ) ) +
		 				CHSHIFTR( ( Rotation.E01 * Rp->E10 ) ) +
					 	CHSHIFTR( ( Rotation.E02 * Rp->E20 ) );

	Scratch->E01	=	CHSHIFTR( ( Rotation.E00 * Rp->E01 ) ) +
					 	CHSHIFTR( ( Rotation.E01 * Rp->E11 ) ) +
					 	CHSHIFTR( ( Rotation.E02 * Rp->E21 ) );

	Scratch->E02	= 	CHSHIFTR( ( Rotation.E00 * Rp->E02 ) ) +
					 	CHSHIFTR( ( Rotation.E01 * Rp->E12 ) ) +
					 	CHSHIFTR( ( Rotation.E02 * Rp->E22 ) );

	Scratch->E10	= 	CHSHIFTR( ( Rotation.E10 * Rp->E00 ) ) +
					 	CHSHIFTR( ( Rotation.E11 * Rp->E10 ) ) +
					 	CHSHIFTR( ( Rotation.E12 * Rp->E20 ) );

	Scratch->E11	= 	CHSHIFTR( ( Rotation.E10 * Rp->E01 ) ) +
					 	CHSHIFTR( ( Rotation.E11 * Rp->E11 ) ) +
					 	CHSHIFTR( ( Rotation.E12 * Rp->E21 ) );

	Scratch->E12	=	CHSHIFTR( ( Rotation.E10 * Rp->E02 ) ) +
					 	CHSHIFTR( ( Rotation.E11 * Rp->E12 ) ) +
					 	CHSHIFTR( ( Rotation.E12 * Rp->E22 ) );

	Scratch->E20	=	CHSHIFTR( ( Rotation.E20 * Rp->E00 ) ) +
					 	CHSHIFTR( ( Rotation.E21 * Rp->E10 ) ) +
					 	CHSHIFTR( ( Rotation.E22 * Rp->E20 ) );

	Scratch->E21	=	CHSHIFTR( ( Rotation.E20 * Rp->E01 ) ) +
					 	CHSHIFTR( ( Rotation.E21 * Rp->E11 ) ) +
					 	CHSHIFTR( ( Rotation.E22 * Rp->E21 ) );

	Scratch->E22	=	CHSHIFTR( ( Rotation.E20 * Rp->E02 ) ) +
					 	CHSHIFTR( ( Rotation.E21 * Rp->E12 ) ) +
					 	CHSHIFTR( ( Rotation.E22 * Rp->E22 ) );

// These lines aren't needed as they always end up as 0 and are already set to that
/*	Scratch->E30	=	CHSHIFTR( ( Rotation.E30 * Rp->E00 ) ) +
						CHSHIFTR( ( Rotation.E31 * Rp->E10 ) ) +
						CHSHIFTR( ( Rotation.E32 * Rp->E20 ) ) +
						Rp->E30;

	Scratch->E31	=	CHSHIFTR( ( Rotation.E30 * Rp->E01 ) ) +
						CHSHIFTR( ( Rotation.E31 * Rp->E11 ) ) +
						CHSHIFTR( ( Rotation.E32 * Rp->E21 ) ) +
						Rp->E31;

	Scratch->E32	=	CHSHIFTR( ( Rotation.E30 * Rp->E02 ) ) +
						CHSHIFTR( ( Rotation.E31 * Rp->E12 ) ) +
						CHSHIFTR( ( Rotation.E32 * Rp->E22 ) ) +
						Rp->E32;
*/

	if( Node->Type == JOINT )	// There may be an attachment here
	{
		int16 *Verts = Node->Attached;
		int16 Vn, Vnx3;
		vec3 T;//, Tn;

		if( Player[ PNum ].HiLo != 0 ) Verts = Node->AttachedLo;

		for( Verts; *Verts >= 0 ; Verts++ )
		{
			Vn = *Verts;
			Vnx3 = Vn * 3;

			T.X = Plyr->CharVerts[ Vnx3 + 0 ] - Node->X;
			T.Y = Plyr->CharVerts[ Vnx3 + 1 ] - Node->Y;
			T.Z = Plyr->CharVerts[ Vnx3 + 2 ] - Node->Z;

//			Vec3_x_Matrix( &T, Scratch, &Tn );
//			inlined version of above
/*

			Tn.X	=	CHSHIFTR( ( T.X * Scratch->E00 ) ) +
						CHSHIFTR( ( T.Y * Scratch->E10 ) ) +
						CHSHIFTR( ( T.Z * Scratch->E20 ) ) +
						TLocal->X;//Scratch->E30 +

			Tn.Y	=	CHSHIFTR( ( T.X * Scratch->E01 ) ) +
						CHSHIFTR( ( T.Y * Scratch->E11 ) ) +
						CHSHIFTR( ( T.Z * Scratch->E21 ) ) +
						TLocal->Y;//Scratch->E31 +

			Tn.Z	=	CHSHIFTR( ( T.X * Scratch->E02 ) ) +
						CHSHIFTR( ( T.Y * Scratch->E12 ) ) +
						CHSHIFTR( ( T.Z * Scratch->E22 ) ) +
						TLocal->Z;//Scratch->E32 +

			ModVerts[ Vn ].vx = -CHSHIFTR( Tn.X );//+ TLocal->X );
			ModVerts[ Vn ].vy = CHSHIFTR( Tn.Y );//+ TLocal->Y );
			ModVerts[ Vn ].vz = CHSHIFTR( Tn.Z );//+ TLocal->Z );
*/

			// Slightly faster version of above inlining
			ModVerts[ PNum ][ Vn ].vx	=	-CHSHIFTR( CHSHIFTR( ( T.X * Scratch->E00 ) ) +
									CHSHIFTR( ( T.Y * Scratch->E10 ) ) +
									CHSHIFTR( ( T.Z * Scratch->E20 ) ) +
									TLocal->X );//Scratch->E30 +

			ModVerts[ PNum ][ Vn ].vy	=	CHSHIFTR( CHSHIFTR( ( T.X * Scratch->E01 ) ) +
									CHSHIFTR( ( T.Y * Scratch->E11 ) ) +
									CHSHIFTR( ( T.Z * Scratch->E21 ) ) +
									TLocal->Y );//Scratch->E31 +

			ModVerts[ PNum ][ Vn ].vz	=	CHSHIFTR( CHSHIFTR( ( T.X * Scratch->E02 ) ) +
									CHSHIFTR( ( T.Y * Scratch->E12 ) ) +
									CHSHIFTR( ( T.Z * Scratch->E22 ) ) +
									TLocal->Z );//Scratch->E32 +
		}
	}

	LimbDepth++;

	for( i = 0; i < Node->NumChildren; i++ )
	{
		if( Node->Child[ i ] != NULL )
		{
			Traverse_Matrix_Skeleton( PNum, (node*)Node->Child[ i ], Scratch, TLocal );
		}
	}
}



void Draw_Skeleton_3d( node *Node, chfloat X, chfloat Y, chfloat Z, chfloat Rx, chfloat Ry, chfloat Rz )
{
/*
	uint16 i;

	SVECTOR A, B;

	long D1, D2, P;
	long Flag;

	// Transform my start, which has been passed on from my parent's end
	A.vx = -Node->X;
	A.vy = Node->Y;
	A.vz = Node->Z;
//	RotTransPers( &A, &D1, &P, &Flag );
 	gte_ldv0(&A);
	gte_rtps();

	OTPtr = &OrderingTable[ otid ][ 0 ];

	gte_stsxy(&D1);

	switch( Node->Type )
	{
		case CHAIN_ROOT:
			setLineF2( &TPrims[ otid ][ Tp ] );
			TPrims[ otid ][ Tp ].r0 = 255;
			TPrims[ otid ][ Tp ].g0 = 0;
			TPrims[ otid ][ Tp ].b0 = 0;
		 	TPrims[ otid ][ Tp ].x0 = -4 + ( D1 >> 16 );
			TPrims[ otid ][ Tp ].y0 = -4 + ( D1 & 0xffff );
		 	TPrims[ otid ][ Tp ].x1 = 4 + ( D1 >> 16 );
			TPrims[ otid ][ Tp ].y1 = 4 + ( D1 & 0xffff );
			AddPrim( OTPtr, &TPrims[ otid ][ Tp ] );
			Tp++;
			setLineF2( &TPrims[ otid ][ Tp ] );
			TPrims[ otid ][ Tp ].r0 = 255;
			TPrims[ otid ][ Tp ].g0 = 0;
			TPrims[ otid ][ Tp ].b0 = 0;
		 	TPrims[ otid ][ Tp ].x0 = +4 + ( D1 >> 16 );
			TPrims[ otid ][ Tp ].y0 = -4 + ( D1 & 0xffff );
		 	TPrims[ otid ][ Tp ].x1 = -4 + ( D1 >> 16 );
			TPrims[ otid ][ Tp ].y1 = 4 + ( D1 & 0xffff );
			AddPrim( OTPtr, &TPrims[ otid ][ Tp ] );
			Tp++;
			break;

		case EFFECTOR:
			setLineF2( &TPrims[ otid ][ Tp ] );
			TPrims[ otid ][ Tp ].r0 = 0;
			TPrims[ otid ][ Tp ].g0 = 0;
			TPrims[ otid ][ Tp ].b0 = 255;
		 	TPrims[ otid ][ Tp ].x0 = -4 + ( D1 >> 16 );
			TPrims[ otid ][ Tp ].y0 = -4 + ( D1 & 0xffff );
		 	TPrims[ otid ][ Tp ].x1 = 4 + ( D1 >> 16 );
			TPrims[ otid ][ Tp ].y1 = 4 + ( D1 & 0xffff );
			AddPrim( OTPtr, &TPrims[ otid ][ Tp ] );
			Tp++;
			setLineF2( &TPrims[ otid ][ Tp ] );
			TPrims[ otid ][ Tp ].r0 = 0;
			TPrims[ otid ][ Tp ].g0 = 0;
			TPrims[ otid ][ Tp ].b0 = 255;
		 	TPrims[ otid ][ Tp ].x0 = +4 + ( D1 >> 16 );
			TPrims[ otid ][ Tp ].y0 = -4 + ( D1 & 0xffff );
		 	TPrims[ otid ][ Tp ].x1 = -4 + ( D1 >> 16 );
			TPrims[ otid ][ Tp ].y1 = 4 + ( D1 & 0xffff );
			AddPrim( OTPtr, &TPrims[ otid ][ Tp ] );
			Tp++;
			break;

		case JOINT:
			if( (node*)Node->Child[ 0 ] == NULL )
			{
				setLineF2( &TPrims[ otid ][ Tp ] );
				TPrims[ otid ][ Tp ].r0 = 0;
				TPrims[ otid ][ Tp ].g0 = 255;
				TPrims[ otid ][ Tp ].b0 = 0;
			 	TPrims[ otid ][ Tp ].x0 = -2 + ( D1 >> 16 );
				TPrims[ otid ][ Tp ].y0 = -2 + ( D1 & 0xffff );
			 	TPrims[ otid ][ Tp ].x1 = 2 + ( D1 >> 16 );
				TPrims[ otid ][ Tp ].y1 = 2 + ( D1 & 0xffff );
				AddPrim( OTPtr, &TPrims[ otid ][ Tp ] );
				Tp++;
				setLineF2( &TPrims[ otid ][ Tp ] );
				TPrims[ otid ][ Tp ].r0 = 0;
				TPrims[ otid ][ Tp ].g0 = 255;
				TPrims[ otid ][ Tp ].b0 = 0;
			 	TPrims[ otid ][ Tp ].x0 = +2 + ( D1 >> 16 );
				TPrims[ otid ][ Tp ].y0 = -2 + ( D1 & 0xffff );
			 	TPrims[ otid ][ Tp ].x1 = -2 + ( D1 >> 16 );
				TPrims[ otid ][ Tp ].y1 = 2 + ( D1 & 0xffff );
				AddPrim( OTPtr, &TPrims[ otid ][ Tp ] );
				Tp++;
			}
			else
			{
				uint8 g;
				for( g = 0; g < Node->NumChildren; g++ )
				{
				B.vx = -((node*)Node->Child[ g ])->X;
				B.vy = ((node*)Node->Child[ g ])->Y;
				B.vz = ((node*)Node->Child[ g ])->Z;
				RotTransPers( &B, &D2, &P, &Flag );

				setLineF2( &TPrims[ otid ][ Tp ] );
				TPrims[ otid ][ Tp ].r0 = 0;
				TPrims[ otid ][ Tp ].g0 = 0;
				TPrims[ otid ][ Tp ].b0 = 0;
			 	TPrims[ otid ][ Tp ].x0 = 0 + ( D1 >> 16 );
				TPrims[ otid ][ Tp ].y0 = 0 + ( D1 & 0xffff );
			 	TPrims[ otid ][ Tp ].x1 = 0 + ( D2 >> 16 );
				TPrims[ otid ][ Tp ].y1 = 0 + ( D2 & 0xffff );
				AddPrim( OTPtr, &TPrims[ otid ][ Tp ] );
				Tp++;
				}
			}
			break;
	}

	// Now loop through my children
	for( i = 0; i < Node->NumChildren; i++ )
	{
		if( Node->Child[ i ] != NULL )
		{
			Draw_Skeleton_3d( (node*)Node->Child[ i ], Node->X, Node->Y, Node->Z, Rx, Ry, Rz );
		}
	}
*/
}









// Separate primitive list (for any primitive type)
int8 AnyType[ 2 ][ 2 * MAX_MODEL_POLYS * sizeof( POLY_GT4 ) ];	// Maximum polys per character * 2, big enough for two players
void *Ptr;

// Call these once each loop
void Refresh_Skeleton( void )
{
	uint16 MyOtid;

	if( ConsolePlayers == 2 )
	{
		if( otid == 0 || otid == 1 ) MyOtid = 0;
		if( otid == 2 || otid == 3 ) MyOtid = 1;
	}
	else
	{
		MyOtid = otid;
	}

	Ptr = &AnyType[ MyOtid ][ 0 ];				// Point to the start of the skeleton primitive list

	// Signal that neither player skeleton has been traversed
	SkeletonModelled[ 0 ] = 0;
	SkeletonModelled[ 1 ] = 0;

//	Ptr = &AnyType[ otid ][ 0 ];
}


SVECTOR N0, N1, N2, N3;
CVECTOR C0, C1, C2, C3;
CVECTOR Gibbon = { 127, 127, 127, 0 };

#define LINE_RED	100
#define LINE_BLUE	100
#define LINE_GREEN	140


// Each player may be drawn twice you know ( split screen mode )
void Draw_Local_Model( uint8 PNum )
{
	SKBADDPRIMLOCALS

	uint32 NumberModPolys =	Plyr->CharNumPolys;	// How many polys this player has
	model_polys *ModPolys = Plyr->CharPolys;	// Point to this players polygon list
	chfloat *Norms = Plyr->CharNorms;			// Point to the players normals
	unsigned short Clut = Plyr->CharClut;
	uint16 BoardPage, BoardClut;

	// Loop through all the polygons in ModPolys
	static long Z[ 300 ], P, Otz, Flag;//, TVert[ 250 ];

	long *TVert = 0x1f800000;

	long Val;
	uint16 i, v0, v1, v2, v3;

	BoardPage = GetTPage( 1, 3, 960, 0 );

	if( PNum == 0 )
		BoardClut = GetClut( 768, 320 );
	else
		BoardClut = GetClut( 768, 321 );

	for( i = 0; i < NumberModVerts[ PNum ]; i++ )
	{
//		gte_RotTransPers( &ModVerts[ i ], &TVert[ i ], &P, &Flag, &Z[ i ] );
//		Z[ i ] = RotTransPers( &ModVerts[ i ], &TVert[ i ], &P, &Flag );
		gte_ldv0(&ModVerts[ PNum ][i]);
        gte_rtps();
        gte_stsxy( &TVert[ i ] );
		gte_stsz(&Z[i]);
	}

	if( Plyr->ConsoleRemoteGhosted == GHOSTED_PLAYER )
	{
		for( i = 0; i < NumberModPolys; i++ )
		{
			v0 = ModPolys[ i ].VertNum[ 0 ];
			v1 = ModPolys[ i ].VertNum[ 1 ];
			v2 = ModPolys[ i ].VertNum[ 2 ];

			if( ModPolys[ i ].NumberOfVerts == 3 )
			{
				// It's a triangle
				gte_NormalClip( TVert[ v0 ], TVert[ v1 ], TVert[ v2 ], &Val );

				if( Val >= 0 )
				{
					LINE_F4 *LF4 = Ptr;

					gte_AverageZ3( Z[ v0 ], Z[ v1 ], Z[ v2 ], &Otz );

			  		*( ( (long*) Ptr) + 1 ) = LINE_RED | ( LINE_BLUE << 8 ) | ( LINE_GREEN << 16 );

			  		*( ( (long*) Ptr) + 2 ) = ( TVert[ v0 ] >> 16 ) | ( TVert[ v0 ] << 16 );
			  		*( ( (long*) Ptr) + 3 ) = ( TVert[ v1 ] >> 16 ) | ( TVert[ v1 ] << 16 );
			  		*( ( (long*) Ptr) + 4 ) = ( TVert[ v2 ] >> 16 ) | ( TVert[ v2 ] << 16 );
			  		*( ( (long*) Ptr) + 5 ) = ( TVert[ v0 ] >> 16 ) | ( TVert[ v0 ] << 16 );

					setLineF4( LF4 );

					if( Otz < OTLENGTH && Otz > 0 )
					{
//						Otz+=30;
//						if( Otz > maxcharot ) maxcharot = Otz;
						SKBADDPRIM( &OrderingTable[ otid ][ Otz ], LF4 );
					}

					Ptr += sizeof( LINE_F4 );
				}
			}
			else
			{
				// It's a quad
				v3 = ModPolys[ i ].VertNum[ 3 ];

				// It's a quad
				gte_NormalClip( TVert[ v0 ], TVert[ v1 ], TVert[ v2 ], &Val );

				if( Val >= 0 )
				{
					LINE_F4 *LF4 = Ptr;
					LINE_F2 *LF2;

					gte_AverageZ4( Z[ v0 ], Z[ v1 ], Z[ v2 ], Z[ v3 ], &Otz );

			  		*( ( (long*) Ptr) + 1 ) = LINE_RED | ( LINE_BLUE << 8 ) | ( LINE_GREEN << 16 );

			  		*( ( (long*) Ptr) + 2 ) = ( TVert[ v0 ] >> 16 ) | ( TVert[ v0 ] << 16 );
			  		*( ( (long*) Ptr) + 3 ) = ( TVert[ v1 ] >> 16 ) | ( TVert[ v1 ] << 16 );
			  		*( ( (long*) Ptr) + 4 ) = ( TVert[ v2 ] >> 16 ) | ( TVert[ v2 ] << 16 );
			  		*( ( (long*) Ptr) + 5 ) = ( TVert[ v3 ] >> 16 ) | ( TVert[ v3 ] << 16 );

					setLineF4( LF4 );

					if( Otz < OTLENGTH && Otz > 0 )
					{
//						Otz+=30;
//						if( Otz > maxcharot ) maxcharot = Otz;
						SKBADDPRIM( &OrderingTable[ otid ][ Otz ], LF4 );
					}

					Ptr += sizeof( LINE_F4 );

					// Plot the last line
					LF2 = Ptr;
			  		*( ( (long*) Ptr) + 1 ) = LINE_RED | ( LINE_BLUE << 8 ) | ( LINE_GREEN << 16 );

			  		*( ( (long*) Ptr) + 2 ) = ( TVert[ v3 ] >> 16 ) | ( TVert[ v3 ] << 16 );
			  		*( ( (long*) Ptr) + 3 ) = ( TVert[ v0 ] >> 16 ) | ( TVert[ v0 ] << 16 );

					setLineF2( LF2 );

					if( Otz < OTLENGTH && Otz > 0 )
					{
//						Otz+=30;
//						if( Otz > maxcharot ) maxcharot = Otz;
						SKBADDPRIM( &OrderingTable[ otid ][ Otz ], LF2 );
					}

					Ptr += sizeof( LINE_F2 );
				}
			}
		}
	}
	else	// Full polygon drawing
	{
		for( i = 0; i < NumberModPolys; i++ )
		{
			v0 = ModPolys[ i ].VertNum[ 0 ];
			v1 = ModPolys[ i ].VertNum[ 1 ];
			v2 = ModPolys[ i ].VertNum[ 2 ];

//			if( ModPolys[ i ].NumberOfVerts == 5 );// && ModPolys[ i ].NumberOfVerts != 4 )
//			{
//				printf("Bollocks\n");
//			}

			if( ModPolys[ i ].NumberOfVerts == 3 )
			{
				// It's a triangle
				gte_NormalClip( TVert[ v0 ], TVert[ v1 ], TVert[ v2 ],&Val );

				if( Val >= 0 )
				{
					switch( ModPolys[ i ].Type )
					{
							case GOURAUD:
							{
								POLY_G3 *G3 = Ptr;

								gte_NormalColorCol3(	&Norms[ v0 * 3 ], &Norms[ v1 * 3 ], &Norms[ v2 * 3 ],
													&ModPolys[ i ].RGB,
													&G3->r0, &G3->r1, &G3->r2 );

								gte_AverageZ3( Z[ v0 ], Z[ v1 ], Z[ v2 ], &Otz );

						  		*( ( (long*) Ptr) + 2 ) = ( TVert[ v0 ] >> 16 ) | ( TVert[ v0 ] << 16 );
						  		*( ( (long*) Ptr) + 4 ) = ( TVert[ v1 ] >> 16 ) | ( TVert[ v1 ] << 16 );
						  		*( ( (long*) Ptr) + 6 ) = ( TVert[ v2 ] >> 16 ) | ( TVert[ v2 ] << 16 );
//						  		*( ( (long*) Ptr) + 2 ) = TVert[ v0 ];
//						  		*( ( (long*) Ptr) + 4 ) = TVert[ v1 ];
//						  		*( ( (long*) Ptr) + 6 ) = TVert[ v2 ];

								setPolyG3( G3 );

								if( Otz < OTLENGTH && Otz > 0 )
								{
//									printf("%ld\n", Otz);
//									Otz+=30;
//									if( Otz > maxcharot ) maxcharot = Otz;
									SKBADDPRIM( &OrderingTable[ otid ][ Otz ], G3 );
								}

								Ptr += sizeof( POLY_G3 );
							}

							case TEXTURED:
							{

								POLY_GT3 *G3 = Ptr;

								gte_NormalColorCol3(	&Norms[ v0 * 3 ], &Norms[ v1 * 3 ], &Norms[ v2 * 3 ],
													&Gibbon,
//													&ModPolys[ i ].RGB,
													&G3->r0, &G3->r1, &G3->r2 );

								gte_AverageZ3( Z[ v0 ], Z[ v1 ], Z[ v2 ], &Otz );

						  		*( ( (long*) Ptr) + 2 ) = ( TVert[ v0 ] >> 16 ) | ( TVert[ v0 ] << 16 );
						  		*( ( (long*) Ptr) + 5 ) = ( TVert[ v1 ] >> 16 ) | ( TVert[ v1 ] << 16 );
						  		*( ( (long*) Ptr) + 8 ) = ( TVert[ v2 ] >> 16 ) | ( TVert[ v2 ] << 16 );
//						  		*( ( (long*) Ptr) + 2 ) = TVert[ v0 ];
//						  		*( ( (long*) Ptr) + 5 ) = TVert[ v1 ];
//						  		*( ( (long*) Ptr) + 8 ) = TVert[ v2 ];

								// Write UV spaces as words (for speed)
						  		*( ( (long*) Ptr) + 3 ) = ModPolys[ i ].U0 | ( ModPolys[ i ].V0 << 8 ) | ( Clut << 16 );
						  		*( ( (long*) Ptr) + 6 ) = ModPolys[ i ].U1 | ( ModPolys[ i ].V1 << 8 ) | ( Plyr->CharPage << 16);
						  		*( ( (long*) Ptr) + 9 ) = ModPolys[ i ].U2 | ( ModPolys[ i ].V2 << 8 );

								setPolyGT3( G3 );
//							SetShadeTex( G3, 1 );

								if( Otz < OTLENGTH && Otz > 0 )
								{
//									printf("%ld\n", Otz);
//									Otz+=30;
//									if( Otz > maxcharot ) maxcharot = Otz;
									SKBADDPRIM( &OrderingTable[ otid ][ Otz ], G3 );
								}

								Ptr += sizeof( POLY_GT3 );
							}
					}
				}
			}	// end of triangle render
			else
			{
				// It's a quad
				v3 = ModPolys[ i ].VertNum[ 3 ];

				gte_NormalClip( TVert[ v0 ], TVert[ v1 ], TVert[ v2 ], &Val );

				if( Val >= 0 )
				{
					switch( ModPolys[ i ].Type )
					{
					 	case GOURAUD:
					  	{
					  		POLY_G4 *G4 = Ptr;

					  		gte_NormalColorCol3( &Norms[ v0 * 3 ], &Norms[ v1 * 3 ], &Norms[ v2 * 3 ],
					  							&ModPolys[ i ].RGB,
					  							&G4->r0, &G4->r1, &G4->r3 );
					  		gte_NormalColorCol(	&Norms[ v3 * 3 ],
			  								&ModPolys[ i ].RGB,
			  								&G4->r2 );

					  		gte_AverageZ4( Z[ v0 ], Z[ v1 ], Z[ v2 ], Z[ v3 ], &Otz );
//					  		Otz = ( Z[ v0 ] + Z[ v1 ] + Z[ v2 ] + Z[ v3 ] ) >> 2;

					  		*( ( (long*) Ptr) + 2 ) = ( TVert[ v0 ] >> 16 ) | ( TVert[ v0 ] << 16 );
					  		*( ( (long*) Ptr) + 4 ) = ( TVert[ v1 ] >> 16 ) | ( TVert[ v1 ] << 16 );
					  		*( ( (long*) Ptr) + 6 ) = ( TVert[ v3 ] >> 16 ) | ( TVert[ v3 ] << 16 );
					  		*( ( (long*) Ptr) + 8 ) = ( TVert[ v2 ] >> 16 ) | ( TVert[ v2 ] << 16 );
//					  		*( ( (long*) Ptr) + 2 ) = TVert[ v0 ];
//					  		*( ( (long*) Ptr) + 4 ) = TVert[ v1 ];
//					  		*( ( (long*) Ptr) + 6 ) = TVert[ v3 ];
//					  		*( ( (long*) Ptr) + 8 ) = TVert[ v2 ];

					  		setPolyG4( G4 );

					  		if( Otz < OTLENGTH && Otz > 0 )
				  			{
//								printf("%ld\n", Otz);
//								Otz+=30;
//								if( Otz > maxcharot ) maxcharot = Otz;
								SKBADDPRIM( &OrderingTable[ otid ][ Otz ], G4 );
					  		}

					  		Ptr += sizeof( POLY_G4 );
					  	}
					  	break;

					 	case TEXTURED:
						case TEXTUREDB:
					  	{
					  		POLY_GT4 *G4 = Ptr;

					  		gte_NormalColorCol3(	&Norms[ v0 * 3 ], &Norms[ v1 * 3 ], &Norms[ v2 * 3 ],
					  							&Gibbon,
//&ModPolys[ i ].RGB,
					  							&G4->r0, &G4->r1, &G4->r3 );
					  		gte_NormalColorCol(	&Norms[ v3 * 3 ],
//  &ModPolys[ i ].RGB,
			  								&Gibbon,
			  								&G4->r2 );

					  		gte_AverageZ4( Z[ v0 ], Z[ v1 ], Z[ v2 ], Z[ v3 ], &Otz );

							// Write VERTS out as words ( instead of G4->x0 for speed )
					  		*( ( (long*) Ptr) + 2 ) = ( TVert[ v0 ] >> 16 ) | ( TVert[ v0 ] << 16 );
					  		*( ( (long*) Ptr) + 5 ) = ( TVert[ v1 ] >> 16 ) | ( TVert[ v1 ] << 16 );
					  		*( ( (long*) Ptr) + 8 ) = ( TVert[ v3 ] >> 16 ) | ( TVert[ v3 ] << 16 );
					  		*( ( (long*) Ptr) + 11 ) = ( TVert[ v2 ] >> 16 ) | ( TVert[ v2 ] << 16 );
//					  		*( ( (long*) Ptr) + 2 ) = TVert[ v0 ];
//					  		*( ( (long*) Ptr) + 5 ) = TVert[ v1 ];
//					  		*( ( (long*) Ptr) + 8 ) = TVert[ v3 ];
//					  		*( ( (long*) Ptr) + 11 ) = TVert[ v2 ];

							// Write UV spaces as words (for speed)

							if( ModPolys[ i ].Type == TEXTUREDB )
							{
						  		*( ( (long*) Ptr) + 3 ) = ModPolys[ i ].U0 | ( ModPolys[ i ].V0 << 8 ) | ( BoardClut << 16 );
						  		*( ( (long*) Ptr) + 6 ) = ModPolys[ i ].U1 | ( ModPolys[ i ].V1 << 8 ) | ( BoardPage << 16);
							}
							else
							{
						  		*( ( (long*) Ptr) + 3 ) = ModPolys[ i ].U0 | ( ModPolys[ i ].V0 << 8 ) | ( Clut << 16 );
						  		*( ( (long*) Ptr) + 6 ) = ModPolys[ i ].U1 | ( ModPolys[ i ].V1 << 8 ) | ( Plyr->CharPage << 16);
							}

					  		*( ( (long*) Ptr) + 9 ) = ModPolys[ i ].U3 | ( ModPolys[ i ].V3 << 8 );
					  		*( ( (long*) Ptr) + 12 ) = ModPolys[ i ].U2 | ( ModPolys[ i ].V2 << 8 );




					  		setPolyGT4( G4 );
//							SetShadeTex( G4, 1 );

					  		if( Otz < OTLENGTH && Otz > 0 )
				  			{
//								printf("%ld\n", Otz);
//								Otz+=30;
//								if( Otz > maxcharot ) maxcharot = Otz;
								SKBADDPRIM( &OrderingTable[ otid ][ Otz ], G4 );
					  		}

					  		Ptr += sizeof( POLY_GT4 );
					  	}
					  	break;
					}
				}
			}	// end of quad render
		}	// end of all polygon rendering
	}
}






