#/* camlogic.c */

/* Machine independant */

#include <stdio.h>
#include "sys/types.h"
#include "LibGTE.h"
#include "LibGPU.h"
#include "LibCD.h"
#include "LibComb.h"
#include "LibEtc.h"
#include "LibGS.h"
#include "LibMath.h"
#include "LibPress.h"
#include "LibSN.h"
#include "LibSnd.h"
#include "LibSPU.h"
#include "ourtypes.h"
#include "main.h"
#include "mathbits.h"
#include "players.h"
#include "camera.h"
#include "pads.h"
#include "camlogic.h"


/* Camera test code for milestone 1 ( 2 routines ) */

#define TEST_TRANS CHFL( 20.0 )

void Camera_Test_Target( uint8 PlayerNumber )
{
	player *Plyr = &Player[ PlayerNumber ];
	uint8 Pad = Plyr->PadNumber;

	/* Mandatory for all camera logic mode routines to set camera to correct type */
	Plyr->Cam.Type = CAMERA_TYPE_TARGET;

	if( Pad_State( Pad ) & PAD_B3 )
	{
		if( Pad_State( Pad ) & PAD_UP )
		{
			Plyr->Cam.Pos.Z += TEST_TRANS;
		}

		if( Pad_State( Pad ) & PAD_DOWN )
		{
			Plyr->Cam.Pos.Z -= TEST_TRANS;
		}

		if( Pad_State( Pad ) & PAD_LEFT )
		{
			Plyr->Cam.Pos.X -= TEST_TRANS;
		}

		if( Pad_State( Pad ) & PAD_RIGHT )
		{
			Plyr->Cam.Pos.X += TEST_TRANS;
		}

		if( Pad_State( Pad ) & PAD_TOPLEFT )
		{
			Plyr->Cam.Pos.Y -= TEST_TRANS;
		}

		if( Pad_State( Pad ) & PAD_TOPRIGHT )
		{
			Plyr->Cam.Pos.Y += TEST_TRANS;
		}
	}
	else
	{
		if( Pad_State( Pad ) & PAD_UP )
		{
			Plyr->Cam.Targ.Z += TEST_TRANS;
		}

		if( Pad_State( Pad ) & PAD_DOWN )
		{
			Plyr->Cam.Targ.Z -= TEST_TRANS;
		}

		if( Pad_State( Pad ) & PAD_LEFT )
		{
			Plyr->Cam.Targ.X -= TEST_TRANS;
		}

		if( Pad_State( Pad ) & PAD_RIGHT )
		{
			Plyr->Cam.Targ.X += TEST_TRANS;
		}

		if( Pad_State( Pad ) & PAD_TOPLEFT )
		{
			Plyr->Cam.Targ.Y -= TEST_TRANS;
		}

		if( Pad_State( Pad ) & PAD_TOPRIGHT )
		{
			Plyr->Cam.Targ.Y += TEST_TRANS;
		}
	}


	if( Pad_Debounced( Pad ) == PAD_B1 )
	{
		Plyr->Cam.Mode = CAMERA_TEST_POSITIONAL;
	}
	if( Pad_Debounced( Pad ) == PAD_B4 )
	{
		Plyr->Cam.Mode = CAMERA_FOLLOW;
		speed=0;
	}

}

void Camera_Test_Positional( uint8 PlayerNumber )
{
	player *Plyr = &Player[ PlayerNumber ];
	uint8 Pad = Plyr->PadNumber;

	/* Mandatory for all camera logic mode routines to set camera to correct type */
	Plyr->Cam.Type = CAMERA_TYPE_POSITIONAL;


	/* Camera Translate */

	if(
	( Pad_State( Pad ) & ( PAD_B1 + PAD_B3 ) ) == 0
	)
	{

		if( Pad_State( Pad ) & PAD_LEFT )
		{
			Plyr->Cam.Pos.X -= TEST_TRANS;
		}

		if( Pad_State( Pad ) & PAD_RIGHT )
		{
			Plyr->Cam.Pos.X += TEST_TRANS;
		}

		if( Pad_State( Pad ) & PAD_UP )
		{
			Plyr->Cam.Pos.Z += TEST_TRANS;
		}

		if( Pad_State( Pad ) & PAD_DOWN )
		{
			Plyr->Cam.Pos.Z -= TEST_TRANS;
		}

		if( Pad_State( Pad ) & ( PAD_TOPRIGHT ) )
		{
			Plyr->Cam.Pos.Y += TEST_TRANS;
		}

		if( Pad_State( Pad ) & ( PAD_TOPLEFT ) )
		{
			Plyr->Cam.Pos.Y -= TEST_TRANS;
		}
	}



	/* Camera Rotate */
	/* Azimuth */
	if( Pad_State( Pad ) == ( PAD_B3 + PAD_RIGHT ) )
	{
		Plyr->Cam.Rot.Y += 10<<12;
		//( CHFL( 4096.0 ) / CHFL( 360.0 ) );
	}

	if( Pad_State( Pad ) == ( PAD_B3 + PAD_LEFT ) )
	{
		Plyr->Cam.Rot.Y -= 10<<12;
		//( CHFL( 4096.0 ) / CHFL( 360.0 ) );
	}

	/* Elevation */
	if( Pad_State( Pad ) == ( PAD_B3 + PAD_UP ) )
	{
		Plyr->Cam.Rot.X += 10<<12;
		//( CHFL( 4096.0 ) / CHFL( 360.0 ) );
	}

	if( Pad_State( Pad ) == ( PAD_B3 + PAD_DOWN ) )
	{
		Plyr->Cam.Rot.X -= 10<<12;
		//( CHFL( 4096.0 ) / CHFL( 360.0 ) );
	}

	/* Twist */
	if( Pad_State( Pad ) == ( PAD_B3 + PAD_TOPLEFT ) )
	{
		Plyr->Cam.Rot.Z += 10<<12;
		//( CHFL( 4096.0 ) / CHFL( 360.0 ) );
	}

	if( Pad_State( Pad ) == ( PAD_B3 + PAD_TOPRIGHT ) )
	{
		Plyr->Cam.Rot.Z -= 10<<12;
		//( CHFL( 4096.0 ) / CHFL( 360.0 ) );
	}

	if ( Pad_State( Pad ) == ( PAD_B3 + PAD_BOTTOMLEFT ) )
	{
		MyLocalLight.m[0][1]-=0x10;
		SetLightMatrix(&MyLocalLight);
	}

	if ( Pad_State( Pad ) == ( PAD_B3 + PAD_BOTTOMRIGHT ) )
	{
		MyLocalLight.m[0][1]+=0x10;
		SetLightMatrix(&MyLocalLight);
	}

	if( Pad_Debounced( Pad ) == PAD_B1 )
	{
		Plyr->Cam.Mode = CAMERA_TEST_TARGET;
	}

	if( Pad_Debounced( Pad ) == PAD_B4 )
	{
		Plyr->Cam.Mode = CAMERA_FOLLOW;
		speed=0;
	}
}


void Camera_Follow(uint8 PlayerNumber)
{
	player *Plyr = &Player[ PlayerNumber ];
	uint8 Pad = Plyr->PadNumber;
	long	xs,zs;

	/* Mandatory for all camera logic mode routines to set camera to correct type */
	Plyr->Cam.Type = CAMERA_FOLLOW;


	/* Camera Translate */

	if(
	( Pad_State( Pad ) & ( PAD_B1 + PAD_B3 ) ) == 0
	)
	{

		if( Pad_State( Pad ) & PAD_LEFT )
		{
			Plyr->Cam.Rot.Y -= TEST_TRANS*2;
		}

		if( Pad_State( Pad ) & PAD_RIGHT )
		{
			Plyr->Cam.Rot.Y += TEST_TRANS*2;
		}

		if( Pad_State( Pad ) & ( PAD_B2 ) && (jumped==0) )
		{
			if (speed<200)
			{
				speed += 4;
			}
		}
		else
		{
			if ((speed>0)&&(jumped==0))
			{
				speed-=2;
			}
		}
		if (jumped==1)
		{
			jumpspeed--;
		}

		if ( Pad_State( Pad ) & ( PAD_TOPRIGHT ) )
		{
			if (jumped==0)
			{
				jumpspeed=6;
				jumped=1;
			}
		}

	}
	xs=rcos((Plyr->Cam.Rot.Y>>12)+1024)*speed;
	zs=rcos(Plyr->Cam.Rot.Y>>12)*speed;

	Plyr->Cam.Pos.X+=xs;
	Plyr->Cam.Pos.Z-=zs;
	if( Pad_Debounced( Pad ) == PAD_B1 )
	{
		Plyr->Cam.Mode = CAMERA_TEST_TARGET;
	}

	if( Pad_Debounced( Pad ) == PAD_B3 )
	{
		Plyr->Cam.Mode = CAMERA_TEST_POSITIONAL;
	}

	if( Pad_Debounced( Pad ) == PAD_B4 )
	{
		InitialisePlayers();
		InitialiseTrackEngine();
	}
	if ( Pad_Debounced ( Pad ) == PAD_BOTTOMLEFT )
	{
		if (ConsolePlayers==1)
		{
			ConsolePlayers++;
			TopScreenDrawn=0;
			display_id=0;
			otid=0;
		}
		else
		{
			ConsolePlayers--;
			TopScreenDrawn=0;
			display_id=0;
			otid=0;
		}
	}



}
