/* mathbits.c */

#include <stdio.h>
#include "sys/types.h"
#include "LibGTE.h"
#include "LibGPU.h"
#include "LibCD.h"
#include "LibComb.h"
#include "LibEtc.h"
#include "LibGS.h"
#include "LibMath.h"
#include "LibPress.h"
#include "LibSN.h"
#include "LibSnd.h"
#include "LibSPU.h"
#include "ourtypes.h"
#include "main.h"
#include "sine.h"
#include "mathbits.h"


void SetVector3(vec3 *vector,int16 x,int16 y,int16 z)
{

	vector->X=x<<12;
	vector->Y=y<<12;
	vector->Z=z<<12;
}

/* Simply creates an identity matrix */
void Matrix_Init( chmatrix *A )
{
	A->E[ 0 ][ 0 ] = CHFL( 1.0 );
	A->E[ 0 ][ 1 ] = CHFL( 0.0 );
	A->E[ 0 ][ 2 ] = CHFL( 0.0 );

	A->E[ 1 ][ 0 ] = CHFL( 0.0 );
	A->E[ 1 ][ 1 ] = CHFL( 1.0 );
	A->E[ 1 ][ 2 ] = CHFL( 0.0 );

	A->E[ 2 ][ 0 ] = CHFL( 0.0 );
	A->E[ 2 ][ 1 ] = CHFL( 0.0 );
	A->E[ 2 ][ 2 ] = CHFL( 1.0 );

	A->E[ 3 ][ 0 ] = CHFL( 0.0 );
	A->E[ 3 ][ 1 ] = CHFL( 0.0 );
	A->E[ 3 ][ 2 ] = CHFL( 0.0 );
}


void Matrix_x_Matrix( chmatrix *A, chmatrix *B, chmatrix *D )
{
	D->E[ 0 ][ 0 ] =	( A->E[ 0 ][ 0 ] * B->E[ 0 ][ 0 ] ) +
           					( A->E[ 0 ][ 1 ] * B->E[ 1 ][ 0 ] ) +
           					( A->E[ 0 ][ 2 ] * B->E[ 2 ][ 0 ] );

  D->E[ 0 ][ 1 ] =	( A->E[ 0 ][ 0 ] * B->E[ 0 ][ 1 ] ) +
             				( A->E[ 0 ][ 1 ] * B->E[ 1 ][ 1 ] ) +
             				( A->E[ 0 ][ 2 ] * B->E[ 2 ][ 1 ] );

  D->E[ 0 ][ 2 ] = 	( A->E[ 0 ][ 0 ] * B->E[ 0 ][ 2 ] ) +
             				( A->E[ 0 ][ 1 ] * B->E[ 1 ][ 2 ] ) +
             				( A->E[ 0 ][ 2 ] * B->E[ 2 ][ 2 ] );

  D->E[ 1 ][ 0 ] = 	( A->E[ 1 ][ 0 ] * B->E[ 0 ][ 0 ] ) +
             				( A->E[ 1 ][ 1 ] * B->E[ 1 ][ 0 ] ) +
             				( A->E[ 1 ][ 2 ] * B->E[ 2 ][ 0 ] );

  D->E[ 1 ][ 1 ] = 	( A->E[ 1 ][ 0 ] * B->E[ 0 ][ 1 ] ) +
             				( A->E[ 1 ][ 1 ] * B->E[ 1 ][ 1 ] ) +
             				( A->E[ 1 ][ 2 ] * B->E[ 2 ][ 1 ] );

  D->E[ 1 ][ 2 ] = 	( A->E[ 1 ][ 0 ] * B->E[ 0 ][ 2 ] ) +
             				( A->E[ 1 ][ 1 ] * B->E[ 1 ][ 2 ] ) +
             				( A->E[ 1 ][ 2 ] * B->E[ 2 ][ 2 ] );

  D->E[ 2 ][ 0 ] = 	( A->E[ 2 ][ 0 ] * B->E[ 0 ][ 0 ] ) +
             				( A->E[ 2 ][ 1 ] * B->E[ 1 ][ 0 ] ) +
             				( A->E[ 2 ][ 2 ] * B->E[ 2 ][ 0 ] );

  D->E[ 2 ][ 1 ] = 	( A->E[ 2 ][ 0 ] * B->E[ 0 ][ 1 ] ) +
             				( A->E[ 2 ][ 1 ] * B->E[ 1 ][ 1 ] ) +
             				( A->E[ 2 ][ 2 ] * B->E[ 2 ][ 1 ] );

  D->E[ 2 ][ 2 ] = 	( A->E[ 2 ][ 0 ] * B->E[ 0 ][ 2 ] ) +
             				( A->E[ 2 ][ 1 ] * B->E[ 1 ][ 2 ] ) +
             				( A->E[ 2 ][ 2 ] * B->E[ 2 ][ 2 ] );

  D->E[ 3 ][ 0 ] = 	( A->E[ 3 ][ 0 ] * B->E[ 0 ][ 0 ] ) +
             				( A->E[ 3 ][ 1 ] * B->E[ 1 ][ 0 ] ) +
             				( A->E[ 3 ][ 2 ] * B->E[ 2 ][ 0 ] ) +
             					B->E[ 3 ][ 0 ];

  D->E[ 3 ][ 1 ] = 	( A->E[ 3 ][ 0 ] * B->E[ 0 ][ 1 ] ) +
             				( A->E[ 3 ][ 1 ] * B->E[ 1 ][ 1 ] ) +
             				( A->E[ 3 ][ 2 ] * B->E[ 2 ][ 1 ] ) +
             					B->E[ 3 ][ 1 ];

  D->E[ 3 ][ 2 ] = 	( A->E[ 3 ][ 0 ] * B->E[ 0 ][ 2 ] ) +
             				( A->E[ 3 ][ 1 ] * B->E[ 1 ][ 2 ] ) +
             				( A->E[ 3 ][ 2 ] * B->E[ 2 ][ 2 ] ) +
             		 			B->E[ 3 ][ 2 ];
}




void Vec3_x_Matrix( vec3 *A, chmatrix *B, vec3 *D )
{
	D->X = ( A->X * B->E[ 0 ][ 0 ] ) +
         ( A->Y * B->E[ 1 ][ 0 ] ) +
         ( A->Z * B->E[ 2 ][ 0 ] ) +
           B->E[ 3 ][ 0 ];

  D->Y = ( A->X * B->E[ 0 ][ 1 ] ) +
         ( A->Y * B->E[ 1 ][ 1 ] ) +
         ( A->Z * B->E[ 2 ][ 1 ] ) +
				 	 B->E[ 3 ][ 1 ];

  D->Z = ( A->X * B->E[ 0 ][ 2 ] ) +
         ( A->Y * B->E[ 1 ][ 2 ] ) +
         ( A->Z * B->E[ 2 ][ 2 ] ) +
           B->E[ 3 ][ 2 ];
}




void Matrix_MakeRotX( chmatrix *A, chfloat Deg )
{
   A->E[1][1] =  Cosine( Deg );
   A->E[2][2] =  Cosine( Deg );
   A->E[2][1] = -Sine( Deg );
   A->E[1][2] =  Sine( Deg );
}

void Matrix_MakeRotY( chmatrix *A, chfloat Deg )
{
   A->E[0][0] =  Cosine( Deg );
   A->E[0][2] = -Sine( Deg );
   A->E[2][0] =  Sine( Deg );
   A->E[2][2] =  Cosine( Deg );
}

void Matrix_MakeRotZ( chmatrix *A, chfloat Deg )
{
   A->E[ 0 ][ 0 ] =  Cosine( Deg );
   A->E[ 1 ][ 1 ] =  Cosine( Deg );
   A->E[ 0 ][ 1 ] =  Sine( Deg );
   A->E[ 1 ][ 0 ] = -Sine( Deg );
}


void Matrix_MakeXYZTransl( chmatrix *A, chfloat X, chfloat Y, chfloat Z )
{
   A->E[3][0] = X;
   A->E[3][1] = Y;
   A->E[3][2] = Z;
}


/*
void AstMatrix4x3::MakeXTranslation( AstFixedpoint_t amount )
{
   data[3][0] = amount;
   data[3][1] = 0;
   data[3][2] = 0;
}
void AstMatrix4x3::MakeYTranslation( AstFixedpoint_t amount )
{
   data[3][0] = 0;
   data[3][1] = amount;
   data[3][2] = 0;
}
void AstMatrix4x3::MakeZTranslation( AstFixedpoint_t amount )
{
   data[3][0] = 0;
   data[3][1] = 0;
   data[3][2] = amount;
}

*/

PSX_Matrix_Init(MATRIX *m)
{
	uint8	Count1,Count2;

	for (Count1=0;Count1<3;Count1++)
	{
		for (Count2=0;Count2<3;Count2++)
		{
			if (Count1==Count2)
			{
				m->m[Count1][Count2]=0x1000;
			}
			else
			{
				m->m[Count1][Count2]=0;
			}
		}
		m->t[Count1]=0;
	}

}

int32 PSX_Dot ( VECTOR *A, VECTOR *B)
{
	return	(( A->vx * B->vx ) +
					( A->vy * B->vy ) +
					( A->vz * B->vz ));
}


chfloat Dot( vec3 *A, vec3 *B )
{
	return	( A->X * B->X ) +
					( A->Y * B->Y ) +
					( A->Z * B->Z );
}

void Cross( vec3 *A, vec3 *B, vec3 *D )
{
	D->X = ( A->Y * B->Z ) - ( A->Z * B->Y );
	D->Y = ( A->Z * B->X ) - ( A->X * B->Z );
	D->Z = ( A->X * B->Y ) - ( A->Y * B->X );
}


chfloat Magnitude( vec3 *A )
{
	chfloat f;
/*	VECTOR	outputvector,squaredvector;

	squaredvector.vx=A->X>>12;
	squaredvector.vy=A->Y>>12;
	squaredvector.vz=A->Z>>12;
	Square0(&squaredvector,&outputvector);
	printf("ivx %d, ivy %d, ivz %d\n",squaredvector.vx,squaredvector.vy,squaredvector.vz);
	printf("ovx %d, ovy %d, ovz %d\n",outputvector.vx,outputvector.vy,outputvector.vz);
	f = outputvector.vx+outputvector.vy+outputvector.vz;

	return SquareRoot0( f<<12 )<<12;   */
	long	d[3],i,j,t;

	d[0]	=	abs(A->X);
	d[1]	=	abs(A->Y);
	d[2]	=	abs(A->Z);

	/* shitsort */
	for ( i=0;i<3;i++ )
	{
		for ( j=1;j<3;j++ )
		{
			if ( d[j]>d[j-1] )
			{
				t		=	d[j];
				d[j]	=	d[j-1];
				d[j-1]	=	t;
			}
		}
	}

	t	=	d[0] + (( d[1] * 11 ) >> 5 ) + ( d[2] >> 2 );

	return(t);


}


void Normalize( vec3 *D )
{
	chfloat M = Magnitude( D );

	D->X = D->X / M;
	D->Y = D->Y / M;
	D->Z = D->Z / M;
}


