/* tracks.h */

#ifndef _TRACKS_H_
#define _TRACKS_H_

//#define OTLENGTH			1800
#define OTLENGTH			3600
#define VMAG 128.0
#define MAX_SEGMENT_SIZE	200         /* now max no of strips */
#define MAX_VIEW_STRIPS		16			/* now max no of viewable strips */
#define	MAX_TRACKS			50
#define MAX_OBJECTS			60

/* object collison stuff */

#define	NONE				0
#define	SIDES				1
#define	TOP					2
#define ROCK				3
#define TRACKEDGE			4
#define	TREESCRAPE			5
#define	TREEBUMP			6
#define	TREESMACK			7
#define	STUMP				8
#define	RAILSLIDE			9
#define	RIVER				10
#define PICKUP				11
#define	MAIN_BOX_COLLIDE	100
#define	OVER_OBJECT			101

#define SKBADDPRIMLOCALS	uint32 OtEntry,PolyTag; \
							uint32 BotBytes=0x00ffffff;\
							uint32 EntryInOT;

#define BYTEMERGE(x,y)  ( ((x) & 0xff000000) | ((y) & BotBytes) )

#define SKBADDPRIM(x,y)     \
{                           \
    OtEntry=(x); \
    PolyTag=(uint32 )(y); \
    EntryInOT=*(uint32 *)OtEntry;           \
    *(uint32 *)PolyTag = BYTEMERGE(*(uint32 *)PolyTag,EntryInOT);   \
    *(uint32 *)OtEntry = BYTEMERGE(EntryInOT,PolyTag);              \
};



/* CD track data streaming buffer */

typedef struct incomingtrackbuffer
{
	uint16 			CurrentStrip;
	uint16			CurrentObject;
	uint16			NumObjects;
	uint16			NumTextures;
	uint16			CurrentTexture;
	uint16			padding[3];
	uint32			CurrentStripPtr;
	uint32			CurrentTexturePtr;
	uint32			CurrentObjectPtr;
	int32			OffsetX;
	int32			OffsetZ;
	uint32  		NumStrips;
	uint8		    TrackData[ 82000 ];     /* 132000 */

}incomingtrackbuffer;

typedef struct trackobject
{
	uint32	StripTag;
	uint32	Type;
	int32	TransX, TransY, TransZ;
	uint32	ScaleX, ScaleY, ScaleZ;
	uint32	RotX, RotY, RotZ;
}trackobject;

typedef struct texttrackpoly
{
	uint32		StripTag;
	uint32		TID;
	SVECTOR	    XYZ[4];
	SVECTOR		Norms[4];
	SVECTOR		UV[4];
	CVECTOR		RGB[4];
}texttrackpoly;

typedef struct trianglemesh
{
	TMESH			TMesh;
	uint32			SplitTag;
	texttrackpoly	*Textaddr;
	uint32			notext;
	trackobject		*Objaddr;
	uint32			noobj;
	uint32			StripNo;
	int32			OffsetX;
	int32			OffsetZ;
}trianglemesh;

/*
typedef struct primbuffer
{
	POLY_G3		GPoly[ MAX_VIEW_STRIPS ][ 80 ];
}primbuffer;    */

/* Track polygon viewlist for rendering engine */
typedef struct trackdispbuffer
{
	trianglemesh  	Meshes[ MAX_VIEW_STRIPS ];
//	primbuffer 		Prims[4];
	uint16 			CurrentStrip;
	uint16 			NumberOfStrips;

}trackdispbuffer;

typedef struct objpolygon
{
	long	numverts;
	long	polytype;
	long	tmatid;
	long 	vertptr;
	SVECTOR normals[4];
	long	textus;
	long	textvs;
	CVECTOR basecolour;
}objpolygon;

typedef struct objcoll
{
	uint32	type;
	VECTOR	info[4];
}objcoll;

typedef struct iscoll
{
	uint8	type;
	vec3	point;
	chfloat angle;
	int32	height;
}iscoll;

typedef struct objheader
{
	uint8		type;	/* for special types like cable cars etc. */
	uint8		loaded;
	SVECTOR 	*vertptr;
	objpolygon  *polyptr;
	objcoll		*collptr;
	long		numpolys;
	long		numverts;
	long		numcolls;
} objheader;

typedef struct objotsort
{
	uint32		*primptr;
	uint16		otvalue;
}objotsort;

typedef	struct moveobj
{
	unsigned char	trackno;
	unsigned char	startstrip;
	unsigned char 	endstrip;
	unsigned char	objectno;
	unsigned short	roty;
	unsigned short	flag;
	long			height;
	long			startx;
	long			starty;
	long			startz;
	long			movex;
	long			movey;
	long			movez;
	long			posx;
	long			posy;
	long			posz;
	long			finx;
	long			finy;
	long			finz;
}moveobj;

typedef struct tcontrol
{
	uint32	entry;
	uint32	exit;
	moveobj	*startptr;
}tcontrol;
/* externed variables */

typedef struct stinfo
{
	uint32	*address;
	uint32	*textaddr;
	uint32	*objaddr;
	uint16	StripNo;
	int32	OffsetX;
	int32   OffsetZ;
	uint8	notext;
	uint8	noobj;
	uint8	unusedadjust;
}stinfo;

typedef struct cableinfo
{
	VECTOR	start;
	VECTOR	end;
	VECTOR	movevector;
	VECTOR	carpos;
	long	strip;
}cableinfo;

extern		uint8 			display_id;
extern		uint8 		 	otid;
extern		uint8 			rotid;
extern		uint8 			TopScreenDrawn;
extern		uint8			setdisplay;
extern		uint32 			OrderingTable[4][OTLENGTH];
extern		uint32 			*OTPtr;
extern		trackdispbuffer Tb[2];
extern		int32			z1;
extern		int32			d2;
extern		VECTOR			Scale;
extern		uint32	currentfile[2];
extern		uint8 howsnowy;
extern		uint16 howfoggy;
extern		uint8 snowspeed;
extern		long		maxobjot;
extern		uint8		firstpoly;
extern		uint16  Cnt;
extern		uint8	novsyncs;
extern		uint8	noframes;
extern		uint8	fps;
extern		long	sxy[200];
extern		long	p[200];
extern		long	ootz[200];
extern		long	maxcharot;
extern		uint32 globaltime;
extern		uint8	clockon;
extern		uint8	musiccnt;
extern		POLY_FT4	polyf4s[4][400];
extern		POLY_FT3	polyf3s[4][100];
extern		POLY_F4		poly4s[4][50];
extern		POLY_F3		poly3s[4][50];
extern		POLY_G3		trackprims[4][1200];
extern		int32 		randarray[40];
extern		long		nopolyf4s;
extern		long		nopoly4s;
extern		long		nopolyf3s;
extern		long		nopoly3s;
extern		uint32		nopolyG3s;
extern		iscoll		thiscollision[4];
extern		uint32		tempchill;
extern		uint32		*vertstoreaddr;
extern		uint32		deltime;
extern		uint8		playwhoah;
extern		uint32		lasttime;
extern		uint32		lastscore;
extern		uint32 		hours;
extern		uint32 		minutes;
extern		uint32 		seconds;
extern		uint32 		decsecs;
extern		uint8		framelock;
extern		RECT 		timecopyrect;
extern		MATRIX		mountmatrix;
extern		VECTOR		B1;
extern		VECTOR		B2;
extern		VECTOR		B3;

#ifndef CD
extern 	uint32	numloadedtracks;
extern	char 	trackname[MAX_TRACKS][100];
extern 	uint32	numloadedobjects;
#else
extern 	uint32	numloadedtracks;
extern	CdlFILE trackname[MAX_TRACKS];
extern 	uint32	numloadedobjects;
#endif
extern	tcontrol trackcontrol[MAX_TRACKS];

extern	long polys_taken;
extern	uint8	bufftest0;
extern	uint8	bufftest;
extern	uint8	bufftest2;
extern	uint32	storepcount;
extern  uint8	nodivs;

void initobjects(void	);
uint8	Collision(uint8 PlayerNumber);
vec3	CollidePoint(uint8 PlayerNumber);
chfloat CollideAngle(uint8 PlayerNumber);
chfloat RailAngle(uint8	PlayerNumber);
void AddPrimJez(long otz,uint32 *address);
void InitialiseTrackEngine( uint8 PlayerNo );
void KillTrackEngine( void );
void TempRenderTrackPolys( uint8 PlayerNumber );
uint8 GetANewTrackPoly( uint8 PlayerNo );
chfloat	GetYAt(chfloat x, chfloat z, uint8 pno);


#endif


