#include <stdio.h>
#include <sys/types.h>
#include <LibGTE.h>
#include <LibGPU.h>
#include <LibCD.h>
#include <LibComb.h>
#include <LibEtc.h>
#include <LibGS.h>
#include <LibMath.h>
#include <LibPress.h>
#include <LibSN.h>
#include <LibSnd.h>
#include <LibSPU.h>
#include <inline_o.h>
#include <gtemac.h>


#include "ourtypes.h"
#include "mathbits.h"

#include "main.h"
#include "skeleton.h"
#include "camera.h"
#include "tracks.h"
#include "players.h"
#include "pads.h"

#include "screen.h"
#include "who.h"
#include "bored.h"
#include "animctrl.h"

#include "bonebag.h"
#include "trixbag.h"
#include "rotmesh.h"
#include "avalanch.h"
#include "plume.h"
#include "logic.h"
#include "camlogic.h"
#include "sound.h"



/************************************************************************/
//	Avalanche.c
//				does not work on splits
/************************************************************************/


VIEW_STRIP	striptab[2][MAX_VIEW_STRIPS];

/* file no & strip nos of each avalanche strip */
VIEW_STRIP	Gstrips_affected[AVALANCHE_NUMBER_OF_STRIPS] = {
							{ 9,69},
							{ 8,69},
							{ 7,69},
							{ 6,69},
							{ 5,69},
							{ 4,69},
							{ 3,69},
							{ 2,69},
							{ 1,69},
							{ 0,69}};

VIEW_STRIP	filesizes[4] = {{37,69},
							{79,70},
							{96,71},
							{48,72}};

long	avalanche_player_strip_count = 0;
long	avalanche_strip_count = 0;

/* flags indicating wether each avalanche strip is in the player track buffer */
long	strips_visible[AVALANCHE_NUMBER_OF_STRIPS] = {1,1,1,1,1,1,1,1,1,1};

long	strips_affected[AVALANCHE_NUMBER_OF_STRIPS] = {0,1,2,3,4,5,6,7,8,9};
long	strips_height[AVALANCHE_NUMBER_OF_STRIPS] = {0,512,1024,1536,2048,2560,3072,3584,4096,0};
long	strip_real = 0;


long	avalanche_active = 0;

long	previous_segment_length[2] = {0,0};


CVECTOR	stripcol = {0xB0,0xB0,0xB0,0x00};
CVECTOR	stripcol0 = {0xFF,0xFF,0xFF,0x00};
CVECTOR	stripcol1 = {0xFF,0xFF,0xFF,0x00};

//DR_TPAGE	avalanche_transparencies[512][2];	/* primitives to turn transparency on/off*/
unsigned short trans_tpage,opaque_pad;
short	wave_table[3] = {10,40,10};
long	strip_count = 0;
short	avalanche_height_lookup[16] = {	20,
										50,
										65,
										80,
										80,
										80,
										80,
										80,
										80,
										80,
										80,
										60,
										40,
										0,
										0,
										0};


long	avalanche_speed = 850;
long	avalanche_collision = 0;
long	avalanche_zoom_count = 0;
long	avalanche_zoom_val = 0;


long	height_strip_step[AVALANCHE_NUMBER_OF_STRIPS][64];
long	height_strip_offset[AVALANCHE_NUMBER_OF_STRIPS][64];
long	height_strip_ctrs[AVALANCHE_NUMBER_OF_STRIPS][64];

void	InitAvalanche()
{
	long	i,j;

	/* init semitrans primitives */
	trans_tpage = GetTPage(1,3,0,0);

	for(i=0;i<AVALANCHE_NUMBER_OF_STRIPS;i++)
	{
		for(j=0;j<64;j++)
		{
			height_strip_step[i][j] = 0;
			height_strip_offset[i][j] = 0;
			height_strip_ctrs[i][j] = 0;
		}

	}
	
	Gstrips_affected[0].fileno = 69;
	Gstrips_affected[0].stripno = 9;
	Gstrips_affected[1].fileno = 69;
	Gstrips_affected[1].stripno = 8;
	Gstrips_affected[2].fileno = 69;
	Gstrips_affected[2].stripno = 7;
	Gstrips_affected[3].fileno = 69;
	Gstrips_affected[3].stripno = 6;
	Gstrips_affected[4].fileno = 69;
	Gstrips_affected[4].stripno = 5;
	Gstrips_affected[5].fileno = 69;
	Gstrips_affected[5].stripno = 4;
	Gstrips_affected[6].fileno = 69;
	Gstrips_affected[6].stripno = 3;
	Gstrips_affected[7].fileno = 69;
	Gstrips_affected[7].stripno = 2;
	Gstrips_affected[8].fileno = 69;
	Gstrips_affected[8].stripno = 1;
	Gstrips_affected[9].fileno = 69;
	Gstrips_affected[9].stripno = 0;

	for(i=0;i<MAX_VIEW_STRIPS;i++)
	{
		striptab[0][i].fileno = 0;
		striptab[0][i].stripno = 0;
		striptab[1][i].fileno = 0;
		striptab[1][i].stripno = 0;
	}

	avalanche_player_strip_count = 0;
	avalanche_strip_count = 0;

	for(i=0;i<AVALANCHE_NUMBER_OF_STRIPS;i++)
	{
		strips_visible[i] = 1;
		strips_affected[i] = i;
	}

	strip_real = 0;
	avalanche_active = 0;
	strip_count = 0;
	avalanche_collision = 0;

	avalanche_zoom_count = 0;
	avalanche_zoom_val = 0;

}

void	StartAvalanche()
{
	/* trigger timer */

}



void	RussRotMeshG3Avalanche(polys,ot,fraction,poly_offset,transparency)
POLY_G3 		*polys;
unsigned long 	*ot;
long			fraction;
long			poly_offset;
long			transparency;
{
	long	i,j,k,l,z,p,bollox,temp;
	SVECTOR	*vert_ptr;
	RUSS_SVECTOR	*sxyz,*end_sxyz,*sxyz2;
	SVECTOR		newverts[4];
	long		start_cell,end_cell,end_height;
	TMESH		*mesh;
	VECTOR		strip_offsets[AVALANCHE_NUMBER_OF_STRIPS];
	long		output_buff0,output_buff1,min_z;
	CVECTOR		col0,col1;
	short		trans_tpage;

	trans_tpage = GetTPage(1,3,0,0);

	min_z = OTLENGTH;




	col0.r = AVALANCHE_R;
	col0.g = AVALANCHE_G;
	col0.b = AVALANCHE_B;

	gte_SetRotMatrix(&(Player[0].Cam.ObjectView));

	for(i=0;i<AVALANCHE_NUMBER_OF_STRIPS;i++)
	{
		strip_offsets[i].vx=(-Player[0].Cam.Pos.X>>12)-Player[0].Track->Meshes[strips_affected[i]].OffsetX;
		strip_offsets[i].vy=(-Player[0].Cam.Pos.Y>>12);
		strip_offsets[i].vz=(-Player[0].Cam.Pos.Z>>12)+Player[0].Track->Meshes[strips_affected[i]].OffsetZ;

		ApplyRotMatrixLV(&(strip_offsets[i]),&(strip_offsets[i]));
	}

	/*******************************************/

	gte_SetRotMatrix(&(Player[0].Cam.ViewMatrix));

	/*start_height = 0;*/
	end_height = 1024;


	/* do first vertex list */
	mesh=&Player[0].Track->Meshes[ strips_affected[0] ].TMesh;

	if(mesh->len == 64)
	{
		start_cell = 10;
		end_cell = 21;
	}
	else
	{
		start_cell = 3;
		end_cell = 14;
	}

	sxyz = (RUSS_SVECTOR *)(0x1f800000);
	end_sxyz = (sxyz + (end_cell - start_cell) + 2);

	vert_ptr = mesh->v + (start_cell * 2);

	i = 0;

	if(strips_visible[0])
	{ 
		Player[0].Cam.TransMatrix.t[1]=strip_offsets[0].vx;
		Player[0].Cam.TransMatrix.t[0]=strip_offsets[0].vy;
		Player[0].Cam.TransMatrix.t[2]=strip_offsets[0].vz;
	
		gte_SetTransMatrix(&(Player[0].Cam.TransMatrix));
	
		
		while(sxyz != end_sxyz)
		{
			bollox = (vert_ptr+1)->vx - (vert_ptr+0)->vx;
			bollox = bollox * fraction;
			bollox = bollox>>12;
			newverts[0].vx = (vert_ptr+0)->vx + bollox;
	
			bollox = (vert_ptr+1)->vy - (vert_ptr+0)->vy;
			bollox = bollox * fraction;
			bollox = bollox>>12;
			newverts[0].vy = (vert_ptr+0)->vy + bollox;
			newverts[0].vy += ((avalanche_height_lookup[i] * strips_height[0])>>12);
	
			bollox = (vert_ptr+1)->vz - (vert_ptr+0)->vz;
			bollox = bollox * fraction;
			bollox = bollox>>12;
			newverts[0].vz = (vert_ptr+0)->vz + bollox;
	
			gte_ldv0(newverts+0);
		   	gte_rtps();
			gte_stsxy(sxyz+0);
		    gte_stdp(&p);
			sxyz->p = (short)p;
			gte_stszotz(&z);
			sxyz->vz = (short)z;
	
			sxyz+=1;
			vert_ptr+=2;
			i++;
		}
	}	

	output_buff0 = 0;
	output_buff1 = 1;
	/*start_height = 1024;*/

	k = poly_offset;
	for(j=1;j<AVALANCHE_NUMBER_OF_STRIPS;j++)
	{
		mesh=&Player[0].Track->Meshes[ strips_affected[j] ].TMesh;

		/* do polys */

		if(mesh->len == 64)
		{
			start_cell = 10;
			end_cell = 21;
		}
		else
		{
			start_cell = 3;
			end_cell = 14;
		}

		
		/* do sxys */
		
		if(strips_visible[j])
		{
			sxyz = (RUSS_SVECTOR *)(0x1f800000);
			sxyz += (64 * output_buff1);
			end_sxyz = (sxyz + (end_cell - start_cell) + 2);
			vert_ptr = mesh->v + (start_cell * 2);
	
			i = 0;
	
	
			Player[0].Cam.TransMatrix.t[1]=strip_offsets[j].vx;
			Player[0].Cam.TransMatrix.t[0]=strip_offsets[j].vy;
			Player[0].Cam.TransMatrix.t[2]=strip_offsets[j].vz;
	
			gte_SetTransMatrix(&(Player[0].Cam.TransMatrix));
	
			while(sxyz != end_sxyz)
			{
				bollox = (vert_ptr+1)->vx - (vert_ptr+0)->vx;
				bollox = bollox * fraction;
				bollox = bollox>>12;
				newverts[0].vx = (vert_ptr+0)->vx + bollox;
	
				bollox = (vert_ptr+1)->vy - (vert_ptr+0)->vy;
				bollox = bollox * fraction;
				bollox = bollox>>12;
				newverts[0].vy = (vert_ptr+0)->vy + bollox;
				newverts[0].vy += ((avalanche_height_lookup[i] * strips_height[j])>>12);
		
				height_strip_ctrs[j][i]--;
				if(height_strip_ctrs[j][i] <= 0)
				{
					if(j==(AVALANCHE_NUMBER_OF_STRIPS-1))
					{
						height_strip_step[j][i] = 0;
						height_strip_ctrs[j][i] = 255;
						height_strip_offset[j][i] = 0;
					}
					else
					{
						height_strip_ctrs[j][i] = 8;
									 
						rand_idx += 7;
						rand_idx = rand_idx & 0x0000001f;
						temp = (randarray[rand_idx] & 0x1f);

						temp = temp - height_strip_offset[j][i];
						temp = temp>>3;
						height_strip_step[j][i] = temp;
					}
				} 
				
				newverts[0].vy += height_strip_offset[j][i];
	
	
				bollox = (vert_ptr+1)->vz - (vert_ptr+0)->vz;
				bollox = bollox * fraction;
				bollox = bollox>>12;
				newverts[0].vz = (vert_ptr+0)->vz + bollox;

				gte_ldv0(newverts+0);
			   	gte_rtps();
				gte_stsxy(sxyz+0);
			    gte_stdp(&p);
				sxyz->p = (short)p;
				gte_stszotz(&z);
				sxyz->vz = (short)z;
	
				sxyz+=1;
				vert_ptr+=2;
				i++;
			}

		}

			/* do polys */

		if(strips_visible[j-1] & strips_visible[j])
		{
	
	
			sxyz = (RUSS_SVECTOR *)(0x1f800000);
			sxyz += (64 * output_buff0);
			sxyz2 = (RUSS_SVECTOR *)(0x1f800000);
			sxyz2 += (64 * output_buff1);
			end_sxyz = (sxyz + (end_cell - start_cell) +1);
	
			i = 0;
	
			col0.r = AVALANCHE_R;
			col0.g = AVALANCHE_G;
			col0.b = AVALANCHE_B;

			col1.r = AVALANCHE_R;
			col1.g = AVALANCHE_G;
			col1.b = AVALANCHE_B;

			
			while(sxyz != end_sxyz)
			{
				z = (long)(sxyz+1)->vz;
				if(z < OTLENGTH-2)
				{
					if(z > MESH_NEAR_Z)
					{
						z -= 250;
						if(transparency)
							z -= 200;
	
						if(z < 0)
							z = 0;
	
						if(z < min_z)
							min_z = z;

						LONG_MOVE(&(sxyz->vx),&polys->x0);
						LONG_MOVE(&((sxyz+1)->vx),&polys->x1);
						LONG_MOVE(&((sxyz2+0)->vx),&polys->x2);
	
						LONG_MOVE(&col0,&polys->r0);
						LONG_MOVE(&col0,&polys->r1);
						LONG_MOVE(&col1,&polys->r2);
	
						setPolyG3(polys);
						SetSemiTrans(polys,transparency);
	
					   	addPrim((ot+z),polys);
	
						LONG_MOVE(&((sxyz2+0)->vx),&(polys+1)->x0);
						LONG_MOVE(&((sxyz+1)->vx),&(polys+1)->x1);
						LONG_MOVE(&((sxyz2+1)->vx),&(polys+1)->x2);
	
						LONG_MOVE(&col1,&(polys+1)->r0);
						LONG_MOVE(&col0,&(polys+1)->r1);
						LONG_MOVE(&col1,&(polys+1)->r2);
	
						setPolyG3(polys+1);
						SetSemiTrans(polys+1,transparency);
						addPrim((ot+z),polys+1);
					}
	
				}
			
			
				polys+=2;
				sxyz++;
				sxyz2++;
				k+=2;
				i+=2;
			}
		}

		/* swap d-cache output buffers for sxys */

		output_buff0 = ((output_buff0) == 1)? 0:1;
		output_buff1 = ((output_buff1) == 1)? 0:1;
	}
}

/*************************************************************************/

void	DrawAvalanche(polys,current_strip,PlayerNumber)
POLY_G3 *polys;
long 	current_strip;
long	PlayerNumber;
{
	player 		*Plyr;
	long		i,j;
	unsigned long	*ot;
	TMESH		*mesh;

	long		strip_fraction;
	long		trigger_tag;
	short	   	screen_x,screen_y;
	VIEW_STRIP	avalanche_player_pos,avalanche_pos;
	long		player_strip_count,avalanche_strip_count;
	long		player_dist;
	long		arse,players_strip;

	players_strip = player_strip[PlayerNumber];

	Plyr = &(Player[PlayerNumber]);
	trigger_tag = Plyr->Track->Meshes[current_strip].SplitTag>>16;
 	if((trigger_tag == 4) && (!avalanche_active))
 	{
 		avalanche_active = 1;
 		GoToAvalancheView(PlayerNumber);
		soundtrigger = 2;
 	}

	if(avalanche_active)
	{
		switch(avalanche_active)
		{
			case (0):	/* don't really need this */
				break;
			case (1):
				/* calculate distance of avalanche from player */
				avalanche_player_pos.fileno = striptab[PlayerNumber][players_strip].fileno;
				avalanche_player_pos.stripno = striptab[PlayerNumber][players_strip].stripno;
				player_strip_count = 0;
				for(i=0;i<(avalanche_player_pos.fileno - 69);i++)
					player_strip_count += filesizes[i].stripno;
				player_strip_count += avalanche_player_pos.stripno;

				avalanche_pos.fileno = Gstrips_affected[5].fileno;
				avalanche_pos.stripno = Gstrips_affected[5].stripno;
				avalanche_strip_count = 0;
				for(i=0;i<(avalanche_pos.fileno - 69);i++)
					avalanche_strip_count += filesizes[i].stripno;
				avalanche_strip_count += avalanche_pos.stripno;

				player_dist = player_strip_count - avalanche_strip_count;

				player_dist = abs(player_dist);


				/* do screen shake */

				if(!paused)
				{
					if(player_dist < 16)
					{
						rand_idx += 7;
						rand_idx = rand_idx & 0x0000001f;
						screen_x = (short)(2 - (randarray[rand_idx]&3));
						rand_idx += 7;
						rand_idx = rand_idx & 0x0000001f;
						screen_y = (short)(2 - (randarray[rand_idx]&3));
#ifdef MUSIC			
						playsoundeffect(0,1);	/* do rumble */
#endif
					}
					else if(player_dist < 32)
					{
						rand_idx += 7;
						rand_idx = rand_idx & 0x0000001f;
						screen_x = (short)(2 - (randarray[rand_idx]&1));
						rand_idx += 7;
						rand_idx = rand_idx & 0x0000001f;
						screen_y = (short)(2 - (randarray[rand_idx]&1));
#ifdef MUSIC	
						playsoundeffect(0,1);	/* do rumble */
#endif
					}
					else
						screen_x = screen_y = 0;
				}
				else
					screen_x = screen_y = 0;


				dispenv[0][0].screen.x = screen_x; 
				dispenv[0][0].screen.y = screen_y; 
				dispenv[0][1].screen.x = screen_x; 
				dispenv[0][1].screen.y = screen_y; 
				
				if(!paused)
				{
					strip_real += avalanche_speed;
	
					if(strip_real >= 4096)
					{
						/* move avalanche on */
						strip_real -= 4096;
						for(i=0;i<AVALANCHE_NUMBER_OF_STRIPS;i++)
						{
							if(Gstrips_affected[i].stripno >= (filesizes[Gstrips_affected[i].fileno - 69].stripno))
							{
								Gstrips_affected[i].stripno = 0;
								Gstrips_affected[i].fileno++;
							}
							else
								Gstrips_affected[i].stripno++;
						}
					}
				}
				strip_fraction = (strip_real & 0x00000FFF);

				/* set the strip visible flags */
				for(i=0;i<AVALANCHE_NUMBER_OF_STRIPS;i++)
				{
					j = 0;
					while(j < MAX_VIEW_STRIPS)
					{
						/* if the strip is in the visible buffer */
						if((Gstrips_affected[i].stripno == striptab[PlayerNumber][j].stripno) &&
							(Gstrips_affected[i].fileno == striptab[PlayerNumber][j].fileno))
						{
							/* if the strip is before the gully */
							if(Gstrips_affected[i].fileno < 72) 
							{
								strips_visible[i] = 1;
								strips_affected[i] = j;
								break;
							}
							else if(Gstrips_affected[i].fileno == 72)
							{
								if(Gstrips_affected[i].stripno < 20)
								{
									strips_visible[i] = 1;
									strips_affected[i] = j;
									break;
								}
							}
						}
						j++;
					}			
					if(j == MAX_VIEW_STRIPS)
						strips_visible[i] = 0;
				}

					/* do collision detection */
				if(avalanche_collision == 0)
				{
					j = 3;
					while(j<AVALANCHE_NUMBER_OF_STRIPS)
					{
						if(strips_visible[j])
						{
							if(strips_affected[j] == players_strip)
							{
								avalanche_collision = 1; 
								break;
							}
						}
						j++;
					}
					if(avalanche_collision)
					{
						Plyr->Style = STY_ST_ROLL;
						SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_ST_ROLL1 ), 5 );
						Plyr->AnimStage = 0;
						Plyr->SpinRot.X = Plyr->BoardElevation;
						Plyr->SpinRot.Y = Plyr->BoardAzimuth - SPONG( 90.0 );
						Plyr->SpinRot.Z = Plyr->BoardRoll;
					}
				}
				if(Plyr->Style == STY_END_ST_ROLL)
					avalanche_collision = 0;

				//arse
				ot = &(OrderingTable[ otid ][0]);
	
	
				/* 0 */
				current_strip = strips_affected[0];
				mesh=&Plyr->Track->Meshes[ current_strip ].TMesh;
	
				RussRotMeshG3Avalanche(polys,ot,4096 - strip_fraction,0,0);
				polys+= 500;
				nopolyG3s+=500;
	
				/* if the avalanche has reached the crevasse, end the demo */
				if(Gstrips_affected[AVALANCHE_NUMBER_OF_STRIPS - 1].fileno == 72)
				{
					if(Gstrips_affected[AVALANCHE_NUMBER_OF_STRIPS - 1].stripno == 20)
					{
						avalanche_active = 2;
											/* clear screen shake */
						dispenv[0][0].screen.x = 0;
						dispenv[0][0].screen.y = 0; 
						dispenv[0][1].screen.x = 0; 
						dispenv[0][1].screen.y = 0; 
					}		
				}
				if(trigger_tag == 5)
				{
					avalanche_active = 0;
					ReturnFromAvalancheView(PlayerNumber);
				
					/* clear screen shake */
					dispenv[0][0].screen.x = 0;
					dispenv[0][0].screen.y = 0; 
					dispenv[0][1].screen.x = 0; 
					dispenv[0][1].screen.y = 0; 
					soundtrigger = 0;
				}
				break;
			case (2):
				/* set zoom back to original */
				if(!paused)
				{
					if(trigger_tag == 5)
					{
						avalanche_active = 0;
						ReturnFromAvalancheView(PlayerNumber);
				
						/* clear screen shake */
						dispenv[0][0].screen.x = 0;
						dispenv[0][0].screen.y = 0; 
						dispenv[0][1].screen.x = 0; 
						dispenv[0][1].screen.y = 0; 
					}			
				}
				break;

			default:
				break;
		}
	}
}	
	
	
	
	
	
	
	
	
	
	


