#include <stdio.h>
#include <sys/types.h>
#include <LibGTE.h>
#include <LibGPU.h>
#include <LibCD.h>
#include <LibComb.h>
#include <LibEtc.h>
#include <LibGS.h>
#include <LibMath.h>
#include <LibPress.h>
#include <LibSN.h>
#include <LibSnd.h>
#include <LibSPU.h>
#include <inline_o.h>
#include <gtemac.h>


#include "ourtypes.h"
#include "mathbits.h"

#include "main.h"
#include "skeleton.h"
#include "camera.h"
#include "tracks.h"
#include "players.h"
#include "pads.h"

#include "screen.h"
#include "who.h"
#include "bored.h"
#include "animctrl.h"

#include "bonebag.h"
#include "trixbag.h"
#include "logic.h"
#include "leaf.h"
#include "plume.h"
#include "rotmesh.h"
#include "avalanch.h"
#include "boulder.h"


long	num_boulders = 0;
long	last_boulder = 0;
long	direction = 0;

/* array defining start positions of boulders */
BOULDER 	boulders[MAX_BOULDERS];

VIEW_STRIP	last_boulder_strip;

CVECTOR		boulder_colour = {127,127,127,0};

long		boulder_y_offset[8] = {3,0,0,3,0,3,0,0};

void 	InitBoulders()
{
	long	i;

	last_boulder = 0;
	last_boulder_strip.stripno = -1;
	last_boulder_strip.fileno = -1;

	for(i=0;i<MAX_BOULDERS;i++)
	{
		boulders[i].state = 	-1;
		boulders[i].strip_fraction = 0;
		boulders[i].direction = 1;
		boulders[i].strip.stripno = 0;
		boulders[i].strip.fileno = 0;
		boulders[i].trigger_strip.stripno = 0;
		boulders[i].trigger_strip.fileno = 0;
		boulders[i].pos.vx = boulders[i].pos.vy = boulders[i].pos.vz = 0;
		boulders[i].rot.vx = boulders[i].rot.vy = boulders[i].rot.vz = 0;
		boulders[i].spin.vx = boulders[i].spin.vy = 0;
		boulders[i].spin.vz = 256;
	}
	num_boulders = 0;
	last_boulder = 0;
	direction = 0;
}					 


					




/******************************************************************/

long	DrawBoulders(long playerno,long current_strip)
{					 
	player 		*Plyr;

	long		trigger_tag,i,j,k,midpoint,objecttype,z,tmatid,visible,boulder_strip,strip_int,strip_remainder,new_fraction,boulder_active,objectheight,playerheight,coll,playerx,playerz,boulder_bump;
	long		vertoffset0,vertoffset1,vertoffset2;
	objheader 	*boulder_object;
	SVECTOR		ObjRotVec,X,*vertptr,start_vert,strip_svector;
	VECTOR		Y,Z,ObjectScale,clear,pos,strip_vector;
	MATRIX		ObjectRotation,ObjectTrans,ObjectZRot;
	RUSS_SVECTOR	*outverts;
	objpolygon	*polyptr;
	POLY_FT4	*ft4_ptr;
	POLY_FT3	*ft3_ptr;
	unsigned long	polyustore,polyvstore,vertsstore;
	long		mask=0x0ff;
	long		fx_played,normalclipresult;


	PushMatrix();
	clear.vx = clear.vy = clear.vz = 0;

	Plyr = &(Player[playerno]);
	trigger_tag = Plyr->Track->Meshes[current_strip].SplitTag>>16;
	if(trigger_tag == 6)
	{
		/* make sure we're not running out of boulders */
		if(num_boulders < MAX_BOULDERS)
		{
			/* check if this boulder has already been activated */

			boulder_active = 0;
			for(i=0;i<num_boulders;i++)
			{
				if(boulders[i].trigger_strip.fileno == striptab[playerno][Plyr->Track->CurrentStrip].fileno)
				{
					if(boulders[i].trigger_strip.stripno == striptab[playerno][Plyr->Track->CurrentStrip].stripno)
					{
						boulder_active = 1;
						break;
					}
				}
			}
			
			if(!boulder_active)
			{
				boulders[last_boulder].trigger_strip.stripno = striptab[playerno][Plyr->Track->CurrentStrip].stripno;
				boulders[last_boulder].trigger_strip.fileno = striptab[playerno][Plyr->Track->CurrentStrip].fileno;

				boulders[last_boulder].strip.stripno = striptab[playerno][Plyr->Track->CurrentStrip].stripno + 12;
				boulders[last_boulder].strip.fileno = striptab[playerno][Plyr->Track->CurrentStrip].fileno;

				if(boulders[last_boulder].strip.stripno >= IBuf[Plyr->Buffer]->NumStrips)
					boulders[last_boulder].strip.fileno++;
				boulders[last_boulder].state = 1;
	
				/*rand_idx += 7;
				rand_idx = rand_idx & 0x0000001f;
				boulders[last_boulder].direction = (randarray[rand_idx] & 1);
				if(boulders[last_boulder].direction == 0)
					 boulders[last_boulder].direction = -1;*/

				direction = ((direction) == 1)? -1:1;
				boulders[last_boulder].direction = direction;

				
				boulders[last_boulder].strip_fraction = 0;
				boulders[last_boulder].rot.vx =
					boulders[last_boulder].rot.vy =
					boulders[last_boulder].rot.vz = 0;

				last_boulder++;
				num_boulders++;
			}
		}
	}

	boulder_object = &(ChillObjects[BOULDER_OBJECT_TYPE]);

	fx_played = 0;	
	for(i=0;i<num_boulders;i++)
	{
		/* check if boulders strip is in the display buffer */

		j = 0;
		while(j<MAX_VIEW_STRIPS)
		{
			if(boulders[i].strip.fileno == striptab[playerno][j].fileno)
			{
				if(boulders[i].strip.stripno == striptab[playerno][j].stripno)
				{
					boulder_strip = j;
					break;
				}
			}
			j++;
		}

		if(boulders[i].state < BOULDER_MAX_TIME)
		{
			boulders[i].state++;
			boulders[i].strip_fraction += (256 * boulders[i].direction);
			boulders[i].rot.vz += (boulders[i].spin.vz * boulders[i].direction);
			if(!fx_played)
			{
				fx_played = 1;
				playsoundeffect(0,1);	/* do rumble */
			}
		}
		/* if the boulders strip was not in the display buffers, continue on to the next boulder */
		if(j < MAX_VIEW_STRIPS)
		{

			
			vertptr = boulder_object->vertptr;

			gte_SetRotMatrix(&Plyr->Cam.ViewMatrix);

			if(boulders[i].direction == 1)
				new_fraction = (2<<12) + boulders[i].strip_fraction;
			else
			{
				new_fraction = (Plyr->Track->Meshes[boulder_strip].TMesh.len - 2)>>1;
				new_fraction = new_fraction<<12;
				new_fraction = new_fraction + boulders[i].strip_fraction;
			}
			strip_int = new_fraction>>12;

			/* get each 2nd vert */
			strip_int = strip_int<<1;

			strip_remainder = new_fraction & 4095;

			midpoint = (Plyr->Track->Meshes[boulder_strip].TMesh.v+strip_int + 2)->vx -
							(Plyr->Track->Meshes[boulder_strip].TMesh.v+strip_int)->vx;
			midpoint = (midpoint * strip_remainder)>>12;

			boulders[i].pos.vx = Plyr->Track->Meshes[boulder_strip].OffsetX +
										(Plyr->Track->Meshes[boulder_strip].TMesh.v+strip_int)->vx +
										midpoint;

			midpoint = (Plyr->Track->Meshes[boulder_strip].TMesh.v+strip_int + 2)->vy -
							(Plyr->Track->Meshes[boulder_strip].TMesh.v+strip_int)->vy;
			midpoint = (midpoint * strip_remainder)>>12;

			/* get an index into the boulders roll y offset array (from 0 - 7) */
			boulder_bump = boulders[i].rot.vz & 0xfff;
			boulder_bump = boulder_bump>>9;
			
			boulders[i].pos.vy = (Plyr->Track->Meshes[boulder_strip].TMesh.v+strip_int)->vy + midpoint + 25 + boulder_y_offset[boulder_bump];

			midpoint = (Plyr->Track->Meshes[boulder_strip].TMesh.v+strip_int + 2)->vz -
							(Plyr->Track->Meshes[boulder_strip].TMesh.v+strip_int)->vz;
			midpoint = (midpoint * strip_remainder)>>12;

			boulders[i].pos.vz = Plyr->Track->Meshes[boulder_strip].OffsetZ + 
									(Plyr->Track->Meshes[boulder_strip].TMesh.v+strip_int)->vz +
									midpoint;


			/* check if the player has hit a boulder */

			objectheight=20;
			playerheight = Plyr->Pos.Y>>12;
			if(collision_ctr == 0)
			{
				if(thiscollision[Plyr->Buffer].type == NONE)
				{
					if (playerheight < (boulders[i].pos.vy+40)) /* check y */
					{
						playerx = abs((Plyr->Pos.X>>12) + boulders[i].pos.vx);
						playerz = abs((Plyr->Pos.Z>>12) - boulders[i].pos.vz);
						if((playerx < 30) && (playerz < 30))
						{		
							/* deflect */
							thiscollision[Plyr->Buffer].type=SIDES;
							collision_ctr = 20;
							thiscollision[Plyr->Buffer].angle=0;
							thiscollision[Plyr->Buffer].point.X=Plyr->LastPos.X;
							thiscollision[Plyr->Buffer].point.Y=Plyr->LastPos.Y;
							thiscollision[Plyr->Buffer].point.Z=Plyr->LastPos.Z;
#ifdef MUSIC
							playsoundeffect(20,1);    
#endif
						}
					}
				}
			}
						   
			X.vx=(boulders[i].pos.vx)+ ((Plyr->Cam.Pos.X)>>12);
			X.vy=(boulders[i].pos.vy)- ((Plyr->Cam.Pos.Y)>>12);
			X.vz=(boulders[i].pos.vz)- ((Plyr->Cam.Pos.Z)>>12);
				
			gte_ApplyRotMatrix(&X,&Y);
			
			ObjectTrans.t[0]=Y.vx;
			ObjectTrans.t[1]=Y.vy;
			ObjectTrans.t[2]=Y.vz;

			/* calc y rot matrix of strip that boulder is on */
			ObjectRotation.m[0][1] = ObjectRotation.m[1][0] =
			ObjectRotation.m[2][1] = ObjectRotation.m[1][2] = 0;
			ObjectRotation.m[1][1] = 4096;
			strip_vector.vx = (Plyr->Track->Meshes[boulder_strip].TMesh.v+4)->vx - (Plyr->Track->Meshes[boulder_strip].TMesh.v+2)->vx;
			strip_vector.vy = 0;
			strip_vector.vz = (Plyr->Track->Meshes[boulder_strip].TMesh.v+4)->vz - (Plyr->Track->Meshes[boulder_strip].TMesh.v+2)->vz;
			VectorNormalS(&strip_vector, &strip_svector);
			ObjectRotation.m[0][0] = -strip_svector.vx;   /* - cos theta */
			ObjectRotation.m[2][0] = -strip_svector.vz;	  /* - sin theta */
			ObjectRotation.m[0][2] = strip_svector.vz;	  /* - sin theta */
			ObjectRotation.m[2][2] = -strip_svector.vx;	  /* + cos theta */

			
			ObjRotVec.vx=boulders[i].rot.vx;
			ObjRotVec.vy=boulders[i].rot.vy;
			ObjRotVec.vz=boulders[i].rot.vz;

			RotMatrix(&ObjRotVec,&ObjectZRot);
			MulMatrix0(&ObjectRotation,&ObjectZRot,&ObjectRotation);

			gte_SetRotMatrix(&Plyr->Cam.ViewMatrix);

			MulRotMatrix(&ObjectRotation);
		
			ObjectScale.vx=4096;
			ObjectScale.vy=4096;
			ObjectScale.vz=4096;

			ScaleMatrix(&ObjectRotation,&ObjectScale);
			gte_SetRotMatrix(&ObjectRotation);
			gte_SetTransMatrix(&ObjectTrans);

			outverts = (RUSS_SVECTOR *)(0x1f800000);
			for (k=0;k<boulder_object->numverts;k++)
			{
				gte_ldv0(vertptr);
            	gte_rtps();
		
			    gte_stsxy((outverts));
				gte_stszotz(&z);
				outverts->vz = (long)z;
				vertptr++;
				outverts++;
			}

			polyptr = boulder_object->polyptr;
			outverts = (RUSS_SVECTOR *)(0x1f800000);
			for (k=0;k<boulder_object->numpolys;k++)
			{
				/* if its a quad */
				if (polyptr->numverts==4)
				{
					vertsstore=polyptr->vertptr;
					z = outverts[vertsstore&mask].vz;
					z -= 50;
					if ((z>=0)&&(z<(OTLENGTH-1)))
					{
						vertoffset0 = vertsstore&mask;
						vertoffset1 = (vertsstore>>8)&mask;
						vertoffset2 = (vertsstore>>24)&mask;
						gte_NormalClip(*(long *)(outverts+vertoffset0),
										*(long*)(outverts+vertoffset1),
										*(long*)(outverts+vertoffset2),&normalclipresult);
						if(normalclipresult < 0)
						{
							ft4_ptr=&polyf4s[otid][nopolyf4s];

							/* do colour */
							LONG_MOVE(&boulder_colour,&(ft4_ptr->r0));

							setPolyFT4(ft4_ptr);

							polyustore=polyptr->textus;
							polyvstore=polyptr->textvs;
							tmatid=polyptr->tmatid;
				
							ft4_ptr->tpage=tids[tmatid].textpage;
							ft4_ptr->clut=tids[tmatid].clutline[0];
					
							/* uv's */
							*(uint16 *)&(ft4_ptr->u0)=(polyvstore<<8)|(polyustore&mask);
							*(uint16 *)&(ft4_ptr->u1)=((polyvstore>>8)&mask)<<8|((polyustore>>8)&mask);
							*(uint16 *)&(ft4_ptr->u2)=((polyvstore>>24)&mask)<<8|((polyustore>>24)&mask);
							*(uint16 *)&(ft4_ptr->u3)=((polyvstore>>16)&mask)<<8|((polyustore>>16)&mask);

							/* verts */
						
							LONG_MOVE(outverts+vertoffset0,&(ft4_ptr->x0));
							LONG_MOVE(outverts+vertoffset1,&(ft4_ptr->x1));
							LONG_MOVE(outverts+vertoffset2,&(ft4_ptr->x2));
					
							vertoffset0 = (vertsstore>>16)&mask;
							LONG_MOVE(outverts+vertoffset0,&(ft4_ptr->x3));
					
							addPrim(&(OrderingTable[otid][z]),ft4_ptr);

							nopolyf4s++;
						}
					}				
				}
				else /* its a triangle */
				{
					vertsstore=polyptr->vertptr;
					z = outverts[vertsstore&mask].vz;
					z -= 50;
					if((z>=0)&&(z<(OTLENGTH-1)))
					{
						vertoffset0 = vertsstore&mask;
						vertoffset1 = (vertsstore>>8)&mask;
						vertoffset2 = (vertsstore>>16)&mask;
						gte_NormalClip(*(long *)(outverts+vertoffset0),
										*(long*)(outverts+vertoffset1),
										*(long*)(outverts+vertoffset2),&normalclipresult);
						if(normalclipresult < 0)
						{						
							ft3_ptr=&polyf3s[otid][nopolyf3s];

							LONG_MOVE(&boulder_colour,&(ft3_ptr->r0));

							setPolyFT3(ft3_ptr);

							polyustore=polyptr->textus;
							polyvstore=polyptr->textvs;
							tmatid=polyptr->tmatid;
				
							ft3_ptr->tpage=tids[tmatid].textpage;
							ft3_ptr->clut=tids[tmatid].clutline[0];
					
							*(uint16 *)&(ft3_ptr->u0)=(polyvstore<<8)|(polyustore&mask);
							*(uint16 *)&(ft3_ptr->u1)=((polyvstore>>8)&mask)<<8|((polyustore>>8)&mask);
							*(uint16 *)&(ft3_ptr->u2)=((polyvstore>>16)&mask)<<8|((polyustore>>16)&mask);
					
							LONG_MOVE(outverts+vertoffset0,&(ft3_ptr->x0));
							LONG_MOVE(outverts+vertoffset1,&(ft3_ptr->x1));
							LONG_MOVE(outverts+vertoffset2,&(ft3_ptr->x2));
					
							addPrim(&(OrderingTable[otid][z]),ft3_ptr);

							nopolyf3s++;
						}
					}			
				}
				polyptr++;
			}
		}
	}		   
			  
	PopMatrix();
	return(1);
	
}					 