#include <stdio.h>
#include <sys/types.h>
#include <LibGTE.h>
#include <LibGPU.h>
#include <LibCD.h>
#include <LibComb.h>
#include <LibEtc.h>
#include <LibGS.h>
#include <LibMath.h>
#include <LibPress.h>
#include <LibSN.h>
#include <LibSnd.h>
#include <LibSPU.h>
#include <inline_o.h>
#include <gtemac.h>


#include "ourtypes.h"
#include "mathbits.h"

#include "main.h"
#include "skeleton.h"
#include "camera.h"
#include "tracks.h"
#include "players.h"
#include "pads.h"

#include "screen.h"
#include "who.h"
#include "bored.h"
#include "animctrl.h"

#include "bonebag.h"
#include "trixbag.h"
#include "logic.h"
#include "leaf.h"
#include "plume.h"
#include "rotmesh.h"


LEAF	leaves[2][MAX_LEAVES];
SVECTOR	leaf_verts[4];
CVECTOR	leaf_colour;
VECTOR	leaf_loc[2][MAX_LEAVES];
long	current_leaf[2],tree_hit[2];



void	InitLeaves()
{
	short	i,j;
	unsigned short trans_tpage,opaque_pad;

	for(i=0;i<MAX_LEAVES;i++)
	{
		leaves[0][i].timer = -1;
		leaves[1][i].timer = -1;

		leaves[0][i].scale.vx = 4096;
		leaves[0][i].scale.vy = 4096;
		leaves[0][i].scale.vz = 4096;

		leaves[1][i].scale.vx = 4096;
		leaves[1][i].scale.vy = 4096;
		leaves[1][i].scale.vz = 4096;

		setPolyF4(&(leaves[0][i].prim[0]));
		setPolyF4(&(leaves[0][i].prim[1]));
		setPolyF4(&(leaves[1][i].prim[0]));
		setPolyF4(&(leaves[1][i].prim[1]));

		if((currentfile >= 52) && (currentfile <= 73))
		{
			setRGB0(&(leaves[0][i].prim[0]),AVALANCHE_LEAF_R,AVALANCHE_LEAF_G,AVALANCHE_LEAF_B);
			/*setRGB1(&(leaves[0][i].prim[0]),AVALANCHE_LEAF_R,AVALANCHE_LEAF_G,AVALANCHE_LEAF_B);
			setRGB2(&(leaves[0][i].prim[0]),AVALANCHE_LEAF_R,AVALANCHE_LEAF_G,AVALANCHE_LEAF_B);
			setRGB3(&(leaves[0][i].prim[0]),AVALANCHE_LEAF_R,AVALANCHE_LEAF_G,AVALANCHE_LEAF_B);*/
			setRGB0(&(leaves[1][i].prim[0]),AVALANCHE_LEAF_R,AVALANCHE_LEAF_G,AVALANCHE_LEAF_B);
			/*setRGB1(&(leaves[1][i].prim[0]),AVALANCHE_LEAF_R,AVALANCHE_LEAF_G,AVALANCHE_LEAF_B);
			setRGB2(&(leaves[1][i].prim[0]),AVALANCHE_LEAF_R,AVALANCHE_LEAF_G,AVALANCHE_LEAF_B);
			setRGB3(&(leaves[1][i].prim[0]),AVALANCHE_LEAF_R,AVALANCHE_LEAF_G,AVALANCHE_LEAF_B);*/
	
			setRGB0(&(leaves[0][i].prim[1]),AVALANCHE_LEAF_R,AVALANCHE_LEAF_G,AVALANCHE_LEAF_B);
			/*setRGB1(&(leaves[0][i].prim[1]),AVALANCHE_LEAF_R,AVALANCHE_LEAF_G,AVALANCHE_LEAF_B);
			setRGB2(&(leaves[0][i].prim[1]),AVALANCHE_LEAF_R,AVALANCHE_LEAF_G,AVALANCHE_LEAF_B);
			setRGB3(&(leaves[0][i].prim[1]),AVALANCHE_LEAF_R,AVALANCHE_LEAF_G,AVALANCHE_LEAF_B);*/
			setRGB0(&(leaves[1][i].prim[1]),AVALANCHE_LEAF_R,AVALANCHE_LEAF_G,AVALANCHE_LEAF_B);
			/*setRGB1(&(leaves[1][i].prim[1]),AVALANCHE_LEAF_R,AVALANCHE_LEAF_G,AVALANCHE_LEAF_B);
			setRGB2(&(leaves[1][i].prim[1]),AVALANCHE_LEAF_R,AVALANCHE_LEAF_G,AVALANCHE_LEAF_B);
			setRGB3(&(leaves[1][i].prim[1]),AVALANCHE_LEAF_R,AVALANCHE_LEAF_G,AVALANCHE_LEAF_B);*/
		}
		else
		{
			setRGB0(&(leaves[0][i].prim[0]),LEAF_R,LEAF_G,LEAF_B);
			/*setRGB1(&(leaves[0][i].prim[0]),LEAF_R,LEAF_G,LEAF_B);
			setRGB2(&(leaves[0][i].prim[0]),LEAF_R,LEAF_G,LEAF_B);
			setRGB3(&(leaves[0][i].prim[0]),LEAF_R,LEAF_G,LEAF_B);*/
			setRGB0(&(leaves[1][i].prim[0]),LEAF_R,LEAF_G,LEAF_B);
			/*setRGB1(&(leaves[1][i].prim[0]),LEAF_R,LEAF_G,LEAF_B);
			setRGB2(&(leaves[1][i].prim[0]),LEAF_R,LEAF_G,LEAF_B);
			setRGB3(&(leaves[1][i].prim[0]),LEAF_R,LEAF_G,LEAF_B);*/
	
			setRGB0(&(leaves[0][i].prim[1]),LEAF_R,LEAF_G,LEAF_B);
			/*setRGB1(&(leaves[0][i].prim[1]),LEAF_R,LEAF_G,LEAF_B);
			setRGB2(&(leaves[0][i].prim[1]),LEAF_R,LEAF_G,LEAF_B);
			setRGB3(&(leaves[0][i].prim[1]),LEAF_R,LEAF_G,LEAF_B);*/

			setRGB0(&(leaves[1][i].prim[1]),LEAF_R,LEAF_G,LEAF_B);
			/*setRGB1(&(leaves[1][i].prim[1]),LEAF_R,LEAF_G,LEAF_B);
			setRGB2(&(leaves[1][i].prim[1]),LEAF_R,LEAF_G,LEAF_B);
			setRGB3(&(leaves[1][i].prim[1]),LEAF_R,LEAF_G,LEAF_B);*/
		}
	}

	/* clear both buffers */

	leaf_verts[0].vx = -2;
	leaf_verts[0].vy = 0;
	leaf_verts[0].vz = 0;
	leaf_verts[1].vx = 0;
	leaf_verts[1].vy = 5;
	leaf_verts[1].vz = 0;
	leaf_verts[2].vx = 0;
	leaf_verts[2].vy = -5;
	leaf_verts[2].vz = 0;
	leaf_verts[3].vx = 2;
	leaf_verts[3].vy = 0;
	leaf_verts[3].vz = 0;
	
	current_leaf[0] = 0;
	current_leaf[1] = 0;
	tree_hit[0] = 0;
	tree_hit[1] = 0;
}


/************************************************/


void AddLeaf(playerno,pos)
long	playerno;
VECTOR 	*pos;
{
	long	i;

	if(leaves[playerno][current_leaf[playerno]].timer < 0)	/* if there is a spare leaf available */
	{
		leaves[playerno][current_leaf[playerno]].timer = LEAF_LIFE;
	
		leaves[playerno][current_leaf[playerno]].global_pos.vx = pos->vx;
		leaves[playerno][current_leaf[playerno]].global_pos.vy = pos->vy;
		leaves[playerno][current_leaf[playerno]].global_pos.vz = pos->vz;

		leaves[playerno][current_leaf[playerno]].inert.vx = 0;
		leaves[playerno][current_leaf[playerno]].inert.vy = 0;
		leaves[playerno][current_leaf[playerno]].inert.vz = 0;	

		leaves[playerno][current_leaf[playerno]].scale.vx = 16000;
		leaves[playerno][current_leaf[playerno]].scale.vy = 16000;
		leaves[playerno][current_leaf[playerno]].scale.vz = 16000;

		rand_idx += 7;
		rand_idx = rand_idx & 0x0000001f;
		leaves[playerno][current_leaf[playerno]].spin.vx = randarray[rand_idx] & 0x000000ff;
		rand_idx += 7;
		rand_idx = rand_idx & 0x0000001f;
		leaves[playerno][current_leaf[playerno]].spin.vy = randarray[rand_idx] & 0x000000ff;
		rand_idx += 7;
		rand_idx = rand_idx & 0x0000001f;
		leaves[playerno][current_leaf[playerno]].spin.vz = randarray[rand_idx] & 0x000000ff;
					
		leaves[playerno][current_leaf[playerno]].rot.vx = leaves[playerno][current_leaf[playerno]].rot.vy = leaves[playerno][current_leaf[playerno]].rot.vz = 0;
	

		leaf_colour.r = LEAF_FALLING_R;
		leaf_colour.g = LEAF_FALLING_G;
		leaf_colour.b = LEAF_FALLING_B;

		LONG_MOVE(&leaf_colour,&(leaves[playerno][current_leaf[playerno]].prim[0].r0));
		LONG_MOVE(&leaf_colour,&(leaves[playerno][current_leaf[playerno]].prim[1].r0));
		current_leaf[playerno]++;
		if(current_leaf[playerno] >= MAX_LEAVES)
			current_leaf[playerno] = 0;
	}
}



/************************************************/

void	DrawLeaves(unsigned long playerno)
{
	SVECTOR	sclear,leaf_spin,leaf_local;
	VECTOR	clear,offset,leaf_scale;
	long	i,address,z,temp,new_otid;
	MATRIX	ObjectTrans,z_mat;
	SVECTOR		rtp_pos[4];


	if(ConsolePlayers == 1)
		new_otid = otid & 1;
	else
	{
		new_otid = otid & 2;
		new_otid = new_otid>>1;
	}

	/* backup rotation matrix */
	PushMatrix();

	sclear.vx = sclear.vy = sclear.vz = 0;
	clear.vx = clear.vy = clear.vz = 0;


	if((Player[playerno].Style == STY_TREESOFT) || (Player[playerno].Style == STY_TREEBUMP))
	{
		if(tree_hit[playerno] == 0)
		{
			tree_hit[playerno] = 1;
			for(i=0;i<16;i++)
			{
				if(leaves[playerno][current_leaf[playerno]].timer < 0)	/* if there is a spare leaf available */
				{
					leaves[playerno][current_leaf[playerno]].timer = LEAF_LIFE;
				
					/* setup random inert & offset part */
					rand_idx += 7;
					rand_idx = rand_idx & 0x0000001f;
					offset.vx = (randarray[rand_idx] - 16384)>>2;
					rand_idx += 7;
					rand_idx = rand_idx & 0x0000001f;
					offset.vy = (randarray[rand_idx] - 16384)>>2;

					rand_idx += 7;
					rand_idx = rand_idx & 0x0000001f;
					temp = (randarray[rand_idx])<<1;

		   			leaves[playerno][current_leaf[playerno]].global_pos.vx = 	Player[playerno].Pos.X;
					leaves[playerno][current_leaf[playerno]].global_pos.vy = 	Player[playerno].Pos.Y + (20 * 4096) + temp;
					leaves[playerno][current_leaf[playerno]].global_pos.vz = 	Player[playerno].Pos.Z;

		   			leaves[playerno][current_leaf[playerno]].global_pos.vx += 	offset.vx;
					leaves[playerno][current_leaf[playerno]].global_pos.vy += 	offset.vy;
						
					leaves[playerno][current_leaf[playerno]].inert.vx = (Player[playerno].Pos.X  - Player[playerno].LastPos.X)>>1;
					leaves[playerno][current_leaf[playerno]].inert.vy = (Player[playerno].Pos.Y  - Player[playerno].LastPos.Y)>>1;
					leaves[playerno][current_leaf[playerno]].inert.vz = (Player[playerno].Pos.Z  - Player[playerno].LastPos.Z)>>1;
					
					leaves[playerno][current_leaf[playerno]].inert.vx += offset.vx;
					leaves[playerno][current_leaf[playerno]].inert.vy += offset.vy;
					
					rand_idx += 7;
					rand_idx = rand_idx & 0x0000001f;
					leaves[playerno][current_leaf[playerno]].spin.vx = randarray[rand_idx] & 0x000000ff;
					rand_idx += 7;
					rand_idx = rand_idx & 0x0000001f;
					leaves[playerno][current_leaf[playerno]].spin.vy = randarray[rand_idx] & 0x000000ff;
					rand_idx += 7;
					rand_idx = rand_idx & 0x0000001f;
					leaves[playerno][current_leaf[playerno]].spin.vz = randarray[rand_idx] & 0x000000ff;
					
					leaves[playerno][current_leaf[playerno]].rot.vx = leaves[playerno][current_leaf[playerno]].rot.vy = leaves[playerno][current_leaf[playerno]].rot.vz = 0;
					
					leaves[playerno][current_leaf[playerno]].scale.vx = 4096;
					leaves[playerno][current_leaf[playerno]].scale.vy = 4096;
					leaves[playerno][current_leaf[playerno]].scale.vz = 4096;


					if((currentfile[0] >= 52) && (currentfile[0] <= 73))
					{
						leaf_colour.r = AVALANCHE_LEAF_R;
						leaf_colour.g = AVALANCHE_LEAF_G;
						leaf_colour.b = AVALANCHE_LEAF_B;
					}
					else
					{
						leaf_colour.r = LEAF_R;
						leaf_colour.g = LEAF_G;
						leaf_colour.b = LEAF_B;
					}
	
					LONG_MOVE(&leaf_colour,&(leaves[playerno][current_leaf[playerno]].prim[0].r0));

					LONG_MOVE(&leaf_colour,&(leaves[playerno][current_leaf[playerno]].prim[1].r0));
				}
				current_leaf[playerno]++;
				if(current_leaf[playerno] >= MAX_LEAVES)
					current_leaf[playerno] = 0;
			}		
		}			
	}
	else		
		tree_hit[playerno] = 0;

	SetRotMatrix(&(Player[playerno].Cam.ViewMatrix));

	for(i=0;i<MAX_LEAVES;i++)
	{
		if(leaves[playerno][i].timer >= 0)
		{
			leaves[playerno][i].timer--;

			leaves[playerno][i].global_pos.vx += leaves[playerno][i].inert.vx;
			leaves[playerno][i].global_pos.vy += leaves[playerno][i].inert.vy;
			leaves[playerno][i].global_pos.vz += leaves[playerno][i].inert.vz;

	   
			leaf_local.vx = (short)((-leaves[playerno][i].global_pos.vx>>12) + (Player[playerno].Cam.Pos.X>>12)); 
			leaf_local.vy = (short)((leaves[playerno][i].global_pos.vy>>12) - (Player[playerno].Cam.Pos.Y>>12)); 
			leaf_local.vz = (short)((leaves[playerno][i].global_pos.vz>>12) - (Player[playerno].Cam.Pos.Z>>12)); 

			ApplyRotMatrix(&leaf_local,&(leaf_loc[playerno][i]));

			if(leaf_loc[playerno][i].vz <  LEAF_NEAR_Z)
				leaves[playerno][i].timer = -1;
			else
			{
				leaves[playerno][i].rot.vx += leaves[playerno][i].spin.vx;
				leaves[playerno][i].rot.vy += leaves[playerno][i].spin.vy;
				leaves[playerno][i].rot.vz += leaves[playerno][i].spin.vz;
			}
		}
	}


	for(i=0;i<MAX_LEAVES;i++)
	{
		if(leaves[playerno][i].timer >= 0)
		{
			ObjectTrans.t[1] = leaf_loc[playerno][i].vx;
			ObjectTrans.t[0] = leaf_loc[playerno][i].vy;
			ObjectTrans.t[2] = leaf_loc[playerno][i].vz;

			SetTransMatrix( &ObjectTrans );

			/* setup random rot */
			RotMatrix(&(leaves[playerno][i].rot),&z_mat);
			
			ScaleMatrix(&z_mat,&(leaves[playerno][i].scale));
			SetRotMatrix(&z_mat);
			
			RotTransPers3(&(leaf_verts[0]),&(leaf_verts[1]),&(leaf_verts[2]),
							&(rtp_pos[0]),&(rtp_pos[1]),&(rtp_pos[2]),&address,&address);
			z = RotTransPers(&(leaf_verts[3]),&(rtp_pos[3]),&address,&address);

			if((z>MESH_NEAR_Z) && (z < OTLENGTH-1))
			{
				leaves[playerno][i].prim[new_otid].x0 = rtp_pos[0].vy;
				leaves[playerno][i].prim[new_otid].y0 = rtp_pos[0].vx;
				leaves[playerno][i].prim[new_otid].x1 = rtp_pos[1].vy;
				leaves[playerno][i].prim[new_otid].y1 = rtp_pos[1].vx;
				leaves[playerno][i].prim[new_otid].x2 = rtp_pos[2].vy;
				leaves[playerno][i].prim[new_otid].y2 = rtp_pos[2].vx;
				leaves[playerno][i].prim[new_otid].x3 = rtp_pos[3].vy;
				leaves[playerno][i].prim[new_otid].y3 = rtp_pos[3].vx;

				setPolyF4(&(leaves[playerno][i].prim[new_otid]));
				addPrim(&(OrderingTable[otid][z]),&(leaves[playerno][i].prim[new_otid]));	 		  
			}
		}
	}
	/* restore old rotation matrix */
	PopMatrix();
}



