#include <stdio.h>
#include <sys/types.h>
#include <LibGTE.h>
#include <LibGPU.h>
#include <LibCD.h>
#include <LibComb.h>
#include <LibEtc.h>
#include <LibGS.h>
#include <LibMath.h>
#include <LibPress.h>
#include <LibSN.h>
#include <LibSnd.h>
#include <LibSPU.h>
#include <inline_o.h>
#include <gtemac.h>


#include "ourtypes.h"
#include "mathbits.h"

#include "main.h"
#include "skeleton.h"
#include "camera.h"
#include "camlogic.h"
#include "tracks.h"
#include "players.h"
#include "pads.h"

#include "screen.h"
#include "who.h"
#include "bored.h"
#include "animctrl.h"

#include "bonebag.h"
#include "trixbag.h"
#include "logic.h"
#include "leaf.h"
#include "plume.h"
#include "rotmesh.h"
#include "avalanch.h"
#include "boulder.h"
#include "parachut.h"


long	last_para_y;

// structure to keep track of all the strips in the last segment
PARA_SEGMENT	landing_zone;


long	parachute_state = 0;
long	parachute_counter = 0;
long	last_parachute_counter;
SVECTOR	parachute_rot;


DR_TPAGE		parachute_transparencies[4];	/* primitives to turn transparency on/off on sub ot */

SVECTOR	fogverts[17] = {
					{ 0,-235,778},
					{ 298,-235,719},
					{ 550,-235,550},
					{ 719,-235,298},
					{ 778,-235,0},
					{ 719,-235,-297},
					{ 550,-235,-550},
					{ 298,-235,-718},
					{ 0,-235,-778},
					{ -297,-235,-718},
					{ -550,-235,-550},
					{ -718,-235,-297},
					{ -778,-235,0},
					{ -718,-235,298},
					{ -550,-235,550},
					{ -297,-235,719},
					{ 0,-235,778}
						};


/*					{ 0   ,-350,778},
					{ 298 ,-350,719},
					{ 550 ,-350,550},
					{ 719 ,-350,298},
					{ 778 ,-350,0},
					{ 719 ,-350,-297},
					{ 550 ,-350,-550},
					{ 298 ,-350,-718},
					{ 0   ,-350,-778},
					{ -297,-350,-718},
					{ -550,-350,-550},
					{ -718,-350,-297},
					{ -778,-350,0},
					{ -718,-350,298},
					{ -550,-350,550},
					{ -297,-350,719},
					{ 0   ,-350,778}
						};*/



long	parachute_height;

void	InitParachute()
{
	long	trans_tpage;

	parachute_state = 0;
	parachute_counter = 0;
	parachute_rot.vx = 0;
	parachute_rot.vy = 1024;
	parachute_rot.vz = 0;


	trans_tpage = GetTPage(1,3,0,0);
	SetDrawTPage(&(parachute_transparencies[0]),1,1,(int)(trans_tpage));
	SetDrawTPage(&(parachute_transparencies[1]),1,1,(int)(trans_tpage));
	SetDrawTPage(&(parachute_transparencies[2]),1,1,(int)(trans_tpage));
	SetDrawTPage(&(parachute_transparencies[3]),1,1,(int)(trans_tpage));



}


void RenderParachute(object,Plyr,scale,rot)
objheader	*object;
player		*Plyr;
VECTOR		*scale;
SVECTOR		*rot;
{
	SVECTOR	*vertptr,X,rotation;
	VECTOR	parachute_loc,Y,ObjectScale;
	MATRIX	ObjectTrans,ObjectRotation;
	RUSS_SVECTOR	*outverts;
	long	k,z,p;
	objpolygon	*polyptr;
	unsigned long	vertsstore,normalclipresult;
	uint32 	polyustore;
	uint32	polyvstore;
	uint32	tmatid;
	long		mask=0x0ff;
//	POLY_G4		*g4_ptr;
	POLY_F4		*f4_ptr;
//	POLY_G3	*g3_ptr;
	POLY_F3		*f3_ptr;
//	POLY_GT3	*gt3_ptr;
	POLY_FT3	*ft3_ptr;
//	POLY_GT4	*gt4_ptr;
	POLY_FT4	*ft4_ptr;

	long		vertoffset0,vertoffset1,vertoffset2,vertoffset3;
	CVECTOR		newcol = {127,127,127,44};



	vertptr = object->vertptr;

	gte_SetRotMatrix(&Plyr->Cam.ViewMatrix);


	parachute_loc.vx = Plyr->Pos.X;
	parachute_loc.vy = Plyr->Pos.Y + 116000;
	parachute_loc.vz = Plyr->Pos.Z;

	X.vx= -(parachute_loc.vx - Plyr->Cam.Pos.X)>>12;
	X.vy= (parachute_loc.vy - Plyr->Cam.Pos.Y)>>12;
	X.vz= (parachute_loc.vz - Plyr->Cam.Pos.Z)>>12;

	gte_ApplyRotMatrix(&X,&Y);

	ObjectTrans.t[0]=Y.vx;
	ObjectTrans.t[1]=Y.vy;
	if(Y.vz < 200)
		Y.vz = 0;
	else
		Y.vz -= 200;
	ObjectTrans.t[2]=Y.vz;

/*	rotation.vx = CHSHIFTR( Plyr->Rot.X );
	rotation.vy = CHSHIFTR( Plyr->Rot.Y );
	rotation.vz = CHSHIFTR( Plyr->Rot.Z );*/

	parachute_rot.vx = rot->vx;
	parachute_rot.vy = -CHSHIFTR( Plyr->Rot.Y );
	parachute_rot.vz = rot->vz;

	RotMatrixYXZ( &parachute_rot, &ObjectRotation );

/*  gte_SetRotMatrix(&Plyr->Cam.ViewMatrix);
  	MulRotMatrix(&ObjectRotation);*/
	MulMatrix0(&Plyr->Cam.ViewMatrix,&ObjectRotation,&ObjectRotation);

	ObjectScale.vx=scale->vx;
	ObjectScale.vy=scale->vy;
	ObjectScale.vz=scale->vz;

	ScaleMatrix(&ObjectRotation,&ObjectScale);
	gte_SetRotMatrix(&ObjectRotation);


	gte_SetTransMatrix(&ObjectTrans);

	outverts = (RUSS_SVECTOR *)(0x1f800000);
	for (k=0;k<object->numverts;k++)
	{
		gte_ldv0(vertptr);
       	gte_rtps();

	    gte_stsxy((outverts));
		gte_stszotz(&z);
        gte_stdp(&p);
		outverts->vz = (long)z;
		vertptr++;
		outverts++;
	}

	polyptr = object->polyptr;
	outverts = (RUSS_SVECTOR *)(0x1f800000);
	if(otid == 0)
	{
		f4_ptr=&poly4s[0][nopoly4s];
		f3_ptr=&poly3s[0][nopoly3s];
		ft4_ptr=&polyf4s[0][nopolyf4s];
		ft3_ptr=&polyf3s[0][nopolyf3s];
	}
	else
	{
		f4_ptr=&poly4s[2][nopoly4s];
		f3_ptr=&poly3s[2][nopoly3s];
		ft4_ptr=&polyf4s[2][nopolyf4s];
		ft3_ptr=&polyf3s[2][nopolyf3s];
	}

	for (k=0;k<object->numpolys;k++)
	{
		/* if its a quad */
		if (polyptr->numverts==4)
		{
			if(polyptr->polytype > 0)	/* if its textured */
			{
				vertsstore=polyptr->vertptr;
				z = outverts[vertsstore&mask].vz;
				if ((z>=0)&&(z<(OTLENGTH-1)))
				{
					vertoffset0 = vertsstore&mask;
					vertoffset1 = (vertsstore>>8)&mask;
					vertoffset2 = (vertsstore>>24)&mask;
					vertoffset3 = (vertsstore>>16)&mask;

					/* do colour */

					LONG_MOVE(&newcol,&(ft4_ptr->r0));

					/* do textured stuff */

					polyustore=polyptr->textus;
					polyvstore=polyptr->textvs;
					tmatid=polyptr->tmatid;
					ft4_ptr->tpage=tids[tmatid].textpage;
					ft4_ptr->clut=tids[tmatid].clutline[0];

					*(uint16 *)&(ft4_ptr->u0)=(polyvstore<<8)|(polyustore&mask);
					*(uint16 *)&(ft4_ptr->u1)=((polyvstore>>8)&mask)<<8|((polyustore>>8)&mask);
					*(uint16 *)&(ft4_ptr->u2)=((polyvstore>>24)&mask)<<8|((polyustore>>24)&mask);
					*(uint16 *)&(ft4_ptr->u3)=((polyvstore>>16)&mask)<<8|((polyustore>>16)&mask);

					setPolyFT4(ft4_ptr);

					LONG_MOVE(outverts+vertoffset0,&(ft4_ptr->x0));
					LONG_MOVE(outverts+vertoffset1,&(ft4_ptr->x1));
					LONG_MOVE(outverts+vertoffset2,&(ft4_ptr->x2));
					LONG_MOVE(outverts+vertoffset3,&(ft4_ptr->x3));

					addPrim(&(OrderingTable[otid][z]),ft4_ptr);

					ft4_ptr++;
					nopolyf4s++;
				}
			}
			else	/* its non textured */
			{
				vertsstore=polyptr->vertptr;
				z = outverts[vertsstore&mask].vz;
				if ((z>=0)&&(z<(OTLENGTH-1)))
				{
					vertoffset0 = vertsstore&mask;
					vertoffset1 = (vertsstore>>8)&mask;
					vertoffset2 = (vertsstore>>24)&mask;
					vertoffset3 = (vertsstore>>16)&mask;

					/* do colour */

					LONG_MOVE(&(polyptr->basecolour),&(f4_ptr->r0));

					setPolyF4(f4_ptr);

					LONG_MOVE(outverts+vertoffset0,&(f4_ptr->x0));
					LONG_MOVE(outverts+vertoffset1,&(f4_ptr->x1));
					LONG_MOVE(outverts+vertoffset2,&(f4_ptr->x2));
					LONG_MOVE(outverts+vertoffset3,&(f4_ptr->x3));

					addPrim(&(OrderingTable[otid][z]),f4_ptr);

					f4_ptr++;
					nopoly4s++;
				}
			}
		}
		else /* its a triangle */
		{
			if(polyptr->polytype > 0 ) /* if its textured */
			{
				vertsstore=polyptr->vertptr;
				z = outverts[vertsstore&mask].vz;
				if((z>=0)&&(z<(OTLENGTH-1)))
			 	{
		 			vertoffset0 = vertsstore&mask;
		 			vertoffset1 = (vertsstore>>8)&mask;
			 		vertoffset2 = (vertsstore>>16)&mask;

					/* do colour */


					LONG_MOVE(&newcol,&(ft3_ptr->r0));

					/* do textured stuff */

					polyustore=polyptr->textus;
					polyvstore=polyptr->textvs;
					tmatid=polyptr->tmatid;
					ft3_ptr->tpage=tids[tmatid].textpage;
					//gt3_ptr->clut=tids[tmatid].clutline[thisp];
					ft3_ptr->clut=tids[tmatid].clutline[0];
					*(uint16 *)&(ft3_ptr->u0)=(polyvstore<<8)|(polyustore&mask);
					*(uint16 *)&(ft3_ptr->u1)=((polyvstore>>8)&mask)<<8|((polyustore>>8)&mask);
					*(uint16 *)&(ft3_ptr->u2)=((polyvstore>>16)&mask)<<8|((polyustore>>16)&mask);



			 		setPolyFT3(ft3_ptr);

			 		LONG_MOVE(outverts+vertoffset0,&(ft3_ptr->x0));
		 			LONG_MOVE(outverts+vertoffset1,&(ft3_ptr->x1));
		 			LONG_MOVE(outverts+vertoffset2,&(ft3_ptr->x2));

			 		addPrim(&(OrderingTable[otid][z]),ft3_ptr);

					ft3_ptr++;
					nopolyf3s++;

			 	}
			}
			else
			{
				vertsstore=polyptr->vertptr;
				z = outverts[vertsstore&mask].vz;
				if((z>=0)&&(z<(OTLENGTH-1)))
			 	{
		 			vertoffset0 = vertsstore&mask;
		 			vertoffset1 = (vertsstore>>8)&mask;
			 		vertoffset2 = (vertsstore>>16)&mask;

					/* do colour */

					LONG_MOVE(&(polyptr->basecolour),&(f3_ptr->r0));

			 		setPolyF3(f3_ptr);

			 		LONG_MOVE(outverts+vertoffset0,&(f3_ptr->x0));
		 			LONG_MOVE(outverts+vertoffset1,&(f3_ptr->x1));
		 			LONG_MOVE(outverts+vertoffset2,&(f3_ptr->x2));

			 		addPrim(&(OrderingTable[otid][z]),f3_ptr);

					f3_ptr++;
					nopoly3s++;

			 	}
			}
		}
		polyptr++;
	}
}




void	DrawParachuteFog(Plyr)
player	*Plyr;
{
	MATRIX	fogmat;
	VECTOR	scale;
	SVECTOR	*vertptr,triverts[3];
	objpolygon *polyptr;
	long	i;
	POLY_G3	*g3_ptr;
	CVECTOR	col0 = {0xff,0xff,0xff,0xff};
	CVECTOR	col1 = {0x00,0x00,0x00,0xff};
	CVECTOR	col2 = {0x00,0x00,0x00,0xff};
	CVECTOR	col3 = {0xdf,0xdf,0xdf,0xff};
	CVECTOR	col4 = {0xbf,0xbf,0xbf,0xff};


	gte_SetGeomOffset(120,160);
	gte_SetGeomScreen(geomscreen);

	SetRotMatrix(&mountmatrix[Plyr->Buffer]);
	gte_ReadRotMatrix(&fogmat);
	scale.vx = 4096;
	scale.vy = 5096;
	scale.vz = 4096;
	ScaleMatrix(&fogmat,&scale);
	SetRotMatrix(&fogmat);

	fogmat.t[0] = skieddown[Plyr->Buffer];
	fogmat.t[1] = 0;
	fogmat.t[2] = 0;

	SetTransMatrix(&fogmat);

	//vertptr = ChillObjects[0].vertptr+32;
	vertptr = fogverts;
	polyptr=ChillObjects[0].polyptr;


	/* do central vert */

	/*triverts[0].vx = 0;
	triverts[0].vy = -235;
	triverts[0].vz = 0;
	gte_ldv0(&(triverts[0].vx));
    gte_rtps();

	gte_stsxy(&(triverts[0].vx));*/

	triverts[0].vx = 160;
	triverts[0].vy = 240;
	triverts[0].vz = 0;

	triverts[1].vx = 100;
	triverts[1].vy = 200;
	triverts[1].vz = 0;

	triverts[2].vx = 200;
	triverts[2].vy = 200;
	triverts[2].vz = 0;


	/* for all the triangles */
	g3_ptr = &(trackprims[otid][nopolyG3s]);
	addPrim(&(OrderingTable[otid][601]),&parachute_transparencies[otid]);
	for(i=0;i<16;i++)
	{
		gte_ldv0(vertptr);
	    gte_rtps();

		gte_stsxy(&(triverts[1].vx));

		gte_ldv0(vertptr+1);
	    gte_rtps();

		gte_stsxy(&(triverts[2].vx));

		/* do colour */

		LONG_MOVE(&col0,&(g3_ptr->r0));
		LONG_MOVE(&col1,&(g3_ptr->r1));
		LONG_MOVE(&col2,&(g3_ptr->r2));

		LONG_MOVE(&col3,&((g3_ptr+1)->r0));
		LONG_MOVE(&col1,&((g3_ptr+1)->r1));
		LONG_MOVE(&col2,&((g3_ptr+1)->r2));

		LONG_MOVE(&col4,&((g3_ptr+2)->r0));
		LONG_MOVE(&col1,&((g3_ptr+2)->r1));
		LONG_MOVE(&col2,&((g3_ptr+2)->r2));


 		setPolyG3(g3_ptr);
 		setPolyG3(g3_ptr+1);
 		setPolyG3(g3_ptr+2);
 		setSemiTrans(g3_ptr,1);
 		setSemiTrans(g3_ptr+1,1);
 		setSemiTrans(g3_ptr+2,1);

 		LONG_MOVE(&(triverts[0]),&(g3_ptr->x0));
 		LONG_MOVE(&(triverts[0]),&((g3_ptr+1)->x0));
 		LONG_MOVE(&(triverts[0]),&((g3_ptr+2)->x0));

		g3_ptr->x1 = triverts[1].vy;
		g3_ptr->y1 = triverts[1].vx;
		g3_ptr->x2 = triverts[2].vy;
		g3_ptr->y2 = triverts[2].vx;

		(g3_ptr+1)->x1 = triverts[1].vy;
		(g3_ptr+1)->y1 = triverts[1].vx;
		(g3_ptr+1)->x2 = triverts[2].vy;
		(g3_ptr+1)->y2 = triverts[2].vx;

		(g3_ptr+2)->x1 = triverts[1].vy;
		(g3_ptr+2)->y1 = triverts[1].vx;
		(g3_ptr+2)->x2 = triverts[2].vy;
		(g3_ptr+2)->y2 = triverts[2].vx;


 		addPrim(&(OrderingTable[otid][600]),g3_ptr);
 		addPrim(&(OrderingTable[otid][600]),g3_ptr+1);
 		addPrim(&(OrderingTable[otid][600]),g3_ptr+2);

		g3_ptr+=3;
		vertptr++;
		nopolyG3s +=3;
	}
	gte_SetGeomOffset(160,120);

}



void	GetLastSegment(playerno)
long	playerno;
{
	long	offset,i;
	unsigned long	*address;
	texttrackpoly 	*textya;

	offset = 0;
	landing_zone.no_strips = IBuf[Player[playerno].Buffer]->NumStrips;
	landing_zone.offsetX = IBuf[Player[playerno].Buffer]->OffsetX;
	landing_zone.offsetZ = IBuf[Player[playerno].Buffer]->OffsetZ;
	landing_zone.no_objects = IBuf[Player[playerno].Buffer]->NumObjects;
	landing_zone.no_textures = IBuf[Player[playerno].Buffer]->NumTextures;
	for(i=0;i<landing_zone.no_strips;i++)
	{
		landing_zone.strips[i].len = IBuf[Player[playerno].Buffer]->TrackData[offset];
		offset += 8;
		landing_zone.strips[i].v = &(IBuf[Player[playerno].Buffer]->TrackData[offset]);
		offset += (landing_zone.strips[i].len<<3);
		landing_zone.strips[i].n = &(IBuf[Player[playerno].Buffer]->TrackData[offset]);
		offset += (landing_zone.strips[i].len<<3);
		landing_zone.strips[i].c = &(IBuf[Player[playerno].Buffer]->TrackData[offset]);
		offset += (landing_zone.strips[i].len<<2);

		/*address=&IBuf[Player[playerno].Buffer]->CurrentStripPtr;
		address+=IBuf[Player[playerno].Buffer]->CurrentTexturePtr+(0*120);
		textya=(texttrackpoly *)address;*/
	}

}


void	DrawParachute(PlayerNumber,current_strip)
long	PlayerNumber;
long	current_strip;
{
	player	*Plyr;
	long	trigger_tag;
	long	arse;
	VECTOR	para_scale;
	SVECTOR	para_rot;
	objheader 	*parachute_object;
	incomingtrackbuffer *ITemp;

	Plyr = &(Player[PlayerNumber]);
	if(parachute_state == 0)
	{
		Plyr = &(Player[PlayerNumber]);
		trigger_tag = Plyr->Track->Meshes[current_strip].SplitTag>>16;
		if(trigger_tag == 7)
		{
			parachute_state = 1;
			//Plyr->JumpPower = CHFL( 9.5 );
			//Plyr->JumpD = 1;
			//Plyr->JumpY = CHFL( 0.0 );
			Plyr->Cam.Mode = CAMERA_PARACHUTE;
			Plyr->Cam.SubMode = PARA_LAUNCH;
			Plyr->Transport = SPACEHOPPER;
			Plyr->Style = PARA_STY_FLY;
			if( Plyr->Vel <= CHFL( 24.0 ) ) Plyr->Vel = CHFL( 24.0 );

		}
	}
	if(parachute_state)
	{
		parachute_counter++;

		switch(parachute_state)
		{
			case (1):	// do laly, just watch him go over the edge until his parachute is ready


				DrawParachuteFog(Plyr);
				if(parachute_counter >= 60)
				{
					parachute_state = 2;
				}
				break;
			case (2): // open parachute in y & continue falling until completely open
				parachute_object = &(ChillObjects[PARACHUTE_OBJECT_TYPE]);


				/*para_scale.vx = (parachute_counter - 48)<<8;
				para_scale.vy = (parachute_counter - 48)<<8;
				para_scale.vz = (parachute_counter - 48)<<8;*/

				para_scale.vx = 1024;
				para_scale.vy = (parachute_counter - 60)<<10;
				para_scale.vz = 1024;

				para_rot.vx = 0;
				para_rot.vy = 0;
				para_rot.vz = 0;

				RenderParachute(parachute_object,Plyr,&para_scale,&para_rot);
				DrawParachuteFog(Plyr);

				if(parachute_counter >= 64)
				{
					parachute_state = 3;
					//Plyr->JumpPower = 4096;
					//gravity = 128;
					//Plyr->Vel = 16000;
					//Plyr->Style = STY_NORMAL;

				}
				break;
			case (3): // open x and z almost instantly
				parachute_object = &(ChillObjects[PARACHUTE_OBJECT_TYPE]);
				para_scale.vx = 1024 + ((parachute_counter - 64)<<10);
				para_scale.vy = 4096;
				para_scale.vz = 1024 + ((parachute_counter - 64)<<10);
				para_rot.vx = 0;
				para_rot.vy = 0;
				para_rot.vz = 0;
				if(parachute_state == 67)
					para_rot.vz = 0;


				DrawParachuteFog(Plyr);
				RenderParachute(parachute_object,Plyr,&para_scale,&para_rot);
				if(parachute_counter >= 68)
				{
					parachute_state = 4;
					Plyr->Style = PARA_STY_AIR;
					Plyr->Grav = CHFL(1.0);
					//InitialiseSpaceHopper( PlayerNumber );
				}
				break;
			case (4): // fly parachute around for a bit

				parachute_object = &(ChillObjects[PARACHUTE_OBJECT_TYPE]);
				para_scale.vx = 4096;
				para_scale.vy = 4096;
				para_scale.vz = 4096;
				para_rot.vx = 0;
				para_rot.vy = 0;
				para_rot.vz = 0;

				DrawParachuteFog(Plyr);
				RenderParachute(parachute_object,Plyr,&para_scale,&para_rot);
				if(parachute_counter >= 128)
				{
					parachute_state = 5;
					InitCameraModel( PlayerNumber );

					Plyr->Track->NumberOfStrips = 0;
					//IBuf[ Plyr->Buffer ]->CurrentStrip = IBuf[ Plyr->Buffer ]->NumStrips;



/****************************************/
#ifndef CD

	  			IBuf[ Plyr->Buffer ]->CurrentStrip = IBuf[ Plyr->Buffer ]->NumStrips;

#endif


/**********************************************************************/




					//if(fileloaded[Plyr->Buffer] == 4)
					if(Plyr->Fakie)
					{
						Plyr->Fakie = 0;
						Plyr->Rot.Y += CHFL(2048);
						Plyr->Cam.SweepAngle = Plyr->Rot.Y;
					}
				}

				break;
			case (5):  // swap to top down view & fly around for a bit
				parachute_object = &(ChillObjects[PARACHUTE_OBJECT_TYPE]);
				/*para_scale.vx = 4096;
				para_scale.vy = 4096;
				para_scale.vz = 4096;*/

				para_scale.vx = 4096 - ((Plyr->Grav / 10));
				para_scale.vy = 4096;
				para_scale.vz = 4096 - ((Plyr->Grav / 10));


				para_rot.vx = 0;
				para_rot.vy = 0;
				para_rot.vz = 0;

				RenderParachute(parachute_object,Plyr,&para_scale,&para_rot);


				// grab details of last segment
				/*if(parachute_counter == 130)
					GetLastSegment(PlayerNumber);*/

//				if(Plyr->Pos.Y <= (-(1270)<<12))
				if(Plyr->Pos.Y <= -51507200)
				{
					parachute_state = 6;
					last_parachute_counter = parachute_counter;
				}
				break;
			case (6): // fold the parachute down, we have landed
				parachute_object = &(ChillObjects[PARACHUTE_OBJECT_TYPE]);
				para_scale.vx = 4096;
				para_scale.vy = 4096;
				para_scale.vz = 4096;

				para_rot.vx = -((parachute_counter - last_parachute_counter)<<5);
				para_rot.vy = 0;
				para_rot.vz = 0;
				RenderParachute(parachute_object,Plyr,&para_scale,&para_rot);
				if((parachute_counter - last_parachute_counter) >= 32)
				{
					parachute_state = 7;
					last_parachute_counter = parachute_counter;
				}
				break;
			case (7): // scale down the parchute in the z to flatten it to the snow
				Plyr->Vel = 0;
				parachute_object = &(ChillObjects[PARACHUTE_OBJECT_TYPE]);
				//para_scale.vx = 4096 - ((parachute_counter - last_parachute_counter)<<10);
				para_scale.vx = 4096;
				para_scale.vy = 4096;
				para_scale.vz = 4096 - ((parachute_counter - last_parachute_counter)<<10);

				para_rot.vx = -1024;
				para_rot.vy = 0;
				para_rot.vz = 0;
				RenderParachute(parachute_object,Plyr,&para_scale,&para_rot);
				if((parachute_counter - last_parachute_counter) >= 4)
				{
					parachute_state = 8;
					last_parachute_counter = parachute_counter;
				}
				break;
			case (8):
				Plyr->Vel = 0;
				parachute_object = &(ChillObjects[PARACHUTE_OBJECT_TYPE]);
				para_scale.vx = 4096;
				para_scale.vy = 4096;
				para_scale.vz = 0;

				para_rot.vx = -1024;
				para_rot.vy = 0;
				para_rot.vz = 0;
				RenderParachute(parachute_object,Plyr,&para_scale,&para_rot);
				if((parachute_counter - last_parachute_counter) > 256)
					parachute_state = 9;
				break;
			case (9):  /* can we go now jeb ? */

				break;

			default:
				break;
		}

	}
}

