/*******************************************************************************/
/*										*/
/*	movie.c:	RWB 							*/
/*										*/
/*			A bunch of functions which allow functionality for 	*/
/*			playing CD-XA films.					*/
/*			Currently assumes cd has been initialised, and plays    */
/*			films based on their actual file name on the CD.	*/
/*			NOTE: all films are currently 24-bit 25fps PAL 320x240  */
/*			   that is 4-sectors 4 film for every 1 sector of ADPCM	*/
/*										*/
/*******************************************************************************/

#include <stdio.h>
#include <sys/types.h>
#include <LibGTE.h>
#include <LibGPU.h>
#include <LibCD.h>
#include <LibEtc.h>
#include <LibMath.h>
#include <LibPress.h>
#include <LibSN.h>
#include <LibSnd.h>

#include "ourtypes.h"
#include "mathbits.h"
#include "tracks.h"
#include "movie.h"
#include "pads.h"
#include "cd.h"



volatile 	DECENV   dec;        /* environment for keeping track of what disp buffer, & what  */
static int  Rewind_Switch;
static int  StrWidth  = 0;  /* This is the width and height of each frame from the str header */
static int  StrHeight = 0;



short curr_frame = 0;
/*
 *  This is the streaming buffer in main ram
 *  the RING_SIZE is the number of cd sectors that the buffer takes up,
 *  and the sector size is the cd sector size which is defined in libcd.h = 2048
 */
/*static u_long   Ring_Buff[RING_SIZE*SECTOR_SIZE];*/
static u_long   *Ring_Buff;

/*
 *	Whenever a frame is de huffmanised from the ring buffer, one of these 2 buffers provides an output area
 */

/*static u_long   vlcbuf0[VLC_BUFF_SIZE];
static u_long   vlcbuf1[VLC_BUFF_SIZE];*/


static u_long   *vlcbuf0;
static u_long   *vlcbuf1;

/*static u_long   *vlcbuf0 = ANIM_PLYR_ADDR + 0x00010000;
static u_long   *vlcbuf1 = ANIM_PLYR_ADDR + 0x00010000 + 0x00028000;*/





/*
 *  these are 2 slices of the output image, 16 pixels wide.
 *  Only one strip is decoded at a time, and then loadimaged to vram.
 *
 */
static u_short  *imgbuf0;
static u_short  *imgbuf1;


unsigned long	*free_mem_start;



/*
 *
 *  (DecDCTout() callback function. When a new frame has been decompressed, this function is called)
 */
static void strCallback()
{
  	RECT snap_rect;
  	int  id;
	extern StCdIntrFlag;

	/* in 24-bit mode StCdIntrFlag has to be checked, reset, and StCdInterrupt has to be called each frame */
    	if(StCdIntrFlag)
	{
       	StCdInterrupt();
        	StCdIntrFlag = 0;
    	}

  	id = dec.imgid;
  	snap_rect = dec.slice;

    	dec.imgid = dec.imgid? 0:1;

    	dec.slice.x += dec.slice.w;

	/* if this slice doesn't go past the edge of the image, decompress it */
    	if (dec.slice.x < dec.rect[dec.rectid].x + dec.rect[dec.rectid].w)
			DecDCTout((unsigned long *)dec.imgbuf[dec.imgid], dec.slice.w*dec.slice.h/2);
    	else /* else tell the main loop that this whole frame has been decompressed */
	{
        	dec.isdone = 1;
        	dec.rectid = dec.rectid? 0: 1;
        	dec.slice.x = dec.rect[dec.rectid].x;
        	dec.slice.y = dec.rect[dec.rectid].y;
    	}

	LoadImage(&snap_rect, (u_long *)dec.imgbuf[id]);
}


/*
 *  CDROM: start the cd reading
 */
static void KickCD(CdlLOC *loc)
{
	long	i;

	i=0;

loop:
  /* seek to the start of the previously identified file at loc */
  while (CdControl(CdlSetloc, (u_char *)loc, 0) == 0);

    /* keep sending read commands to the cd until you get success */
    /* the mode is streaming, at 300k/s, including ADPCM sectors */
	if(CdRead2(CdlModeStream|CdlModeSpeed|CdlModeRT) == 0)
//	if(CdRead2(CdlModeStream|CdlModeSpeed) == 0)
    goto loop;

}

static void strNextVlc(unsigned long end_frame)
{
    /*int cnt = WAIT_TIME;*/
    int cnt = WAIT_TIME;
    u_long  *next;
    static u_long *strNext();

    while ((next = strNext(&dec,end_frame)) == 0) {
        if (--cnt == 0)
            return;
    }

    dec.vlcid = dec.vlcid? 0: 1;

    /*DecDCTvlc(next, dec.vlcbuf[dec.vlcid]);*/
	DecDCTvlc2(next,dec.vlcbuf[dec.vlcid] , free_mem_start);

    StFreeRing(next);

    return;
}

static u_long *strNext(DECENV *dec,unsigned long end_frame)
{
    u_long      *addr= 0x00000000;
    StHEADER    *sector = 0x00000000;
    int     cnt = WAIT_TIME;
	long	arse = 1;

	while(StGetNext((u_long **)&addr,(u_long **)&sector))
	{
		if (--cnt == 0)
       		return(0);
    }



    if(sector->frameCount >= end_frame)
       	Rewind_Switch = 1;

	if(StrWidth != sector->width || StrHeight != sector->height)
	{
	        RECT    rect;

		setRECT(&rect, 0, 0, SCR_WIDTH * PPW, SCR_HEIGHT);
	        ClearImage(&rect, 0, 0, 0);
       	StrWidth  = sector->width;
		StrHeight = sector->height;
    	}

 	dec->rect[0].w = dec->rect[1].w = StrWidth*PPW;
    	dec->rect[0].h = dec->rect[1].h = StrHeight;
    	dec->slice.h   = StrHeight;
    	return(addr);
}

static void strSync(int mode /* VOID */)
{
    volatile u_long cnt = WAIT_TIME;

    while (dec.isdone == 0) {
        if (--cnt == 0) {
#ifdef DEBUG
            printf("time out in decoding !\n");
#endif
            dec.isdone = 1;
            dec.rectid = dec.rectid? 0: 1;
            dec.slice.x = dec.rect[dec.rectid].x;
            dec.slice.y = dec.rect[dec.rectid].y;
        }
    }
    dec.isdone = 0;
}


void	InitMoviePlayer()
{
	/*free_mem_start = (unsigned long *)(0x80400000);*/


	free_mem_start = (unsigned long *)Ib1;

	dec.vlcbuf[0] = (unsigned long *)free_mem_start;
	/*free_mem_start = free_mem_start + VLC_BUFF_SIZE;*/
	free_mem_start = (unsigned long *)Ib1;
	free_mem_start+=45000;
    dec.vlcbuf[1] = free_mem_start;
	free_mem_start = free_mem_start + VLC_BUFF_SIZE;

    dec.vlcid     = 0;

	free_mem_start = (unsigned long *)cdbuffer;
	dec.Ring_Buff = (unsigned long *)free_mem_start;
	free_mem_start = free_mem_start + (RING_SIZE * SECTOR_SIZE);


    dec.imgbuf[0] = (unsigned short *)free_mem_start;
	free_mem_start = free_mem_start + SLICE_LONG_IMG_SIZE;
    dec.imgbuf[1] = (unsigned short *)free_mem_start;
	free_mem_start = free_mem_start + SLICE_LONG_IMG_SIZE;

    dec.imgid     = 0;


    dec.rect[0].x = 0;
    dec.rect[0].y = 0;
    dec.rect[1].x = 0;
    dec.rect[1].y = 256;
    dec.rectid    = 0;

    dec.rect[0].w = dec.rect[1].w = SCR_WIDTH*PPW;
    dec.rect[0].h = dec.rect[1].h = SCR_HEIGHT;
    dec.slice.h   = SCR_HEIGHT;

    dec.slice.x = 0;
    dec.slice.y = 0;
    dec.slice.w = 16*PPW;

    dec.isdone    = 0;



 	DecDCTReset(0);

	DecDCTvlcBuild(free_mem_start);



}


void	FreeMoviePlayer()
{
/*	free_mem_start = free_mem_start - SLICE_LONG_IMG_SIZE - SLICE_LONG_IMG_SIZE;
    dec.imgbuf[0] = dec.imgbuf[1] = NULL;
	free_mem_start = free_mem_start - VLC_BUFF_SIZE - VLC_BUFF_SIZE;
    dec.vlcbuf[0] = dec.vlcbuf[1] = NULL;
	free_mem_start = free_mem_start - (RING_SIZE * SECTOR_SIZE);
	dec.Ring_Buff = NULL;*/

    dec.vlcid     = 0;
    dec.imgid     = 0;

    dec.rect[0].x = 0;
    dec.rect[0].y = 0;
    dec.rect[1].x = 512;
    dec.rect[1].y = 0;
    dec.rectid    = 0;

    dec.rect[0].w = dec.rect[1].w = SCR_WIDTH*PPW;
    dec.rect[0].h = dec.rect[1].h = SCR_HEIGHT;
    dec.slice.h   = SCR_HEIGHT;

    dec.slice.x = 0;
    dec.slice.y = 0;
    dec.slice.w = 16*PPW;

    dec.isdone    = 0;

	/* the anim has stopped, because the full anim has been played */
   	DecDCToutCallback(0);

   	StUnSetRing();


}



int	CountFrames()
{
	curr_frame++;
}


int	EndFrames()
{
/*	printf("num of frames = %d\n",curr_frame);*/
}


long	PlayMovie(name,start_frame,end_frame)
char	*name;
unsigned long	start_frame,end_frame;
{
	DISPENV disp;
	CdlFILE	file;
	long	cnt = 16000;
	unsigned long	*addr;
	StHEADER	*sector;
	long id;
	RECT	clear_size;
	unsigned long	*ring_ptr;


	printf("playmovie called, movie name is %s\n",name);
	clear_size.x = clear_size.y = 0;
	clear_size.w = 1024;
	clear_size.h = 256;
	ClearImage(&clear_size, 0, 0, 0);
	DrawSync(0);		/* wait for the clear image */
	VSync(0);

	curr_frame = 0;
	/* Seek the cd to the start of the animation */
    if (CdSearchFile(&file, name) == NULL)
	{
#ifdef DEBUG
        	printf("file %s not found\n",name);
#endif
        	return 0;
	}
	printf("file size = %d\n",file.size);
	printf("sector no =%d\n",file.size>>14);


 	DecDCTReset(0);
   	DecDCToutCallback(strCallback);
   	StSetRing(dec.Ring_Buff, RING_SIZE);

   	StSetStream(1,start_frame , 0xffffffff, 0, 0);
/*    	StSetStream(1, START_FRAME, 0xffffffff, &CountFrames, &EndFrames);*/

#ifdef DEBUG
	pollhost();
#endif
	/* start the film */
	KickCD(&(file.pos));

	strNextVlc(end_frame);

	Rewind_Switch = 0;
#ifdef DEBUG
	pollhost();
#endif

	while(1)
	{
 		DecDCTin(dec.vlcbuf[dec.vlcid], DCT_MODE);
        DecDCTout((unsigned long *)dec.imgbuf[dec.imgid], dec.slice.w*dec.slice.h/2);
		cnt = 16000;
		strNextVlc(end_frame);

		/* wait a while for the last dct to have happened before swapping display buffers */
		/* if it takes too long, reset the stuff and carry on to the next frame */
	        strSync(0);

		VSync(2);
		VSync(2);

		/* sort out what display buffer to use */
		id = dec.rectid? 0: 1;
	    /*SetDefDispEnv(&disp, (id)*512, 0, SCR_WIDTH, SCR_HEIGHT);*/
	    SetDefDispEnv(&disp, 0, (id)*256, SCR_WIDTH, SCR_HEIGHT);
	    disp.isrgb24 = 1;
		PutDispEnv(&disp);
	    SetDispMask(1);
		Pad_Update();
	    if((Pad_Debounced(0) & PAD_START)||(Pad_Debounced(0) & PAD_B2))
		{
			printf("pressed\n");
			break;
		}


	    if(Rewind_Switch == 1)
 			break;
#ifdef DEBUG
	pollhost();
#endif

	}

    CdControlB(CdlPause,0,0);

	clear_size.x = clear_size.y = 0;
	clear_size.w = 340;
	clear_size.h = 512;
	ClearImage(&clear_size, 0, 0, 0);
	DrawSync(0);		/* wait for the clear image */
	VSync(0);
	return(1);
}






