#include <stdio.h>
#include <sys/types.h>
#include <LibGTE.h>
#include <LibGPU.h>
#include <LibCD.h>
#include <LibComb.h>
#include <LibEtc.h>
#include <LibGS.h>
#include <LibMath.h>
#include <LibPress.h>
#include <LibSN.h>
#include <LibSnd.h>
#include <LibSPU.h>
#include <inline_o.h>
#include <gtemac.h>


#include "ourtypes.h"
#include "mathbits.h"

#include "main.h"
#include "skeleton.h"
#include "camera.h"
#include "tracks.h"
#include "players.h"
#include "pads.h"

#include "screen.h"
#include "who.h"
#include "bored.h"
#include "animctrl.h"

#include "bonebag.h"
#include "trixbag.h"
#include "rotmesh.h"
#include "rotmeshs.h"
#include "avalanch.h"
#include "parachut.h"


long	start_cell = 3;
long	end_cell = 6;
MATRIX	cam,rev,colourMatrix,lightMatrix;
RLIGHT snow_lights[3];
long	no_divided_polys;
long	player_strip[2];
long	player_cell[2];

long	buffer_filenumbers[2][2];	// one for each player & one for each buffer	


/**************************************************************************************/

long	GetIcy(PlayerNumber)
long	PlayerNumber;
{
	long	return_val;
	unsigned long trigger_tag;

	trigger_tag = Player[PlayerNumber].Track->Meshes[player_strip[PlayerNumber]].SplitTag>>16;
	switch(trigger_tag)
	{
		case (8):
			if((player_cell[PlayerNumber] >= 3) && 
				(player_cell[PlayerNumber] <= 7))
				return_val = 7000; 
			else
				return_val = 0;
			break;
		case (9):
			if((player_cell[PlayerNumber] >= 4) && 
				(player_cell[PlayerNumber] <= 8))
				return_val = 7000; 
			else
				return_val = 0;
			break;
		case (10):
			if((player_cell[PlayerNumber] >= 14) && 
				(player_cell[PlayerNumber] <= 18))
				return_val = 7000; 
			else
				return_val = 0;
			break;
		case (11):
			if((player_cell[PlayerNumber] >= 4) && 
				(player_cell[PlayerNumber] <= 18))
				return_val = 7000; 
			else
				return_val = 0;
			break;
		default:
				return_val = 0;
			break;

	}

	return(return_val);
}




/*************************************************************************************/

void	RMGetPlayerStrip(PlayerNumber)
long	PlayerNumber;
{
	player	*Plyr;
	long	old_collision,arse;

	Plyr = &(Player[PlayerNumber]);

	// gotta do this cos CPoly[?] is not updated when the collision = TOP or RAILSLIDE
	if ((thiscollision[Plyr->Buffer].type==TOP)||(thiscollision[Plyr->Buffer].type==RAILSLIDE))
	{	
		old_collision = thiscollision[Plyr->Buffer].type;
		thiscollision[Plyr->Buffer].type = NONE;
		arse = GetYAt( Plyr->Pos.X, Plyr->Pos.Z, PlayerNumber );
		thiscollision[Plyr->Buffer].type = old_collision;
	}
	else
		arse = GetYAt( Plyr->Pos.X, Plyr->Pos.Z, PlayerNumber );
	player_strip[PlayerNumber] = CPoly[PlayerNumber];
}

/*************************************************************************************/

long	ApproxMagnitude(distance)
VECTOR	*distance;
{
	long	max,med,min;

	max = abs(distance->vx);
	med = abs(distance->vy);
	min = abs(distance->vz);
	if(max<med)
	{
		/* swap max & med */
		max ^= med;
		med ^= max;
		max ^= med;
	}
	if(max<min)
	{
	 	/* swap max & min */
		max ^= min;
		min ^= max;
		max ^= min;
	}
	med = med + min;
	max = max + (med>>2);
	return(max);
}

long	ApproxMagnitudeS(distance)
SVECTOR	*distance;
{
	long	max,med,min;

	max = abs(distance->vx);
	med = abs(distance->vy);
	min = abs(distance->vz);
	if(max<med)
	{
		/* swap max & med */
		max ^= med;
		med ^= max;
		max ^= med;
	}
	if(max<min)
	{
	 	/* swap max & min */
		max ^= min;
		min ^= max;
		max ^= min;
	}
	med = med + min;
	max = max + (med>>2);
	return(max);
}




/********************************************************************************************************/
/*	CalcRotsMats:	mat = matrix to get eye vec back to z-axis and up vec back to y-axis		*/
/*			rev_mat = matrix for reverse operation						*/
/*******************************************************************************************************/




void	CalcRotsMats(eye,up,mat,rev_mat)
SVECTOR	*eye;
SVECTOR	*up;
MATRIX	*mat;
MATRIX	*rev_mat;
{
	SVECTOR	P2,P3;
	long    xdiff,ydiff,zdiff;
	long	D1,D2,D3;
	MATRIX	mat0,mat1,mat2;
	MATRIX	rev_mat0,rev_mat1,rev_mat2;
	VECTOR	clear;
	SVECTOR	sclear;
	VECTOR	P21,P31,P22,P32;
	long	flag;
	SVECTOR	SP2,SP3;	/* short versions for rot trans */

	clear.vx = clear.vy = clear.vz = 0;
	sclear.vx = sclear.vy = sclear.vz = 0;

	/* step 2 , rotate about y axis so P1 (scumbo_cam.location) --> P2 lies in yz-plane */

	P2.vx = eye->vx;
	P2.vy = eye->vy;
	P2.vz = eye->vz;

	P3.vx = up->vx;
	P3.vy = up->vy;
	P3.vz = up->vz;

	zdiff = P2.vz;
	xdiff = P2.vx;
	/*if(zdiff < 0)
		zdiff = -zdiff;*/
	D1 = ((zdiff) * (zdiff)) + ((xdiff) * (xdiff));
	D1 = csqrt(D1);
	D1 = D1>>6;		/* cos csqrt returns 64 * the sqrt */
	if(D1 == 0)
	{

		TransMatrix(&mat0,&clear);
		RotMatrix(&sclear,&mat0);

		TransMatrix(&rev_mat0,&clear);
		RotMatrix(&sclear,&rev_mat0);
	}
	else
	{
		mat0.m[0][1] = mat0.m[1][0] = mat0.m[1][2] = mat0.m[2][1] = 0;
		mat0.m[1][1] = 4096;
		mat0.m[0][0] = -((short)((-zdiff * 4096) / D1));
		mat0.m[2][2] = -((short)((-zdiff * 4096) / D1));
		mat0.m[0][2] = ((short)((-xdiff * 4096) / D1));
		mat0.m[2][0] = -((short)((-xdiff * 4096) / D1));		/* T2 calc'd */
		TransMatrix(&mat0,&clear);

		rev_mat0.m[0][1] = rev_mat0.m[1][0] = rev_mat0.m[1][2] = rev_mat0.m[2][1] = 0;
		rev_mat0.m[1][1] = 4096;
		rev_mat0.m[0][0] = -((short)((-zdiff * 4096) / D1));
		rev_mat0.m[2][2] = -((short)((-zdiff * 4096) / D1));
		rev_mat0.m[0][2] = ((short)((xdiff * 4096) / D1));
		rev_mat0.m[2][0] = -((short)((xdiff * 4096) / D1)); 		/* rev T2 calc'd */
		TransMatrix(&rev_mat0,&clear);
	}

	/* do rt on eye_vec & up_vec to get P2' & P3' */

	SetTransMatrix(&mat0);
	SetRotMatrix(&mat0);
	RotTrans(&P2,&P21,&flag);
	RotTrans(&P3,&P31,&flag);


	/* step 3 , rotate about x axis so P1 (scumbo_cam.location) --> P2 lies on negative z-plane */


	xdiff = P21.vx;
	ydiff = P21.vy;
	zdiff = P21.vz;
	D2 = (xdiff * xdiff) + (ydiff * ydiff) + (zdiff * zdiff);
	D2 = csqrt(D2);
	D2 = D2>>6;		/* cos csqrt returns 64 * the sqrt */

	mat1.m[0][0] = 4096;
	mat1.m[0][1] = mat1.m[0][2] = mat1.m[1][0] = mat1.m[2][0] = 0;
	mat1.m[1][1] = mat1.m[2][2] = ((short)((zdiff * 4096) / D2));
	mat1.m[1][2] = -((short)((ydiff * 4096) / D2));
	mat1.m[2][1] = ((short)((ydiff * 4096) / D2));		/* T3 calc'd */
	TransMatrix(&mat1,&clear);


	rev_mat1.m[0][0] = 4096;
	rev_mat1.m[0][1] = rev_mat1.m[0][2] = rev_mat1.m[1][0] = rev_mat1.m[2][0] = 0;
	rev_mat1.m[1][1] = rev_mat1.m[2][2] = ((short)((zdiff * 4096) / D2));
	rev_mat1.m[1][2] = -((short)((-ydiff * 4096) / D2));
	rev_mat1.m[2][1] = ((short)((-ydiff * 4096) / D2));		/* rev T3 calc'd */
	TransMatrix(&rev_mat1,&clear);


	/* do rt on (T2'd eye_vec & up_vec) to get P2'' & P3'' */

	SP2.vx = (short)P21.vx;
	SP2.vy = (short)P21.vy;
	SP2.vz = (short)P21.vz;

	SP3.vx = (short)P31.vx;
	SP3.vy = (short)P31.vy;
	SP3.vz = (short)P31.vz;

	/* do rt on P2' & P3' to get P2'' & P3'' */

	SetTransMatrix(&mat1);
	SetRotMatrix(&mat1);
	RotTrans(&SP2,&P22,&flag);
	RotTrans(&SP3,&P32,&flag);

	/* step 4 , rotate about z axis so P1 (scumbo_cam.location) --> P3 lies on yz-plane in the pos y */

	ydiff = P32.vy;
	xdiff = P32.vx;
	D3 = (xdiff * xdiff) + (ydiff * ydiff);
	D3 = csqrt(D3);
	D3 = D3>>6;		/* cos csqrt returns 64 * the sqrt */

	mat2.m[0][2] = mat2.m[1][2] = mat2.m[2][0] = mat2.m[2][1] = 0;
	mat2.m[2][2] = 4096;
	mat2.m[0][0] = mat2.m[1][1] = (short)((ydiff * 4096) / D3);
	mat2.m[0][1] = -(short)((xdiff * 4096) / D3);
	mat2.m[1][0] = (short)((xdiff * 4096) / D3);
	TransMatrix(&mat2,&clear);				/* T4 calc'd */

	rev_mat2.m[0][2] = rev_mat2.m[1][2] = rev_mat2.m[2][0] = rev_mat2.m[2][1] = 0;
	rev_mat2.m[2][2] = 4096;
	rev_mat2.m[0][0] = rev_mat2.m[1][1] = (short)((ydiff * 4096) / D3);
	rev_mat2.m[0][1] = -(short)((-xdiff * 4096) / D3);
	rev_mat2.m[1][0] = (short)((-xdiff * 4096) / D3);
	TransMatrix(&rev_mat2,&clear);			       /* rev T4 calc'd */


	MulMatrix(&mat1,&mat0);
	MulMatrix0(&mat2,&mat1,mat);
	TransMatrix(mat,&clear);				/* clear the translation stuff */
	MulMatrix(&rev_mat1,&rev_mat2);
	MulMatrix0(&rev_mat0,&rev_mat1,rev_mat);
	TransMatrix(mat,&clear);				/* clear the translation stuff */
	TransMatrix(rev_mat,&clear);				/* clear the translation stuff */

}

/*************************************************************/

void	DoCamMats(Plyr)
player	*Plyr;
{
	VECTOR	Z,clear;
	SVECTOR	up,eye;
	VECTOR	CamScale={12288,12288,12288};

	PushMatrix();

	clear.vx = clear.vy = clear.vz = 0;

	Z.vx = CHSHIFTR( Plyr->Cam.Targ.X ) - CHSHIFTR( Plyr->Cam.Pos.X );
	Z.vy = CHSHIFTR( Plyr->Cam.Targ.Y ) - CHSHIFTR( Plyr->Cam.Pos.Y );
	Z.vz = CHSHIFTR( Plyr->Cam.Targ.Z ) - CHSHIFTR( Plyr->Cam.Pos.Z );

	VectorNormalS(&Z,&eye);

	up.vx = 0;
	up.vy = -4096;
	up.vz = 0;

	CalcRotsMats(&eye,&up,&cam,&rev);
	ScaleMatrix(&cam,&CamScale);
	ScaleMatrix(&rev,&CamScale);
	PopMatrix();

}



long	maxz(long z1,long z2,long z3)
{
	long	return_z;

	if(z1>z2)
	{
		if(z1>z3)
			return_z=z1;
		else
			return_z=z3;
	}
	else
	{
		if(z2>z3)
			return_z=z2;
		else
			return_z=z3;
	}
	return(return_z);
}

/********************************************/
// RussRotMeshG3:
//		Written to replace RotMeshPrimS_GC3()
//		aim is to do proper depth sorting
/*******************************************/

void	RussRotMeshG3(TMESH *mesh,POLY_G3 *polys,unsigned long *ot,long stripno)
{
	long	z,p,temp;
	SVECTOR	*vert_ptr;
	RUSS_SVECTOR	*sxyz,*end_sxyz;
	CVECTOR			*cols;

	/* do polys */
	sxyz = (RUSS_SVECTOR *)(0x1f800000);
	end_sxyz = (sxyz + mesh->len - 2);
	cols = mesh->c - 2;
	vert_ptr = mesh->v;
	while(sxyz != end_sxyz)
	{
		gte_ldv3(vert_ptr,vert_ptr+1,vert_ptr+2);
	    gte_rtpt();
		vert_ptr+=2;
    	gte_stsxy3(sxyz,sxyz+1,sxyz+2);
   	    gte_stdp(&p);
		gte_stszotz(&z);

		gte_ldv0(vert_ptr+1);
	    gte_rtps();
 		cols+=2;
    	gte_stsxy(sxyz+3);

		if(z < OTLENGTH-2)
		{
			if(z > MESH_NEAR_Z)
			{
				gte_NormalClip(*(long *)(sxyz),*(long*)(sxyz+1),*(long*)(sxyz+2),&temp);
				if(temp >= 0 )
				{
 					gte_ldrgb3(cols,cols+1,cols+2);
					gte_lddp(p);
					gte_dpct();
					LONG_MOVE(&(sxyz->vx),&polys->x0);
					LONG_MOVE(&((sxyz+1)->vx),&polys->x1);
					LONG_MOVE(&((sxyz+2)->vx),&polys->x2);
					gte_strgb3(&((polys+0)->r0),&((polys+0)->r1),&((polys+0)->r2));


					setPolyG3(polys);
				   	addPrim((ot+z),polys);
				}
				gte_NormalClip(*(long *)(sxyz+1),*(long*)(sxyz+3),*(long*)(sxyz+2),&temp);
		 		if(temp >= 0 )
			 	{

					gte_ldrgb3(cols+2,cols+1,cols+3);
					gte_lddp(p);
					gte_dpct();
					LONG_MOVE(&((sxyz+2)->vx),&(polys+1)->x0);
					LONG_MOVE(&((sxyz+1)->vx),&(polys+1)->x1);
					LONG_MOVE(&((sxyz+3)->vx),&(polys+1)->x2);
					gte_strgb3(&((polys+1)->r0),&((polys+1)->r1),&((polys+1)->r2));

					/*if(stripno == strip_affected)
					{

						setRGB0(polys+1,0xff,0x00,0x00);
						setRGB1(polys+1,0xff,0x00,0x00);
						setRGB2(polys+1,0xff,0x00,0x00);

					} */

					setPolyG3(polys+1);
					addPrim((ot+z),polys+1);

				}
			}
		}
	 	polys+=2;
		sxyz+=2;
	}
}

/********************************************************************************/

void	RussRotMeshG3maxz(TMESH *mesh,POLY_G3 *polys,unsigned long *ot,long stripno)
{
	long	z,p,temp,z0,z1,z2,z3,z_max,cell,icy,col;
	SVECTOR	*vert_ptr;
	RUSS_SVECTOR	*sxyz,*end_sxyz;
	CVECTOR			*cols;
	SVECTOR			*normal_ptr;
	CVECTOR			col0,col1,col2,col3;


	icy = 0;

	if( (striptab[0][stripno].fileno >= 26) &&
		(striptab[0][stripno].fileno <= 28))
		icy = 1;

	/* do polys */
	cell = 0;

	sxyz = (RUSS_SVECTOR *)(0x1f800000);
	end_sxyz = (sxyz + mesh->len - 2);
	cols = mesh->c;
	vert_ptr = mesh->v;
	normal_ptr = mesh->n;

	/* do 1st 2 verts */

	gte_ldv0(vert_ptr);
    gte_rtps();
    gte_stsxy(sxyz);
 	gte_stszotz(&z);
	sxyz->vz = z;

	gte_ldv0(normal_ptr);
	gte_ldrgb(cols);
	gte_ncds();
	gte_strgb(&col0);

	gte_ldv0(vert_ptr+1);
    gte_rtps();
    gte_stsxy(sxyz+1);
 	gte_stszotz(&z);
	(sxyz+1)->vz = z;

	gte_ldv0(normal_ptr+1);
	gte_ldrgb(cols+1);
	gte_ncds();
	gte_strgb(&col1);


	while(sxyz != end_sxyz)
	{
		gte_ldv0(vert_ptr+2);
	    gte_rtps();
    	gte_stsxy(sxyz+2);
	 	gte_stszotz(&z);
		(sxyz+2)->vz = z;

		gte_ldv0(normal_ptr+2);
		
		if((cell > start_cell) && (cell < end_cell) && (icy))
		{
			LONG_MOVE(cols+2,&col2);
			col2.r = col2.r - 24;

			col2.r = 0;
			col2.g = 0;
			col2.b = 0;
	
			/*col = (long)(col2.g + 24);
			if(col > 255)
				col = 255;
			col2.g = col;

			col = (long)(col2.b + 24);
			if(col > 255)
				col = 255;
			col2.b = col;*/
			gte_ldrgb(&col2);
		}
		else
			gte_ldrgb(cols+2);

		gte_ncds();
		gte_strgb(&col2);

		gte_ldv0(vert_ptr+3);
	    gte_rtps();
    	gte_stsxy(sxyz+3);
        gte_stdp(&p);
		gte_stszotz(&z);
		(sxyz+3)->vz = z;

		gte_ldv0(normal_ptr+3);
		if((cell > start_cell) && (cell < end_cell) && (icy))
		{
			LONG_MOVE(cols+3,&col3);
			col3.r = col3.r - 24;
			
			col3.r = 0;
			col3.g = 0;
			col3.b = 0;
			/*col = (long)(col3.g + 24);
			if(col > 255)
				col = 255;
			col3.g = col;
	
			col = (long)(col3.b + 24);
			if(col > 255)
				col = 255;
			col3.b = col;*/
			gte_ldrgb(&col3);
		}
		else
			gte_ldrgb(cols+3);
		
		gte_ncds();
		gte_strgb(&col3);


		z0 = (sxyz+0)->vz;
		z1 = (sxyz+1)->vz;
		z2 = (sxyz+2)->vz;
		z3 = (sxyz+3)->vz;

		if(z1 > z0)
			z0 = z1;
		if(z3 > z2)
			z2 = z3;

		if(z0 > z2)
		   	z_max = z0;
		else
			z_max = z2;


		if(z_max < OTLENGTH-2)
		{
			if(z_max > MESH_NEAR_Z)
			{
				gte_NormalClip(*(long *)(sxyz),*(long*)(sxyz+1),*(long*)(sxyz+2),&temp);
				if(temp >= 0 )
				{

					LONG_MOVE(&(sxyz->vx),&polys->x0);
					LONG_MOVE(&((sxyz+1)->vx),&polys->x1);
					LONG_MOVE(&((sxyz+2)->vx),&polys->x2);
					LONG_MOVE(&col0,&polys->r0);
					LONG_MOVE(&col1,&polys->r1);
					LONG_MOVE(&col2,&polys->r2);


					setPolyG3(polys);
				   	addPrim((ot+z_max),polys);
				}
				gte_NormalClip(*(long *)(sxyz+1),*(long*)(sxyz+3),*(long*)(sxyz+2),&temp);
		 		if(temp >= 0 )
			 	{

					LONG_MOVE(&((sxyz+2)->vx),&(polys+1)->x0);
					LONG_MOVE(&((sxyz+1)->vx),&(polys+1)->x1);
					LONG_MOVE(&((sxyz+3)->vx),&(polys+1)->x2);

					LONG_MOVE(&col2,&(polys+1)->r0);
					LONG_MOVE(&col1,&(polys+1)->r1);
					LONG_MOVE(&col3,&(polys+1)->r2);

					setPolyG3(polys+1);
					addPrim((ot+z_max),polys+1);
				}
			}
		}
	 	polys+=2;
		sxyz+=2;
 		cols+=2;
		vert_ptr+=2;
		normal_ptr+=2;
		LONG_MOVE(&col2,&col0);
		LONG_MOVE(&col3,&col1);
		cell++;
	}
}


void	RussRotMeshG3maxzPara(TMESH *mesh,POLY_G3 *polys,unsigned long *ot,long stripno)
{
	long	z,p,temp,z0,z1,z2,z3,z_max;
	SVECTOR	*vert_ptr;
	RUSS_SVECTOR	*sxyz,*end_sxyz;
	CVECTOR			*cols;
	SVECTOR			*normal_ptr;
	CVECTOR			col0,col1,col2,col3;

	/* do polys */
	sxyz = (RUSS_SVECTOR *)(0x1f800000);
	end_sxyz = (sxyz + mesh->len - 2);
	cols = mesh->c;
	vert_ptr = mesh->v;
	normal_ptr = mesh->n;

	/* do 1st 2 verts */

	gte_ldv0(vert_ptr);
    gte_rtps();
    gte_stsxy(sxyz);
 	gte_stszotz(&z);
	sxyz->vz = z;

	gte_ldv0(normal_ptr);
	gte_ldrgb(cols);
	gte_ncds();
	gte_strgb(&col0);

	gte_ldv0(vert_ptr+1);
    gte_rtps();
    gte_stsxy(sxyz+1);
 	gte_stszotz(&z);
	(sxyz+1)->vz = z;

	gte_ldv0(normal_ptr+1);
	gte_ldrgb(cols+1);
	gte_ncds();
	gte_strgb(&col1);



	while(sxyz != end_sxyz)
	{
		gte_ldv0(vert_ptr+2);
	    gte_rtps();
    	gte_stsxy(sxyz+2);
	 	gte_stszotz(&z);
		(sxyz+2)->vz = z;

		gte_ldv0(normal_ptr+2);
		gte_ldrgb(cols+2);
		gte_ncds();
		gte_strgb(&col2);

		gte_ldv0(vert_ptr+3);
	    gte_rtps();
    	gte_stsxy(sxyz+3);
        gte_stdp(&p);
		gte_stszotz(&z);
		(sxyz+3)->vz = z;

		gte_ldv0(normal_ptr+3);
		gte_ldrgb(cols+3);
		gte_ncds();
		gte_strgb(&col3);


		z0 = (sxyz+0)->vz;
		z1 = (sxyz+1)->vz;
		z2 = (sxyz+2)->vz;
		z3 = (sxyz+3)->vz;

		if(z1 > z0)
			z0 = z1;
		if(z3 > z2)
			z2 = z3;

		if(z0 > z2)
		   	z_max = z0;
		else
			z_max = z2;


		if(z_max > MESH_NEAR_Z)
		{
			z_max = OTLENGTH-2;
			gte_NormalClip(*(long *)(sxyz),*(long*)(sxyz+1),*(long*)(sxyz+2),&temp);
			if(temp >= 0 )
			{
				LONG_MOVE(&(sxyz->vx),&polys->x0);
				LONG_MOVE(&((sxyz+1)->vx),&polys->x1);
				LONG_MOVE(&((sxyz+2)->vx),&polys->x2);
				LONG_MOVE(&col0,&polys->r0);
				LONG_MOVE(&col1,&polys->r1);
				LONG_MOVE(&col2,&polys->r2);

				setPolyG3(polys);
			   	addPrim((ot+z_max),polys);
			}
			gte_NormalClip(*(long *)(sxyz+1),*(long*)(sxyz+3),*(long*)(sxyz+2),&temp);
	 		if(temp >= 0 )
		 	{
				LONG_MOVE(&((sxyz+2)->vx),&(polys+1)->x0);
				LONG_MOVE(&((sxyz+1)->vx),&(polys+1)->x1);
				LONG_MOVE(&((sxyz+3)->vx),&(polys+1)->x2);

				LONG_MOVE(&col2,&(polys+1)->r0);
				LONG_MOVE(&col1,&(polys+1)->r1);
				LONG_MOVE(&col3,&(polys+1)->r2);

				setPolyG3(polys+1);
				addPrim((ot+z_max),polys+1);
			}
		}
	 	polys+=2;
		sxyz+=2;
 		cols+=2;
		vert_ptr+=2;
		normal_ptr+=2;
		LONG_MOVE(&col2,&col0);
		LONG_MOVE(&col3,&col1);
	}
}




/********************************************/
// RussRotMeshG3Multi:
//		Written to replace RussRotMeshG3()
//		aim is to render multiple strips
/*******************************************/


void	RussDivideG3(sxyz0,sxyz1,sxyz2,cols0,cols1,cols2,polys,ot,ndiv)
DVECTOR	sxyz0,sxyz1,sxyz2;
CVECTOR	cols0,cols1,cols2;
POLY_G3	**polys;
unsigned long *ot;
long	ndiv;
{
	DVECTOR	sxyz3,sxyz4,sxyz5;
	CVECTOR	cols3,cols4,cols5;

	long	col_diff,long_col0,long_col1;

	/* do mid verts  */

	sxyz3.vx = sxyz0.vx + ((sxyz1.vx - sxyz0.vx)>>1);
	sxyz3.vy = sxyz0.vy + ((sxyz1.vy - sxyz0.vy)>>1);

	sxyz4.vx = sxyz2.vx + ((sxyz1.vx - sxyz2.vx)>>1);
	sxyz4.vy = sxyz2.vy + ((sxyz1.vy - sxyz2.vy)>>1);

	sxyz5.vx = sxyz0.vx + ((sxyz2.vx - sxyz0.vx)>>1);
	sxyz5.vy = sxyz0.vy + ((sxyz2.vy - sxyz0.vy)>>1);

	/* do mid cols */

	LONG_MOVE(&cols0,&long_col0);
	LONG_MOVE(&cols1,&long_col1);
	long_col0 = long_col0 & 0x00fefeff;
	long_col1 = long_col1 & 0x00fefeff;
	col_diff = long_col0 + long_col1;
	col_diff = col_diff>>1;
	LONG_MOVE(&col_diff,&cols3);


	LONG_MOVE(&cols2,&long_col0);
	LONG_MOVE(&cols1,&long_col1);
	long_col0 = long_col0 & 0x00fefeff;
	long_col1 = long_col1 & 0x00fefeff;
	col_diff = long_col0 + long_col1;
	col_diff = col_diff>>1;
	LONG_MOVE(&col_diff,&cols4);

	LONG_MOVE(&cols0,&long_col0);
	LONG_MOVE(&cols2,&long_col1);
	long_col0 = long_col0 & 0x00fefeff;
	long_col1 = long_col1 & 0x00fefeff;
	col_diff = long_col0 + long_col1;
	col_diff = col_diff>>1;
	LONG_MOVE(&col_diff,&cols5);

	ndiv--;
	if(ndiv == 0)
	{
		/* do poly 0 */

		LONG_MOVE(&sxyz0,&((*polys)->x0));
		LONG_MOVE(&sxyz3,&((*polys)->x1));
		LONG_MOVE(&sxyz5,&((*polys)->x2));
		LONG_MOVE(&cols0,&((*polys)->r0));
		LONG_MOVE(&cols3,&((*polys)->r1));
		LONG_MOVE(&cols5,&((*polys)->r2));

		setPolyG3(*polys);
		addPrim(ot,*polys);
		(*polys)++;

	/* do poly 1 */

		LONG_MOVE(&sxyz5,&((*polys)->x0));
		LONG_MOVE(&sxyz4,&((*polys)->x1));
		LONG_MOVE(&sxyz2,&((*polys)->x2));
		LONG_MOVE(&cols5,&((*polys)->r0));
		LONG_MOVE(&cols4,&((*polys)->r1));
		LONG_MOVE(&cols2,&((*polys)->r2));

		setPolyG3(*polys);
		addPrim(ot,*polys);
		(*polys)++;


	/* do poly 2 */

		LONG_MOVE(&sxyz5,&((*polys)->x0));
		LONG_MOVE(&sxyz3,&((*polys)->x1));
		LONG_MOVE(&sxyz4,&((*polys)->x2));
		LONG_MOVE(&cols5,&((*polys)->r0));
		LONG_MOVE(&cols3,&((*polys)->r1));
		LONG_MOVE(&cols4,&((*polys)->r2));

		setPolyG3(*polys);
		addPrim(ot,*polys);
		(*polys)++;


	/* do poly 3 */

		LONG_MOVE(&sxyz3,&((*polys)->x0));
		LONG_MOVE(&sxyz1,&((*polys)->x1));
		LONG_MOVE(&sxyz4,&((*polys)->x2));
		LONG_MOVE(&cols3,&((*polys)->r0));
		LONG_MOVE(&cols1,&((*polys)->r1));
		LONG_MOVE(&cols4,&((*polys)->r2));

		setPolyG3(*polys);
		addPrim(ot,*polys);
		(*polys)++;
	}
	else
	{
		RussDivideG3(sxyz0,sxyz3,sxyz5,cols0,cols3,cols5,polys,ot,ndiv);
		RussDivideG3(sxyz5,sxyz4,sxyz2,cols5,cols4,cols2,polys,ot,ndiv);
		RussDivideG3(sxyz5,sxyz3,sxyz4,cols5,cols3,cols4,polys,ot,ndiv);
		RussDivideG3(sxyz3,sxyz1,sxyz4,cols3,cols1,cols4,polys,ot,ndiv);
	}
}

/*void	RussDivideG3by2(sxyz0,sxyz1,sxyz2,cols0,cols1,cols2,polys,ot)
DVECTOR	sxyz0,sxyz1,sxyz2;
CVECTOR	cols0,cols1,cols2;
POLY_G3	**polys;
unsigned long *ot;
{
	DVECTOR	sxyz3,sxyz4,sxyz5,sxyz6,sxyz7,sxyz8,sxyz9,sxyz10,sxyz11,sxyz12,sxyz13,sxyz14,half_diff,quarter_diff;
	CVECTOR	cols3,cols4,cols5,cols6,cols7,cols8,cols9,cols10,cols11,cols12,cols13,cols14;

	DVECTOR	new_sxy;
	long	col_diff,col_half_diff,col_quart_diff,long_col0,long_col1,long_vert0,long_vert1,vert_diff;
	CVECTOR	new_col;
	unsigned long	temp0,temp1;


	quarter_diff.vx = (sxyz1.vx - sxyz0.vx)>>2;
	quarter_diff.vy = (sxyz1.vy - sxyz0.vy)>>2;




	half_diff.vx = (sxyz1.vx - sxyz0.vx)>>1;
	half_diff.vy = (sxyz1.vy - sxyz0.vy)>>1;

	sxyz4.vx = sxyz0.vx + quarter_diff.vx;
	sxyz4.vy = sxyz0.vy + quarter_diff.vy;

	sxyz7.vx = sxyz0.vx + half_diff.vx;
	sxyz7.vy = sxyz0.vy + half_diff.vy;

	sxyz11.vx = sxyz1.vx - quarter_diff.vx;
	sxyz11.vy = sxyz1.vy - quarter_diff.vy;

	quarter_diff.vx = (sxyz2.vx - sxyz0.vx)>>2;
	quarter_diff.vy = (sxyz2.vy - sxyz0.vy)>>2;

	half_diff.vx = (sxyz2.vx - sxyz0.vx)>>1;
	half_diff.vy = (sxyz2.vy - sxyz0.vy)>>1;

	sxyz3.vx = sxyz0.vx + quarter_diff.vx;
	sxyz3.vy = sxyz0.vy + quarter_diff.vy;

	sxyz5.vx = sxyz0.vx + half_diff.vx;
	sxyz5.vy = sxyz0.vy + half_diff.vy;

	sxyz8.vx = sxyz2.vx - quarter_diff.vx;
	sxyz8.vy = sxyz2.vy - quarter_diff.vy;

	quarter_diff.vx = (sxyz1.vx - sxyz2.vx)>>2;
	quarter_diff.vy = (sxyz1.vy - sxyz2.vy)>>2;

	half_diff.vx = (sxyz1.vx - sxyz2.vx)>>1;
	half_diff.vy = (sxyz1.vy - sxyz2.vy)>>1;

	sxyz12.vx = sxyz2.vx + quarter_diff.vx;
	sxyz12.vy = sxyz2.vy + quarter_diff.vy;

	sxyz13.vx = sxyz2.vx + half_diff.vx;
	sxyz13.vy = sxyz2.vy + half_diff.vy;

	sxyz14.vx = sxyz1.vx - quarter_diff.vx;
	sxyz14.vy = sxyz1.vy - quarter_diff.vy;

	half_diff.vx = (sxyz7.vx - sxyz5.vx)>>1;
	half_diff.vy = (sxyz7.vy - sxyz5.vy)>>1;

	sxyz6.vx = sxyz5.vx + half_diff.vx;
	sxyz6.vy = sxyz5.vy + half_diff.vy;

	half_diff.vx = (sxyz7.vx - sxyz13.vx)>>1;
	half_diff.vy = (sxyz7.vy - sxyz13.vy)>>1;

	sxyz10.vx = sxyz13.vx + half_diff.vx;
	sxyz10.vy = sxyz13.vy + half_diff.vy;

	half_diff.vx = (sxyz7.vx - sxyz2.vx)>>1;
	half_diff.vy = (sxyz7.vy - sxyz2.vy)>>1;

	sxyz9.vx = sxyz2.vx + half_diff.vx;
	sxyz9.vy = sxyz2.vy + half_diff.vy;



	LONG_MOVE(&cols0,&long_col0);
	LONG_MOVE(&cols1,&long_col1);
	long_col0 = long_col0 & 0x00fefeff;
	long_col1 = long_col1 & 0x00fefeff;
	col_diff = long_col0 + long_col1;
	col_diff = col_diff>>1;
	LONG_MOVE(&col_diff,&cols7);

	LONG_MOVE(&cols0,&long_col0);
	LONG_MOVE(&cols2,&long_col1);
	long_col0 = long_col0 & 0x00fefeff;
	long_col1 = long_col1 & 0x00fefeff;
	col_diff = long_col0 + long_col1;
	col_diff = col_diff>>1;
	LONG_MOVE(&col_diff,&cols5);

	LONG_MOVE(&cols2,&long_col0);
	LONG_MOVE(&cols1,&long_col1);
	long_col0 = long_col0 & 0x00fefeff;
	long_col1 = long_col1 & 0x00fefeff;
	col_diff = long_col0 + long_col1;
	col_diff = col_diff>>1;
	LONG_MOVE(&col_diff,&cols13);

	LONG_MOVE(&cols0,&long_col0);
	LONG_MOVE(&cols7,&long_col1);
	long_col0 = long_col0 & 0x00fefeff;
	long_col1 = long_col1 & 0x00fefeff;
	col_diff = long_col0 + long_col1;
	col_diff = col_diff>>1;
	LONG_MOVE(&col_diff,&cols4);

	LONG_MOVE(&cols1,&long_col0);
	LONG_MOVE(&cols7,&long_col1);
	long_col0 = long_col0 & 0x00fefeff;
	long_col1 = long_col1 & 0x00fefeff;
	col_diff = long_col0 + long_col1;
	col_diff = col_diff>>1;
	LONG_MOVE(&col_diff,&cols11);

	LONG_MOVE(&cols5,&long_col0);
	LONG_MOVE(&cols0,&long_col1);
	long_col0 = long_col0 & 0x00fefeff;
	long_col1 = long_col1 & 0x00fefeff;
	col_diff = long_col0 + long_col1;
	col_diff = col_diff>>1;
	LONG_MOVE(&col_diff,&cols3);

	LONG_MOVE(&cols2,&long_col0);
	LONG_MOVE(&cols5,&long_col1);
	long_col0 = long_col0 & 0x00fefeff;
	long_col1 = long_col1 & 0x00fefeff;
	col_diff = long_col0 + long_col1;
	col_diff = col_diff>>1;
	LONG_MOVE(&col_diff,&cols8);

	LONG_MOVE(&cols5,&long_col0);
	LONG_MOVE(&cols7,&long_col1);
	long_col0 = long_col0 & 0x00fefeff;
	long_col1 = long_col1 & 0x00fefeff;
	col_diff = long_col0 + long_col1;
	col_diff = col_diff>>1;
	LONG_MOVE(&col_diff,&cols6);

	LONG_MOVE(&cols7,&long_col0);
	LONG_MOVE(&cols13,&long_col1);
	long_col0 = long_col0 & 0x00fefeff;
	long_col1 = long_col1 & 0x00fefeff;
	col_diff = long_col0 + long_col1;
	col_diff = col_diff>>1;
	LONG_MOVE(&col_diff,&cols10);

	LONG_MOVE(&cols7,&long_col0);
	LONG_MOVE(&cols2,&long_col1);
	long_col0 = long_col0 & 0x00fefeff;
	long_col1 = long_col1 & 0x00fefeff;
	col_diff = long_col0 + long_col1;
	col_diff = col_diff>>1;
	LONG_MOVE(&col_diff,&cols9);

	LONG_MOVE(&cols13,&long_col0);
	LONG_MOVE(&cols2,&long_col1);
	long_col0 = long_col0 & 0x00fefeff;
	long_col1 = long_col1 & 0x00fefeff;
	col_diff = long_col0 + long_col1;
	col_diff = col_diff>>1;
	LONG_MOVE(&col_diff,&cols12);

	LONG_MOVE(&cols1,&long_col0);
	LONG_MOVE(&cols13,&long_col1);
	long_col0 = long_col0 & 0x00fefeff;
	long_col1 = long_col1 & 0x00fefeff;
	col_diff = long_col0 + long_col1;
	col_diff = col_diff>>1;
	LONG_MOVE(&col_diff,&cols14);






	LONG_MOVE(&sxyz0,&((*polys)->x0));
	LONG_MOVE(&sxyz4,&((*polys)->x1));
	LONG_MOVE(&sxyz3,&((*polys)->x2));
	LONG_MOVE(&cols0,&((*polys)->r0));
	LONG_MOVE(&cols4,&((*polys)->r1));
	LONG_MOVE(&cols3,&((*polys)->r2));

		setPolyG3(*polys);
		addPrim(ot,*polys);


	LONG_MOVE(&sxyz3,&(((*polys)+1)->x0));
	LONG_MOVE(&sxyz4,&(((*polys)+1)->x1));
	LONG_MOVE(&sxyz5,&(((*polys)+1)->x2));
	LONG_MOVE(&cols3,&(((*polys)+1)->r0));
	LONG_MOVE(&cols4,&(((*polys)+1)->r1));
	LONG_MOVE(&cols5,&(((*polys)+1)->r2));

		setPolyG3( (*polys)+1);
		addPrim(ot,(*polys)+1);


	LONG_MOVE(&sxyz4,&(((*polys)+2)->x0));
	LONG_MOVE(&sxyz5,&(((*polys)+2)->x1));
	LONG_MOVE(&sxyz6,&(((*polys)+2)->x2));
	LONG_MOVE(&cols4,&(((*polys)+2)->r0));
	LONG_MOVE(&cols5,&(((*polys)+2)->r1));
	LONG_MOVE(&cols6,&(((*polys)+2)->r2));

		setPolyG3( (*polys)+2);
		addPrim(ot,(*polys)+2);


	LONG_MOVE(&sxyz4,&(((*polys)+3)->x0));
	LONG_MOVE(&sxyz6,&(((*polys)+3)->x1));
	LONG_MOVE(&sxyz7,&(((*polys)+3)->x2));
	LONG_MOVE(&cols4,&(((*polys)+3)->r0));
	LONG_MOVE(&cols6,&(((*polys)+3)->r1));
	LONG_MOVE(&cols7,&(((*polys)+3)->r2));

		setPolyG3( (*polys)+3);
		addPrim(ot,(*polys)+3);


  	LONG_MOVE(&sxyz5,&(((*polys)+4)->x0));
	LONG_MOVE(&sxyz8,&(((*polys)+4)->x1));
	LONG_MOVE(&sxyz9,&(((*polys)+4)->x2));
	LONG_MOVE(&cols5,&(((*polys)+4)->r0));
	LONG_MOVE(&cols8,&(((*polys)+4)->r1));
	LONG_MOVE(&cols9,&(((*polys)+4)->r2));

		setPolyG3( (*polys)+4);
		addPrim(ot,(*polys)+4);


	LONG_MOVE(&sxyz5,&(((*polys)+5)->x0));
	LONG_MOVE(&sxyz6,&(((*polys)+5)->x1));
	LONG_MOVE(&sxyz9,&(((*polys)+5)->x2));
	LONG_MOVE(&cols5,&(((*polys)+5)->r0));
	LONG_MOVE(&cols6,&(((*polys)+5)->r1));
	LONG_MOVE(&cols9,&(((*polys)+5)->r2));

		setPolyG3( (*polys)+5);
		addPrim(ot,(*polys)+5);


	LONG_MOVE(&sxyz6,&(((*polys)+6)->x0));
	LONG_MOVE(&sxyz7,&(((*polys)+6)->x1));
	LONG_MOVE(&sxyz9,&(((*polys)+6)->x2));
	LONG_MOVE(&cols6,&(((*polys)+6)->r0));
	LONG_MOVE(&cols7,&(((*polys)+6)->r1));
	LONG_MOVE(&cols9,&(((*polys)+6)->r2));

		setPolyG3( (*polys)+6);
		addPrim(ot,(*polys)+6);


	LONG_MOVE(&sxyz7,&(((*polys)+7)->x0));
	LONG_MOVE(&sxyz9,&(((*polys)+7)->x1));
	LONG_MOVE(&sxyz10,&(((*polys)+7)->x2));
	LONG_MOVE(&cols7,&(((*polys)+7)->r0));
	LONG_MOVE(&cols9,&(((*polys)+7)->r1));
	LONG_MOVE(&cols10,&(((*polys)+7)->r2));

		setPolyG3( (*polys)+7);
		addPrim(ot,(*polys)+7);


	LONG_MOVE(&sxyz7,&(((*polys)+8)->x0));
	LONG_MOVE(&sxyz10,&(((*polys)+8)->x1));
	LONG_MOVE(&sxyz11,&(((*polys)+8)->x2));
	LONG_MOVE(&cols7,&(((*polys)+8)->r0));
	LONG_MOVE(&cols10,&(((*polys)+8)->r1));
	LONG_MOVE(&cols11,&(((*polys)+8)->r2));

		setPolyG3( (*polys)+8);
		addPrim(ot,(*polys)+8);


	LONG_MOVE(&sxyz2,&(((*polys)+9)->x0));
	LONG_MOVE(&sxyz8,&(((*polys)+9)->x1));
	LONG_MOVE(&sxyz9,&(((*polys)+9)->x2));
	LONG_MOVE(&cols2,&(((*polys)+9)->r0));
	LONG_MOVE(&cols8,&(((*polys)+9)->r1));
	LONG_MOVE(&cols9,&(((*polys)+9)->r2));

		setPolyG3( (*polys)+9);
		addPrim(ot,(*polys)+9);


	LONG_MOVE(&sxyz2,&(((*polys)+10)->x0));
	LONG_MOVE(&sxyz9,&(((*polys)+10)->x1));
	LONG_MOVE(&sxyz12,&(((*polys)+10)->x2));
	LONG_MOVE(&cols2,&(((*polys)+10)->r0));
	LONG_MOVE(&cols9,&(((*polys)+10)->r1));
	LONG_MOVE(&cols12,&(((*polys)+10)->r2));

		setPolyG3( (*polys)+10);
		addPrim(ot,(*polys)+10);


	LONG_MOVE(&sxyz9,&(((*polys)+11)->x0));
	LONG_MOVE(&sxyz12,&(((*polys)+11)->x1));
	LONG_MOVE(&sxyz13,&(((*polys)+11)->x2));
	LONG_MOVE(&cols9,&(((*polys)+11)->r0));
	LONG_MOVE(&cols12,&(((*polys)+11)->r1));
	LONG_MOVE(&cols13,&(((*polys)+11)->r2));

		setPolyG3( (*polys)+11);
		addPrim(ot,(*polys)+11);


	LONG_MOVE(&sxyz9,&(((*polys)+12)->x0));
	LONG_MOVE(&sxyz10,&(((*polys)+12)->x1));
	LONG_MOVE(&sxyz13,&(((*polys)+12)->x2));
	LONG_MOVE(&cols9,&(((*polys)+12)->r0));
	LONG_MOVE(&cols10,&(((*polys)+12)->r1));
	LONG_MOVE(&cols13,&(((*polys)+12)->r2));

		setPolyG3( (*polys)+12);
		addPrim(ot,(*polys)+12);


	LONG_MOVE(&sxyz10,&(((*polys)+13)->x0));
	LONG_MOVE(&sxyz11,&(((*polys)+13)->x1));
	LONG_MOVE(&sxyz13,&(((*polys)+13)->x2));
	LONG_MOVE(&cols10,&(((*polys)+13)->r0));
	LONG_MOVE(&cols11,&(((*polys)+13)->r1));
	LONG_MOVE(&cols13,&(((*polys)+13)->r2));

		setPolyG3( (*polys)+13);
		addPrim(ot,(*polys)+13);


	LONG_MOVE(&sxyz11,&(((*polys)+14)->x0));
	LONG_MOVE(&sxyz13,&(((*polys)+14)->x1));
	LONG_MOVE(&sxyz14,&(((*polys)+14)->x2));
	LONG_MOVE(&cols11,&(((*polys)+14)->r0));
	LONG_MOVE(&cols13,&(((*polys)+14)->r1));
	LONG_MOVE(&cols14,&(((*polys)+14)->r2));

		setPolyG3( (*polys)+14);
		addPrim(ot,(*polys)+14);


	LONG_MOVE(&sxyz1,&(((*polys)+15)->x0));
	LONG_MOVE(&sxyz11,&(((*polys)+15)->x1));
	LONG_MOVE(&sxyz14,&(((*polys)+15)->x2));
	LONG_MOVE(&cols1,&(((*polys)+15)->r0));
	LONG_MOVE(&cols11,&(((*polys)+15)->r1));
	LONG_MOVE(&cols14,&(((*polys)+15)->r2));

		setPolyG3( (*polys)+15);
		addPrim(ot,(*polys)+15);


	(*polys) += 16;

}*/




/****************************************************/

void	RussRotMeshG3Multi(player *Plyr,POLY_G3 *polys)
{
	long		i,address,z,temp,z0,z1,z2,z3,z_max;
	SVECTOR		*vert_ptr,*normal_ptr,sclear;
	CVECTOR		*cols,*end_cols;
	VECTOR		scale_vec;
	VECTOR		stripoffsets[128],clear;
	unsigned long	*ot;
	long		current_strip,no_polys,ndiv;
	POLY_G3		codeprim;
	TMESH		*mesh;
	MATRIX		unknown1,rev;
	RUSS_SVECTOR		*sxyz_ptr;
	CVECTOR		temp_col = {255,255,255,255};
	CVECTOR		col0,col1,col2,col3;

	VECTOR		input,output;
	long		distance;
	long		player_strip;

	no_divided_polys = 0;
	clear.vx = clear.vy = clear.vz = 0;
	sclear.vx = sclear.vy = sclear.vz = 0;

	SetPolyG3(&codeprim);

	ot = &(OrderingTable[ otid ][0]);
	ndiv = 2;
	current_strip = Plyr->Track->CurrentStrip;		/* CP = Current Polygon */

    input.vx = (-Plyr->Cam.Targ.X) -  (-Plyr->Cam.Pos.X);
    input.vy = Plyr->Cam.Targ.Y  -  Plyr->Cam.Pos.Y;
    input.vz = Plyr->Cam.Targ.Z  -  Plyr->Cam.Pos.Z;


	gte_SetRotMatrix(&(Plyr->Cam.ObjectView));

	for ( i = 0; i < Plyr->Track->NumberOfStrips; i++ )
	{
			 /* do strip ring buffer wrap around */
		if (current_strip == MAX_VIEW_STRIPS)
			current_strip=0;

		stripoffsets[current_strip].vx=(-Plyr->Cam.Pos.X>>12)-Plyr->Track->Meshes[ current_strip ].OffsetX;
		stripoffsets[current_strip].vy=(-Plyr->Cam.Pos.Y>>12);
		stripoffsets[current_strip].vz=(-Plyr->Cam.Pos.Z>>12)+Plyr->Track->Meshes[ current_strip ].OffsetZ;

		ApplyRotMatrixLV(&(stripoffsets[current_strip]),&(stripoffsets[current_strip]));
		current_strip++;

	}

	gte_SetRotMatrix(&(Plyr->Cam.ViewMatrix));

	current_strip = Plyr->Track->CurrentStrip;		/* CP = Current Polygon */
	for ( i = 0; i < Plyr->Track->NumberOfStrips; i++ )
	{


		/* do strip ring buffer wrap around */
		if (current_strip == MAX_VIEW_STRIPS)
			current_strip=0;

		mesh=&Plyr->Track->Meshes[ current_strip ].TMesh;

		Plyr->Cam.TransMatrix.t[1]=stripoffsets[current_strip].vx;
		Plyr->Cam.TransMatrix.t[0]=stripoffsets[current_strip].vy;
		Plyr->Cam.TransMatrix.t[2]=stripoffsets[current_strip].vz;

		gte_SetTransMatrix(&(Plyr->Cam.TransMatrix));

		/* if this strip is the players strip, or one in front or one behind, subdivide some polys */

		if ((current_strip==bufftest0)||(current_strip==bufftest)||(current_strip==bufftest2))
		{

			/* do polys */

			sxyz_ptr = (RUSS_SVECTOR *)0x1f800000;

			cols = mesh->c - 2;
			end_cols = (cols + mesh->len - 2);
			vert_ptr = mesh->v;
			normal_ptr = mesh->n;

			/* do 1st 2 verts */

			gte_ldv0(vert_ptr);
		    gte_rtps();
			gte_stsxy(sxyz_ptr);
			gte_stszotz(&(sxyz_ptr->vz));

			gte_ldv0(normal_ptr);
			gte_ldrgb(cols);
			gte_ncds();
			gte_strgb(&col0);

			vert_ptr++;
			sxyz_ptr++;
			normal_ptr++;
			cols++;

			gte_ldv0(vert_ptr);
		    gte_rtps();
			gte_stsxy(sxyz_ptr);
			gte_stszotz(&(sxyz_ptr->vz));

			gte_ldv0(normal_ptr);
			gte_ldrgb(cols);
			gte_ncds();
			gte_strgb(&col1);

			vert_ptr++;
			sxyz_ptr++;
			normal_ptr++;
			cols++;

			while(cols != end_cols)
			{
			  	/* find distance to cell */

				RotTrans(vert_ptr,&output,&address);
				distance = ApproxMagnitude(&output);


				gte_ldv0(vert_ptr);
				gte_rtps();
				gte_stsxy(sxyz_ptr);
				gte_stszotz(&(sxyz_ptr->vz));

				gte_ldv0(normal_ptr);
				gte_ldrgb(cols);
				gte_ncds();
				gte_strgb(&col2);
				normal_ptr++;
				cols++;

				gte_ldv0(vert_ptr+1);
				gte_rtps();
				gte_stsxy(sxyz_ptr+1);
		   		gte_stszotz(&((sxyz_ptr+1)->vz));

				gte_ldv0(normal_ptr);
				gte_ldrgb(cols);
				gte_ncds();
				gte_strgb(&col3);
				normal_ptr++;
				cols++;


				/* if this cell is next to the player, subdivide */
			    //	if((cols >= start_subd) && (cols <= end_subd))
				if(distance < MESH_SUBDIV)
				{
					/* find max z */
					z0 = (sxyz_ptr-2)->vz;
					z1 = (sxyz_ptr-1)->vz;
					z2 = (sxyz_ptr-0)->vz;
					z3 = (sxyz_ptr+1)->vz;

					if(z1 > z0)
						z0 = z1;

					if(z3 > z2)
						z2 = z3;

					if(z0 > z2)
					   	z_max = z0;
					else
						z_max = z2;

					if(z_max > MESH_NEAR_Z)
					{

							RussDivideG3(*(DVECTOR *)(&((sxyz_ptr-2)->vx)),
									   *(DVECTOR *)(&((sxyz_ptr-1)->vx)),
									   *(DVECTOR *)(&((sxyz_ptr-0)->vx)),
										col0,col1,col2,
										&polys,
										ot + z_max,ndiv);

							nodivs++;
							no_polys = ndiv<<1;
							no_polys = 1<<no_polys;
							nopolyG3s+=no_polys;
							no_divided_polys += no_polys;
					}

					if(z_max > 0)
					{
						RussDivideG3(*(DVECTOR *)(&((sxyz_ptr-1)->vx)),
									   *(DVECTOR *)(&((sxyz_ptr-0)->vx)),
									   *(DVECTOR *)(&((sxyz_ptr+1)->vx)),
								col1,col2,col3,
								&polys,
								ot + z_max,ndiv);


						no_polys = ndiv<<1;
						no_polys = 1<<no_polys;
						nopolyG3s+=no_polys;
						nodivs++;
						no_divided_polys += no_polys;
					}
				}
				else
				{
					/* find max z */
					z0 = (sxyz_ptr-2)->vz;
					z1 = (sxyz_ptr-1)->vz;
					z2 = (sxyz_ptr-0)->vz;
					z3 = (sxyz_ptr+1)->vz;

					if(z1 > z0)
						z0 = z1;

					if(z3 > z2)
						z2 = z3;

					if(z0 > z2)
					   	z = z0;
					else
						z = z2;


					if(z < OTLENGTH-2)
					{
						if(z > MESH_NEAR_Z)
						{
							gte_NormalClip(*(long *)(sxyz_ptr-2),*(long*)(sxyz_ptr-1),*(long*)(sxyz_ptr-0),&temp);
							if(temp >= 0 )
							{
								LONG_MOVE(&col0,&polys->r0);
								LONG_MOVE(&col1,&polys->r1);
								LONG_MOVE(&col2,&polys->r2);

								LONG_MOVE(&((sxyz_ptr-2)->vx),&polys->x0);
								LONG_MOVE(&((sxyz_ptr-1)->vx),&polys->x1);
								LONG_MOVE(&((sxyz_ptr-0)->vx),&polys->x2);

								setPolyG3(polys);
								addPrim((ot+z),polys);
							}
							gte_NormalClip(*(long *)(sxyz_ptr-0),*(long*)(sxyz_ptr-1),*(long*)(sxyz_ptr+1),&temp);
							if(temp >= 0 )
						 	{
								LONG_MOVE(&col1,&(polys+1)->r0);
								LONG_MOVE(&col2,&(polys+1)->r1);
								LONG_MOVE(&col3,&(polys+1)->r2);

								LONG_MOVE(&((sxyz_ptr-1)->vx),&(polys+1)->x0);
								LONG_MOVE(&((sxyz_ptr-0)->vx),&(polys+1)->x1);
								LONG_MOVE(&((sxyz_ptr+1)->vx),&(polys+1)->x2);


								setPolyG3(polys+1);
								addPrim((ot+z),polys+1);
							}
						}
					}
		 			polys+=2;
					nopolyG3s+=2;
				}
				vert_ptr +=2;
				sxyz_ptr	+=2;
				LONG_MOVE(&col2,&col0);
				LONG_MOVE(&col3,&col1);
			}
		}
		//else
		{
//			RussRotMeshG3S(mesh,polys,ot);
//			RussRotMeshG3(mesh,polys,ot,current_strip);
//			RussRotMeshG3maxz(mesh,polys,ot,current_strip);
			RussRotMeshG3Szmax(mesh,polys,ot);
			polys+=mesh->len-2;
			nopolyG3s+=mesh->len-2;
		}
		current_strip++;
	}
}




/****************************************************/


SVECTOR	normalspot = {0,4096,0,0};

void	RussRotMeshG3Para(player *Plyr,POLY_G3 *polys)
{
	long		i,address,z,temp,z0,z1,z2,z3,z_max;
	SVECTOR		*vert_ptr,*normal_ptr,sclear,snew;
	CVECTOR		*cols,*end_cols;
	VECTOR		scale_vec;
	VECTOR		stripoffset,clear,output;
	unsigned long	*ot;
	long		no_polys,flag;
	POLY_G3		codeprim;
	TMESH		newmesh;
	MATRIX		unknown1,rev;
	RUSS_SVECTOR		*sxyz_ptr;
	CVECTOR		colours[128],*colsout_ptr;

	long		no_strips;
	long		offsetx,offsetz;
	long		offset;
	long		player_strip;
	long		templen;

	clear.vx = clear.vy = clear.vz = 0;
	sclear.vx = sclear.vy = sclear.vz = 0;

	ot = &(OrderingTable[ otid ][OTLENGTH - 9]);

	gte_SetRotMatrix(&(Plyr->Cam.ObjectView));

	no_strips = IBuf[Plyr->Buffer]->NumStrips;
	offsetx = IBuf[Plyr->Buffer]->OffsetX;
	offsetz = IBuf[Plyr->Buffer]->OffsetZ;

	offset = 0;

	stripoffset.vx=(-Plyr->Cam.Pos.X>>12)-offsetx;
	stripoffset.vy=(-Plyr->Cam.Pos.Y>>12);
	stripoffset.vz=(-Plyr->Cam.Pos.Z>>12)+offsetz;



	//ApplyRotMatrixLV(&stripoffset,&stripoffset);

	TransMatrix(&rev,&clear);
	SetTransMatrix(&rev);
	snew.vx = (short)stripoffset.vx;
	snew.vy = (short)stripoffset.vy;
	snew.vz = (short)stripoffset.vz;
	RotTrans(&snew,&stripoffset,&flag);


	gte_SetRotMatrix(&(Plyr->Cam.ViewMatrix));


	Plyr->Cam.TransMatrix.t[1]=stripoffset.vx;
	Plyr->Cam.TransMatrix.t[0]=stripoffset.vy;
	Plyr->Cam.TransMatrix.t[2]=stripoffset.vz;

	gte_SetTransMatrix(&(Plyr->Cam.TransMatrix));

	player_strip = striptab[0][bufftest0].stripno;
	for (i = 0; i < no_strips; i++ )
	{
		newmesh.len = IBuf[Plyr->Buffer]->TrackData[offset];
		offset += 8;
		newmesh.v = &(IBuf[Plyr->Buffer]->TrackData[offset]);

		offset += (newmesh.len<<3);
		newmesh.n = &(IBuf[Plyr->Buffer]->TrackData[offset]);
		offset += (newmesh.len<<3);
		newmesh.c = &(IBuf[Plyr->Buffer]->TrackData[offset]);
		offset += (newmesh.len<<2);




		//if(abs(i - player_strip) < parachute_strips)
		if(i > 10)
		{
			templen = newmesh.len;
			RussRotMeshG3SzmaxPara(&newmesh,polys,ot);
			//RussRotMeshG3Szmax(&newmesh,polys,ot);
			//RussRotMeshG3maxzPara(&newmesh,polys,ot,0);
			//polys+=newmesh.len-2;
			polys+=(templen-2);
			//nopolyG3s+=newmesh.len-2;
			nopolyG3s+=(templen-2);
		}
	}
}




