#include <stdio.h>
#include <sys/types.h>
#include <LibGTE.h>
#include <LibGPU.h>
#include <LibCD.h>
#include <LibComb.h>
#include <LibEtc.h>
#include <LibGS.h>
#include <LibMath.h>
#include <LibPress.h>
#include <LibSN.h>
#include <LibSnd.h>
#include <LibSPU.h>
#include <inline_o.h>
#include <gtemac.h>


#include "ourtypes.h"
#include "mathbits.h"

#include "main.h"
#include "skeleton.h"
#include "camera.h"
#include "tracks.h"
#include "players.h"
#include "pads.h"

#include "screen.h"
#include "who.h"
#include "bored.h"
#include "animctrl.h"

#include "bonebag.h"
#include "trixbag.h"
#include "logic.h"
#include "snow.h"
#include "plume.h"
#include "rotmesh.h"

SNOW	flakes[MAX_SNOW];
long	no_flakes,current_flake;
VECTOR	flake_loc[MAX_SNOW];

void	InitSnow()
{
	long	i;

	for(i=0;i<MAX_SNOW;i++)
	{
		flakes[i].available = 1;

		flakes[i].last_pos.vx = 0;
		flakes[i].last_pos.vy = 0;
		flakes[i].last_pos.vz = 0;

		flakes[i].global_pos.vx = 0;
		flakes[i].global_pos.vy = 0;
		flakes[i].global_pos.vz = 0;

		setLineG2(&(flakes[i].prim[0]));
		setLineG2(&(flakes[i].prim[1]));

		setRGB0(&(flakes[i].prim[0]),SNOW_R,SNOW_G,SNOW_B);
		setRGB1(&(flakes[i].prim[0]),0x00,0x00,0x00);
		setRGB0(&(flakes[i].prim[1]),SNOW_R,SNOW_G,SNOW_B);
		setRGB1(&(flakes[i].prim[0]),0x00,0x00,0x00);

		setTile(&(flakes[i].bathroom[0]));
		setTile(&(flakes[i].bathroom[1]));
		setRGB0(&(flakes[i].bathroom[0]),SNOW_R,SNOW_G,SNOW_B);
		setRGB0(&(flakes[i].bathroom[1]),SNOW_R,SNOW_G,SNOW_B);
		flakes[i].bathroom[0].w = 2;
		flakes[i].bathroom[0].h = 2;
		flakes[i].bathroom[1].w = 2;
		flakes[i].bathroom[1].h = 2;



	}
	no_flakes = 0;
	current_flake = 0;

}




/**************************************************************/

void	GetNewFlakeLoc(output,Plyr)
VECTOR *output;
player *Plyr;
{
	MATRIX	rev_cam_mat,trans_mat,arse_mat;
	VECTOR	scale_vec,loc,offset,clear;
	SVECTOR	temp;
	long	address;
	SVECTOR	eye,up;
	VECTOR	Z;

	/* backup rotation matrix */
	PushMatrix();

	/* clear translation matrix */
	clear.vx = clear.vy = clear.vz = 0;
	TransMatrix(&trans_mat,&clear);
	SetTransMatrix(&trans_mat);
			
	SetRotMatrix(&rev);
	ReadRotMatrix(&arse_mat);

	Z.vx = Z.vy = Z.vz = 4096 / 6;
	ScaleMatrix(&arse_mat,&Z);
	SetRotMatrix(&arse_mat);

	rand_idx++;
	rand_idx = rand_idx & 0x0000001f;
	temp.vx = (randarray[rand_idx] - 16384)>>3;

	rand_idx++;
	rand_idx = rand_idx & 0x0000001f;
	temp.vy = 1024 + ((randarray[rand_idx] & 1023) - 512);

	rand_idx++;
	rand_idx = rand_idx & 0x0000001f;
	temp.vz =  1024 + (randarray[rand_idx] & 2047);



	RotTrans(&temp,&offset,&address);

	offset.vx = offset.vx<<12;
	offset.vy = offset.vy<<12;
	offset.vz = offset.vz<<12;

	output->vx = Plyr->Cam.Pos.X + offset.vx;
	output->vy = Plyr->Cam.Pos.Y + offset.vy;
	output->vz = Plyr->Cam.Pos.Z + offset.vz;

	/* restore rotation matrix */
	PopMatrix();

}


/**********************************************************************/

void   	DrawSnow(unsigned long amount,unsigned long playerno)
{
	unsigned char col,a,b,c;
	long	address,i,z,size;
	SVECTOR	local_pos,temp,eye,up;
	MATRIX	trans_mat,forward_cam_mat,rev_cam_mat,ObjectTrans,z_mat;
	VECTOR	output,clear,offset,scale_vec;
						  
	/* backup rotation matrix */
	PushMatrix();

	clear.vx = clear.vy = clear.vz = 0;

	DoCamMats(&(Player[playerno]));

   	if ((amount*5) > no_flakes)
	{
		for (no_flakes;no_flakes<(amount*5);no_flakes++)
			GetNewFlakeLoc(&(flakes[no_flakes].global_pos),&(Player[playerno]));
	}
	else
		no_flakes=amount*5;

	for(i=0;i<no_flakes;i++)
		flakes[i].global_pos.vy -= 20480;

	/* clear translation matrix */
	TransMatrix(&trans_mat,&clear);
	SetTransMatrix(&trans_mat);
	SetGeomOffset(160,120);
	SetRotMatrix(&(Player[playerno].Cam.ViewMatrix));
		
	for(i=0;i<no_flakes;i++)
	{
		rand_idx++;
		rand_idx = rand_idx & 0x0000001f;
		flakes[i].global_pos.vx += (randarray[rand_idx] - 16384)>>1;

		output.vx = (-flakes[i].global_pos.vx + Player[playerno].Cam.Pos.X)>>12; 
		output.vy =	(flakes[i].global_pos.vy - Player[playerno].Cam.Pos.Y)>>12; 
		output.vz =	(flakes[i].global_pos.vz - Player[playerno].Cam.Pos.Z)>>12; 

		local_pos.vx = (short)(output.vx); 
		local_pos.vy = (short)(output.vy); 
		local_pos.vz = (short)(output.vz); 

		if(local_pos.vy < -500)
		{				  
			GetNewFlakeLoc(&(flakes[i].global_pos),&(Player[playerno]));
			flakes[i].available = 1;
		}
		else
		{
			RotTrans(&local_pos,&output,&address);

			if((output.vz > 1) && (output.vz < 16000) && 
			(output.vx > -output.vz) && (output.vx < output.vz) &&
			(output.vy > -output.vz) && (output.vy < output.vz))
			{
				z = RotTransPers(&local_pos,&temp,&address,&address);

				if(z<256)
					size = 2;
				else 
					size = 1;

				flakes[i].bathroom[otid].w = size;
				flakes[i].bathroom[otid].h = size;
			
				col = 128;
				col += (127 - (z>>3));

				setRGB0(&(flakes[i].bathroom[otid]),col,col,col);
				/*LONG_MOVE(&temp,&(flakes[i].prim[otid].x0));
				LONG_MOVE(&temp,&(flakes[i].prim[otid].x1));*/
				LONG_MOVE(&temp,&(flakes[i].bathroom[otid].x0));
			
				/*if(flakes[i].available)
				{
					LONG_MOVE(&temp,&(flakes[i].last_pos.vx));
					flakes[i].available = 0;
				}
				LONG_MOVE(&(flakes[i].last_pos.vx),&(flakes[i].prim[otid].x1));
				LONG_MOVE(&temp,&(flakes[i].last_pos.vx));*/
				/*addPrim(&(OrderingTable[otid][0]),&(flakes[i].prim[otid]));*/
				addPrim(&(OrderingTable[otid][(z>>1)]),&(flakes[i].bathroom[otid]));
			}
			else
			{				  
				GetNewFlakeLoc(&(flakes[i].global_pos),&(Player[playerno]));
				flakes[i].available = 1;
			}
		}
	}
	/* restore old rotation matrix */
	PopMatrix();
	SetGeomOffset(120,160);		   
}

