#include <stdio.h>
#include "sys/types.h"
#include "LibGTE.h"
#include "LibGPU.h"
#include "LibCD.h"
#include "LibEtc.h"
#include "LibMath.h"
#include "LibPress.h"
#include "LibSN.h"
#include "LibSnd.h"
#include "LibSPU.h"
#include "ourtypes.h"
#include "main.h"
#include "cd.h"
#include "sound.h"
#include "mathbits.h"
#include "screen.h"
#include "camera.h"
#include "tracks.h"
#include "players.h"
#include "pads.h"
#include "profile.h"
#include "skeleton.h"
#include "who.h"
#include "frontend.h"
#include "plume.h"
#include "rotmeshs.h"
#include "animctrl.h"
#include <inline_o.h>
#include <gtemac.h>
#include "avalanch.h"
#include "boulder.h"
#include "parachut.h"
#include "bonebag.h"
#include "logic.h"
#include "tree.h"
#include "rotmesh.h"


TREE_TYPE	tree_falling[MAX_TREES];
long		number_of_allocated_trees;

void 	InitTrees()
{
	long	i;

	for(i=0;i<MAX_TREES;i++)
	{
		tree_falling[i].fileno = 0;
		tree_falling[i].stripno = 0;
		tree_falling[i].treeno = 0;
		tree_falling[i].fall_state = 0;
		tree_falling[i].fall_direction = 1;
		tree_falling[i].counter = 32;
		tree_falling[i].max_rot = 1024;
	}


	tree_falling[0].max_rot = 1024; // 0deg
	tree_falling[0].counter = 24;
	tree_falling[1].max_rot = 1138;  // +10deg
	tree_falling[1].counter = 24;
	tree_falling[2].max_rot = 910; // -10deg
	tree_falling[2].counter = 24;
	tree_falling[3].max_rot = 910; // -10deg
	tree_falling[3].counter = 24;
	tree_falling[4].max_rot = 1138; // +10deg
	tree_falling[4].counter = 24;
	tree_falling[5].max_rot = 1138; // +10deg
	tree_falling[5].counter = 24;
	tree_falling[6].max_rot = 910; // -10deg
	tree_falling[6].counter = 24;
	tree_falling[7].max_rot = 1080; // +5deg
	tree_falling[7].counter = 24;
	tree_falling[8].max_rot = 967; // -5deg
	tree_falling[8].counter = 32;
	tree_falling[9].max_rot = 1024; // 0deg
	tree_falling[9].counter = 32;
	tree_falling[10].max_rot = 1024; // 0deg
	tree_falling[10].counter = 32;
	tree_falling[11].max_rot = 967; // -5deg
	tree_falling[11].counter = 48;
	tree_falling[12].max_rot = 1024; // 0deg
	tree_falling[12].counter = 32;
	tree_falling[13].max_rot = 1024; // +10deg
	tree_falling[13].counter = 32;
	tree_falling[14].max_rot = 1024; // +10deg
	tree_falling[14].counter = 32;
	tree_falling[15].max_rot = 1024; // 0deg
	tree_falling[15].counter = 32;
	tree_falling[16].max_rot = 967; // -5deg
	tree_falling[16].counter = 48;
	tree_falling[17].max_rot = 1024; // 0deg
	tree_falling[17].counter = 48;
	tree_falling[18].max_rot = 1024; // 0deg
	tree_falling[18].counter = 48;
	tree_falling[19].max_rot = 1024; // +5deg
	tree_falling[19].counter = 48;
	tree_falling[20].max_rot = 1024; // 0deg
	tree_falling[20].counter = 48;
	tree_falling[21].max_rot = 967; // -5deg
	tree_falling[21].counter = 48;
	tree_falling[22].max_rot = 1054;// 
	tree_falling[22].counter = 48;
	tree_falling[23].max_rot = 910; // -10deg
	tree_falling[23].counter = 48;
	tree_falling[24].max_rot = 1138;// +10deg
	tree_falling[24].counter = 54;
	tree_falling[25].max_rot = 967; // -5deg
	tree_falling[25].counter = 54;
	tree_falling[26].max_rot = 1138;// +10deg
	tree_falling[26].counter = 48;
	tree_falling[27].max_rot = 910; // -10deg
	tree_falling[27].counter = 48;
	tree_falling[28].max_rot = 1024; // +5deg
	tree_falling[28].counter = 48;
	tree_falling[29].max_rot = 1024; // 0deg
	tree_falling[29].counter = 48;
	tree_falling[30].max_rot = 1024; // 0deg
	tree_falling[30].counter = 40;
	tree_falling[31].max_rot = 1024; // 0deg
	tree_falling[31].counter = 40;

	number_of_allocated_trees = 1;
}


/**************************************************/
//	GetTreeIdx:
//			Check if the tree in treeloc already has a tree structure
//		allocated to it, and if so return the index to it. Otherwise
//		allocate a new one & return the new index.
/**************************************************/

long	GetTreeIdx(treeloc)
TREE_TYPE *treeloc;
{
	long	i;
	long	return_val;

	return_val = -1;
	i=1;
	while(i<(number_of_allocated_trees+1))
	{
		if(tree_falling[i].stripno == treeloc->stripno)
		{
			if(tree_falling[i].fileno == treeloc->fileno)
			{
				if(tree_falling[i].treeno == treeloc->treeno)
				{
				   return_val = i;
				   break;
				}
			}
		}
		i++;
	}

	if(return_val == -1) // if this tree has no tree structure, allocate one
	{
		// if its the first tree, change the music
		//if(number_of_allocated_trees == 1)
		//	soundtrigger = 1;
		return_val = number_of_allocated_trees;
		tree_falling[return_val].fileno = treeloc->fileno;
		tree_falling[return_val].stripno = treeloc->stripno;
		tree_falling[return_val].treeno = treeloc->treeno;
		number_of_allocated_trees++;
	}

	return(return_val);
}

/**************************************************/


void	DrawTrees(playerno)
long	playerno;
{
	long	i;

	//if(number_of_allocated_trees > 1)
	{
		if((striptab[playerno][player_strip[playerno]].fileno == 8) && 
			(striptab[playerno][player_strip[playerno]].stripno ==91))
				soundtrigger = 0;
		if((striptab[playerno][player_strip[playerno]].fileno == 7) && 
			(striptab[playerno][player_strip[playerno]].stripno ==90))
				soundtrigger = 1;

	}

	if(!paused)
	{	
		for(i=1;i<number_of_allocated_trees;i++)
		{
			tree_falling[i].counter--;
			if(tree_falling[i].counter < 0)	// don't start falling until ready
			{
				//if(tree_falling[i].fall_state < 1024)
				if(tree_falling[i].fall_state < tree_falling[i].max_rot)
				{
					tree_falling[i].fall_state += 32;
				}
			}
		}
	}
}


/********************************************************************/

void UpdateTree(objectptr,P,thiscp,ObjectRotation,ObjectTrans,playerno)
trackobject	*objectptr;
player		*P;
long		thiscp;
MATRIX		*ObjectRotation;
MATRIX		*ObjectTrans;
long		playerno;
{
	SVECTOR	X,ObjRotVec,tree_leaf_offset,player_offset,RevRot;
	VECTOR	Y,output,tree_pos,plume_pos,leaf_pos;
	long	tree_idx,plume_idx;
	MATRIX	rev_mat;

	/* do collision detection */


	gte_SetRotMatrix(&P->Cam.ViewMatrix);


	X.vx=(objectptr->TransX+P->Cam.Pos.X)>>12;
	X.vy=(objectptr->TransY-P->Cam.Pos.Y)>>12;
	X.vz=(objectptr->TransZ-P->Cam.Pos.Z)>>12;
	X.vx+=P->Track->Meshes[ thiscp ].OffsetX;
	X.vz+=P->Track->Meshes[ thiscp ].OffsetZ;

	gte_ApplyRotMatrix(&X,&Y);
	ObjectTrans->t[1]=Y.vx;
	ObjectTrans->t[0]=Y.vy;
	ObjectTrans->t[2]=Y.vz;
			 	
	tree_idx = (objectptr->Type & 0x00ff0000)>>16;
		 	
	ObjRotVec.vx = tree_falling[tree_idx].fall_state;
	ObjRotVec.vy = -(objectptr->RotY);
	ObjRotVec.vz = 0;
	RotMatrixYXZ(&ObjRotVec,ObjectRotation);


	tree_pos.vx = objectptr->TransX;
	tree_pos.vx = (tree_pos.vx)>>12;
	tree_pos.vx += P->Track->Meshes[ thiscp ].OffsetX;
	tree_pos.vx = -(tree_pos.vx<<12);

	tree_pos.vy = objectptr->TransY; 
		 	
	tree_pos.vz = objectptr->TransZ; 
	tree_pos.vz = (tree_pos.vz)>>12;
	tree_pos.vz += P->Track->Meshes[ thiscp ].OffsetZ;
	tree_pos.vz = (tree_pos.vz)<<12;

	if(thiscollision[P->Buffer].type == NONE)
	{
		player_offset.vx = (short)((P->Pos.X - tree_pos.vx)>>12);
		player_offset.vy = (short)((P->Pos.Y - tree_pos.vy)>>12);
		player_offset.vz = (short)((P->Pos.Z - tree_pos.vz)>>12);
		
		RevRot.vx = -tree_falling[tree_idx].fall_state;		
		RevRot.vy = (objectptr->RotY);
		RevRot.vz = 0;
		RotMatrix(&RevRot,&rev_mat);
		gte_SetRotMatrix(&rev_mat);

		gte_ApplyRotMatrix(&player_offset,&Y);

		if(abs(Y.vx) < 24)
		{
			if(abs(Y.vz) < 24)
			{
				if(Y.vy < 155)
				{ 
					if(Y.vy < 60)
					{

						//prad=coldx;
						thiscollision[P->Buffer].type=TREESMACK;	
						thiscollision[P->Buffer].angle=2048;
						thiscollision[P->Buffer].point.X=P->LastPos.X;
						thiscollision[P->Buffer].point.Y=P->LastPos.Y;
						thiscollision[P->Buffer].point.Z=P->LastPos.Z;
#ifdef MUSIC
						playsoundeffect(20,1);    /* 20 */
						playsoundeffect(21,1);
						playsoundeffect(7,1);
						deltime=globaltime[P->Buffer];
						playwhoah=1;
#endif
					}
					else
					{
						thiscollision[P->Buffer].type=TREEBUMP;
						rand_idx += 7;
						rand_idx = rand_idx & 0x0000001f;

						thiscollision[P->Buffer].angle=rand_idx;
						thiscollision[P->Buffer].point.X=P->LastPos.X;
						thiscollision[P->Buffer].point.Y=P->LastPos.Y;
						thiscollision[P->Buffer].point.Z=P->LastPos.Z;
#ifdef MUSIC
						playsoundeffect(18,1);
#endif
					}
				}
			}
		}
	}	

  
	
	if(!paused)
	{
		if((tree_falling[tree_idx].fall_state < tree_falling[tree_idx].max_rot) && (tree_falling[tree_idx].counter < 0))
		{
	 		tree_leaf_offset.vx = 0;
			tree_leaf_offset.vy = 8192;
			rand_idx += 7;
			rand_idx = rand_idx & 0x0000001f;
			tree_leaf_offset.vy += randarray[rand_idx] & 0x00003fff;
			tree_leaf_offset.vz = 0;

			gte_SetRotMatrix(ObjectRotation);
			gte_ApplyRotMatrix(&tree_leaf_offset,&output);
				
			output.vx = output.vx<<5;
			output.vy = output.vy<<5;
			output.vz = output.vz<<5;

			AddPlume(&tree_pos);

			leaf_pos.vx = tree_pos.vx + output.vx;
			leaf_pos.vy = tree_pos.vy + output.vy;
			leaf_pos.vz = tree_pos.vz + output.vz;

			AddLeaf(playerno,&leaf_pos);
		}
	
		if(tree_falling[tree_idx].fall_state == tree_falling[tree_idx].max_rot)
		{
			/* make sure this only happens once */
			tree_falling[tree_idx].fall_state++;

		 	gte_SetRotMatrix(ObjectRotation);
			
			for(plume_idx=0;plume_idx<8;plume_idx++)
			{
				tree_leaf_offset.vx = 0;
				tree_leaf_offset.vy = 10000;
				rand_idx += 7;
				rand_idx = rand_idx & 0x0000001f;
				tree_leaf_offset.vy += (plume_idx<<11);
				tree_leaf_offset.vz = -4096;

				gte_ApplyRotMatrix(&tree_leaf_offset,&output);
				
				output.vx = output.vx<<5;
				output.vy = output.vy<<5;
				output.vz = output.vz<<5;

				plume_pos.vx = tree_pos.vx + output.vx;
				plume_pos.vy = tree_pos.vy + output.vy;
				plume_pos.vz = tree_pos.vz + output.vz;
	
				AddPlume(&plume_pos);
			}
		}
	}
	gte_SetRotMatrix(&P->Cam.ObjectView);
	MulRotMatrix(ObjectRotation);
}
