// sound.c
#include "sys/types.h"
#include "LibGTE.h"
#include "LibGPU.h"
//#include <libds.h>
#include "LibCD.h"
#include "LibEtc.h"
#include "LibMath.h"
#include "LibPress.h"
#include "LibSN.h"
#include "LibSnd.h"
#include "LibSPU.h"
#include "gtemac.h"
#include "inline_c.h"

/* rod. john will supply you with a .vab file and a number of .seq files.
   use vabplit.exe to split the .vab into a .vh and a .vb - header and body files.
   once the .vb is in sound ram, it can be trashed, but the .vh MUST be held intact!
   the individual .seq files are convert like so:

   seq2sep -o output.sep sfx1.seq sfx2.seq sfx3.seq ....

   the number of .seq files loaded must be adjusted in the code, or it will hang.
   when using playsoundeffect, the no refers to the sample added in the order
   given to seq2sep i.e. playsoundeffect(0,4) will play 4 repeats of sfx1.seq.
   SS_INFINITY can be used to get a constant repeat.
   */





short sfxsep1;
short sfxvab;

char seq_table[SS_SEQ_TABSIZ * 1 * 32];     /* you might nead multiple seq files */

char transferbuffer[200000]; /* however big your vab body is, find a suitable buffer to load it in to */
char sfxsep[4096];
char sfxhead[8192];
long fin,arsesize;

void playsoundeffect(uint8 no,uint8 repeat)	/* this function can be used directly after the init */
{
	if (SsIsEos(sfxsep1,no)==0)
	{
		SsSepSetVol (sfxsep1, no, 127 ,127); /* left, right volume */
		SsSepPlay(sfxsep1, no, SSPLAY_PLAY, repeat);
	}
}

void InitialiseSound(uint8 where)
{
	SsInit ();
	SsSetTableSize (seq_table, 1, 32); /* gives 1 sep file with 32 seq in it */
	SsSetTickMode (SS_TICK60);

//	SsSetReservedVoice(22);    /* you may want to set this - it reserves x out of 24 channels for sfx */

#ifndef CD
	fin			=	PCopen( "music/sfx/sfx.vh", 0, 0 );               /* load samples */
	arsesize	=	PClseek( fin, 0, 2 );
	PClseek( fin, 0, 0 );
	PCread( fin, (char *)&sfxhead, arsesize );
	PCclose( fin );
	fin			=	PCopen( "music/sfx/sfx.vb", 0, 0 );
	arsesize	=	PClseek( fin, 0, 2 );
	PClseek( fin, 0, 0 );
	PCread( fin, (char *)&transferbuffer, arsesize );
	PCclose( fin );

/* not to be hard coded! */
	fin			=	PCopen( "music/sfx/sfx.sep", 0, 0 );            /* load sfx 'tunes' */
	arsesize	=	PClseek( fin, 0, 2 );
	PClseek( fin, 0, 0 );
	PCread( fin, (char *)&sfxsep, arsesize );
	PCclose( fin );
#else
	load_cd("\\MUSIC\\SFX\\SFX_LO.VH;1",(char *)&sfxhead);
	do
	{

	}
	while (CdReadSync(1,0) > 0 );
	load_cd("\\MUSIC\\SFX\\SFX_LO.VB;1",(char *)&transferbuffer);
	do
	{

	}
	while (CdReadSync(1,0) > 0 );
	load_cd("\\MUSIC\\SFX\\SFX.SEP;1",(char *)&sfxsep);
	do
	{

	}
 	while (CdReadSync(1,0) > 0 );
#endif

	sfxvab = SsVabOpenHead ((unsigned char*)&sfxhead, 2);
	if( sfxvab == -1 ) {
	  printf( "sfxVAB headder open failed\n" );
	  exit(0);
	}
        sfxvab = SsVabTransBody ((unsigned char*)&transferbuffer, sfxvab);
	if( sfxvab == -1 ) {
	  printf( "mixVAB body open failed\n" );
	  exit(0);
	}
	SsVabTransCompleted (SS_WAIT_COMPLETED);

	SsStart2 ();

	sfxsep1 = SsSepOpen((unsigned long *)&sfxsep, sfxvab, 31);  /* 31 == no sound effects in file */

	SsSetMVol (127, 127);

	SsUtSetReverbType(1);                            /* set stereo and reverb if needed */
	SsUtReverbOn();
	for(waitcount=0;waitcount<200000;waitcount++)
	{

	}
	SsUtSetReverbDepth(80,80);

	if (currentsave.options[STEREO_SET]==1)
	{
		SsSetStereo();
	}
	else
	{
		SsSetMono();
	}

}

void KillSoundFE(void)
{
//	StopCallback();
	uint8	i;
	SpuCommonAttr	CommonAttrib;

	musicon=0;
	SsSetMVol (0, 0);
	SsSetMarkCallback(sep1,0,0);

//	SsSepSetVol(sep1,0,0,0);
//	SsSepStop(endsep[0],0);
	SsSepStop(sfxsep1,i);
	SsSepClose (sfxsep1);
	SsVabClose (sfxvab);
    SsEnd ();
	SsInit ();
	SsSetMVol (127, 127);
	CommonAttrib.mask = (SPU_COMMON_CDVOLL | SPU_COMMON_CDVOLR | SPU_COMMON_CDREV | SPU_COMMON_CDMIX);
	CommonAttrib.cd.volume.left  = 0x7fff;
	CommonAttrib.cd.volume.right  = 0x7fff;
	CommonAttrib.cd.mix			  = SPU_ON;
	CommonAttrib.cd.reverb		  = SPU_OFF;
	SpuSetCommonAttr (&CommonAttrib);				/* Sound common attributes set */
//	SsQuit ();
}


void KillSound(void)
{
	SsSetMarkCallback(sep1,0,0);
	StopCallback();
	SsSepClose (sep1);
    SsVabClose (vab1);
	SsSepClose (endsep[(whichload+1)%2]);
	SsVabClose (mixvab);
    SsEnd ();
	SsQuit ();
}



