// Skeleton.c

#include <stdio.h>
#include <sys/types.h>
#include <LibGTE.h>
#include <LibGPU.h>
#include <LibCD.h>
#include <LibComb.h>
#include <LibEtc.h>
#include <LibGS.h>
#include <LibMath.h>
#include <LibPress.h>
#include <LibSN.h>
#include <LibSnd.h>
#include <LibSPU.h>

#include "ourtypes.h"
#include "mathbits.h"

#include "main.h"
#include "skeleton.h"
#include "camera.h"
#include "tracks.h"
#include "players.h"
#include "pads.h"

#include "screen.h"

uint16 Tp;
uint16 Nm;

MATRIX TransMat;
MATRIX RotMat;


LINE_F2 TPrims[ 2 ][ 150 ];
POLY_F4 NodeMarker[ 2 ][ 150 ];

void Prepare_Skeleton( void )
{
	Tp = 0;
	Nm = 0;


	/* Stick the temporary bone manipulator here */
/*	if( Pad_State( 1 ) & PAD_LEFT )
	{
		NOODLE1.Rz -= CHFL( ( 4096.0 / 360.0 ) * 11.25 );
	}

	if( Pad_State( 1 ) & PAD_RIGHT )
	{
		NOODLE1.Rz += CHFL( ( 4096.0 / 360.0 ) * 11.25 );
	}

	if( Pad_State( 1 ) & PAD_BL )
	{
		NOODLE2.Rz -= CHFL( ( 4096.0 / 360.0 ) * 11.25 );
	}

	if( Pad_State( 1 ) & PAD_BR )
	{
		NOODLE2.Rz += CHFL( ( 4096.0 / 360.0 ) * 11.25 );
	}
*/
}




void Traverse_Calculate_Skeleton( node *Node, chfloat X, chfloat Y, chfloat Z, chfloat Rx, chfloat Ry, chfloat Rz )
{
	uint16 i;

	// BEWARE when using static variables in recursive routines
	static SVECTOR A;
	static VECTOR D;
	static MATRIX Mat;

	// First rotate Node->Tx, Ty, Tz around 0, 0, 0
	// Put the result in Node->X, Y, Z
	A.vx = CHSHIFTR( Node->Rx + Rx );
	A.vy = CHSHIFTR( Node->Ry + Ry );
	A.vz = CHSHIFTR( Node->Rz + Rz );

	RotMatrix( &A, &Mat );

	A.vx = Node->Tx;
	A.vy = Node->Ty;
	A.vz = Node->Tz;

	ApplyMatrix( &Mat, &A, &D );
	Node->X = D.vx;
	Node->Y = D.vy;
	Node->Z = D.vz;

	// Now add on the X, Y, Z routune parameters, again put the result in Node->X, Y, Z
	Node->X += X;
	Node->Y += Y;
	Node->Z += Z;

	// Pass the result on recursively
	for( i = 0; i < Node->NumChildren; i++ )
	{
		if( Node->Child[ i ] != NULL )
		{
			Traverse_Calculate_Skeleton( (node*)Node->Child[ i ], Node->X, Node->Y, Node->Z, Node->Rx + Rx, Node->Ry + Ry, Node->Rz + Rz );
		}
	}
}



void Draw_Skeleton_2d( node *Node, chfloat X, chfloat Y )
{
	uint16 i;

	// Plop a green blob at the node marker to help debug
	{
		NodeMarker[ otid ][ Nm ].x0 = CHSHIFTR( Node->X * 5 ) + 158;
		NodeMarker[ otid ][ Nm ].y0 = CHSHIFTR( Node->Y * 5 ) + 97;
		NodeMarker[ otid ][ Nm ].x1 = CHSHIFTR( Node->X * 5 ) + 162;
		NodeMarker[ otid ][ Nm ].y1 = CHSHIFTR( Node->Y * 5 ) + 97;
		NodeMarker[ otid ][ Nm ].x2 = CHSHIFTR( Node->X * 5 ) + 158;
		NodeMarker[ otid ][ Nm ].y2 = CHSHIFTR( Node->Y * 5 ) + 102;
		NodeMarker[ otid ][ Nm ].x3 = CHSHIFTR( Node->X * 5 ) + 162;
		NodeMarker[ otid ][ Nm ].y3 = CHSHIFTR( Node->Y * 5 ) + 102;
		NodeMarker[ otid ][ Nm ].r0 = 0;
		NodeMarker[ otid ][ Nm ].g0 = 255;
		NodeMarker[ otid ][ Nm ].b0 = 0;
		setPolyF4( &NodeMarker[ otid ][ Nm ] );
		OTPtr=&(OrderingTable[otid][0]);
		AddPrim( OTPtr, &NodeMarker[ otid ][ Nm ] );
		Nm++;
	}

	for( i = 0; i < Node->NumChildren; i++ )
	{
		if( Node->Child[ i ] != NULL )
		{
			setLineF2( &TPrims[ otid ][ Tp ] );

			TPrims[ otid ][ Tp ].x0 = CHSHIFTR( Node->X * 5 ) + 160;
			TPrims[ otid ][ Tp ].y0 = CHSHIFTR( Node->Y * 5 ) + 100;
			TPrims[ otid ][ Tp ].x1 = CHSHIFTR( ( ( (node*)Node->Child[ i ] )->X + X ) * 5 ) + 160;
			TPrims[ otid ][ Tp ].y1 = CHSHIFTR( ( ( (node*)Node->Child[ i ] )->Y + Y ) * 5 ) + 100;

			TPrims[ otid ][ Tp ].r0 = 0;//( Tp * 16 ) & 0xff;
			TPrims[ otid ][ Tp ].g0 = 0;//( Tp * 32 ) & 0xff;
			TPrims[ otid ][ Tp ].b0 = 0;//( Tp * 48 ) & 0xff;

			OTPtr=&(OrderingTable[otid][0]);
  			AddPrim( OTPtr, &TPrims[ otid ][ Tp ] );
			Tp++;

			Draw_Skeleton_2d( (node*)Node->Child[ i ], X, Y );
		}
	}
}




void Draw_Skeleton_3d( node *Node, chfloat X, chfloat Y, chfloat Z, chfloat Rx, chfloat Ry, chfloat Rz )
{
// Essentially this draws and connects joints like the 2d one it's just that things are transformed by the
// ViewMatrix before doing so

	uint16 i;

	static SVECTOR A;
	static long D, P;
	static long Flag;

	// First of all transform this node to get an x0, y0
	A.vx = CHSHIFTR( Node->X + X );
	A.vy = CHSHIFTR( Node->Y + Y );
	A.vz = CHSHIFTR( Node->Z + Z );

	RotTransPers( &A, &D, &P, &Flag );

 	TPrims[ otid ][ Tp ].x0 = D >> 16;;
	TPrims[ otid ][ Tp ].y0 = D & 0xffff;


	// Plop a green blob on the joint to mark it
	{
		NodeMarker[ otid ][ Nm ].x0 = ( D >> 16 ) - 2;
		NodeMarker[ otid ][ Nm ].y0 = ( D & 0xffff ) - 2;
		NodeMarker[ otid ][ Nm ].x1 = ( D >> 16 ) + 2;
		NodeMarker[ otid ][ Nm ].y1 = ( D & 0xffff ) - 2;
		NodeMarker[ otid ][ Nm ].x2 = ( D >> 16 ) - 2;
		NodeMarker[ otid ][ Nm ].y2 = ( D & 0xffff ) + 2;
		NodeMarker[ otid ][ Nm ].x3 = ( D >> 16 ) + 2;
		NodeMarker[ otid ][ Nm ].y3 = ( D & 0xffff ) + 2;
		NodeMarker[ otid ][ Nm ].r0 = 0;
		NodeMarker[ otid ][ Nm ].g0 = 255;
		NodeMarker[ otid ][ Nm ].b0 = 0;
		setPolyF4( &NodeMarker[ otid ][ Nm ] );
		OTPtr=&(OrderingTable[otid][0]);
		AddPrim( OTPtr, &NodeMarker[ otid ][ Nm ] );
		Nm++;
	}



	// Now loop through my children
	for( i = 0; i < Node->NumChildren; i++ )
	{
		if( Node->Child[ i ] != NULL )
		{
			setLineF2( &TPrims[ otid ][ Tp ] );

			// Transform the child
			A.vx = CHSHIFTR( ( (node*)Node->Child[ i ] )->X + X );
			A.vy = CHSHIFTR( ( (node*)Node->Child[ i ] )->Y + Y );
			A.vz = CHSHIFTR( ( (node*)Node->Child[ i ] )->Z + Z );
			RotTransPers( &A, &D, &P, &Flag );

		 	TPrims[ otid ][ Tp ].x1 = D >> 16;;
			TPrims[ otid ][ Tp ].y1 = D & 0xffff;

			TPrims[ otid ][ Tp ].r0 = 0;//( Tp * 16 ) & 0xff;
			TPrims[ otid ][ Tp ].g0 = 0;//( Tp * 32 ) & 0xff;
			TPrims[ otid ][ Tp ].b0 = 0;//( Tp * 48 ) & 0xff;

			OTPtr=&(OrderingTable[otid][0]);
  			AddPrim( OTPtr, &TPrims[ otid ][ Tp ] );
			Tp++;

			Draw_Skeleton_3d( (node*)Node->Child[ i ], X, Y, Z, Rx, Ry, Rz );
		}
	}
}

// Here lies the code to insert in Jeb's tracks.c just before the DrawSync()

//	Prepare_Skeleton();
//	Traverse_Calculate_Skeleton( &NOODLE, CHFL( 0.0 ), CHFL( 0.0 ), CHFL( 0.0 ), CHFL( 0.0 ), CHFL( 0.0 ), CHFL( 0.0 ) );
//	Draw_Skeleton_2d( &NOODLE, CHFL( 0.0 ), CHFL( 0.0 ) );
//
//	SetRotMatrix(&(Plyr->Cam.ViewMatrix));
//	SetTransMatrix(&(Plyr->Cam.ViewMatrix));
//	Draw_Skeleton_3d( &NOODLE, CHFL( 0.0 ) - Plyr->Cam.Pos.X, CHFL( -90.0) - Plyr->Cam.Pos.Y, CHFL( 100.0 ), CHFL( 0.0 ), CHFL( 0.0 ), CHFL( 0.0 ) - Plyr->Cam.Pos.Z );
//
//	SetRotMatrix(&(Plyr->Cam.ViewMatrix));
//	SetTransMatrix(&(Plyr->Cam.ViewMatrix));
//	Temp_Draw_Box( PlayerNumber );








// Definition of the local model

// the polys ( these never change )
uint16 AllNumberModPolys[ 4 ];
model_polys AllModPolys[ 4 ][ MAX_MODEL_POLYS ];

// and the verts ( these change all the time )
uint16 NumberModVerts;
SVECTOR ModVerts[ MAX_MODEL_VERTS ];


void Initialise_Skeleton( void )
{
	static uint8 CalledBefore = 0;
	uint8 i;

	printf("Exec : Initialise_Skeleton");

	if( CalledBefore == 0 )
	{
		// Cold Initialise
		printf(" - Cold");

		CalledBefore = 1;

		for( i = 0; i < 4; i++ )
		{
			Player[ i ].Character = CUBE;	// Each player initially wants to be a cube
			Player[ i ].Loaded = NONE;		// But no character is loaded
		}

	}

	printf("\n");

	// Warm Initialise
	for( i = 0; i < 4; i++ )
	{
		if( Player[ i ].Character != Player[ i ].Loaded )
		{
			// This player does not have the required skeleton.  We need to load it.

			switch( Player[ i ].Character )
			{
				case GOAT:
				{
				}
				break;

				case LOOP:
				{
				}
				break;

				case TRIX:
				{
				}
				break;

				case YOYO:
				{
				}
				break;

				case YETI:
				{
				}
				break;

				case CUBE:
				{
					// Now, it will temporarily set up polys for a cube
					AllNumberModPolys[ i ] = 5;

					AllModPolys[ i ][ 0 ].Type = FLAT;
					AllModPolys[ i ][ 0 ].NumberOfVerts = 4;
					AllModPolys[ i ][ 0 ].VertNum[ 0 ] = 0;
					AllModPolys[ i ][ 0 ].VertNum[ 1 ] = 1;
					AllModPolys[ i ][ 0 ].VertNum[ 2 ] = 2;
					AllModPolys[ i ][ 0 ].VertNum[ 3 ] = 3;
					AllModPolys[ i ][ 0 ].R[ 0 ] = 255;
					AllModPolys[ i ][ 0 ].G[ 0 ] = 255;
					AllModPolys[ i ][ 0 ].B[ 0 ] = 255;

					AllModPolys[ i ][ 1 ].Type = GOURAUD;
					AllModPolys[ i ][ 1 ].NumberOfVerts = 4;
					AllModPolys[ i ][ 1 ].VertNum[ 0 ] = 4;
					AllModPolys[ i ][ 1 ].VertNum[ 1 ] = 5;
					AllModPolys[ i ][ 1 ].VertNum[ 2 ] = 1;
					AllModPolys[ i ][ 1 ].VertNum[ 3 ] = 0;

					AllModPolys[ i ][ 2 ].Type = GOURAUD;
					AllModPolys[ i ][ 2 ].NumberOfVerts = 4;
					AllModPolys[ i ][ 2 ].VertNum[ 0 ] = 1;
					AllModPolys[ i ][ 2 ].VertNum[ 1 ] = 5;
					AllModPolys[ i ][ 2 ].VertNum[ 2 ] = 6;
					AllModPolys[ i ][ 2 ].VertNum[ 3 ] = 2;


					AllModPolys[ i ][ 3 ].Type = GOURAUD;
					AllModPolys[ i ][ 3 ].NumberOfVerts = 3;
					AllModPolys[ i ][ 3 ].VertNum[ 0 ] = 4;
					AllModPolys[ i ][ 3 ].VertNum[ 1 ] = 0;
					AllModPolys[ i ][ 3 ].VertNum[ 2 ] = 3;

					AllModPolys[ i ][ 4 ].Type = FLAT;
					AllModPolys[ i ][ 4 ].NumberOfVerts = 3;
					AllModPolys[ i ][ 4 ].VertNum[ 0 ] = 4;
					AllModPolys[ i ][ 4 ].VertNum[ 1 ] = 3;
					AllModPolys[ i ][ 4 ].VertNum[ 2 ] = 7;

				}
				break;

			}
			Player[ i ].Loaded = Player[ i ].Character;
		}
	}
}





//
//	____________
//	\4		   5\
//	|\		    |\
//	| \		    | \
//	|  \		|  \
//	|   \_______|___\
//	|	 |0	    |  1|
//	|	 |	    |   |
//	|7___|_____6|   |
//	 \   |	    \   |
//	  \  |	     \  |
//	   \ |	      \ |
//	    \|	       \|
//	     \3________2|
//

void Create_Local_Model( void )
{
	// This routine will eventually set up the local vertices




	// now it temporarily set up verts for a cube
	NumberModVerts = 8;
	ModVerts[ 0 ].vx = -10;
	ModVerts[ 0 ].vy = 10;
	ModVerts[ 0 ].vz = -10;

	ModVerts[ 1 ].vx = 10;
	ModVerts[ 1 ].vy = 10;
	ModVerts[ 1 ].vz = -10;

	ModVerts[ 2 ].vx = 10;
	ModVerts[ 2 ].vy = -10;
	ModVerts[ 2 ].vz = -10;

	ModVerts[ 3 ].vx = -10;
	ModVerts[ 3 ].vy = -10;
	ModVerts[ 3 ].vz = -10;

	ModVerts[ 4 ].vx = -10;
	ModVerts[ 4 ].vy = 10;
	ModVerts[ 4 ].vz = 10;

	ModVerts[ 5 ].vx = 10;
	ModVerts[ 5 ].vy = 10;
	ModVerts[ 5 ].vz = 10;

	ModVerts[ 6 ].vx = 10;
	ModVerts[ 6 ].vy = -10;
	ModVerts[ 6 ].vz = 10;

	ModVerts[ 7 ].vx = -10;
	ModVerts[ 7 ].vy = -10;
	ModVerts[ 7 ].vz = 10;

	// later it should transform the basic vertices around their attached transformed points
	// in the oriented skeleton and put them into ModVerts
}

// POLY_G4
// POLY_GT4
// POLY_FT4


int8 AnyType[ 2 ][ 400 * 14 ];	// (this is only big enough for one character at the moment)
void *Ptr;


// Call these once each loop
void Refresh_Skeleton( void )
{
	Ptr = &AnyType[ otid ][ 0 ];	// Point to the start
}

// Each player may be drawn twice you know ( split screen mode )
void Draw_Local_Model( uint8 PlayerNumber )
{
	// Set up these important variables
	uint16 NumberModPolys =	AllNumberModPolys[ PlayerNumber ];	// How many polys this player has
	model_polys *ModPolys = &AllModPolys[ PlayerNumber ][ 0 ];	// Point to this players polygon list
	player *Plyr = &Player[ PlayerNumber ];

	// Loop through all the polygons in ModPolys
	long TVert[ 8 ], P, Otz, Flag;
	uint16 i;

//	int32 RotX = 0, RotY = 0, RotZ = 0;
//	int32 TranX = 0, TranY = 0, TranZ = 0;

	int32 RotX = Plyr->Rot.X;
	int32 RotY = Plyr->Rot.Y;
	int32 RotZ = Plyr->Rot.Z;
	int32 TranX = Plyr->Pos.X;
	int32 TranY = Plyr->Pos.Y;
	int32 TranZ = Plyr->Pos.Z;

	SVECTOR RotVec, X;
	VECTOR Scale, Y;

//	Plyr->Rot.X += ( 1 << 12 );
	Plyr->Rot.Y += ( 1 << 12 );
//	Plyr->Rot.Z += ( 1 << 12 );

	PSX_Matrix_Init( &TransMat );
	PSX_Matrix_Init( &RotMat );

	// Make sure the GTE is in the state we want
	SetGeomOffset( 160, 120 );
	SetRotMatrix( &Plyr->Cam.ViewMatrix );
	SetTransMatrix( &Plyr->Cam.ViewMatrix );

	/* Jeb's superb object ViewMatrix calculations */
	RotVec.vx = RotY >> 12;
	RotVec.vy = RotX >> 12;
	RotVec.vz = ( RotZ >> 12 ) + 1024;
	RotVec.vx += Plyr->Cam.Rot.Y >> 12;
	RotVec.vy += -Plyr->Cam.Rot.X >> 12;
	RotVec.vz += Plyr->Cam.Rot.Z >> 12;

	X.vx = TranX >> 12;
	X.vy = TranY >> 12;
	X.vz = TranZ >> 12;
	X.vx += Plyr->Cam.Pos.X >> 12;
	X.vy -= Plyr->Cam.Pos.Y >> 12;
	X.vz -= Plyr->Cam.Pos.Z >> 12;

	ApplyRotMatrix( &X, &Y );

	TransMat.t[ 1 ] = Y.vx;
	TransMat.t[ 0 ] = Y.vy;
	TransMat.t[ 2 ] = Y.vz;

	RotMatrix( &RotVec, &RotMat );

//	Scale.vx = 1 << 12;
//	Scale.vy = 1 << 12;
//	Scale.vz = 1 << 12;
//	ScaleMatrix( &RotMat, &Scale );

	SetRotMatrix( &RotMat );
	SetTransMatrix( &TransMat );
	/* End of Jeb's superb ViewMatrix calculations */

	// Now transform and render each polygon
	for( i = 0; i < NumberModPolys; i++ )
	{
		if( ModPolys[ i ].NumberOfVerts == 4 )
		{
			// It's a QUAD

			RotAverageNclip4(
				&ModVerts[ ModPolys[ i ].VertNum[ 3 ] ], &ModVerts[ ModPolys[ i ].VertNum[ 2 ] ], &ModVerts[ ModPolys[ i ].VertNum[ 1 ] ], &ModVerts[ ModPolys[ i ].VertNum[ 0 ] ],
				&TVert[ 0 ], &TVert[ 1 ], &TVert[ 2 ], &TVert[ 3 ],
				&P, &Otz, &Flag );

			if( Otz < 4096 && Otz > 0 )
			{
				switch( ModPolys[ i ].Type )
				{
					case FLAT:
					{
						POLY_F4 *F4 = Ptr;
						Ptr += sizeof( POLY_F4 );

						setPolyF4( F4 );
						F4->r0 = 0;
						F4->g0 = 128;
						F4->b0 = 0;

						F4->x0 = TVert[ 0 ] >> 16;
						F4->y0 = TVert[ 0 ] & 0xffff;
						F4->x1 = TVert[ 1 ] >> 16;
						F4->y1 = TVert[ 1 ] & 0xffff;
						F4->x2 = TVert[ 3 ] >> 16;
						F4->y2 = TVert[ 3 ] & 0xffff;
						F4->x3 = TVert[ 2 ] >> 16;
						F4->y3 = TVert[ 2 ] & 0xffff;

						AddPrim( &OrderingTable[ otid ][ 0 ], F4 );
					}
					break;

					case GOURAUD:
					{
						POLY_G4 *G4 = Ptr;
						Ptr += sizeof( POLY_G4 );

						setPolyG4( G4 );

/*
						G4->r0 = 255;
						G4->g0 = 0;
						G4->b0 = 0;
						G4->r1 = 0;
						G4->g1 = 255;
						G4->b1 = 0;
						G4->r2 = 0;
						G4->g2 = 0;
						G4->b2 = 255;
						G4->r3 = 255;
						G4->g3 = 255;
						G4->b3 = 255;
*/
						G4->r0 = 0;
						G4->g0 = 0;
						G4->b0 = 128;
						G4->r1 = 0;
						G4->g1 = 0;
						G4->b1 = 128;
						G4->r2 = 0;
						G4->g2 = 0;
						G4->b2 = 128;
						G4->r3 = 0;
						G4->g3 = 0;
						G4->b3 = 128;


						G4->x0 = TVert[ 0 ] >> 16;
						G4->y0 = TVert[ 0 ] & 0xffff;
						G4->x1 = TVert[ 1 ] >> 16;
						G4->y1 = TVert[ 1 ] & 0xffff;
						G4->x2 = TVert[ 3 ] >> 16;
						G4->y2 = TVert[ 3 ] & 0xffff;
						G4->x3 = TVert[ 2 ] >> 16;
						G4->y3 = TVert[ 2 ] & 0xffff;

						AddPrim( &OrderingTable[ otid ][ 0 ], G4 );
					}
					break;

					case TEXTURED:
					{
						POLY_GT4 *GT4 = Ptr;
						printf("We haven't catered for textured polys - yet\n");
					}
					break;



				}
			}
			// End of QUAD render
		}
		else
		{
			// It's a TRIANGLE

			RotAverageNclip3(
				&ModVerts[ ModPolys[ i ].VertNum[ 2 ] ], &ModVerts[ ModPolys[ i ].VertNum[ 1 ] ], &ModVerts[ ModPolys[ i ].VertNum[ 0 ] ],
				&TVert[ 0 ], &TVert[ 1 ], &TVert[ 2 ],
				&P, &Otz, &Flag );

			if( Otz < 4096 && Otz > 0 )
			{
				switch( ModPolys[ i ].Type )
				{
					case FLAT:
					{
						POLY_F3 *F3 = Ptr;
						Ptr += sizeof( POLY_F3 );

						setPolyF3( F3 );
						F3->r0 = 0;
						F3->g0 = 128;
						F3->b0 = 0;

						F3->x0 = TVert[ 0 ] >> 16;
						F3->y0 = TVert[ 0 ] & 0xffff;
						F3->x1 = TVert[ 1 ] >> 16;
						F3->y1 = TVert[ 1 ] & 0xffff;
						F3->x2 = TVert[ 2 ] >> 16;
						F3->y2 = TVert[ 2 ] & 0xffff;

		 				AddPrim( &OrderingTable[ otid ][ 0 ], F3 );
					}
					break;

					case GOURAUD:
					{
						POLY_G3 *G3 = Ptr;
						Ptr += sizeof( POLY_G3 );

						setPolyG3( G3 );

/*
						G4->r0 = 255;
						G4->g0 = 0;
						G4->b0 = 0;
						G4->r1 = 0;
						G4->g1 = 255;
						G4->b1 = 0;
						G4->r2 = 0;
						G4->g2 = 0;
						G4->b2 = 255;
*/
						G3->r0 = 128;
						G3->g0 = 128;
						G3->b0 = 0;
						G3->r1 = 128;
						G3->g1 = 0;
						G3->b1 = 128;
						G3->r2 = 0;
						G3->g2 = 0;
						G3->b2 = 128;


						G3->x0 = TVert[ 0 ] >> 16;
						G3->y0 = TVert[ 0 ] & 0xffff;
						G3->x1 = TVert[ 1 ] >> 16;
						G3->y1 = TVert[ 1 ] & 0xffff;
						G3->x2 = TVert[ 2 ] >> 16;
						G3->y2 = TVert[ 2 ] & 0xffff;

						AddPrim( &OrderingTable[ otid ][ 0 ], G3 );
					}
					break;

					case TEXTURED:
					{
						POLY_GT3 *GT3 = Ptr;
						printf("We haven't catered for textured polys - yet\n");
					}
					break;
				}
			}
			// End of TRIANGLE render
		}
	}

	// Restore the old state of the GTE
	SetGeomOffset( 120, 160 );
	SetRotMatrix( &Plyr->Cam.ViewMatrix );
	SetTransMatrix( &Plyr->Cam.TransMatrix );
}





