// LOGIC.C

#include <stdio.h>
#include "sys/types.h"
#include "LibGTE.h"
#include "LibGPU.h"
#include "LibCD.h"
#include "LibComb.h"
#include "LibEtc.h"
#include "LibGS.h"
#include "LibMath.h"
#include "LibPress.h"
#include "LibSN.h"
#include "LibSnd.h"
#include "LibSPU.h"
#include "ourtypes.h"
#include "main.h"
#include "mathbits.h"
#include "camera.h"
#include "pads.h"
#include "tracks.h"
#include "players.h"
#include "logic.h"
#include "animctrl.h"
#include "bonebag.h"
#include "scores.h"
#include "ghostrep.h"
#include "parachut.h"
#include "camlogic.h"
#include "tracks.h"

//#include "m2debug.h"

#define JEBS_COLLISION	1
#define PARACHUTE_CODE	1
#define ADES_COLLISION	0
#define USE_PADD				1
#define USE_RAND				1

#include "sine.h"

uint16 Modes[] =
{
	MD_ALLOWTUCK + MD_ALLOWAUTOJUMP + MD_ALLOWBRAKE,		// 00 STY_NORMAL
	MD_ALLOWTUCK + MD_ALLOWAUTOJUMP,						// 01 STY_TURNING
	MD_ALLOWTUCK + MD_ALLOWAUTOJUMP,						// 02 STY_PRE_TURNING
	MD_NOVEL,												// 03 STY_DOLMIO
	MD_ALLOWAUTOJUMP,										// 04 STY_PRE_JUMP
	MD_ALLOWAUTOJUMP,										// 05 STY_PREP_JUMP
	0,														// 06 STY_SJUMPM
	0,														// 07 STY_SJUMPPREPLANDM
	MD_ALLOWGRAB + MD_AIRTURNS,								// 08 STY_SJUMPL
	MD_AIRTURNS,											// 09 STY_AUTOJUMP (mode not used)
	MD_NOGROUNDORIENTBOARD + MD_RECOVERY + MD_ALLOWGRAB,	// 10 STY_FRONTSPIN
	0,														// 11 STY_FSLAND
	MD_NOGROUNDORIENTBOARD + MD_RECOVERY + MD_ALLOWGRAB,	// 12 STY_BACKSPIN
	0,														// 13 STY_BSLAND
	MD_NOVEL + MD_NOGROUNDORIENTBOARD,						// 14 STY_RECOVERY
	MD_NOVEL + MD_NOGROUNDORIENTBOARD,						// 15 STY_RECOVERED
	MD_NOVEL + MD_NOGROUNDORIENTBOARD,						// 16 STY_RECLAND1
	MD_NOVEL,												// 17 STY_RECLAND2
	MD_ALLOWGRAB + MD_AIRTURNS,								// 18 STY_SJUMPM_AIR
	MD_NOGROUNDORIENTBOARD,									// 19 STY_LANDONHEAD
	MD_NOGROUNDORIENTBOARD + MD_NOVEL,						// 20 STY_LANDONLEFT
	MD_NOGROUNDORIENTBOARD,									// 21 STY_LANDONRIGHT
	MD_ALLOWAUTOJUMP,										// 22 STY_TUCK
	MD_NOVEL,												// 23 STY_TREESOFT
	MD_NOVEL,												// 24 STY_TREEBUMP
	MD_NOVEL,												// 25 STY_TREESMACK
	MD_NOVEL,												// 26 STY_BRAKE
	0,														// 27 STY_ENDBRAKE
	MD_NOVEL,												// 28 STY_LATEGRAB
	MD_NOGROUNDORIENTBOARD + MD_ALLOWGRAB,					// 29 STY_TWISTR
	MD_NOGROUNDORIENTBOARD,									// 30 STY_TWISTRLAND
	MD_NOGROUNDORIENTBOARD + MD_ALLOWGRAB,					// 31 STY_TWISTL
	MD_NOGROUNDORIENTBOARD + MD_RECOVERY + MD_ALLOWGRAB,	// 32 STY_TWISTANDSPIN
	MD_NOGROUNDORIENTBOARD + MD_NOVEL + MD_NOBONK,			// 33 STY_STUMPTRIP
	MD_NOVEL,												// 34 STY_BONK
	MD_NOGROUNDORIENTBOARD,									// 35 STY_TWISTLB
	MD_NOGROUNDORIENTBOARD,									// 36 STY_TWISTLF
	MD_NOVEL,												// 37 STY_BONK1
	MD_NOVEL + MD_NOGROUNDORIENTBOARD + MD_NOCOLLISION,		// 38 STY_RAILSLIDE
	MD_NOVEL + MD_NOGROUNDORIENTBOARD,						// 39 STY_ENDRAILSLIDE
	MD_NOVEL + MD_NOGROUNDORIENTBOARD,						// 40 STY_ST_ROLL
	MD_NOVEL + MD_NOGROUNDORIENTBOARD,						// 41 STY_END_ST_ROLL
	MD_NOVEL + MD_NOGROUNDORIENTBOARD,						// 42 STY_ST_RECOVERY
	MD_NOVEL + MD_NOGROUNDORIENTBOARD,						// 43 STY_ST_RECOVERY2
	MD_NOVEL + MD_NOGROUNDORIENTBOARD,						// 44 STY_OFFRAIL
	MD_NOVEL + MD_NOGROUNDORIENTBOARD,						// 45 STY_CHILLTRICK
	0,														// 46 STY_OLLIE
	0,	// 47
	0,	// 48
	0,	// 49
	0,	// 50
	0,	// 51
	0,	// 52
};


void BoardModel( uint8 PlayerNumber )
{
	player *Plyr = &Player[ PlayerNumber ];

	switch( Plyr->Transport )
	{
		case TESTCAM:
			/* TESTCAM mode has no actual logic to move the board */
			break;

		case SPACEHOPPER:
			SpaceHopper( PlayerNumber );
			break;

		case SNOWBOARD:
			Snowboard( PlayerNumber );
			break;

	}
}


void InitBoardModel( uint8 PlayerNumber )
{
	player *Plyr = &Player[ PlayerNumber ];

	switch( Plyr->Transport )
	{
		case TESTCAM:
			InitialiseTestCam( PlayerNumber );
			break;

		case SPACEHOPPER:
			InitialiseSpaceHopper( PlayerNumber );
			break;

		case SNOWBOARD:
			InitialiseSnowboard( PlayerNumber );
			break;
	}
}



void InitialiseTestCam( uint8 PlayerNumber )
{
	player *Plyr = &Player[ PlayerNumber ];

	// As this is a debug mode then set these start parameters to your convenience
	SetVec3( &Plyr->Cam.Pos, CHFL( 0.0 ), CHFL( 0.0 ), CHFL( -50.0 ) );
	SetVec3( &Plyr->Cam.Rot, CHFL( 0.0 ), CHFL( 0.0 ), CHFL( 0.0 ) );
}



void InitialiseSpaceHopper( uint8 PlayerNumber )
{
	player *Plyr = &Player[ PlayerNumber ];

	printf("InitialiseSpaceHopper\n");

	Plyr->Transport = SPACEHOPPER;

//	Plyr->Pos.Y = GetYAt( Plyr->Pos.X, Plyr->Pos.Z, PlayerNumber ) + CHFL( 500.0 );

	Plyr->TurnInert = CHFL( 0.0 );
	Plyr->Vel = CHFL( 0.0 );

	//Plyr->Cam.Mode = CAMERA_PARACHUTE;
	Plyr->Style = PARA_STY_AIR;
	//InitCameraModel( PlayerNumber );

}



#define MAX_TURN_INERT	CHFL( 5.0 * ( 4096.0 / 360.0 ) )
chfloat DigPadTurnSense = CHFL( 0.35 );


/* SpaceHopper logic - Milestone 2 simple boarding engine, could be used for a SpaceHopper cheat mode later on */

void SpaceHopper( uint8 PlayerNumber )
{
	vec3 MV, FMV;
	player *Plyr = &Player[ PlayerNumber ];
	uint32	Pad = Pad_State( Plyr->PadNumber );
#if USE_PADD
	uint32	PadD = Pad_Debounced( Plyr->PadNumber );
#endif
	long	para_swap;

	MV.X = MV.Y = MV.Z = CHFL( 0.0 );

	Plyr->LastPos.X = Plyr->Pos.X;
	Plyr->LastPos.Y = Plyr->Pos.Y;
	Plyr->LastPos.Z = Plyr->Pos.Z;

#if PARACHUTE_CODE
	switch( Plyr->Style )
	{
		case PARA_STY_AIR:

		// Is the pad a digital pad ?
			if( Pad_Type( Plyr->PadNumber ) == PAD_DIGITAL )
			{
				// Steering from the D-Pad left and right buttons
				if( Pad & ( PAD_LEFT | PAD_RIGHT ) )	/* Is left or right being pressed */
				{
					/* Left or Right is pressed, decide which one and then fiddle the turning velocity */

					if( Pad & PAD_LEFT )
					{
						if( Plyr->TurnInert > -MAX_TURN_INERT )
						{
							Plyr->TurnInert -= DigPadTurnSense;

							if( Plyr->TurnInert > CHFL( 0.0 ) )
							{
								Plyr->TurnInert -= DigPadTurnSense;
							}
						}
						else
						{
							Plyr->TurnInert = -MAX_TURN_INERT;
						}
					}

					if( Pad & PAD_RIGHT )
					{
						if( Plyr->TurnInert < MAX_TURN_INERT )
						{
							Plyr->TurnInert += DigPadTurnSense;

							if( Plyr->TurnInert < CHFL( 0.0 ) )
							{
								Plyr->TurnInert += DigPadTurnSense;
							}
						}
						else
						{
							Plyr->TurnInert = MAX_TURN_INERT;
						}
					}
				}
				else
				{
					/* None is pressed, fiddle the turning velocity to gradually decrease to 0 ( inertia effect for digital pads ) */

					if( Plyr->TurnInert > CHFL( 0.0 ) )
					{
						Plyr->TurnInert -= ( DigPadTurnSense * 2 );

						if( Plyr->TurnInert < CHFL( 0.0 ) )	/* Back to 0 ? */
						{
							Plyr->TurnInert = CHFL( 0.0 );
						}
					}

					if( Plyr->TurnInert < CHFL( 0.0 ) )
					{
						Plyr->TurnInert += ( DigPadTurnSense * 2 );

						if( Plyr->TurnInert > CHFL( 0.0 ) )	/* Back to 0 ? */
						{
							Plyr->TurnInert = CHFL( 0.0 );
						}
					}
				}
			}

			Plyr->Rot.Y += Plyr->TurnInert;


			/*
			if( Pad & PAD_B2 )
			{
				if( Plyr->Vel < CHFL( 16.0 ) )
					Plyr->Vel += CHFL( 0.5 );
			}
			else
			{
				Plyr->Vel -= CHFL( 0.5 );
				if( Plyr->Vel < CHFL( 0.0 ) )
				{
					Plyr->Vel = CHFL( 0.0 );
				}
			}
			*/



			if( Pad & PAD_B2 )
			{
				// Flare
				Plyr->Vel -= CHFL( 0.01 );
				Plyr->Grav += CHFL( 0.05 );

				if( Plyr->Vel <= CHFL( 0.0 ) ) Plyr->Vel = CHFL( 0.0 );
				if( Plyr->Vel > CHFL( 1.5 ) ) Plyr->Vel = CHFL( 1.5 );
				if( Plyr->Grav >= CHFL( 5.0 ) ) Plyr->Grav = CHFL( 5.0 );

			}
			else
			{
				// No flare

				Plyr->Vel += CHFL( 0.01 );

				if(Plyr->Grav > CHFL(1.0))
					Plyr->Grav -= CHFL(0.1);
				else
				{
					Plyr->Grav += CHFL( 0.025 );
					if( Plyr->Grav >= CHFL( 1.0 ) ) Plyr->Grav = CHFL( 1.0 );
				}

				if( Plyr->Vel > CHFL( 1.5 ) ) Plyr->Vel = CHFL( 1.5 );
			}

			MV.Z = Plyr->Vel;

			/* Now rotate the movement vector */
			FMV.X = ( MV.X * Cosine( Plyr->Rot.Y>>PSX_PREC ) ) - ( MV.Z * Sine( Plyr->Rot.Y>>PSX_PREC ) );
			FMV.Z = ( MV.Z * Cosine( Plyr->Rot.Y>>PSX_PREC ) ) + ( MV.X * Sine( Plyr->Rot.Y>>PSX_PREC ) );
			FMV.Y = CHFL( 0.0 );	// This will be jumping

			// Going forwards
			Plyr->Pos.X -= CHSHIFTR( FMV.X );
			Plyr->Pos.Z += CHSHIFTR( FMV.Z );

			Plyr->Pos.Y -= Plyr->Grav;

			//Plyr->TerrainY = -51503104;
			Plyr->TerrainY = GetYAt( Plyr->Pos.X, Plyr->Pos.Z, PlayerNumber );

			Plyr->JumpY = Plyr->Pos.Y - Plyr->TerrainY;


			//CHFL( -1500.0 );
			//GetYAt( Plyr->Pos.X, Plyr->Pos.Z, PlayerNumber );

			if(parachute_counter > 130)
			{
				if( Plyr->Pos.Y <= Plyr->TerrainY )
				{
					Plyr->Pos.Y = Plyr->TerrainY;
					Plyr->Style = PARA_STY_LANDED;
					Plyr->Cam.SubMode = PARA_LANDED;
					russ_cam_rot = 128;
					Plyr->Cam.XSweepAngle = CHFL(512.0);
					Plyr->Cam.SweepAngle = CHFL(0.0);
				}
			}
			break;

		case PARA_STY_LANDED:
			break;

		case PARA_STY_FLY:
			/*if( Pad_Type( Plyr->PadNumber ) == PAD_DIGITAL )
			{
				if( Pad & ( PAD_LEFT | PAD_RIGHT ) )
				{

					if( Pad & PAD_LEFT )
					{
						if( Plyr->TurnInert > -MAX_TURN_INERT )
						{
							Plyr->TurnInert -= DigPadTurnSense;

							if( Plyr->TurnInert > CHFL( 0.0 ) )
							{
								Plyr->TurnInert -= DigPadTurnSense;
							}
						}
						else
						{
							Plyr->TurnInert = -MAX_TURN_INERT;
						}
					}

					if( Pad & PAD_RIGHT )
					{
						if( Plyr->TurnInert < MAX_TURN_INERT )
						{
							Plyr->TurnInert += DigPadTurnSense;

							if( Plyr->TurnInert < CHFL( 0.0 ) )
							{
								Plyr->TurnInert += DigPadTurnSense;
							}
						}
						else
						{
							Plyr->TurnInert = MAX_TURN_INERT;
						}
					}
				}
				else
				{

					if( Plyr->TurnInert > CHFL( 0.0 ) )
					{
						Plyr->TurnInert -= ( DigPadTurnSense * 2 );

						if( Plyr->TurnInert < CHFL( 0.0 ) )
						{
							Plyr->TurnInert = CHFL( 0.0 );
						}
					}

					if( Plyr->TurnInert < CHFL( 0.0 ) )
					{
						Plyr->TurnInert += ( DigPadTurnSense * 2 );

						if( Plyr->TurnInert > CHFL( 0.0 ) )
						{
							Plyr->TurnInert = CHFL( 0.0 );
						}
					}
				}
			}

			Plyr->Rot.Y += Plyr->TurnInert;*/


			/*if( Pad & PAD_B2 )
			{
				// Flare
				Plyr->Vel -= CHFL( 0.01 );

				Plyr->Grav += CHFL( 0.05 );

				if( Plyr->Vel <= CHFL( 0.0 ) ) Plyr->Vel = CHFL( 0.0 );
				if( Plyr->Grav >= CHFL( 5.0 ) ) Plyr->Grav = CHFL( 5.0 );

			}
			else
			{
				// No flare
				Plyr->Vel += CHFL( 0.01 );
				Plyr->Grav -= CHFL( 0.025 );

				if( Plyr->Vel > CHFL( 1.5 ) ) Plyr->Vel = CHFL( 1.5 );
				if( Plyr->Grav <= CHFL( 0.5 ) ) Plyr->Grav = CHFL( 0.5 );

			} */

			Plyr->Grav += CHFL(0.50);
			if( Plyr->Grav >= CHFL( 12.0 ) )
				Plyr->Grav = CHFL( 12.0 );


			if(Plyr->Fakie)
				MV.Z = -Plyr->Vel;
			else
				MV.Z = Plyr->Vel;

			/* Now rotate the movement vector */
			FMV.X = ( MV.X * Cosine( Plyr->Rot.Y>>PSX_PREC ) ) - ( MV.Z * Sine( Plyr->Rot.Y>>PSX_PREC ) );
			FMV.Z = ( MV.Z * Cosine( Plyr->Rot.Y>>PSX_PREC ) ) + ( MV.X * Sine( Plyr->Rot.Y>>PSX_PREC ) );
			FMV.Y = CHFL( 0.0 );	// This will be jumping

			// Going forwards
			Plyr->Pos.X -= CHSHIFTR( FMV.X );
			Plyr->Pos.Z += CHSHIFTR( FMV.Z );

			Plyr->Pos.Y -= Plyr->Grav;

			Plyr->TerrainY = GetYAt( Plyr->Pos.X, Plyr->Pos.Z, PlayerNumber );

			Plyr->JumpY = Plyr->Pos.Y - Plyr->TerrainY;


			/*if( Plyr->Pos.Y <= Plyr->TerrainY )
			{
				Plyr->Pos.Y = Plyr->TerrainY;
				Plyr->Style = PARA_STY_LANDED;
				Plyr->Cam.SubMode = PARA_LANDED;
			} */
			break;


	}

	if(parachute_counter == 129)
	{
		SetPose( PlayerNumber, B_STW2 );

		Plyr->Pos.X = -102298231;
		Plyr->Pos.Y = -48000000;
		Plyr->Pos.Z = 309129196;

		IBuf[0]->CurrentStrip = 18;
		Plyr->Track->NumberOfStrips = 0;
		//InitCameraModel( PlayerNumber );

	}

	parachute_height = Plyr->Pos.Y - GetYAt( Plyr->Pos.X, Plyr->Pos.Z, PlayerNumber );

//	Plyr->Pos.Y += MV.Y;
//	Plyr->Pos.Y = GetYAt( Plyr->Pos.X, Plyr->Pos.Z, PlayerNumber );// + CHFL( 32.0 );
#endif
}














// *************************************
// *** The actual snowboarding logic ***
// *************************************

// Adjustable stuff

// Note : In adjusting any of these don't forget that most values combine with others.  Therefore
// they don't need adjusting by large amounts in particular the board and character stats.  For example
// making one character do a much larger jump than anybody else will make him do a ridiculous jump when
// combined with a board of a similar nature.

#define SWING_SPEED			5.0			// How fast you can turn your upper body
#define CROUCH_SWING_SPEED	0.75		// How fast you can turn your upper body whilst crouching to jump
#define TUCK_SWING_SPEED	1.5			// How fast you can turn your upper body whilst in the tuck
#define SWING_BACK_SPEED	5.0			// How fast the upper body snaps back when you apply no turning to it 2.0

#define AIR_TURN_SPEED		2.5			// How fast you can turn your body in the air

#define MIN_JUMP_SPEED		4.0			// The minimum jumping speed
#define MAX_JUMP_SPEED		9.5			// The maximum jumping speed
#define HOLD_JUMP_KEY_ACCEL	0.05		// The jumping speed increase between MIN and MAX when holding jump
#define SMALL_JUMP_MAX		7.0			// The cut off speed to distinguish a small jump from a big one

#define MAX_FALL_SPEED		30.0		// The maximum fall speed

#define CENTRE_STAGE		50			// Central stance frame stage


// Adjust these to tweak the basic turning speeds for all characters
chfloat TurnSpeeds[] = {
	CHFL( 1.500 ),CHFL( 0.550 ),CHFL( 0.600 ),CHFL( 0.700 ),CHFL( 0.800 ),
	CHFL( 0.900 ),CHFL( 1.033 ),CHFL( 1.133 ),CHFL( 1.233 ),CHFL( 1.333 ),
	CHFL( 1.333 ),CHFL( 1.233 ),CHFL( 1.133 ),CHFL( 1.033 ),CHFL( 1.000 ),
	CHFL( 1.000 ),CHFL( 0.900 ),CHFL( 0.900 ),CHFL( 0.900 ),CHFL( 0.800 ),
	CHFL( 0.800 ),CHFL( 0.800 ),CHFL( 0.700 ),CHFL( 0.700 ),CHFL( 0.700 ),
	CHFL( 0.600 ),CHFL( 0.600 ),CHFL( 0.600 ),CHFL( 0.600 ),CHFL( 0.500 )
};

// Adjust these to tweak the individual turning speeds of each character
// The number is added onto the number plucked from TurnSpeeds
chfloat Character_Turning[] = {
	CHFL( 0.0 ),	// Ignore this
	CHFL( 0.0 ),	// BONE
	CHFL( 0.0 ),	// TRIX
	CHFL( 0.0 ),	// GOAT
	CHFL( 0.0 ),	// YOYO
	CHFL( 0.0 )		// YETI
};

// Adjust these to tweak the maximum speeds of each player ( 14.0 = current normal speed )
chfloat Character_Speeds[] = {
	CHFL( 14.0 ),	// Ignore this
	CHFL( 14.0 ),	// BONE
	CHFL( 14.0 ),	// TRIX
	CHFL( 14.0 ),	// GOAT
	CHFL( 14.0 ),	// YOYO
	CHFL( 14.0 )	// YETI
};

// Adjust these to tweak the MIN_JUMP_SPEED of each character ( this is added on to MIN/MAX_JUMP_SPEED so don't adjust it much )
chfloat Character_Jumps[] = {
	CHFL( 0.0 ),	// Ignore this
	CHFL( 0.0 ),	// BONE
	CHFL( 0.0 ),	// TRIX
	CHFL( 0.0 ),	// GOAT
	CHFL( 0.0 ),	// YOYO
	CHFL( 0.0 )		// YETI
};

//chfloat Board_Turning[] = { };
//chfloat Board_Speeds[] = { };
//chfloat Board_Jumping[] = { };

uint8 Dolmio = 0;

void InitialiseSnowboard( uint8 PlayerNumber )
{
	player *Plyr = &Player[ PlayerNumber ];

	// This stuff should be overridden
	SetVec3( &Plyr->Pos, CHFL( 0.0 ), CHFL( 0.0 ), CHFL( 0.0 ) );
	SetVec3( &Plyr->Rot, CHFL( 0.0 ), CHFL( 0.0 ), CHFL( 0.0 ) );
	Plyr->LastTerrainY = Plyr->TerrainY = CHFL( 0.0 );
	Plyr->MoveAzimuth = Plyr->LastBoardAzimuth = Plyr->BoardAzimuth = CHFL( 0.0 );





	Plyr->BoardRoll = Plyr->LastBoardRoll = CHFL( 0.0 );
	Plyr->BoardElevation = Plyr->LastBoardElevation = CHFL( 0.0 );

	Plyr->TurnInert = CHFL( 0.0 );
	Plyr->Vel = CHFL( 0.0 );

	Plyr->TurnX = CHFL( 0.0 );
	Plyr->TurnY = CHFL( 0.0 );

	Plyr->Style = STY_NORMAL;

	Plyr->Fakie = 0;

	SetTween( PlayerNumber, FRAMEEQU( PlayerNumber, B_STA1_H1 ), FRAMEEQU( PlayerNumber, B_STA2_H1 ), 50, 25 );
	Plyr->AnimStage = CENTRE_STAGE;
	Plyr->AnimTween = 25;
	Plyr->AnimFrame = 0;
	Plyr->TurnYStage = 0;
	Plyr->JumpHeld = 0;

	Plyr->SideSlip = CHFL( 0.0 );

	Plyr->GrabHeld = 0;
	Plyr->HaveGrabbed = 0;

	Plyr->JumpPower = CHFL( 0.0 );
	Plyr->JumpY = CHFL( 0.0 );
	Plyr->JumpD = 0;

	Plyr->SlowTransition = 0;
}


// ****************
// Animation tables
// ****************

// Boring frame tables for the animation logic
uint32 TurnTable[ 4 ][ 8 ] =
{
	{ B_LEFTX_H1, B_LEFT3_H1, B_LEFT2_H1, B_LEFT1_H1, B_RIGHT1_H1, B_RIGHT2_H1, B_RIGHT3_H1, B_RIGHTX_H1, },
	{ B_LEFTX_H2, B_LEFT3_H2, B_LEFT2_H2, B_LEFT1_H2, B_RIGHT1_H2, B_RIGHT2_H2, B_RIGHT3_H2, B_RIGHTX_H2, },
	{ B_LEFTX_H3, B_LEFT3_H3, B_LEFT2_H3, B_LEFT1_H3, B_RIGHT1_H3, B_RIGHT2_H3, B_RIGHT3_H3, B_RIGHTX_H3, },
	{ B_LEFTX_H4, B_LEFT3_H4, B_LEFT2_H4, B_LEFT1_H4, B_RIGHT1_H4, B_RIGHT2_H4, B_RIGHT3_H4, B_RIGHTX_H4, }
};

uint32 FakieTurnTable[ 4 ][ 8 ] =
{
	{ B_LEFTX_H1, B_FLEFT3_H1, B_FLEFT2_H1, B_FLEFT1_H1, B_FRIGHT1_H1, B_FRIGHT2_H1, B_FRIGHT3_H1, B_RIGHTX_H1, },
	{ B_LEFTX_H2, B_FLEFT3_H2, B_FLEFT2_H2, B_FLEFT1_H2, B_FRIGHT1_H2, B_FRIGHT2_H2, B_FRIGHT3_H2, B_RIGHTX_H2, },
	{ B_LEFTX_H3, B_FLEFT3_H3, B_FLEFT2_H3, B_FLEFT1_H3, B_FRIGHT1_H3, B_FRIGHT2_H3, B_FRIGHT3_H3, B_RIGHTX_H3, },
	{ B_LEFTX_H4, B_FLEFT3_H4, B_FLEFT2_H4, B_FLEFT1_H4, B_FRIGHT1_H4, B_FRIGHT2_H4, B_FRIGHT3_H4, B_RIGHTX_H4, }
};

uint32 NormalTable[ 4 ][ 4 ] =
{
	{ B_LEAN2_H1, B_STA1_H1, B_STA2_H1, B_LEAN1_H1 },
	{ B_LEAN2_H2, B_STA1_H2, B_STA2_H2, B_LEAN1_H2 },
	{ B_LEAN2_H3, B_STA1_H3, B_STA2_H3, B_LEAN1_H3 },
	{ B_LEAN2_H4, B_STA1_H4, B_STA2_H4, B_LEAN1_H4 },
};

uint32 FakieNormalTable[ 4 ][ 4 ] =
{
	{ B_LEAN2_H1, B_FSTA1_H1, B_FSTA2_H1, B_LEAN1_H1 },
	{ B_LEAN2_H2, B_FSTA1_H2, B_FSTA2_H2, B_LEAN1_H2 },
	{ B_LEAN2_H3, B_FSTA1_H3, B_FSTA2_H3, B_LEAN1_H3 },
	{ B_LEAN2_H4, B_FSTA1_H4, B_FSTA2_H4, B_LEAN1_H4 },
};



int16 RockAround[] = { 6,6,5,5,4,4,3,3,2,2,2,1,1,1,1,0,0,0,0,0,1,1,1,1,2,2,2,3,3,4,4,5,5,6,6,6,7,7,7,8,8,9,9,10,10,11,11,12,12,12,13,13,13,13,14,14,14,14,14,13,13,13,13,12,12,12,11,11,10,10,9,9,8,8,7,7,-128 };

uint32 TuckTable[ 4 ][ 3 ] =
{
	{ B_SP1_H1, B_SP2_H1, B_SP3_H1 },
	{ B_SP1_H2, B_SP2_H2, B_SP3_H2 },
	{ B_SP1_H3, B_SP2_H3, B_SP3_H3 },
	{ B_SP1_H4, B_SP2_H4, B_SP3_H4 },
};

uint32 TuckLeftTable[ 4 ][ 2 ] =
{
	{ B_SP3_H1, B_SPL_H1 },
	{ B_SP3_H2, B_SPL_H2 },
	{ B_SP3_H3, B_SPL_H3 },
	{ B_SP3_H4, B_SPL_H4 },
};

uint32 TuckRightTable[ 4 ][ 2 ] =
{
	{ B_SP3_H1, B_SPR_H1 },
	{ B_SP3_H2, B_SPR_H2 },
	{ B_SP3_H3, B_SPR_H3 },
	{ B_SP3_H4, B_SPR_H4 },
};


uint32 FTuckTable[ 4 ][ 3 ] =
{
	{ B_FSP1_H1, B_FSP2_H1, B_FSP3_H1 },
	{ B_FSP1_H2, B_FSP2_H2, B_FSP3_H2 },
	{ B_FSP1_H3, B_FSP2_H3, B_FSP3_H3 },
	{ B_FSP1_H4, B_FSP2_H4, B_FSP3_H4 },
};

uint32 FTuckLeftTable[ 4 ][ 2 ] =
{
	{ B_FSP3_H1, B_FSPL_H1 },
	{ B_FSP3_H2, B_FSPL_H2 },
	{ B_FSP3_H3, B_FSPL_H3 },
	{ B_FSP3_H4, B_FSPL_H4 },
};

uint32 FTuckRightTable[ 4 ][ 2 ] =
{
	{ B_FSP3_H1, B_FSPR_H1 },
	{ B_FSP3_H2, B_FSPR_H2 },
	{ B_FSP3_H3, B_FSPR_H3 },
	{ B_FSP3_H4, B_FSPR_H4 },
};





uint32 PreTurnTableL[] = { B_LEFT1_H1, B_LEFT1_H2, B_LEFT1_H3, B_LEFT1_H4 };
uint32 PreTurnTableR[] = { B_RIGHT1_H1, B_RIGHT1_H2, B_RIGHT1_H3, B_RIGHT1_H4 };

uint32 FakiePreTurnTableL[] = { B_FLEFT1_H1, B_FLEFT1_H2, B_FLEFT1_H3, B_FLEFT1_H4 };
uint32 FakiePreTurnTableR[] = { B_FRIGHT1_H1, B_FRIGHT1_H2, B_FRIGHT1_H3, B_FRIGHT1_H4 };

uint32 FakieSJUMPM_List[] = { B_FJUMP_4, B_FJUMP_5, B_FJUMP_6 };
uint32 SJUMPM_List[] = { B_SJUMPM_4, B_SJUMPM_5, B_SJUMPM_6 };
int16 SJUMPM_Time[] = { 4,3,4 };

uint32 FakieSJUMPPREPLANDM_List[] = { B_FJUMP_8, B_FJUMP_9, B_FJUMP_10 };
uint32 SJUMPPREPLANDM_List[] = { B_SJUMPM_8, B_SJUMPM_9, B_SJUMPM_10 };
int16 SJUMPPREPLANDM_Time[] = { 5,5,8 };

uint32 SJUMPL_List[] = { B_SJUMPL_1, B_SJUMPL_2, B_SJUMPL_2, B_SJUMPM_9, B_SJUMPM_10 };
int16 SJUMPL_Time[] = { 3,10,1,5,5 };

uint32 FRONTSPIN_List[] = { B_FS2, B_FS3, B_FS4, B_FS5, B_FS6 };
int16 FRONTSPIN_Time[] = { 5, 7, 5, 3, 6, -128 };

uint32 BACKSPIN_List[] = { B_FS2, B_FS3, B_BS4, B_FS5, B_FS6 };
int16 BACKSPIN_Time[] = { 5, 7, 5, 3, 6, -128 };

uint32 RECOVERY_List[] = { B_RCYC1, B_RCYC2, B_RCYC3, B_RCYC4, B_RCYC5, B_RCYC6 };
int16 RECOVERY_Time[] = { 5,5,5,5,5,5,-128 };

uint32 RECOVERED_List[] = { B_RCYC8, B_RCYC9, B_RCYC8 };
// B_RCYC10, B_RCYC11, B_RCYC12, B_RCYC13 };
int16 RECOVERED_Time[] = { 15,15,15,-128,4,4,4,4,-128 };

uint32 RECLAND2_List[] = { B_RCYC16, B_RCYC17, B_RCYC18, B_RCYC19, B_RCYC20 };
int16 RECLAND2_Time[] = { 1,3,3,4,4,-128 };

uint32 BONEGRAB1_List[] = {
//B_MUT1, B_MUT2, B_MUT3, B_MUT4,
B_MUT5, B_MUT6, B_MUT7 };
int16 BONEGRAB1_Time[] = {
//8,8,5,3,
4,4,4,-128 };

uint32 BONEGRAB2_List[] = {
//B_MET1, B_MET2, B_MET3, B_MET4,
B_MET5, B_MET6, B_MET7 };
int16 BONEGRAB2_Time[] = {
//10,3,4,4,
4,4,4,-128 };

uint32 LANDONHEAD_List[] = { B_HCR2, B_HCR3, B_HCR2, B_HCR3,B_HCR2, B_HCR3, B_HCR2, B_HCR3, B_HCR4, B_HCR5, B_HCR6, B_HCR7, B_HCR8, B_HCR9, B_HCR10, B_HCR11, B_HCR12 };
int16 LANDONHEAD_Time[] = { 3,3,3,3,7,7,7,7,3,5,5,5,5,5,10,20,20, -128 };



vec3 LANDONHEAD_Rots[] =
{
	{ SPONG( 90.0 ), SPONG( 0.0 ), SPONG( 0.0 ) },
	{ SPONG( 90.0 ), SPONG( 0.0 ), SPONG( 0.0 ) },
	{ SPONG( 90.0 ), SPONG( 0.0 ), SPONG( 0.0 ) },
	{ SPONG( 90.0 ), SPONG( 0.0 ), SPONG( 0.0 ) },
	{ SPONG( 90.0 ), SPONG( 0.0 ), SPONG( 0.0 ) },
	{ SPONG( 90.0 ), SPONG( 0.0 ), SPONG( 0.0 ) },
	{ SPONG( 90.0 ), SPONG( 0.0 ), SPONG( 0.0 ) },
	{ SPONG( 90.0 ), SPONG( 0.0 ), SPONG( 0.0 ) },
	{ SPONG( 45.0 ), SPONG( 0.0 ), SPONG( 0.0 ) },
	{ SPONG( 45.0 ), SPONG( 0.0 ), SPONG( 0.0 ) },
	{ SPONG( 45.0 ), SPONG( 0.0 ), SPONG( 0.0 ) },
	{ SPONG( 35.0 ), SPONG( -22.5 ), SPONG( -10.0 ) },
	{ SPONG( 10.0 ), SPONG( -22.5 ), SPONG( -10.0 ) },
	{ SPONG( 0.0 ), SPONG( 22.5 ), SPONG( -10.0 ) },
	{ SPONG( 0.0 ), SPONG( 22.5 ), SPONG( 10.0 ) },
	{ SPONG( 0.0 ), SPONG( 15.0 ), SPONG( 10.0 ) },
	{ SPONG( 0.0 ), SPONG( -15.0 ), SPONG( 10.0 ) },
};

uint32 LANDONLEFT_List[] = { B_LCR3, B_LCR4,
B_LCR5, B_LCR6,B_LCR5, B_LCR6,
B_LCR5, B_LCR6,B_LCR5, B_LCR6,
B_LCR5, B_LCR6,B_LCR5, B_LCR6,
B_LCR5, B_LCR6,B_LCR5, B_LCR6,
B_LCR5, B_LCR6,B_LCR5, B_LCR6,

B_LCR7, B_LCR8, B_LCR9, B_LCR10, B_LCR11, B_LCR12, B_LCR13, B_LCR14 };

int16 LANDONLEFT_Time[] = { 5,5,
8,8,8,8,
5,5,5,5,
5,5,5,5,
5,5,5,5,
5,5,5,5,
3,4,6,6,10,5,5,5, -128 };

vec3 LANDONLEFT_Rots[] =
{
	{ SPONG( 60.0 ), SPONG( 0.0 ), SPONG( 0.0 ) },
	{ SPONG( 30.0 ), SPONG( 0.0 ), SPONG( 15.0 ) },

	{ SPONG( 0.0 ), SPONG( -22.5 ), SPONG( 0.0 ) },
	{ SPONG( 0.0 ), SPONG( -22.5 ), SPONG( 0.0 ) },
	{ SPONG( 0.0 ), SPONG( -22.5 ), SPONG( 0.0 ) },
	{ SPONG( 0.0 ), SPONG( -22.5 ), SPONG( 0.0 ) },

	{ SPONG( 0.0 ), SPONG( -82.0 ), SPONG( 0.0 ) },
	{ SPONG( 0.0 ), SPONG( -82.0 ), SPONG( 0.0 ) },
	{ SPONG( 0.0 ), SPONG( -82.0 ), SPONG( 0.0 ) },
	{ SPONG( 0.0 ), SPONG( -82.0 ), SPONG( 0.0 ) },

	{ SPONG( 0.0 ), SPONG( -48.125 ), SPONG( 0.0 ) },
	{ SPONG( 0.0 ), SPONG( -35.625 ), SPONG( 0.0 ) },
	{ SPONG( 0.0 ), SPONG( -35.625 ), SPONG( 0.0 ) },
	{ SPONG( 0.0 ), SPONG( -35.625 ), SPONG( 0.0 ) },

	{ SPONG( 0.0 ), SPONG( -36.75 ), SPONG( 0.0 ) },
	{ SPONG( 0.0 ), SPONG( -36.75 ), SPONG( 0.0 ) },
	{ SPONG( 0.0 ), SPONG( -36.75 ), SPONG( 0.0 ) },
	{ SPONG( 0.0 ), SPONG( -36.75 ), SPONG( 0.0 ) },

	{ SPONG( 0.0 ), SPONG( -37.5 ), SPONG( 0.0 ) },
	{ SPONG( 0.0 ), SPONG( -37.5 ), SPONG( 0.0 ) },
	{ SPONG( 0.0 ), SPONG( -37.5 ), SPONG( 0.0 ) },
	{ SPONG( 0.0 ), SPONG( -37.5 ), SPONG( 0.0 ) },

	{ SPONG( 0.0 ), SPONG( -5.0 ), SPONG( 45.0 ) },
	{ SPONG( 0.0 ), SPONG( -5.0 ), SPONG( 45.0 ) },
	{ SPONG( 0.0 ), SPONG( -5.0 ), SPONG( 30.0 ) },
	{ SPONG( 0.0 ), SPONG( -5.0 ), SPONG( 30.0 ) },
	{ SPONG( 0.0 ), SPONG( -5.0 ), SPONG( 15.0 ) },
	{ SPONG( 0.0 ), SPONG( -5.0 ), SPONG( 15.0 ) },
	{ SPONG( 0.0 ), SPONG( -5.0 ), SPONG( 0.0 ) },
	{ SPONG( 0.0 ), SPONG( -5.0 ), SPONG( 0.0 ) },
};


uint32 LANDONRIGHT_List[] = { B_RCR3, B_RCR4, B_RCR5, B_RCR6, B_RCR7,
 B_RCR8, B_RCR9, B_RCR10, B_RCR11, B_RCR12 };

int16 LANDONRIGHT_Time[] = { 3,3,3,7,7,7,5,5,10,10, -128 };

vec3 LANDONRIGHT_Rots[] = {
	{ SPONG( 90.0 ), SPONG( 0.0 ), SPONG( -90.0 ) },
	{ SPONG( 0.0 ), SPONG( -30.0 ), SPONG( 0.0 ) },
	{ SPONG( 0.0 ), SPONG( -30.0 ), SPONG( 0.0 ) },
	{ SPONG( 0.0 ), SPONG( -30.0 ), SPONG( -30.0 ) },
	{ SPONG( 0.0 ), SPONG( 0.0 ), SPONG( -90.0 ) },
	{ SPONG( -15.0 ), SPONG( 0.0 ), SPONG( -90.0 ) },
	{ SPONG( -15.0 ), SPONG( 35.0 ), SPONG( -60.0 ) },
	{ SPONG( 15.0 ), SPONG( 35.0 ), SPONG( 0.0 ) },
	{ SPONG( 15.0 ), SPONG( 35.0 ), SPONG( 0.0 ) },
	{ SPONG( 0.0 ), SPONG( -15.0 ), SPONG( 0.0 ) },
};

uint32 TREESOFT_List[] = { B_TREESOFT2_H1, B_TREESOFT3_H1, B_TREESOFT4_H1, B_TREESOFT5_H1, B_TREESOFT6_H1 };
uint32 FTREESOFT_List[] = { B_FTREESOFT2_H1, B_FTREESOFT3_H1, B_FTREESOFT4_H1, B_FTREESOFT5_H1, B_FTREESOFT6_H1 };
int16 TREESOFT_Time[] = { 2,4,4,4,5, -128 };

uint32 TREEBUMP_List[] = { B_TREEBUMP2_H1, B_TREEBUMP3_H1, B_TREEBUMP4_H1, B_TREEBUMP5_H1, B_TREEBUMP6_H1, B_TREEBUMP7_H1, B_TREEBUMP8_H1 };
uint32 FTREEBUMP_List[] = { B_FTREEBUMP2_H1, B_FTREEBUMP3_H1, B_FTREEBUMP4_H1, B_FTREEBUMP5_H1, B_FTREEBUMP6_H1, B_FTREEBUMP7_H1, B_FTREEBUMP8_H1 };
int16 TREEBUMP_Time[] = { 3,2,3,7,4,3,9, -128 };
/*
vec3 TREEBUMP_Rots[] = {
	{ SPONG( 0.0 ), SPONG( 0.0 ), SPONG( 0.0 ) },
	{ SPONG( 0.0 ), SPONG( 0.0 ), SPONG( 10.0 ) },
	{ SPONG( 0.0 ), SPONG( 0.0 ), SPONG( 10.0 ) },
	{ SPONG( 0.0 ), SPONG( 0.0 ), SPONG( 10.0 ) },
	{ SPONG( 0.0 ), SPONG( 0.0 ), SPONG( -10.0 ) },
	{ SPONG( 0.0 ), SPONG( 0.0 ), SPONG( -10.0 ) },
	{ SPONG( 0.0 ), SPONG( 0.0 ), SPONG( -10.0 ) },
};
*/

uint32 TREESMACK_List[] = {
B_HAR1,
B_HAR2, B_HAR3, B_HAR4, B_HAR5,
B_HAR6, B_HAR7, B_HAR8, B_HAR9,
B_HAR10, B_HAR11, B_HAR12, B_HAR13,
B_HAR14, B_HAR15, B_HAR16, B_HAR17,
B_HAR18, B_HAR19 };

int16 TREESMACK_Time[] = {
5,
5,4,6,10,
3,5,7,8,
5,20,10,3,
3,6,8,4,
6,10,
-128 };

vec3 TREESMACK_Rots[] = {
	{ SPONG( 0.0 ), SPONG( -90.0 ), SPONG( 0.0 ) },

	{ SPONG( 0.0 ), SPONG( 0.0 ), SPONG( 0.0 ) },
	{ SPONG( 0.0 ), SPONG( 0.0 ), SPONG( 30.0 ) },
	{ SPONG( 0.0 ), SPONG( 0.0 ), SPONG( -30.0 ) },
	{ SPONG( 0.0 ), SPONG( 0.0 ), SPONG( 0.0 ) },

	{ SPONG( 0.0 ), SPONG( 0.0 ), SPONG( 0.0 ) },
	{ SPONG( 0.0 ), SPONG( 0.0 ), SPONG( 90.0 ) },
	{ SPONG( 0.0 ), SPONG( 0.0 ), SPONG( 0.0 ) },
	{ SPONG( 0.0 ), SPONG( 0.0 ), SPONG( 0.0 ) },

	{ SPONG( 0.0 ), SPONG( 0.0 ), SPONG( 0.0 ) },
	{ SPONG( 0.0 ), SPONG( 0.0 ), SPONG( 0.0 ) },
	{ SPONG( 0.0 ), SPONG( 0.0 ), SPONG( 0.0 ) },
	{ SPONG( 0.0 ), SPONG( 0.0 ), SPONG( 0.0 ) },

	{ SPONG( 0.0 ), SPONG( 0.0 ), SPONG( -10.0 ) },
	{ SPONG( 0.0 ), SPONG( 0.0 ), SPONG( -20.0 ) },
	{ SPONG( 0.0 ), SPONG( 0.0 ), SPONG( -30.0 ) },
	{ SPONG( 0.0 ), SPONG( 0.0 ), SPONG( -30.0 ) },

	{ SPONG( 0.0 ), SPONG( 0.0 ), SPONG( 0.0 ) },
	{ SPONG( 0.0 ), SPONG( 0.0 ), SPONG( 0.0 ) },
};



uint32 BRAKE_List[] = { B_BRK1, B_BRK2, B_BRK3, B_BRK4, B_BRK5, B_BRK6 };
uint32 FBRAKE_List[] = { B_FBRK1, B_FBRK2, B_FBRK3, B_FBRK4, B_FBRK5, B_FBRK6 };
int16 BRAKE_Time[] = { 6,6,6,6,6,6, -128 };
//vec3 BRAKE_Rots[] = {
//	{ SPONG( 0.0 ), SPONG( -10.0 ), SPONG( 0.0 ) },
//	{ SPONG( 0.0 ), SPONG( -15.0 ), SPONG( 0.0 ) },
//	{ SPONG( 0.0 ), SPONG( -20.0 ), SPONG( 0.0 ) },
//	{ SPONG( 0.0 ), SPONG( -30.0 ), SPONG( 0.0 ) },
//	{ SPONG( 0.0 ), SPONG( -15.0 ), SPONG( 0.0 ) },
//	{ SPONG( 0.0 ), SPONG( 0.0 ), SPONG( 0.0 ) },
//};

chfloat BRAKE_Power[] = { CHFL( 0.1 ), CHFL( 0.1 ), CHFL( 0.25 ), CHFL( 0.25 ), CHFL( 0.5 ), CHFL( 0.5 ), CHFL( 0.75 ) };

uint32 ENDBRAKE_List[] = { B_BRK11, B_BRK10, B_BRK9, B_BRK8, B_BRK7, B_BRK6 };
uint32 FENDBRAKE_List[] = { B_FBRK11, B_FBRK10, B_FBRK9, B_FBRK8, B_FBRK7, B_FBRK6 };
int16 ENDBRAKE_Time[] = { 6,6,6,6,6,2, -128 };
//vec3 ENDBRAKE_Rots[] = {
//	{ SPONG( 0.0 ), SPONG( 10.0 ), SPONG( 0.0 ) },
//	{ SPONG( 0.0 ), SPONG( 15.0 ), SPONG( 0.0 ) },
//	{ SPONG( 0.0 ), SPONG( 20.0 ), SPONG( 0.0 ) },
//	{ SPONG( 0.0 ), SPONG( 30.0 ), SPONG( 0.0 ) },
//	{ SPONG( 0.0 ), SPONG( 15.0 ), SPONG( 0.0 ) },
//	{ SPONG( 0.0 ), SPONG( 0.0 ), SPONG( 0.0 ) },
//};

uint32 LATEGRAB_List[] = { B_LATE2, B_LATE3, B_LATE4, B_LATE5, B_LATE6, B_LATE7, B_LATE8, B_LATE9, B_LATE10, B_LATE11 };
int16 LATEGRAB_Time[] = { 3,3,4,4,5,3,4,6,5,5, -128 };

uint32 TWISTR_List[] = { B_SJUMPM_3, B_RTW2, B_RTW3, B_RTW4, B_RTW5 };
int16 TWISTR_Time[] = { 7,6,6,6,6, -128 };

uint32 TWISTSPIN_List[] = { B_STW1, B_STW2, B_STW3, B_STW4, B_STW5 };
int16 TWISTSPIN_Time[] = { 7,6,6,6,6, -128 };

uint32 ROLL_List[] = { B_ROLL1, B_ROLL2, B_ROLL3, B_ROLL4, B_ROLL5, B_ROLL6, B_ROLL7, B_ROLL8, B_ROLL9, B_ROLL10, B_ROLL11, B_ROLL12 };
int16 ROLL_Time[] = { 5,5,5,5,5,4,2,3,3,2,3,8, -128 };
vec3 ROLL_Rots[] = {
	{ SPONG( 30.0 ), SPONG( 0.0 ), SPONG( 0.0 ) },
	{ SPONG( 30.0 ), SPONG( 0.0 ), SPONG( 0.0 ) },
	{ SPONG( 90.0 ), SPONG( 0.0 ), SPONG( 0.0 ) },
	{ SPONG( 20.0 ), SPONG( 0.0 ), SPONG( 0.0 ) },
	{ SPONG( 10.0 ), SPONG( 0.0 ), SPONG( 0.0 ) },
	{ SPONG( 10.0 ), SPONG( 0.0 ), SPONG( 0.0 ) },
	{ SPONG( 10.0 ), SPONG( 0.0 ), SPONG( 0.0 ) },
	{ SPONG( 20.0 ), SPONG( 0.0 ), SPONG( 0.0 ) },
	{ SPONG( 20.0 ), SPONG( 0.0 ), SPONG( 0.0 ) },
	{ SPONG( 30.0 ), SPONG( 0.0 ), SPONG( 0.0 ) },
	{ SPONG( 45.0 ), SPONG( 0.0 ), SPONG( 0.0 ) },
	{ SPONG( 45.0 ), SPONG( 0.0 ), SPONG( 0.0 ) },
};

uint32 TWISTLB_List[] = { B_TWLB1, B_TWLB2, B_TWLB3, B_TWLB4, B_TWLB5, B_TWLB6, B_TWLB7, B_TWLB8 };
int16  TWISTLB_Time[] = { 5,3,4,3,3,3,3,4, -128 };

uint32 TWISTLF_List[] = { B_TWLF1, B_TWLF2, B_TWLF3, B_TWLF4, B_TWLF5, B_TWLF6, B_TWLF7 };
int16  TWISTLF_Time[] = { 5,3,4,3,3,4,4, -128 };

//uint32 BONK_List[] = { B_BONK1, B_BONK2, B_BONK3, B_BONK4, B_BONK5 };
//int16 BONK_Time[] = { 5,5,5,5,5, -128 };

uint32 TRC_List[] = { B_TRC2, B_TRC3, B_TRC4, B_TRC5, B_TRC6, B_TRC7, B_TRC8, B_TRC9, B_TRC10, B_TRC11, B_TRC12, B_TRC13 };
int16 TRC_Time[] = { 5,5,5,5,5,5,5,5,5,5,5,5, -128 };

uint32 OFFRAIL_List[] = { B_RCYC15, B_RCYC16, B_RCYC17, B_RCYC18, B_RCYC19, B_RCYC20 };
int16 OFFRAIL_Time[] = { 5,5,3,5,5,5,-128 };

uint32 CHILLTRICK_List[] = { B_CT2, B_CT3, B_CT4, B_CT5, B_CT6, B_CT7, B_CT8, B_CT9, B_CT10 };
int16 CHILLTRICK_Time[] = { 5,5,25,5,5,5,5,5,5, -128 };

uint32 OLLIE_List[] = { B_OLL2, B_OLL3, B_OLL4, B_OLL5, B_OLL6 };
uint32 FOLLIE_List[] = { B_FOLL2, B_FOLL3, B_FOLL4, B_FOLL5, B_FOLL6 };
int16 OLLIE_Time[] = { 6, 6, 6, 2, 6, -128 };



void Snowboard( uint8 PlayerNumber )
{
	vec3 MV, FMV;
	long Long0;
	vec3 Fore, Aft, Left, Right;
	vec3 TVec;

	uint8 Coll;

	chfloat Desired;
	uint16	Ix, Iy;
	int16 Tween, Lerps;

	player *Plyr = &Player[ PlayerNumber ];
	uint16	Pad = Pad_State( Plyr->PadNumber );
#if USE_PADD
	uint16	PadD = Pad_Debounced( Plyr->PadNumber );
#endif

	if( PermitMove == 0 )
	{
		Pad = 0;
		PadD = 0;
	}

	MV.X = MV.Y = MV.Z = CHFL( 0.0 );

#if (ADES_COLLISION==0)
	Plyr->LastPos.X = Plyr->Pos.X;	// Store away the (soon to be) last position of the player
	Plyr->LastPos.Y = Plyr->Pos.Y;
	Plyr->LastPos.Z = Plyr->Pos.Z;
	Plyr->LastTerrainY = Plyr->TerrainY;
	Plyr->LastBoardElevation = Plyr->BoardElevation;
	Plyr->LastBoardAzimuth = Plyr->BoardAzimuth;
#endif

	TestScoring( PlayerNumber );	// This temporary handling routine should be removed when each version has their own handling code

// *********
// Collision
// *********

#if JEBS_COLLISION
	if( Plyr->ConsoleRemoteGhosted == GHOSTED_PLAYER || Plyr->ConsoleRemoteGhosted == REMOTE_PLAYER )
		RetrieveCollisionPacket( PlayerNumber );
#endif

	if( ( Coll = Collision( PlayerNumber ) ) && ( Modes[ Plyr->Style ] & MD_NOCOLLISION ) == 0 )
	{
		//	There has been some sort of collision
		//		vec3 CollidePoint( uint8 PlayerNumber )
		//		chfloat CollideAngle( uint8 PlayerNumber )
		//		case RAILSLIDE:
		//		chfloat RailAngle( uint8 PlayerNumber );
#if JEBS_COLLISION
		if( Plyr->ConsoleRemoteGhosted == CONSOLE_PLAYER )
			InsertCollisionPacket( PlayerNumber, Coll );
#endif
		switch( Coll )
		{
			case TOP:
				if( ( Modes[ Plyr->Style ] & MD_NOBONK ) == 0 && Plyr->Fakie == 0 )
				{
					if( Plyr->JumpY <= CHFL( 0.0 ) )
					{
						printf("Bonk\n");
						Plyr->Style = STY_BONK;
						SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_BONK1 ), 5 );
						Plyr->JumpD = 0;
						Plyr->Vel = ( Plyr->Vel * 2 ) / 3;
//						Plyr->Vel = CHFL( 0.0 );
					}
				}
				break;

			case RAILSLIDE:
				printf("Returning railslide\n");
				if( Plyr->Style != STY_RAILSLIDE && Plyr->Style != STY_ENDRAILSLIDE && Plyr->Style != STY_OFFRAIL )
				{
					if( Plyr->JumpY <= CHFL( 0.0 ) )	// Make sure the dude lands on it
					{
						Initialise_Railslide(Plyr, PlayerNumber);
					}
				}
				break;

			case SIDES:	// Smacks into the side of anything
				{
					chfloat Phase;

					printf("SIDES\n");

					Plyr->Pos.X = CollidePoint( PlayerNumber ).X;
					Plyr->Pos.Y = CollidePoint( PlayerNumber ).Y;
					Plyr->Pos.Z = CollidePoint( PlayerNumber ).Z;

//					printf("%ld, %ld\n", CHSHIFTR( Plyr->MoveAzimuth ) & 4095, CHSHIFTR( CollideAngle( PlayerNumber ) ) ) & 4095;

					Plyr->TurnX = CHFL( 0.0 );

					Phase = CHSHIFTR( CHFL( CHSHIFTR( CollideAngle( PlayerNumber ) ) & 4095 ) - CHFL( CHSHIFTR( Plyr->MoveAzimuth ) & 4095 ) ) & 4095;

//					if( Phase < ( 4096 - 1024 ) && Phase > 1024 )
					if( Phase > ( 4096 - 768 ) || Phase < 768 )
					{
// 2						chfloat Painful;

						// Deflect
						printf("Deflection\n");
						Plyr->Rot.Y = Plyr->BoardAzimuth = Plyr->MoveAzimuth = CHFL( CHSHIFTR( CollideAngle( PlayerNumber ) ) & 4095 ) + CHFL( 8192.0 );

						if( Plyr->Cam.SweepAngle > Plyr->Rot.Y )
						{
							while( Plyr->Cam.SweepAngle > Plyr->Rot.Y + CHFL( 2048.0 ) )
							{
								Plyr->Cam.SweepAngle -= CHFL( 4096.0 );
							}
						}
						else
						{
							while( Plyr->Cam.SweepAngle < Plyr->Rot.Y - CHFL( 2048.0 ) )
							{
								Plyr->Cam.SweepAngle += CHFL( 4096.0 );
							}
						}




/*						if( Plyr->JumpY <= CHFL( 0.0 ) )
						{
							Plyr->TurnX = CHFL( 0.0 );
							if( Plyr->Fakie == 0 )
							{
								SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_TREESOFT1_H1 ), 2 );
							}
							else
							{
								SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_FTREESOFT1_H1 ), 2 );
							}
							Plyr->Style = STY_TREESOFT;
							Plyr->AnimStage = 0;
						}
*/

// 2						Painful = ( CHSHIFTR( Plyr->Rot.Y ) & 4095 ) - ( CHSHIFTR( Plyr->Cam.SweepAngle ) & 4095 );
// 2						Plyr->Cam.SweepAngle = Plyr->Rot.Y - CHFL( Painful );

// 1						GoToCrashView( PlayerNumber );
// 1						GoToNormalView( PlayerNumber );

						Plyr->Vel -= CHFL( 5.0 );
						if( Plyr->Vel < CHFL( 5.0 ) ) Plyr->Vel = CHFL( 5.0 );
					}
					else
					{
						// Full on crash
						printf("Head on collision\n");
						Plyr->Fakie = 0;
						Plyr->SideSlip = CHFL( 0.0 );
						Plyr->Style = STY_TREESMACK;
//						Plyr->SpinRot.X = Plyr->Rot.X;
//						Plyr->SpinRot.Y = Plyr->Rot.Y;
//						Plyr->SpinRot.Z = Plyr->Rot.Z;
						Plyr->Vel = CHFL( 0.0 );
						SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, TREESMACK_List[ 0 ] ), TREESMACK_Time[ 0 ] );
//						TVec.X = Plyr->SpinRot.X + TREESMACK_Rots[ 0 ].X;
//						TVec.Y = Plyr->SpinRot.Y + TREESMACK_Rots[ 0 ].Y;
//						TVec.Z = Plyr->SpinRot.Z + TREESMACK_Rots[ 0 ].Z;
//						SetLerpRots( PlayerNumber, &Plyr->SpinRot, &TVec, TREESMACK_Time[ 0 ], &Plyr->SpinRot );
						Plyr->AnimStage = 1;
						GoToArsView( PlayerNumber );
					}
				}
				break;

			case STUMP:
				if( Plyr->Style != STY_STUMPTRIP && Plyr->Style != STY_ST_ROLL )
				{

					Plyr->Style = STY_ST_ROLL;
					SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_ST_ROLL1 ), 5 );
					Plyr->AnimStage = 0;
					Plyr->SpinRot.X = Plyr->BoardElevation;
					Plyr->SpinRot.Y = Plyr->BoardAzimuth - SPONG( 90.0 );
					Plyr->SpinRot.Z = Plyr->BoardRoll;
					GoToCrashView( PlayerNumber );
					Plyr->Fakie = 0;
//					printf("Stump\n");
//					Plyr->Style = STY_STUMPTRIP;
//					Plyr->AnimStage = 1;
//					SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_ROLL1 ), 2 );
//					Plyr->JumpPower = CHFL( 8.0 );
//					Plyr->JumpD = 1;
//					Plyr->Vel = CHFL( 2.0 );
//					GoToCrashView( PlayerNumber );
				}
				break;

			case TREESCRAPE:
				if( Plyr->Style != STY_TREESOFT && Plyr->Style != STY_LANDONLEFT )
				{
					if( Plyr->Vel > CHFL( 8.0 ) )
					{
						Plyr->TurnX = CHFL( 0.0 );
						if( Plyr->Fakie == 0 )
						{
							SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_TREESOFT1_H1 ), 2 );
						}
						else
						{
							SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_FTREESOFT1_H1 ), 2 );
						}
						Plyr->Style = STY_TREESOFT;
						Plyr->AnimStage = 0;
						Plyr->Vel -= CHFL( 5.0 );
					}
				}
				break;

			case TREEBUMP:
			case TREESMACK:
				if( Plyr->Style == STY_LANDONLEFT )
				{
					Plyr->MoveAzimuth += SPONG( 2.0 );
				}
				else
				{
					if( Plyr->Style != STY_TREEBUMP )
					{
						Plyr->TurnX = CHFL( 0.0 );
						if( Plyr->Fakie == 0 )
						{
							SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_TREEBUMP1_H1 ), 3 );
						}
						else
						{
							SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_FTREEBUMP1_H1 ), 3 );
						}
						Plyr->Style = STY_TREEBUMP;
						Plyr->AnimStage = 0;

/*
						Plyr->SpinRot.X = Plyr->BoardElevation;
						Plyr->SpinRot.Y = Plyr->BoardAzimuth;
						Plyr->SpinRot.Z = Plyr->BoardRoll;
						if( Plyr->Fakie == 1 )
						{
							Plyr->SpinRot.X = -Plyr->BoardAzimuth;
							Plyr->SpinRot.Y -= CHFL( 2048.0 );
							Plyr->SpinRot.Z = -Plyr->BoardRoll;
						}
*/
						Plyr->Vel -= CHFL( 10.0 );
						if( Plyr->Vel <= CHFL( 3.0 ) ) Plyr->Vel = CHFL( 3.0 );
					}
				}
				break;

/*			case TREESMACK:
				printf("TREESMACK\n");
				Plyr->Style = STY_TREESMACK;
				Plyr->SpinRot.X = Plyr->Rot.X;
				Plyr->SpinRot.Y = Plyr->Rot.Y;
				Plyr->SpinRot.Z = Plyr->Rot.Z;
				Plyr->Vel = CHFL( 0.0 );
				SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, TREESMACK_List[ 0 ] ), TREESMACK_Time[ 0 ] );
//				TVec.X = Plyr->SpinRot.X + TREESMACK_Rots[ 0 ].X;
//				TVec.Y = Plyr->SpinRot.Y + TREESMACK_Rots[ 0 ].Y;
//				TVec.Z = Plyr->SpinRot.Z + TREESMACK_Rots[ 0 ].Z;
//				SetLerpRots( PlayerNumber, &Plyr->SpinRot, &TVec, TREESMACK_Time[ 0 ], &Plyr->SpinRot );
				Plyr->AnimStage = 1;
				GoToArsView( PlayerNumber );
				Plyr->Pos.X = CollidePoint( PlayerNumber ).X;
				Plyr->Pos.Y = CollidePoint( PlayerNumber ).Y;
				Plyr->Pos.Z = CollidePoint( PlayerNumber ).Z;
				break;
*/
			default:
				// Do bugger all!
				break;

		}

	}


#if (ADES_COLLISION)
	Plyr->LastPos.X = Plyr->Pos.X;	// Store away the (soon to be) last position of the player
	Plyr->LastPos.Y = Plyr->Pos.Y;
	Plyr->LastPos.Z = Plyr->Pos.Z;
	Plyr->LastTerrainY = Plyr->TerrainY;
	Plyr->LastBoardElevation = Plyr->BoardElevation;
	Plyr->LastBoardAzimuth = Plyr->BoardAzimuth;
#endif

// **********************
// Calculate Player Input
// **********************

	if( Config_BRAKE( Plyr->PadNumber ) )
	{
		if( ( Modes[ Plyr->Style ] & MD_ALLOWBRAKE ) != 0 )
		{
			Plyr->Style = STY_BRAKE;
			Plyr->AnimStage = 0;
		}
	}

	if( Config_TUCK( Plyr->PadNumber ) )
	{
		// Attempting to go into TUCK frames
		if( ( Modes[ Plyr->Style ] & MD_ALLOWTUCK ) != 0 )
		{
			Plyr->Style = STY_TUCK;
			Plyr->AnimStage = 0;
		}
	}



	// Is the pad a digital pad ?
	if( Pad_Type( Plyr->PadNumber ) == PAD_DIGITAL )
	{
		// *************
		// Digital input
		// *************

		// Steering from the D-Pad left and right buttons
		if( Pad & ( PAD_LEFT | PAD_RIGHT ) )	// Is left or right being pressed
		{
			// Left or Right is pressed, decide which one and then fiddle the turning velocity

			// Normal steer
			if( Plyr->Style == STY_NORMAL || Plyr->Style == STY_TURNING )
			{
				if( Pad & PAD_LEFT )
				{
					if( Plyr->Style == STY_NORMAL )
					{
						Plyr->Style = STY_PRE_TURNING;
						if( Plyr->Fakie == 0 )
							SetLerpToTween( PlayerNumber, FRAMEEQU( PlayerNumber, PreTurnTableL[ Plyr->TurnY / 20 ] ), FRAMEEQU( PlayerNumber, PreTurnTableR[ ( Plyr->TurnY / 20 ) + 1 ] ), 5, 20, Plyr->TurnY % 20 );
						else
							SetLerpToTween( PlayerNumber, FRAMEEQU( PlayerNumber, FakiePreTurnTableL[ Plyr->TurnY / 20 ] ), FRAMEEQU( PlayerNumber, PreTurnTableR[ ( Plyr->TurnY / 20 ) + 1 ] ), 5, 20, Plyr->TurnY % 20 );
					}

					if( Plyr->Style == STY_TURNING )
					{
						Plyr->TurnX -= CHFL( SWING_SPEED );
						if( Plyr->TurnX < CHFL( -40.0 ) ) Plyr->TurnX = CHFL( -40.0 );
					}
				}

				if( Pad & PAD_RIGHT )
				{
					if( Plyr->Style == STY_NORMAL )
					{
						Plyr->Style = STY_PRE_TURNING;

						if( Plyr->Fakie == 0 )
							SetLerpToTween( PlayerNumber, FRAMEEQU( PlayerNumber, PreTurnTableR[ Plyr->TurnY / 20 ] ), FRAMEEQU( PlayerNumber, PreTurnTableR[ ( Plyr->TurnY / 20 ) + 1 ] ), 5, 20, Plyr->TurnY % 20 );
						else
							SetLerpToTween( PlayerNumber, FRAMEEQU( PlayerNumber, FakiePreTurnTableR[ Plyr->TurnY / 20 ] ), FRAMEEQU( PlayerNumber, PreTurnTableR[ ( Plyr->TurnY / 20 ) + 1 ] ), 5, 20, Plyr->TurnY % 20 );

					}

					if( Plyr->Style == STY_TURNING )
					{
						Plyr->TurnX += CHFL( SWING_SPEED );
						if( Plyr->TurnX > CHFL( 40.0 ) ) Plyr->TurnX = CHFL( 40.0 );
					}
				}
			}

			// Steering whilst preparing to jump
			if( Plyr->Style == STY_PRE_JUMP || Plyr->Style == STY_PREP_JUMP )
			{
				if( Pad & PAD_LEFT )
				{
					Plyr->TurnX -= CHFL( CROUCH_SWING_SPEED );
					if( Plyr->TurnX < CHFL( -40.0 ) ) Plyr->TurnX = CHFL( -40.0 );
				}

				if( Pad & PAD_RIGHT )
				{
					Plyr->TurnX += CHFL( CROUCH_SWING_SPEED );
					if( Plyr->TurnX > CHFL( 40.0 ) ) Plyr->TurnX = CHFL( 40.0 );
				}
			}

			// Steering whilst tucking
			if( Plyr->Style == STY_TUCK )
			{
				if( Pad & PAD_LEFT )
				{
					Plyr->TurnX -= CHFL( TUCK_SWING_SPEED );
					if( Plyr->TurnX < CHFL( -40.0 ) ) Plyr->TurnX = CHFL( -40.0 );
				}

				if( Pad & PAD_RIGHT )
				{
					Plyr->TurnX += CHFL( TUCK_SWING_SPEED );
					if( Plyr->TurnX > CHFL( 40.0 ) ) Plyr->TurnX = CHFL( 40.0 );
				}
			}

			// Steering whilst in the air
			if( ( Modes[ Plyr->Style ] & MD_AIRTURNS ) != 0 )
			{
				if( Pad & PAD_LEFT )
				{
					Plyr->TurnX -= CHFL( AIR_TURN_SPEED );
					if( Plyr->TurnX < CHFL( -40.0 ) ) Plyr->TurnX = CHFL( -40.0 );
				}

				if( Pad & PAD_RIGHT )
				{
					Plyr->TurnX += CHFL( AIR_TURN_SPEED );
					if( Plyr->TurnX > CHFL( 40.0 ) ) Plyr->TurnX = CHFL( 40.0 );
				}
			}
		}
		else	// Neither left or right is being pressed
		{
//			if( Plyr->Style == STY_TURNING )
//			{
				// Gradually drag TurnX back to 0 and then return to STY_NORMAL
				if( Plyr->TurnX < CHFL( 0.0 ) )
				{
					Plyr->TurnX += CHFL( SWING_BACK_SPEED );
					if( Plyr->TurnX > CHFL( 0.0 ) ) Plyr->TurnX = CHFL( 0.0 );
				}

				if( Plyr->TurnX > CHFL( 0.0 ) )
				{
					Plyr->TurnX -= CHFL( SWING_BACK_SPEED );
					if( Plyr->TurnX < CHFL( 0.0 ) ) Plyr->TurnX = CHFL( 0.0 );
				}

				// When it is 0, then the player is boarding straight again
				if( Plyr->TurnX == CHFL( 0.0 ) )
				{
					if( Plyr->Style == STY_TURNING )
					{
						Plyr->Style = STY_NORMAL;
						if( Plyr->Fakie == 0 )
							SetTween( PlayerNumber, FRAMEEQU( PlayerNumber, B_STA1_H1 ), FRAMEEQU( PlayerNumber, B_STA2_H1 ), 50, 25 );
						else
							SetTween( PlayerNumber, FRAMEEQU( PlayerNumber, B_FSTA1_H1 ), FRAMEEQU( PlayerNumber, B_FSTA2_H1 ), 50, 25 );
						Plyr->AnimStage = CENTRE_STAGE;
						Plyr->AnimTween = 25;
						Plyr->AnimFrame = 0;
					}
				}
//			}
		}
	}

	// Is the pad an analog pad ?
	if( Pad_Type( Plyr->PadNumber ) == PAD_ANALOG_CONTROLLER || Pad_Type( Plyr->PadNumber ) == PAD_ANALOG || Pad_Type( Plyr->PadNumber ) == PAD_NEGCON )
	{
		// ************
		// Analog input
		// ************

		chfloat DesiredTurnX;

		if( Analog_Config_Steer( Plyr->PadNumber ) < 112 || Analog_Config_Steer( Plyr->PadNumber ) > 144 )
		{
			DesiredTurnX = CHFL( ( Analog_Config_Steer( Plyr->PadNumber ) - 128 ) / 3 );

			if( DesiredTurnX < CHFL( -40.0 ) ) DesiredTurnX = CHFL( -40.0 );
			if( DesiredTurnX > CHFL( 40.0 ) ) DesiredTurnX = CHFL( 40.0 );
		}
		else
		{
			DesiredTurnX = CHFL( 0.0 );
		}

		if( Plyr->Style == STY_NORMAL || Plyr->Style == STY_TURNING )
		{
			Analog_Swing( Plyr, CHFL( SWING_SPEED ), DesiredTurnX );

			if( Plyr->TurnX == CHFL( 0.0 ) )
			{
				if( Plyr->Style == STY_TURNING ) Plyr->Style = STY_NORMAL;
			}
			else
			{
				if( Plyr->Style == STY_NORMAL ) Plyr->Style = STY_TURNING;
			}
		}

		if( Plyr->Style == STY_PRE_JUMP || Plyr->Style == STY_PREP_JUMP )
		{
			Analog_Swing( Plyr, CHFL( CROUCH_SWING_SPEED ), DesiredTurnX );
		}

		if( Plyr->Style == STY_TUCK )
		{
			Analog_Swing( Plyr, CHFL( TUCK_SWING_SPEED ), DesiredTurnX );
		}
	}

// Is the pad an negcon pad ?
/*	if( Pad_Type( 1 ) == PAD_NEGCON )
	{
		// ************
		// Negcon input
		// ************

		Plyr->Style = STY_TURNING;

		Plyr->TurnX = ( Analog_Config_Steer( 1 ) - 128 ) / 3;

		if( Plyr->TurnX < -40 ) Plyr->TurnX = -40;

		if( Plyr->TurnX > 40 ) Plyr->TurnX = 40;

		Plyr->TurnX <<= 12;
	}
*/

/*	if( Pad_Type( 1 ) == PAD_DIGITAL )
	{
		if( Pad & PAD_TR ) Plyr->Vel = CHFL( 16.0 );
		if( Pad & PAD_TL ) Plyr->Vel = CHFL( 0.0 );
	}
*/


//	if( PadD & PAD_B4 )
//	{
//		GoToOnRailsView( PlayerNumber, 0 );
//	}


// *************
// Jump controls
// *************

	// Initiate jump only from normal boarding modes
	if( Plyr->Style == STY_TURNING || Plyr->Style == STY_NORMAL || Plyr->Style == STY_PRE_TURNING )
	{
		if( Config_JUMP( Plyr->PadNumber ) && Plyr->JumpY <= CHFL( 0.0 ) )// && ( Plyr->JumpHeld == 0 ) )
		{

			if( Plyr->Vel <= CHFL( 0.0 ) )
			{
				// Not moving, so do the Ollie
				if( trackdirection[ Plyr->Buffer ] == 1 )
				{
					Plyr->Style = STY_OLLIE;
					Plyr->AnimStage = 0;
					if( Plyr->Fakie == 0 )
						SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_OLL1 ), 5 );
					else
						SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_FOLL1 ), 5 );
				}
			}
			else
			{
				// Normal jumping code
				Plyr->JumpPower = CHFL( MIN_JUMP_SPEED ) + Character_Jumps[ Plyr->Character ];
				Plyr->JumpHeld = 1;
				if( Plyr->Fakie == 0 )
					SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_SJUMPM_1 ), 5 );	// Lerp to first jump frames quickly
				else
					SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_FJUMP_1 ), 5 );	// Lerp to first jump frames quickly
				Plyr->Style = STY_PREP_JUMP;

 				// Initialise airborne control parameters
				Plyr->SpinRot.X = Plyr->Rot.X;
				Plyr->SpinRot.Y = Plyr->Rot.Y;
				Plyr->SpinRot.Z = Plyr->Rot.Z;
				Plyr->SpinHeld = 0;
				Plyr->TwistHeld = 0;
				Plyr->SpinSpeed = CHFL( 0.0 );
				Plyr->TwistSpeed = CHFL( 0.0 );
			}
		}
	}

	// If preparing to jump or crouching to jump then...
	if( Plyr->Style == STY_PREP_JUMP || Plyr->Style == STY_PRE_JUMP )
	{
		// (1) increase the jump power as the button is held down
		if( Config_JUMP( Plyr->PadNumber ) )
		{
//			printf("Jump held\n");
			Plyr->JumpHeld = 1;
			Plyr->JumpPower += CHFL( HOLD_JUMP_KEY_ACCEL );
			if( Plyr->JumpPower > CHFL( MAX_JUMP_SPEED ) + Character_Jumps[ Plyr->Character ] ) Plyr->JumpPower = CHFL( MAX_JUMP_SPEED ) + Character_Jumps[ Plyr->Character ];
		}

		// (2) test for release of the jump button
		if( !Config_JUMP( Plyr->PadNumber ) )	// Jump has been released
		{
			// Initialise jump movement vars
//			printf("Jump released\n");
			Plyr->JumpD = 0;
			Plyr->JumpY = CHFL( 0.0 );
//			Plyr->JumpPower = CHFL( MIN_JUMP_SPEED );
			Plyr->TurnX = 0;
			Plyr->JumpHeld = 0;


			// Special move ?
			if( Config_CHILLTRICK( Plyr->PadNumber ) && Plyr->Vel > CHFL( 5.0 ) && Plyr->Fakie == 0 )
			{
				Plyr->Style = STY_CHILLTRICK;
				Plyr->AnimStage = 0;

				// Each chill trick is likely to be different
				SetLerp( PlayerNumber, B_CT1, 5 );

				Plyr->JumpD = 1;
			}
		}
	}

	//if( parachute_state == 1 )
	//{
	//	Plyr->JumpPower = CHFL( MAX_JUMP_SPEED );
	//	Plyr->JumpD = 1;
	//	Plyr->JumpY = CHFL( 0.0 );
	//	Plyr->Cam.Mode = CAMERA_PARACHUTE;
	//	Plyr->Cam.SubMode = PARA_LAUNCH;
	//	if( Plyr->Vel <= CHFL( 8.0 ) ) Plyr->Vel = CHFL( 8.0 );
	//}

// **************************************
// Airborne control parameters (pre-jump)
// **************************************

	// Spin & Twist recovery check
/*
	if( Plyr->JumpY > CHFL( 0.0 ) )
	{
		if( Config_JUMP( Plyr->PadNumber ) && ( ( Modes[ Plyr->Style ] & MD_RECOVERY ) != 0 ) )
		{
//			printf("Allowing a recovery\n");
			switch( Plyr->Style )
			{
				case STY_FRONTSPIN:
				case STY_BACKSPIN:
					Plyr->Style = STY_RECOVERY;
					Plyr->AnimStage = 0;
					SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_RCYC1 ), 8 );
					break;

				case STY_TWISTANDSPIN:
					Plyr->Style = STY_ST_RECOVERY;
					Plyr->AnimStage = 0;
					SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_TRC1 ), 5 );
					break;
			}
		}
	}
*/
	// Spin and Twist power build up
	if( Plyr->JumpHeld == 1 )
	{
		if( Config_FRONTSPIN( Plyr->PadNumber ) || Config_BACKSPIN( Plyr->PadNumber ) )
		{
			if( Config_FRONTSPIN( Plyr->PadNumber ) )
			{
				Plyr->SpinSpeed += CHFL( 10.0 );
				if( Plyr->SpinSpeed >= CHFL( 300.0 ) ) Plyr->SpinSpeed = CHFL( 300.0 );
				Plyr->SpinHeld = 1;
			}

			if( Config_BACKSPIN( Plyr->PadNumber ) )
			{
				Plyr->SpinSpeed -= CHFL( 5.0 );
				if( Plyr->SpinSpeed <= CHFL( -200.0 ) ) Plyr->SpinSpeed = CHFL( -200.0 );
				Plyr->SpinHeld = 1;
			}
		}
		else
			Plyr->SpinSpeed = CHFL( 0.0 );

		if( Config_LEFTTWIST( Plyr->PadNumber ) || Config_RIGHTTWIST( Plyr->PadNumber ) )
		{
			if( Config_LEFTTWIST( Plyr->PadNumber ) )
			{
				Plyr->TwistSpeed -= CHFL( 2.5 );
				if( Plyr->TwistSpeed <= CHFL( -150.0 ) ) Plyr->TwistSpeed = CHFL( -150.0 );
				Plyr->TwistHeld = 1;
			}

			if( Config_RIGHTTWIST( Plyr->PadNumber ) )
			{
				Plyr->TwistSpeed += CHFL( 2.5 );
				if( Plyr->TwistSpeed >= CHFL( 150.0 ) ) Plyr->TwistSpeed = CHFL( 150.0 );
				Plyr->TwistHeld = 1;
			}
		}
		else
			Plyr->TwistSpeed = CHFL( 0.0 );
	}

// *************
// Grab controls
// *************

	// B1 and B2 are the grab controls
	if( Plyr->GrabHeld == 0 && Plyr->Fakie == 0 )
	{
		if( ( Config_GRAB1( Plyr->PadNumber ) || Config_GRAB2( Plyr->PadNumber ) ) && ( Modes[ Plyr->Style ] & MD_ALLOWGRAB ) )
		{
			// Do a grab
			if( Config_GRAB1( Plyr->PadNumber ) ) Plyr->GrabHeld = 1;
			if( Config_GRAB2( Plyr->PadNumber ) ) Plyr->GrabHeld = 2;
//			Plyr->GrabRotX = CHFL( 0.0 );
//			Plyr->GrabRotZ = CHFL( 0.0 );
			Plyr->GrabStage = 0;
			Plyr->HaveGrabbed = 0;
			Plyr->GrabDuration = 0;
		}
	}
	else
	{
		if( ( Modes[ Plyr->Style ] & MD_ALLOWGRAB ) == 0 )
		{
			Plyr->GrabHeld = 0;
			Plyr->HaveGrabbed = 0;
		}

		// Cancel the grab, if it's not held anymore
		if( Plyr->GrabHeld == 1 )
		{
			if( !Config_GRAB1( Plyr->PadNumber ) )
			{
				Plyr->GrabHeld = 3;
				Plyr->HaveGrabbed = 1;
				GrabScoring(Plyr, PlayerNumber);
			}
		}

		if( Plyr->GrabHeld == 2 )
		{
			if( !Config_GRAB2( Plyr->PadNumber ) )
			{
				Plyr->GrabHeld = 4;
				Plyr->HaveGrabbed = 1;
				GrabScoring(Plyr, PlayerNumber);
			}
		}
	}

// **************************************
// Temporary get out of the shit controls
// **************************************

	#if I_AM == JEZ
	if( Pad_Type( 1 ) == PAD_NEGCON )
	{
		if( Pad_Negcon_Button2( 1 ) > 10 )
		{
			Plyr->Vel = CHFL( 0.0 );
		}

		if( Pad_Negcon_Button1( 1 ) > 10 )
		{
			Plyr->Vel = ( CHFL( 30.0 ) / 255 ) * Pad_Negcon_Button1( 1 );
		}
	}
	#endif

#if USE_PADD
//	#if I_AM != NOBODY
	if( PadD & PAD_SELECT )
	{
		Dolmio ^= 1;

		if( Dolmio != 0 )
		{
			Plyr->Style = STY_DOLMIO;
		}
		else
		{
			LerpToNormalStance(Plyr, PlayerNumber);
		}
	}
//	#endif
#endif

// *********************
// Movement calculations
// *********************

	// Jumping up calculations
	if( Plyr->JumpD == 1 )
	{
		Plyr->JumpY += Plyr->JumpPower;
		Plyr->JumpPower -= CHFL( 0.75 );
		if ( Plyr->JumpPower < CHFL( 0.0 ) )
		{
			Plyr->JumpPower = CHFL( 0.0 );
			Plyr->JumpD = 2;
		}
	}

	// Falling down calculations
	if( Plyr->JumpD == 2 )
	{
		Plyr->JumpY -= Plyr->JumpPower;
		if( Plyr->JumpY < CHFL( 0.0 ) )
		{
			Plyr->JumpY = CHFL( 0.0 );
			Plyr->JumpD = 3;	// End all the jumping
		}
		Plyr->JumpPower += CHFL( 0.75 );
		if( Plyr->JumpPower > CHFL( MAX_FALL_SPEED ) ) Plyr->JumpPower = CHFL( MAX_FALL_SPEED );
#if PARACHUTE_CODE
		if( parachute_state == 1 && Plyr->JumpPower > CHFL( 4.0 ) )
			Plyr->JumpPower = CHFL( 4.0 );
#endif
	}

	if( Plyr->JumpY > CHFL( 0.0 ) && ( Plyr->JumpD == 0 || Plyr->JumpD == 3 ) )
	{
		if( Plyr->Style != STY_DOLMIO )
		{
			Plyr->JumpD = 2;
			Plyr->JumpPower = CHFL( 0.1 );
		}
	}

	// Calculate board turning inertia from upper body twist extent
	if( Plyr->TurnX == 	CHFL( 0.0 ) )
	{
		Plyr->TurnInert = CHFL( 0.0 );
	}
	else
	{
//		chfloat	TurnThang = ( CHFL( 1.0 ) / ( CHSHIFTR( Plyr->Vel ) + 1 ) ) + CHFL( 5.0 );
//		chfloat TurnThang = CHFL( 1.333 );
		chfloat TurnThang = TurnSpeeds[ CHSHIFTR( Plyr->Vel ) ] + Character_Turning[ Plyr->Character ];// + Board_Turning[ Plyr->BoardNumber ];

		Plyr->TurnInert = CHSHIFTR( ( TurnThang * Plyr->TurnX ) );

		// Restricted turning speed
		if( Plyr->Style == STY_PRE_JUMP || Plyr->Style == STY_PREP_JUMP )
			Plyr->TurnInert = CHSHIFTR( ( TurnThang - CHFL( 0.5 ) ) * Plyr->TurnX );

		if( Plyr->Style == STY_TUCK )
			Plyr->TurnInert = CHSHIFTR( ( TurnThang - CHFL( 0.4 ) ) * Plyr->TurnX );

		if( ( Modes[ Plyr->Style ] & MD_AIRTURNS ) != 0 )
			Plyr->TurnInert = CHSHIFTR( ( TurnThang - CHFL( 0.2 ) ) * Plyr->TurnX );
	}



	if( Plyr->JumpY <= CHFL( 0.0 ) )
	{
		// Change : Try and catch the two up
		Plyr->MoveAzimuth = Plyr->BoardAzimuth;
	}
	else
	{
		chfloat Diff = Plyr->MoveAzimuth - Plyr->BoardAzimuth;
		// If not on the ground, the BoardAzimuth may never be more than 90 degrees away

		if( Diff > SPONG( 120.0 ) )
		{
			Plyr->BoardAzimuth = Plyr->MoveAzimuth + SPONG( 120.0 );
		}

		if( Diff < SPONG( -120.0 ) )
		{
			Plyr->BoardAzimuth = Plyr->MoveAzimuth - SPONG( 120.0 );
		}

	}

	// Calculate movement vector
	MV.X = Plyr->SideSlip;	// Side slip motion ?
//	MV.Y = CHFL( 0.0 );
	MV.Z = Plyr->Vel;

	/* Now rotate the movement vector */
	FMV.X = ( MV.X * Cosine( CHSHIFTR( Plyr->MoveAzimuth ) ) ) - ( MV.Z * Sine( CHSHIFTR( Plyr->MoveAzimuth ) ) );
	FMV.Z = ( MV.Z * Cosine( CHSHIFTR( Plyr->MoveAzimuth ) ) ) + ( MV.X * Sine( CHSHIFTR( Plyr->MoveAzimuth ) ) );
//	FMV.Y = MV.Y;

	Plyr->Pos.X -= CHSHIFTR( FMV.X );
	Plyr->Pos.Z += CHSHIFTR( FMV.Z );
	Plyr->TerrainY = GetYAt( Plyr->Pos.X, Plyr->Pos.Z, PlayerNumber );


	// Have we come up against a wall type of thing here
/*
#if JEBS_COLLISION
	// PlayStation version
	if( Plyr->ConsoleRemoteGhosted == GHOSTED_PLAYER || Plyr->ConsoleRemoteGhosted == REMOTE_PLAYER )
	{
		if( RetrieveHitTheWallPacket( PlayerNumber ) == 1 )
		{
			printf("Replay - Hit the wall\n");
			Plyr->Fakie = 0;
			Plyr->SideSlip = CHFL( 0.0 );
			Plyr->Style = STY_TREESMACK;
			Plyr->Vel = CHFL( 0.0 );
			SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, TREESMACK_List[ 0 ] ), TREESMACK_Time[ 0 ] );
			Plyr->AnimStage = 1;
			GoToArsView( PlayerNumber );
		}
	}
	else
	{
		if( Plyr->TerrainY > ( Plyr->LastTerrainY + CHFL( 16.0 ) ) && Plyr->JumpY <= CHFL( 0.0 ) )
		{
			if( Collision( PlayerNumber ) == 0 )
			{
				printf("Hit the wall\n");
				Plyr->Fakie = 0;
				Plyr->SideSlip = CHFL( 0.0 );
				Plyr->Style = STY_TREESMACK;
				Plyr->Pos.X = Plyr->LastPos.X;
				Plyr->Pos.Y = Plyr->LastPos.Y;
				Plyr->Pos.Z = Plyr->LastPos.Z;
				Plyr->Vel = CHFL( 0.0 );
				SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, TREESMACK_List[ 0 ] ), TREESMACK_Time[ 0 ] );
				Plyr->AnimStage = 1;
				GoToArsView( PlayerNumber );
				InsertHitTheWallPacket( PlayerNumber );
			}
		}
	}
#else
	// Saturn version
	if( Plyr->TerrainY > ( Plyr->LastTerrainY + CHFL( 16.0 ) ) && Plyr->JumpY <= CHFL( 0.0 ) )
	{
		if( Collision( PlayerNumber ) == 0 )
		{
			printf("Hit the wall\n");
			Plyr->Fakie = 0;
			Plyr->SideSlip = CHFL( 0.0 );
			Plyr->Style = STY_TREESMACK;
			Plyr->Pos.X = Plyr->LastPos.X;
			Plyr->Pos.Y = Plyr->LastPos.Y;
			Plyr->Pos.Z = Plyr->LastPos.Z;
			Plyr->Vel = CHFL( 0.0 );
			SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, TREESMACK_List[ 0 ] ), TREESMACK_Time[ 0 ] );
			Plyr->AnimStage = 1;
			GoToArsView( PlayerNumber );
			InsertHitTheWallPacket( PlayerNumber );
		}
	}
#endif
*/



	// Alter Plyr->JumpY for undulating terrain
	if( Plyr->JumpY > CHFL( 0.0 ) )
	{
		if( Plyr->TerrainY < Plyr->LastTerrainY )
		{
			Plyr->JumpY += ( Plyr->LastTerrainY - Plyr->TerrainY );
		}

		if( Plyr->TerrainY > Plyr->LastTerrainY )
		{
			Plyr->JumpY -= ( Plyr->TerrainY - Plyr->LastTerrainY );
		}
	}

	// Prevent him going below the ground at all times
	if( Plyr->JumpY < CHFL( 0.0 ) )
	{
		Plyr->JumpY = CHFL( 0.0 );
	}

	//RWB
	//if(parachute_state < 4)
		Plyr->Pos.Y = Plyr->TerrainY + Plyr->JumpY;
	//else if(parachute_state == 4)
	//{
	//	Plyr->Pos.Y -= Plyr->Grav;
	//	Plyr->Vel = 4096;
	//	Plyr->MoveAzimuth = Plyr->BoardAzimuth;
	//}

	// Player is now at the his new and correct XYZ


// ******************************
// Board orientation calculations
// ******************************

	// Calculate the new orientation of the board to the ground

	// Position of start and end of board by rotating static approximate start and end points
	Fore.X = 0 - ( CHFL( 15.0 ) * Sine( CHSHIFTR( Plyr->Rot.Y ) ) );
	Fore.Z = ( CHFL( 15.0 ) * Cosine( CHSHIFTR( Plyr->Rot.Y ) ) ) + 0;
	Aft.X = 0 - ( CHFL( -15.0 ) * Sine( CHSHIFTR( Plyr->Rot.Y ) ) );
	Aft.Z = ( CHFL( -15.0 ) * Cosine( CHSHIFTR( Plyr->Rot.Y ) ) ) + 0;

	// GetYAt these positions
	Fore.Y = GetYAt( CHSHIFTR( -Fore.X ) + Plyr->Pos.X, CHSHIFTR( Fore.Z ) + Plyr->Pos.Z, PlayerNumber );
	Aft.Y = GetYAt( CHSHIFTR( -Aft.X ) + Plyr->Pos.X, CHSHIFTR( Aft.Z ) + Plyr->Pos.Z, PlayerNumber );

	// arctan( dx, dy ) to get the board elevation angle
	Long0 = chatan2( CHFL( 15.0 ) - CHFL( -15.0 ), Fore.Y - Aft.Y );

	// Get the board angle
	Plyr->BoardElevation = ( CHFL( Long0 )  ) - CHFL( 1024.0 );

	if( Plyr->Fakie == 1 ) Plyr->BoardElevation = -Plyr->BoardElevation;



//	if( Plyr->JumpY > CHFL( 0.0 ) ) Plyr->BoardElevation = Plyr->LastBoardElevation;

	// Position of start and end of board by rotating static approximate start and end points
	Right.X = ( CHFL( 15.0 ) * Cosine( CHSHIFTR( Plyr->Rot.Y ) ) ) - 0;
	Right.Z = 0 + ( CHFL( 15.0 ) * Sine( CHSHIFTR( Plyr->Rot.Y ) ) );
	Left.X = ( CHFL( -15.0 ) * Cosine( CHSHIFTR( Plyr->Rot.Y ) ) ) - 0;
	Left.Z = 0 + ( CHFL( -15.0 ) * Sine( CHSHIFTR( Plyr->Rot.Y ) ) ) ;

	// GetYAt these positions
	Right.Y = GetYAt( CHSHIFTR( -Right.X ) + Plyr->Pos.X, CHSHIFTR( Right.Z ) + Plyr->Pos.Z, PlayerNumber );
	Left.Y = GetYAt( CHSHIFTR( -Left.X ) + Plyr->Pos.X, CHSHIFTR( Left.Z ) + Plyr->Pos.Z, PlayerNumber );

	// arctan( dx, dy ) to get the board elevation angle
	Long0 = chatan2( CHFL( 15.0 ) - CHFL( -15.0 ), Right.Y - Left.Y );

	// Get the board angle
	// +ve is leaning left
	// -ve is leaning right
	Plyr->BoardRoll = ( CHFL( Long0 ) ) - CHFL( 1024.0 );

	if( Plyr->Fakie == 1 ) Plyr->BoardRoll = -Plyr->BoardRoll;

// ******************
// Player orientation
// ******************

	// Turn the board
	Plyr->BoardAzimuth += Plyr->TurnInert;

	CalcRotXYZ( Plyr );

//	if( Plyr->GrabHeld != 0 )
//	{
//		Plyr->Rot.X += Plyr->GrabRotX;
//		Plyr->Rot.Z = Plyr->GrabRotZ;
//	}

// ********************
// Automatic maneouvres
// ********************

	// Side Slipping (when stood sideways on a steep slope)
	if( ( Plyr->BoardRoll < CHFL( -384.0 ) || Plyr->BoardRoll > CHFL( 384.0 ) ) && Plyr->JumpY <= CHFL( 0.0 ) )
	{
		if( Plyr->BoardRoll < 0 )
		{
			Plyr->SideSlip -= CHFL( 1.0 );
		}

		if( Plyr->BoardRoll > 0 )
		{
			Plyr->SideSlip += CHFL( 1.0 );
		}
	}
	else
	{
		if( Plyr->SideSlip > CHFL( 0.0 ) )
		{
			Plyr->SideSlip -= CHFL( 1.5 );
			if( Plyr->SideSlip < CHFL( 0.0 ) ) Plyr->SideSlip = CHFL( 0.0 );
		}

		if( Plyr->SideSlip < CHFL( 0.0 ) )
		{
			Plyr->SideSlip += CHFL( 1.5 );
			if( Plyr->SideSlip > CHFL( 0.0 ) ) Plyr->SideSlip = CHFL( 0.0 );
		}
	}

	// Automated jumping (when going off a ledge or something)
	if( Plyr->JumpY == CHFL( 0.0 ) )	// Are we on the ground
	{
		if( ( Modes[ Plyr->Style ] & MD_ALLOWAUTOJUMP ) != 0 )	// Change to Style capabilities check
		{
			// Basically if the board elevation changes violently
			if( Plyr->BoardElevation > ( Plyr->LastBoardElevation + SPONG( 15.0 ) ) )
			{
//					printf("Automatic leap\n");
//					Plyr->Style = STY_AUTO_JUMP;

					// Crouching ?
					if( Plyr->JumpHeld == 1 )
					{
						// If crouching for a jump we don't really want him to take off for a small bump
						if( Plyr->BoardElevation < ( Plyr->LastBoardElevation + SPONG( 20.0 ) ) )
						{
							// Kill the auto jump if it's only a small one
//							Plyr->JumpD = 0;
//							Plyr->JumpY = CHFL( 0.0 );
						}
						else
						{
							// It's a big auto jump, kill the crouch instead
							LerpToNormalStance(Plyr, PlayerNumber);

							// Do the jump
							Plyr->JumpD = 1;
							Plyr->TurnX = 0;
							Plyr->JumpY = CHFL( 0.25 );
							Plyr->JumpPower = ( Plyr->BoardElevation - ( Plyr->LastBoardElevation + CHFL( 64.0 ) ) ) / 256;
							Plyr->BoardElevation = Plyr->LastBoardElevation;
							if( Plyr->JumpPower > CHFL( MIN_JUMP_SPEED ) + Character_Jumps[ Plyr->Character ] )
							{
								// No, no, no, you can only do a jump this powerful if you initiated it yourself
								Plyr->JumpPower = CHFL( MIN_JUMP_SPEED ) + Character_Jumps[ Plyr->Character ];
							}

						}
					}
					else
					{
						// Do the jump
						Plyr->JumpD = 1;
						Plyr->TurnX = 0;
						Plyr->JumpY = CHFL( 0.25 );
						Plyr->JumpPower = ( Plyr->BoardElevation - ( Plyr->LastBoardElevation + CHFL( 64.0 ) ) ) / 256;
						Plyr->BoardElevation = Plyr->LastBoardElevation;
						if( Plyr->JumpPower > CHFL( MIN_JUMP_SPEED ) + Character_Jumps[ Plyr->Character ] )
						{
							// No, no, no, you can only do a jump this powerful if you initiated it yourself
							Plyr->JumpPower = CHFL( MIN_JUMP_SPEED ) + Character_Jumps[ Plyr->Character ];
						}
					}
			}
		}
	}


// *********************
// Velocity calculations
// *********************

	if( Plyr->JumpY == CHFL( 0.0 ) && ( ( Modes[ Plyr->Style ] & MD_NOVEL ) == 0 ) )
	{
		if( Plyr->BoardElevation < 0 )	// Going up - slow down
		{
			Plyr->Desired = CHFL( 0.0 );	// If going up then desired is 0

			if( Plyr->BoardElevation < SPONG( -25.0 ) )
			{
				// > 35 degrees (very steep = slow down fast)
				Plyr->Vel -= CHFL( 4.0 );


				if( Plyr->Vel <= CHFL( 0.0 ) )
				{
					Plyr->Vel = CHFL( 0.0 );

					// Don't let it call the transition twice
					if( Plyr->SlowTransition == 0 )
					{
						if( Plyr->Fakie == 0 )
						{
							// Go to fakie stance
							Plyr->SlowTransition = 1;

							Plyr->Fakie = 1;
							Plyr->BoardAzimuth += CHFL( 2048.0 );
							Plyr->Cam.SweepAngle += CHFL( 2048.0 );
							Plyr->MoveAzimuth = Plyr->BoardAzimuth;
							Plyr->BoardElevation = -Plyr->BoardElevation;
							Plyr->LastBoardElevation = Plyr->BoardElevation;
							Plyr->Cam.SweepBonus = CHFL( -2048.0 );

							LerpToFakieStance(Plyr, PlayerNumber);
							RotateToFakieView( PlayerNumber );
						}
						else
						{
							// Go back to normal stance
							Plyr->SlowTransition = 1;

							Plyr->Fakie = 0;
							Plyr->BoardAzimuth += CHFL( 2048.0 );
							Plyr->Cam.SweepAngle += CHFL( 2048.0 );
							Plyr->MoveAzimuth = Plyr->BoardAzimuth;
							Plyr->BoardElevation = -Plyr->BoardElevation;
							Plyr->LastBoardElevation = Plyr->BoardElevation;
							Plyr->Cam.SweepBonus = CHFL( -2048.0 );

							LerpToNormalStance(Plyr, PlayerNumber);	// Incorporates GoToNormalView();
							RotateToFakieView( PlayerNumber );
						}

						// Better do this again, considering you've just messed it all up
						CalcRotXYZ( Plyr );
					}
				}
			}
			else
			{
				// < 35 degrees (slow down smoothly)

				// Slow down even smoother if tucked
				if( Plyr->Style == STY_TUCK && Plyr->AnimStage > 15 )
				{
					Plyr->Vel -= CHFL( 0.125 );
				}
				else
				{
					Plyr->Vel -= CHFL( 0.25 );
				}

				if( Plyr->Vel < CHFL( 0.0 ) ) Plyr->Vel = CHFL( 0.0 );
			}

		}
		else	// Going down the slope
		{
			// Calculate the desired speed
			Plyr->Desired = ( chabs( Plyr->BoardElevation ) / 60 ) + Character_Speeds[ Plyr->Character ];

			if( Plyr->SlowTransition != 0 )
			{
				if( Plyr->SlowTransition == 1 )
				{
					Plyr->Vel = CHFL( 0.0 );
				}
				else
				{
					Plyr->Desired /= 50;
					Plyr->SlowTransition++;
					Plyr->SlowTransition&=7;
				}
			}

			if( Plyr->Fakie == 1 ) Plyr->Desired = ( Plyr->Desired / 5 ) * 4;

			// Slow down on a rock
			if( Collision( PlayerNumber ) == ROCK )
			{
				Plyr->Desired = ( Plyr->Desired / 3 ) * 1;
				if( Plyr->Vel > CHFL( 3.0 ) )
					Plyr->Vel -= CHFL( 0.50 );
			}

			if( Plyr->Style == STY_TUCK && Plyr->AnimStage > 15 )	Plyr->Desired += CHFL( 4.0 );

			if( Plyr->Vel < Plyr->Desired )
			{
				Plyr->Vel += ( ( Plyr->Desired - Plyr->Vel ) / 25 );
			}
			else
			{
				if( Plyr->Style == STY_TUCK && Plyr->AnimStage > 15 )
				{
					Plyr->Vel -= CHFL( 0.025 );
				}
				else
				{
					Plyr->Vel -= CHFL( 0.05 );
				}
			}

			// If SlowTransition == 1 then don't let it move at all costs
			if( Plyr->SlowTransition == 1 ) Plyr->Vel = CHFL( 0.0 );

			if( Plyr->Vel <= CHFL( 0.0 ) ) Plyr->Vel = CHFL( 0.0 );
		}
	}


	if( PermitMove == 0 )
	{
		Plyr->Vel = CHFL( 0.0 );
	}

// **********************
// Player animation logic
// **********************

	switch( Plyr->Style )
	{
		case STY_NORMAL:	// Temporary basic cycle

			if( Anim_Status( PlayerNumber ) == AS_STATIC )
			{
				int16 Temp;
				// Relative frame anim list for rocking back and forth
				CalculateCrouch(Plyr);

				if( Plyr->BoardElevation < 0 )
				{
					// Board pointing up
					Temp = ( ( - CHSHIFTR( Plyr->BoardElevation ) ) / 12 ) + 50;
					if( Temp < 50 ) Temp = 50;
					if( Temp > 85 ) Temp = 85;
				}
				else
				{
					// Board pointing down
					Temp = 50 - ( CHSHIFTR( Plyr->BoardElevation ) / 10 );
					if( Temp < 0 ) Temp = 0;
					if( Temp > 50 ) Temp = 50;
				}

				if( Plyr->AnimStage < Temp ) Plyr->AnimStage++;
				if( Plyr->AnimStage > Temp ) Plyr->AnimStage--;

				Iy = Plyr->TurnY / 20;

				Temp = Plyr->AnimStage + RockAround[ Plyr->AnimFrame ];


				if( Plyr->Fakie == 0 )
				{
					SetTweenOfTwoTweens( PlayerNumber,
						FRAMEEQU( PlayerNumber, NormalTable[ Iy ][ 1 ] ), FRAMEEQU( PlayerNumber, NormalTable[ Iy ][ 2 ] ),
						FRAMEEQU( PlayerNumber, NormalTable[ Iy + 1 ][ 1 ] ), FRAMEEQU( PlayerNumber, NormalTable[ Iy + 1 ][ 2 ] ),
						100, Temp,
						100, Temp,
						20, Plyr->TurnY % 20 );
				}
				else
				{
					SetTweenOfTwoTweens( PlayerNumber,
						FRAMEEQU( PlayerNumber, FakieNormalTable[ Iy ][ 1 ] ), FRAMEEQU( PlayerNumber, FakieNormalTable[ Iy ][ 2 ] ),
						FRAMEEQU( PlayerNumber, FakieNormalTable[ Iy + 1 ][ 1 ] ), FRAMEEQU( PlayerNumber, FakieNormalTable[ Iy + 1 ][ 2 ] ),
						100, Temp,
						100, Temp,
						20, Plyr->TurnY % 20 );
				}



/*				if( Temp < 25 )
				{
					SetTweenOfTwoTweens( PlayerNumber,
						FRAMEEQU( PlayerNumber, NormalTable[ Iy ][ 0 ] ), FRAMEEQU( PlayerNumber, NormalTable[ Iy ][ 1 ] ),
						FRAMEEQU( PlayerNumber, NormalTable[ Iy + 1 ][ 0 ] ), FRAMEEQU( PlayerNumber, NormalTable[ Iy + 1 ][ 1 ] ),
						25, Temp,
						25, Temp,
						20, Plyr->TurnY % 20 );
				}

				if( Temp >= 25 && Temp < 75 )
				{

					SetTweenOfTwoTweens( PlayerNumber,
						FRAMEEQU( PlayerNumber, NormalTable[ Iy ][ 1 ] ), FRAMEEQU( PlayerNumber, NormalTable[ Iy ][ 2 ] ),
						FRAMEEQU( PlayerNumber, NormalTable[ Iy + 1 ][ 1 ] ), FRAMEEQU( PlayerNumber, NormalTable[ Iy + 1 ][ 2 ] ),
						50, Temp - 25,
						50, Temp - 25,
						20, Plyr->TurnY % 20 );

				}

				if( Temp >= 75 )
				{
						SetTweenOfTwoTweens( PlayerNumber,
							FRAMEEQU( PlayerNumber, NormalTable[ Iy ][ 2 ] ), FRAMEEQU( PlayerNumber, NormalTable[ Iy ][ 3 ] ),
							FRAMEEQU( PlayerNumber, NormalTable[ Iy + 1 ][ 2 ] ), FRAMEEQU( PlayerNumber, NormalTable[ Iy + 1 ][ 3 ] ),
							25, Temp - 75,
							25, Temp - 75,
							20, Plyr->TurnY % 20 );
				}
*/
				Plyr->AnimFrame++;
				if( RockAround[ Plyr->AnimFrame ] == -128 ) Plyr->AnimFrame = 0;

			}
			break;

		case STY_TURNING:	// Player is turning

//			printf("Bigot\n");
			CalculateCrouch(Plyr);

			Iy = Plyr->TurnY / 20;

			Tween = CHSHIFTR( Plyr->TurnX );

			if( Tween < 0 )
			{
				// left turning frames
				if( Tween <= -15 )
				{
					Ix = 1;
					Tween = -Tween - 15;
					Lerps = 25;
				}
				else
				{
					Ix = 2;
					Tween =	-Tween;
					Lerps = 15;
				}

				if( Plyr->Fakie == 0 )
				{
					SetTweenOfTwoTweens( PlayerNumber,
						FRAMEEQU( PlayerNumber, TurnTable[ Iy ][ Ix + 1 ] ), FRAMEEQU( PlayerNumber, TurnTable[ Iy ][ Ix ] ),
						FRAMEEQU( PlayerNumber, TurnTable[ Iy + 1 ][ Ix + 1 ] ), FRAMEEQU( PlayerNumber, TurnTable[ Iy + 1 ][ Ix ] ),
						Lerps, Tween,
						Lerps, Tween,
						20, Plyr->TurnY % 20 );
				}
				else
				{
					SetTweenOfTwoTweens( PlayerNumber,
						FRAMEEQU( PlayerNumber, FakieTurnTable[ Iy ][ Ix + 1 ] ), FRAMEEQU( PlayerNumber, FakieTurnTable[ Iy ][ Ix ] ),
						FRAMEEQU( PlayerNumber, FakieTurnTable[ Iy + 1 ][ Ix + 1 ] ), FRAMEEQU( PlayerNumber, FakieTurnTable[ Iy + 1 ][ Ix ] ),
						Lerps, Tween,
						Lerps, Tween,
						20, Plyr->TurnY % 20 );
				}

			}
			else
			{
				// right turning frames
				if( Tween <= 15 )
				{
					Ix = 4;
					Lerps = 15;
				}
				else
				{
					Ix = 5;
					Lerps = 25;
					Tween = Tween - 15;
				}

				if( Plyr->Fakie == 0 )
				{
					SetTweenOfTwoTweens( PlayerNumber,
						FRAMEEQU( PlayerNumber, TurnTable[ Iy ][ Ix ] ), FRAMEEQU( PlayerNumber, TurnTable[ Iy ][ Ix + 1 ] ),
						FRAMEEQU( PlayerNumber, TurnTable[ Iy + 1 ][ Ix ] ), FRAMEEQU( PlayerNumber, TurnTable[ Iy + 1 ][ Ix + 1 ] ),
						Lerps, Tween,
						Lerps, Tween,
						20, Plyr->TurnY % 20 );
				}
				else
				{
					SetTweenOfTwoTweens( PlayerNumber,
						FRAMEEQU( PlayerNumber, FakieTurnTable[ Iy ][ Ix ] ), FRAMEEQU( PlayerNumber, FakieTurnTable[ Iy ][ Ix + 1 ] ),
						FRAMEEQU( PlayerNumber, FakieTurnTable[ Iy + 1 ][ Ix ] ), FRAMEEQU( PlayerNumber, FakieTurnTable[ Iy + 1 ][ Ix + 1 ] ),
						Lerps, Tween,
						Lerps, Tween,
						20, Plyr->TurnY % 20 );
				}
			}
			break;

		case STY_PRE_TURNING:	// Player is going to first turn frame from basic cycle
			if( Anim_Status( PlayerNumber ) == AS_STATIC )
			{
				Plyr->Style = STY_TURNING;
			}
			break;

		case STY_DOLMIO:
			if( Pad & PAD_B4 )
			{
				Plyr->Vel = CHFL( 20.0 );
			}
			else
			{
				Plyr->Vel = CHFL( 0.0 );
			}

			if( Pad & PAD_B2 )
			{
				Plyr->Vel = CHFL( -10.0 );
			}

			if( ( Pad & PAD_LEFT ) || ( Pad & PAD_RIGHT ) )
			{
				if( Pad & PAD_LEFT ) Plyr->TurnX = CHFL( -60.0 );
				if( Pad & PAD_RIGHT ) Plyr->TurnX = CHFL( 60.0 );
			}
			else
			{
				Plyr->TurnX = 0;
			}

			if( Pad & PAD_UP )	Plyr->JumpY += CHFL( 5.0 );
			if( Pad & PAD_DOWN )	Plyr->JumpY -= CHFL( 5.0 );
			if( Pad & PAD_B1 )	Plyr->JumpY = CHFL( 0.0 );
			break;

		case STY_PRE_JUMP:
			// If the jump button has been released then start the jump
			if( !Config_JUMP( Plyr->PadNumber ) )
			{
				Plyr->SpinRot.X = Plyr->Rot.X;
				Plyr->SpinRot.Y = Plyr->Rot.Y;
				Plyr->SpinRot.Z = Plyr->Rot.Z;

				Plyr->TwistDist = CHFL( 0.0 );
				Plyr->SpinDist = CHFL( 0.0 );

				// Just a quick check to see whether or not it's a twist or a spin jump
				if( Plyr->SpinSpeed == CHFL( 0.0 ) && Plyr->TwistSpeed == CHFL( 0.0 ) )
				{
					// It's a jump ( the normal medium height one
					Plyr->Style = STY_SJUMPM;
					Plyr->AnimStage = 0;
				}
				else
				{
					if( Plyr->SpinSpeed != CHFL( 0.0 ) && Plyr->TwistSpeed != CHFL( 0.0 ) )
					{
						if( Plyr->Fakie == 0 )	// You can't do this from the fakie
						{
							// Both Spin AND Twist have been initiated
							Plyr->Style = STY_TWISTANDSPIN;
							GoToTwistView( PlayerNumber );
							Plyr->AnimStage = 0;
							Plyr->JumpD = 1;
						}
						else
						{
							Plyr->Style = STY_SJUMPM;
							Plyr->AnimStage = 0;
						}
					}
					else
					{
						if( Plyr->Fakie == 0 )	// Prevent spins from fakie
						{
							// Front Spin ?
							if( Plyr->SpinSpeed > CHFL( 0.0 ) )
							{
								Plyr->Style = STY_FRONTSPIN;
								GoToSpinView( PlayerNumber );
								Plyr->AnimStage = 0;
//								Plyr->JumpD = 1;
//								Plyr->Vel = CHFL( 0.1 );
//								Plyr->JumpPower = CHFL( 10.0 );
							}

							// Back Spin ?
							if( Plyr->SpinSpeed < CHFL( 0.0 ) )
							{
								Plyr->Style = STY_BACKSPIN;
								GoToSpinView( PlayerNumber );
								Plyr->AnimStage = 0;
								Plyr->JumpD = 1;
							}
						}
						else
						{
							Plyr->Style = STY_SJUMPM;
							Plyr->AnimStage = 0;
						}

						// Right Twist ?
						if( Plyr->TwistSpeed > CHFL( 0.0 ) )
						{
							Plyr->Style = STY_TWISTR;
							GoToTwistView( PlayerNumber );
							Plyr->AnimStage = 0;
							Plyr->JumpD = 1;
						}

						// Left Twist ?
						if( Plyr->TwistSpeed < CHFL( 0.0 ) )
						{
							Plyr->Style = STY_TWISTL;
							GoToTwistView( PlayerNumber );
							Plyr->AnimStage = 0;
							Plyr->JumpD = 1;
						}
					}
				}
			}
/*			else
			{
				// If not in or going to the correct crouch frame then go to another

				// B_SJUMPM_2 is the standard crouch frame
				// B_FS1 is the front spin crouch frame
				// B_BS1 is the back spin crouch frame

				// Crouch frame for Back Spin
				if( Plyr->SpinSpeed < CHFL( 0.0 ) )
				{
					if( DestLerpFrame( PlayerNumber ) != FRAMEEQU( PlayerNumber, B_BS1 ) )
						SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_BS1 ), 5 );
				}

				// Crouch frame for Forward Spin
				if( Plyr->SpinSpeed > CHFL( 0.0 ) )
				{
					if( DestLerpFrame( PlayerNumber ) != FRAMEEQU( PlayerNumber, B_FS1 ) )
						SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_FS1 ), 5 );
				}

				// Return to normal jump crouch
				if( !( Pad & PAD_UP || Pad & PAD_DOWN ) )
				{
					if( DestLerpFrame( PlayerNumber ) != FRAMEEQU( PlayerNumber, B_SJUMPM_2 ) )
					{
						SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_SJUMPM_2 ), 5 );
					}
					Plyr->SpinSpeed = CHFL( 0.0 );
				}
			}
*/
			break;

		case STY_PREP_JUMP:
			if( Anim_Status( PlayerNumber ) == AS_STATIC )
			{
				if( Plyr->Fakie == 0 )
				{
					if( DestLerpFrame( PlayerNumber ) == FRAMEEQU( PlayerNumber, B_SJUMPM_1 ) )
					{
						SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_SJUMPM_2 ), 5 );
					}
					else
					{
						Plyr->Style = STY_PRE_JUMP;
					}
				}
				else
				{
					if( DestLerpFrame( PlayerNumber ) == FRAMEEQU( PlayerNumber, B_FJUMP_1 ) )
					{
						SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_FJUMP_2 ), 5 );
					}
					else
					{
						Plyr->Style = STY_PRE_JUMP;
					}
				}
			}

			// If the jump key is no longer being pressed then go straight to the little jump rather than do a crouch
			if( !Config_JUMP( Plyr->PadNumber ) && Plyr->Fakie == 0 )
			{
				Plyr->Style = STY_SJUMPL;
				Plyr->AnimStage = 0;
				Plyr->JumpD = 1;
				SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, SJUMPL_List[ Plyr->AnimStage ] ), SJUMPL_Time[ Plyr->AnimStage ] );
			}
			break;

		case STY_SJUMPM:
			if( Anim_Status( PlayerNumber ) == AS_STATIC )
			{
				if( Plyr->Fakie == 1 )
				{
					SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, FakieSJUMPM_List[ Plyr->AnimStage ] ), SJUMPM_Time[ Plyr->AnimStage ] );
					if( DestLerpFrame( PlayerNumber ) == FRAMEEQU( PlayerNumber, B_FJUMP_4 ) )	// Ready to leave the ground yet ?
					{
						Plyr->JumpD = 1;
						Plyr->Style = STY_SJUMPM_AIR;
					}
				}
				else
				{
					SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, SJUMPM_List[ Plyr->AnimStage ] ), SJUMPM_Time[ Plyr->AnimStage ] );
					if( DestLerpFrame( PlayerNumber ) == FRAMEEQU( PlayerNumber, B_SJUMPM_4 ) )	// Ready to leave the ground yet ?
					{
						Plyr->JumpD = 1;
						Plyr->Style = STY_SJUMPM_AIR;
					}
				}
				Plyr->AnimStage++;
			}
			break;

		case STY_SJUMPM_AIR:
			// If grab is held and we are off the ground
			if( Plyr->GrabHeld != 0 )
			{
				Grab(Plyr, PlayerNumber);
				if( Plyr->JumpY <= CHFL( 0.0 ) )
				{
					SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_LATE1 ), 3 );
					Plyr->Style = STY_LATEGRAB;
					Plyr->AnimStage = 0;
				}
			}
			else
			{
				if( Plyr->HaveGrabbed != 0 )
				{
					// We have to recover him to a decent frame
					if( Plyr->Fakie == 0 )
						SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_SJUMPM_6 ), 15 );
					else
						SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_FJUMP_6 ), 15 );
					Plyr->HaveGrabbed = 0;
				}

				if( Plyr->Fakie == 0 )
				{
					if( Anim_Status( PlayerNumber ) == AS_STATIC && DestLerpFrame( PlayerNumber ) != FRAMEEQU( PlayerNumber, B_SJUMPM_6 ) )
					{
						SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, SJUMPM_List[ Plyr->AnimStage ] ), SJUMPM_Time[ Plyr->AnimStage ] );
						Plyr->AnimStage++;
					}
				}
				else
				{
					if( Anim_Status( PlayerNumber ) == AS_STATIC && DestLerpFrame( PlayerNumber ) != FRAMEEQU( PlayerNumber, B_FJUMP_6 ) )
					{
						SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, FakieSJUMPM_List[ Plyr->AnimStage ] ), SJUMPM_Time[ Plyr->AnimStage ] );
						Plyr->AnimStage++;
					}
				}

				// Make him land as he nears the ground
				if( Plyr->JumpY < CHFL( 40.0 ) && Plyr->JumpD != 1 )			// 80.0 for a massive fall
				{
					if( Plyr->Fakie == 0 )
						SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_SJUMPM_7 ), 2 );
					else
						SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_FJUMP_7 ), 2 );
					Plyr->Style = STY_SJUMPPREPLANDM;
					Plyr->AnimStage = 0;
				}
			}
			break;

		case STY_SJUMPPREPLANDM:
			if( Anim_Status( PlayerNumber ) == AS_STATIC )
			{
				if( Plyr->Fakie == 0 )
				{
					if( DestLerpFrame( PlayerNumber ) == FRAMEEQU( PlayerNumber, B_SJUMPM_10 ) )
						LerpToNormalStance(Plyr, PlayerNumber);
					else
					{
						SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, SJUMPPREPLANDM_List[ Plyr->AnimStage ] ), SJUMPPREPLANDM_Time[ Plyr->AnimStage ] );
						Plyr->AnimStage++;
					}
				}
				else
				{
					if( DestLerpFrame( PlayerNumber ) == FRAMEEQU( PlayerNumber, B_FJUMP_10 ) )
						LerpToNormalStance(Plyr, PlayerNumber);
					else
					{
						SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, FakieSJUMPPREPLANDM_List[ Plyr->AnimStage ] ), SJUMPPREPLANDM_Time[ Plyr->AnimStage ] );
						Plyr->AnimStage++;
					}
				}
			}
			break;

		case STY_SJUMPL:	// Straight jump, little
			if( Plyr->GrabHeld != 0 )//&& Plyr->JumpY > CHFL( 0.0 ) )
			{
					Grab(Plyr, PlayerNumber);
					if( Plyr->JumpY <= CHFL( 0.0 ) )
					{
						SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_LATE1 ), 3 );
						Plyr->Style = STY_LATEGRAB;
						Plyr->AnimStage = 0;
					}
			}
			else
			if( Anim_Status( PlayerNumber ) == AS_STATIC )
			{
				if( Plyr->HaveGrabbed != 0 )
				{
					// We have to recover him to a decent frame
					Plyr->HaveGrabbed = 0;
					Plyr->AnimStage = 2;
					SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, SJUMPL_List[ Plyr->AnimStage ] ), SJUMPL_Time[ Plyr->AnimStage ] );
					Plyr->AnimStage++;
				}

				if( DestLerpFrame( PlayerNumber ) == FRAMEEQU( PlayerNumber, B_SJUMPM_10 ) )
				{
					// Return to normal stance
					LerpToNormalStance(Plyr, PlayerNumber);
				}
				else
				{
					// Do the next frame
					if( DestLerpFrame( PlayerNumber ) == FRAMEEQU( PlayerNumber, B_SJUMPL_2 ) )
					{
						if( Plyr->JumpY < CHFL( 10.0 ) && Plyr->JumpD != 1 )
						{
							Plyr->AnimStage = 3;
							SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, SJUMPL_List[ Plyr->AnimStage ] ), SJUMPL_Time[ Plyr->AnimStage ] );
							Plyr->AnimStage++;
						}
					}
					else
					{
						SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, SJUMPL_List[ Plyr->AnimStage ] ), SJUMPL_Time[ Plyr->AnimStage ] );
						Plyr->AnimStage++;
					}

				}
			}
			break;

		case STY_AUTO_JUMP:
			printf("I didn't think this was used anymore!\n");
			if( Plyr->JumpY <= CHFL( 0.0 ) )
			{
				LerpToNormalStance(Plyr, PlayerNumber);
			}
			break;

		case STY_FRONTSPIN:
			if( Plyr->GrabHeld != 0 && Plyr->AnimStage == 3 )
			{
				Grab(Plyr, PlayerNumber);
			}
			else
			{
				if( Plyr->HaveGrabbed != 0 )
				{
					Plyr->AnimStage = 3;
					Plyr->HaveGrabbed = 0;
					SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, FRONTSPIN_List[ 2 ] ), 5 );
				}

				if( Anim_Status( PlayerNumber ) == AS_STATIC && Plyr->AnimStage <= 2 )
				{
					if( Plyr->AnimStage == 1 ) Plyr->JumpD = 1;
					SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, FRONTSPIN_List[ Plyr->AnimStage ] ), FRONTSPIN_Time[ Plyr->AnimStage ] );
					Plyr->AnimStage++;
				}
			}


			// Is it on the spinning frame
			if( Plyr->AnimStage == 3 )
			{
				if( !Config_FRONTSPIN( Plyr->PadNumber ) ) Plyr->SpinHeld = 0;

				Plyr->SpinRot.X += Plyr->SpinSpeed;
				Plyr->SpinDist += Plyr->SpinSpeed;

				if( Plyr->SpinRot.X >= CHFL( 4096.0 ) ) Plyr->SpinRot.X -= CHFL( 4096.0 );	// Stop it getting bigger than 4096

				// Has the spin has been cancelled
				if( Plyr->SpinHeld == 0 )
				{
					if(
//					( CHSHIFTR( Plyr->SpinRot.X ) & 4095 ) > 3840 ||
					( CHSHIFTR( Plyr->SpinRot.X ) & 4095 ) < 512 )
					{
						printf("Spin & Land\n");
						Plyr->Style = STY_FSLAND;
						break;
					}
				}
			}


			if( Plyr->JumpY <= CHFL( 0.0 ) && Plyr->JumpD != 1 && Plyr->AnimStage > 1 )
			{
				Plyr->Style = STY_ST_ROLL;
				SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_ST_ROLL1 ), 5 );
				Plyr->AnimStage = 0;
				Plyr->SpinRot.X = Plyr->BoardElevation;
				Plyr->SpinRot.Y = Plyr->BoardAzimuth - SPONG( 90.0 );
				Plyr->SpinRot.Z = Plyr->BoardRoll;
				GoToCrashView( PlayerNumber );
				Plyr->Fakie = 0;
			}
			break;

		case STY_FSLAND:
			if( Anim_Status( PlayerNumber ) == AS_STATIC )
			{
					if( FRONTSPIN_Time[ Plyr->AnimStage ] != -128 )
					{
						SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, FRONTSPIN_List[ Plyr->AnimStage ] ), FRONTSPIN_Time[ Plyr->AnimStage ] );
						Plyr->AnimStage++;
					}
			}

			if( Plyr->JumpY <= CHFL( 0.0 ) )
			{
				Spin_Scoring(Plyr, PlayerNumber);
				SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_SJUMPM_8 ), 2 );
				Plyr->Style = STY_SJUMPPREPLANDM;
				Plyr->AnimStage = 1;
			}
			break;

		case STY_BACKSPIN:
			if( Plyr->GrabHeld != 0 && Plyr->AnimStage == 3 )
			{
				Grab(Plyr, PlayerNumber);
			}
			else
			{
				if( Plyr->HaveGrabbed != 0 )
				{
					Plyr->AnimStage = 3;
					Plyr->HaveGrabbed = 0;
					SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, BACKSPIN_List[ 2 ] ), 5 );
				}

				if( Anim_Status( PlayerNumber ) == AS_STATIC && Plyr->AnimStage <= 2 )
				{
					if( Plyr->AnimStage == 1 ) Plyr->JumpD = 1;
					SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, BACKSPIN_List[ Plyr->AnimStage ] ), BACKSPIN_Time[ Plyr->AnimStage ] );
					Plyr->AnimStage++;
				}
			}


			// Is it on the spinning frame
			if( Plyr->AnimStage == 3 )
			{
				if( !Config_BACKSPIN( Plyr->PadNumber ) ) Plyr->SpinHeld = 0;

				Plyr->SpinRot.X += Plyr->SpinSpeed;
				Plyr->SpinDist += Plyr->SpinSpeed;

				if( Plyr->SpinRot.X >= CHFL( 4096.0 ) ) Plyr->SpinRot.X -= CHFL( 4096.0 );	// Stop it getting bigger than 4096

				// Has the spin has been cancelled
				if( Plyr->SpinHeld == 0 )
				{
					if(
//					( CHSHIFTR( Plyr->SpinRot.X ) & 4095 ) > 3840 ||
					( CHSHIFTR( Plyr->SpinRot.X ) & 4095 ) < 512 )
					{
						printf("Spin & Land\n");
						Plyr->Style = STY_BSLAND;
						break;
					}
				}
			}


			if( Plyr->JumpY <= CHFL( 0.0 ) && Plyr->JumpD != 1 && Plyr->AnimStage > 1 )
			{
				Plyr->Style = STY_ST_ROLL;
				SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_ST_ROLL1 ), 5 );
				Plyr->AnimStage = 0;
				Plyr->SpinRot.X = Plyr->BoardElevation;
				Plyr->SpinRot.Y = Plyr->BoardAzimuth - SPONG( 90.0 );
				Plyr->SpinRot.Z = Plyr->BoardRoll;
				GoToCrashView( PlayerNumber );
				Plyr->Fakie = 0;
			}
			break;

		case STY_BSLAND:
			if( Anim_Status( PlayerNumber ) == AS_STATIC )
			{
					if( BACKSPIN_Time[ Plyr->AnimStage ] != -128 )
					{
						SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, BACKSPIN_List[ Plyr->AnimStage ] ), FRONTSPIN_Time[ Plyr->AnimStage ] );
						Plyr->AnimStage++;
					}
			}

			if( Plyr->JumpY <= CHFL( 0.0 ) )
			{
				Spin_Scoring(Plyr, PlayerNumber);
				SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_SJUMPM_8 ), 2 );
				Plyr->Style = STY_SJUMPPREPLANDM;
				Plyr->AnimStage = 1;
			}
			break;

		case STY_RECOVERY:
			if( Anim_Status( PlayerNumber ) == AS_STATIC )
			{
				SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, RECOVERY_List[ Plyr->AnimStage ] ), RECOVERY_Time[ Plyr->AnimStage ] );
				Plyr->AnimStage++;

				if( RECOVERY_Time[ Plyr->AnimStage ] == -128 )
					Plyr->AnimStage = 0;
			}

			if( ( Plyr->SpinRot.X > ( Plyr->BoardElevation - CHFL( 256.0 ) ) ) && ( Plyr->SpinRot.X < ( Plyr->BoardElevation + CHFL( 256.0 ) ) ) )
			{
				if( Plyr->SpinSpeed >= CHFL( -50.0 ) && Plyr->SpinSpeed <= CHFL( 50.0 ) )
				{
					Plyr->SpinSpeed = CHFL( 0.0 );
//					SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_RCYC7 ), 20 );	// TWEENEY
					Plyr->AnimStage = 0;
//					Plyr->Style = STY_RECOVERED;

					SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_RCYC7 ), 20 );
					Plyr->Style = STY_RECLAND1;
				}
			}

			if( Plyr->SpinSpeed < CHFL( 25.0 ) )
			{
				Plyr->SpinSpeed += CHFL( 25.0 );
				if( Plyr->SpinSpeed > CHFL( -25.0 ) ) Plyr->SpinSpeed = CHFL( -25.0 );
			}

			if( Plyr->SpinSpeed > CHFL( 25.0 ) )
			{
				Plyr->SpinSpeed -= CHFL( 25.0 );
				if( Plyr->SpinSpeed < CHFL( 25.0 ) ) Plyr->SpinSpeed = CHFL( 25.0 );
			}

			Plyr->SpinRot.X += Plyr->SpinSpeed;
			Plyr->SpinDist += Plyr->SpinSpeed;
			if( Plyr->SpinRot.X < CHFL( 0.0 ) ) Plyr->SpinRot.X += CHFL( 4096.0 );
			if( Plyr->SpinRot.X > CHFL( 4096.0 ) ) Plyr->SpinRot.X -= CHFL( 4096.0 );



			// Near the ground - detect the landing type
			if( Plyr->JumpY < CHFL( 40.0 ) && Plyr->JumpD != 1 )
			{
				chfloat PhaseRot = Plyr->SpinRot.X;

//				if( PhaseRot < CHFL( 0.0 ) ) PhaseRot += CHFL( 4096.0 );
//				if( PhaseRot >= CHFL( 4096.0 ) ) PhaseRot -= CHFL( 4096.0 );

				// landing on head (180)
				if( PhaseRot > CHFL( 1536.0 ) || PhaseRot < CHFL( 2560.0 ) )
				{
					Plyr->Style = STY_LANDONHEAD;
					SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_HCR1 ), 5 );
					Plyr->AnimStage = 0;
					Plyr->JumpD = 0;
					RMode[ PlayerNumber ] = AS_STATIC;
				}

				// landing on left side (90)
				if( PhaseRot > CHFL( 1024.0 ) || PhaseRot <= CHFL( 1536.0 ) )
				{
					Plyr->Style = STY_LANDONLEFT;
					GoToTwistView( PlayerNumber );
					Plyr->AnimStage = 0;
					SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_LCR2 ), 3 );
				}

				// Near the ground - landing on right side (270)
				if( PhaseRot >= CHFL( 2560.0 ) || PhaseRot <= CHFL( 3072.0 ) )
				{
					Plyr->Style = STY_LANDONRIGHT;
					Plyr->AnimStage = 0;
					SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_RCR2 ), 3 );
				}
			}

			break;

		// doesn't look like I use this anymore
		case STY_RECOVERED:
			if( Anim_Status( PlayerNumber ) == AS_STATIC )
			{
				SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, RECOVERED_List[ Plyr->AnimStage ] ), RECOVERED_Time[ Plyr->AnimStage ] );
				Plyr->AnimStage++;

				if( RECOVERED_Time[ Plyr->AnimStage ] == -128 )
					Plyr->AnimStage = 1;
			}

			if( Plyr->JumpY <= CHFL( 40.0 ) )
			{
				SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_RCYC12 ), 4 );
				Plyr->Style = STY_RECLAND1;
			}
			break;

		case STY_RECLAND1:
			if( Anim_Status( PlayerNumber ) == AS_STATIC )
			{
				if( DestLerpFrame( PlayerNumber ) == FRAMEEQU( PlayerNumber, B_RCYC14 ) )
				{
					SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_RCYC15 ), 2 );
					Plyr->AnimStage = 0;
					Plyr->Style = STY_RECLAND2;
				}
				if( DestLerpFrame( PlayerNumber ) == FRAMEEQU( PlayerNumber, B_RCYC12 ) )
					SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_RCYC13 ), 5 );

				if( DestLerpFrame( PlayerNumber ) == FRAMEEQU( PlayerNumber, B_RCYC7 ) )
					SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_RCYC12 ), 15 );

			}

			if( Plyr->JumpY < CHFL( 30.0 ) && DestLerpFrame( PlayerNumber ) != FRAMEEQU( PlayerNumber, B_RCYC14 ) )
			{
				SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_RCYC14 ), 4 );
			}


			if( Plyr->JumpY <= CHFL( 0.0 ) )
			{
					SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_RCYC15 ), 2 );
					Plyr->AnimStage = 0;
					Plyr->Style = STY_RECLAND2;
					Spin_Scoring(Plyr, PlayerNumber);
			}
			break;

		case STY_RECLAND2:
			if( Anim_Status( PlayerNumber ) == AS_STATIC )
			{
				SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, RECLAND2_List[  Plyr->AnimStage ] ), RECLAND2_Time[ Plyr->AnimStage ] );

				if( FRAMEEQU( PlayerNumber, RECLAND2_List[ Plyr->AnimStage ] ) == FRAMEEQU( PlayerNumber, B_RCYC18 ) )
				{
					Plyr->JumpD = 1;
					Plyr->JumpPower = CHFL( 2.0 );
				}

				Plyr->AnimStage++;
				if( RECLAND2_Time[ Plyr->AnimStage ] == -128 )
				{
					LerpToNormalStance(Plyr, PlayerNumber);
				}
			}
			break;

		case STY_LANDONHEAD:
			if( Plyr->Vel > CHFL( 5.0 ) ) Plyr->Vel -= CHFL( 0.25 );

			if( Anim_Status( PlayerNumber ) == AS_STATIC )
			{
					if( LANDONHEAD_Time[ Plyr->AnimStage ] == -128 )
					{
						LerpToNormalStance(Plyr, PlayerNumber);
						Plyr->JumpY = CHFL( 0.0 );
						Plyr->JumpD = 0;
					}
					else
					{
						SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, LANDONHEAD_List[ Plyr->AnimStage ] ), LANDONHEAD_Time[ Plyr->AnimStage ] );

						TVec.X = Plyr->SpinRot.X + LANDONHEAD_Rots[ Plyr->AnimStage ].X;
						TVec.Y = Plyr->SpinRot.Y + LANDONHEAD_Rots[ Plyr->AnimStage ].Y;
						TVec.Z = Plyr->SpinRot.Z + LANDONHEAD_Rots[ Plyr->AnimStage ].Z;

						if( LANDONHEAD_Rots[ Plyr->AnimStage ].X == SPONG( 0.0 ) )	// Lerp orient it with the ground
						{
							Plyr->SpinRot.X = TVec.X = KILLFRAC( Plyr->BoardElevation );
						}

						SetLerpRots( PlayerNumber, &Plyr->SpinRot, &TVec, LANDONHEAD_Time[ Plyr->AnimStage ], &Plyr->SpinRot );//, &Plyr->Base );

						if( DestLerpFrame( PlayerNumber ) == FRAMEEQU( PlayerNumber, B_HCR4 ) )
						{
							Plyr->JumpD = 1;
							Plyr->JumpPower = CHFL( 1.5 );
						}

						Plyr->AnimStage++;
					}
			}
			break;

		case STY_LANDONLEFT:

			// This is the one where he spins round on his belly
			if( Plyr->Vel > CHFL( 5.0 ) ) Plyr->Vel -= CHFL( 0.50 );

			if( Anim_Status( PlayerNumber ) == AS_STATIC )
			{
				if( LANDONLEFT_Time[ Plyr->AnimStage ] == -128 )
				{
					LerpToNormalStance(Plyr, PlayerNumber);
					Plyr->JumpY = CHFL( 0.0 );
					Plyr->JumpD = 0;
					Plyr->Vel = CHFL( 0.0 );
				}
				else
				{
					SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, LANDONLEFT_List[ Plyr->AnimStage ] ), LANDONLEFT_Time[ Plyr->AnimStage ] );

					TVec.X = Plyr->SpinRot.X + LANDONLEFT_Rots[ Plyr->AnimStage ].X;
					TVec.Y = Plyr->SpinRot.Y + LANDONLEFT_Rots[ Plyr->AnimStage ].Y;
					TVec.Z = Plyr->SpinRot.Z + LANDONLEFT_Rots[ Plyr->AnimStage ].Z;
					SetLerpRots( PlayerNumber, &Plyr->SpinRot, &TVec, LANDONLEFT_Time[ Plyr->AnimStage ], &Plyr->SpinRot );

					Plyr->AnimStage++;
				}
			}
			break;




		case STY_LANDONRIGHT:
			if( Anim_Status( PlayerNumber ) == AS_STATIC )
			{
				if( LANDONRIGHT_Time[ Plyr->AnimStage ] == -128 )
				{
					LerpToNormalStance(Plyr, PlayerNumber);
					Plyr->JumpY = CHFL( 0.0 );
					Plyr->JumpD = 0;
				}
				else
				{
					SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, LANDONRIGHT_List[ Plyr->AnimStage ] ), LANDONRIGHT_Time[ Plyr->AnimStage ] );

					TVec.X = Plyr->SpinRot.X + LANDONRIGHT_Rots[ Plyr->AnimStage ].X;
					TVec.Y = Plyr->SpinRot.Y + LANDONRIGHT_Rots[ Plyr->AnimStage ].Y;
					TVec.Z = Plyr->SpinRot.Z + LANDONRIGHT_Rots[ Plyr->AnimStage ].Z;
					SetLerpRots( PlayerNumber, &Plyr->SpinRot, &TVec, LANDONRIGHT_Time[ Plyr->AnimStage ], &Plyr->SpinRot );

					if( DestLerpFrame( PlayerNumber ) == FRAMEEQU( PlayerNumber, B_RCR6 ) )
					{
						Plyr->JumpD = 1;
						Plyr->JumpPower = CHFL( 7.5 );
					}

					Plyr->AnimStage++;
				}
			}
			break;

		case STY_TUCK:
//			if( Anim_Status( PlayerNumber ) == AS_STATIC )
//			{
				CalculateCrouch(Plyr);

				Iy = Plyr->TurnY / 20;


				if( Plyr->AnimStage == 18 )
				{
					if( Plyr->TurnX == 0 )
					{
						if( Plyr->Fakie == 0 )
						SetTweenOfTwoTweens( PlayerNumber,
							FRAMEEQU( PlayerNumber, TuckTable[ Iy ][ 1 ] ), FRAMEEQU( PlayerNumber, TuckTable[ Iy ][ 2 ] ),
							FRAMEEQU( PlayerNumber, TuckTable[ Iy + 1 ][ 1 ] ), FRAMEEQU( PlayerNumber, TuckTable[ Iy + 1 ][ 2 ] ),
							12, Plyr->AnimStage - 6,
							12, Plyr->AnimStage - 6,
							10, ( Plyr->TurnY % 20 ) >> 1 );
						else
						SetTweenOfTwoTweens( PlayerNumber,
							FRAMEEQU( PlayerNumber, FTuckTable[ Iy ][ 1 ] ), FRAMEEQU( PlayerNumber, FTuckTable[ Iy ][ 2 ] ),
							FRAMEEQU( PlayerNumber, FTuckTable[ Iy + 1 ][ 1 ] ), FRAMEEQU( PlayerNumber, FTuckTable[ Iy + 1 ][ 2 ] ),
							12, Plyr->AnimStage - 6,
							12, Plyr->AnimStage - 6,
							10, ( Plyr->TurnY % 20 ) >> 1 );
					}
					else
					{
						if( Plyr->TurnX < 0 )
						{
							// TuckLeftTable
							if( Plyr->Fakie == 0 )
							SetTweenOfTwoTweens( PlayerNumber,
								FRAMEEQU( PlayerNumber, TuckLeftTable[ Iy ][ 0 ] ), FRAMEEQU( PlayerNumber, TuckLeftTable[ Iy ][ 1 ] ),
								FRAMEEQU( PlayerNumber, TuckLeftTable[ Iy + 1 ][ 0 ] ), FRAMEEQU( PlayerNumber, TuckLeftTable[ Iy + 1 ][ 1 ] ),
								20, -CHSHIFTR( Plyr->TurnX )>>1,
								20, -CHSHIFTR( Plyr->TurnX )>>1,
								20, Plyr->TurnY % 20 );
							else
							SetTweenOfTwoTweens( PlayerNumber,
								FRAMEEQU( PlayerNumber, FTuckLeftTable[ Iy ][ 0 ] ), FRAMEEQU( PlayerNumber, FTuckLeftTable[ Iy ][ 1 ] ),
								FRAMEEQU( PlayerNumber, FTuckLeftTable[ Iy + 1 ][ 0 ] ), FRAMEEQU( PlayerNumber, FTuckLeftTable[ Iy + 1 ][ 1 ] ),
								20, -CHSHIFTR( Plyr->TurnX )>>1,
								20, -CHSHIFTR( Plyr->TurnX )>>1,
								20, Plyr->TurnY % 20 );
						}

						if( Plyr->TurnX > 0 )
						{
							// TuckRightTable
							if( Plyr->Fakie == 0 )
							SetTweenOfTwoTweens( PlayerNumber,
								FRAMEEQU( PlayerNumber, TuckRightTable[ Iy ][ 0 ] ), FRAMEEQU( PlayerNumber, TuckRightTable[ Iy ][ 1 ] ),
								FRAMEEQU( PlayerNumber, TuckRightTable[ Iy + 1 ][ 0 ] ), FRAMEEQU( PlayerNumber, TuckRightTable[ Iy + 1 ][ 1 ] ),
								20, CHSHIFTR( Plyr->TurnX )>>1,
								20, CHSHIFTR( Plyr->TurnX )>>1,
								20, Plyr->TurnY % 20 );
							else
							SetTweenOfTwoTweens( PlayerNumber,
								FRAMEEQU( PlayerNumber, FTuckRightTable[ Iy ][ 0 ] ), FRAMEEQU( PlayerNumber, FTuckRightTable[ Iy ][ 1 ] ),
								FRAMEEQU( PlayerNumber, FTuckRightTable[ Iy + 1 ][ 0 ] ), FRAMEEQU( PlayerNumber, FTuckRightTable[ Iy + 1 ][ 1 ] ),
								20, CHSHIFTR( Plyr->TurnX )>>1,
								20, CHSHIFTR( Plyr->TurnX )>>1,
								20, Plyr->TurnY % 20 );
						}
	 				}
				}
				else
				{
					if( Plyr->AnimStage < 6 )
					{
						if( Plyr->Fakie == 0 )
						SetTweenOfTwoTweens( PlayerNumber,
							FRAMEEQU( PlayerNumber, TuckTable[ Iy ][ 0 ] ), FRAMEEQU( PlayerNumber, TuckTable[ Iy ][ 1 ] ),
							FRAMEEQU( PlayerNumber, TuckTable[ Iy + 1 ][ 0 ] ), FRAMEEQU( PlayerNumber, TuckTable[ Iy + 1 ][ 1 ] ),
							6, Plyr->AnimStage,
							6, Plyr->AnimStage,
							20, Plyr->TurnY % 20 );
						else
						SetTweenOfTwoTweens( PlayerNumber,
							FRAMEEQU( PlayerNumber, FTuckTable[ Iy ][ 0 ] ), FRAMEEQU( PlayerNumber, FTuckTable[ Iy ][ 1 ] ),
							FRAMEEQU( PlayerNumber, FTuckTable[ Iy + 1 ][ 0 ] ), FRAMEEQU( PlayerNumber, FTuckTable[ Iy + 1 ][ 1 ] ),
							6, Plyr->AnimStage,
							6, Plyr->AnimStage,
							20, Plyr->TurnY % 20 );
					}
					else
					{
						if( Plyr->Fakie == 0 )
						SetTweenOfTwoTweens( PlayerNumber,
							FRAMEEQU( PlayerNumber, TuckTable[ Iy ][ 1 ] ), FRAMEEQU( PlayerNumber, TuckTable[ Iy ][ 2 ] ),
							FRAMEEQU( PlayerNumber, TuckTable[ Iy + 1 ][ 1 ] ), FRAMEEQU( PlayerNumber, TuckTable[ Iy + 1 ][ 2 ] ),
							12, Plyr->AnimStage - 6,
							12, Plyr->AnimStage - 6,
							20, Plyr->TurnY % 20 );
						else
						SetTweenOfTwoTweens( PlayerNumber,
							FRAMEEQU( PlayerNumber, FTuckTable[ Iy ][ 1 ] ), FRAMEEQU( PlayerNumber, FTuckTable[ Iy ][ 2 ] ),
							FRAMEEQU( PlayerNumber, FTuckTable[ Iy + 1 ][ 1 ] ), FRAMEEQU( PlayerNumber, FTuckTable[ Iy + 1 ][ 2 ] ),
							12, Plyr->AnimStage - 6,
							12, Plyr->AnimStage - 6,
							20, Plyr->TurnY % 20 );
					}

					Plyr->AnimStage++;
				}

				if( !Config_TUCK( Plyr->PadNumber ) ) LerpToNormalStance(Plyr, PlayerNumber);
//			}
			break;

		case STY_TREESOFT:
			if( Anim_Status( PlayerNumber ) == AS_STATIC )
			{
				if( TREESOFT_Time[ Plyr->AnimStage ] == -128 )
				{
					LerpToNormalStance(Plyr, PlayerNumber);
				}
				else
				{
					if( Plyr->Fakie == 0 )
					{
						SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, TREESOFT_List[ Plyr->AnimStage ] ), TREESOFT_Time[ Plyr->AnimStage ] );
					}
					else
					{
						SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, FTREESOFT_List[ Plyr->AnimStage ] ), TREESOFT_Time[ Plyr->AnimStage ] );
					}
					Plyr->AnimStage++;
				}
			}
			break;

		case STY_TREEBUMP:
			// Rot around Z
			if( Anim_Status( PlayerNumber ) == AS_STATIC )
			{
				if( TREEBUMP_Time[ Plyr->AnimStage ] == -128 )
				{
					LerpToNormalStance(Plyr, PlayerNumber);
				}
				else
				{
					if( Plyr->Fakie == 0 )
					{
						SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, TREEBUMP_List[ Plyr->AnimStage ] ), TREEBUMP_Time[ Plyr->AnimStage ] );
					}
					else
					{
						SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, FTREEBUMP_List[ Plyr->AnimStage ] ), TREEBUMP_Time[ Plyr->AnimStage ] );
					}

/*					TVec.X = Plyr->SpinRot.X + TREEBUMP_Rots[ Plyr->AnimStage ].X;
					TVec.Y = Plyr->SpinRot.Y + TREEBUMP_Rots[ Plyr->AnimStage ].Y;
					TVec.Z = Plyr->SpinRot.Z + TREEBUMP_Rots[ Plyr->AnimStage ].Z;
					SetLerpRots( PlayerNumber, &Plyr->SpinRot, &TVec, TREEBUMP_Time[ Plyr->AnimStage ], &Plyr->SpinRot );
*/
					Plyr->AnimStage++;
				}
			}
			Plyr->TurnX = CHFL( 0.0 );
			break;

		case STY_TREESMACK:
			if( Anim_Status( PlayerNumber ) == AS_STATIC )
			{

				if( DestLerpFrame( PlayerNumber ) == FRAMEEQU( PlayerNumber, B_HAR11 ) && ( Pad == 0 ) )
				{

				}
				else
				{
					if( TREESMACK_Time[ Plyr->AnimStage ] == -128 )
					{
						Plyr->Vel = CHFL( 0.1 );
						LerpToNormalStance(Plyr, PlayerNumber);
					}
					else
					{
						SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, TREESMACK_List[ Plyr->AnimStage ] ), TREESMACK_Time[ Plyr->AnimStage ] );

//					TVec.X = Plyr->SpinRot.X + TREESMACK_Rots[ Plyr->AnimStage ].X;
//					TVec.Y = Plyr->SpinRot.Y + TREESMACK_Rots[ Plyr->AnimStage ].Y;
//					TVec.Z = Plyr->SpinRot.Z + TREESMACK_Rots[ Plyr->AnimStage ].Z;
//					SetLerpRots( PlayerNumber, &Plyr->SpinRot, &TVec, TREESMACK_Time[ Plyr->AnimStage ], &Plyr->SpinRot );

						Plyr->AnimStage++;

						if( DestLerpFrame( PlayerNumber ) == FRAMEEQU( PlayerNumber, B_HAR2 ) ) Plyr->Vel = -CHFL( 5.0 );
					}
				}
			}


			if( DestLerpFrame( PlayerNumber ) >= FRAMEEQU( PlayerNumber, B_HAR2 ) && DestLerpFrame( PlayerNumber ) <= FRAMEEQU( PlayerNumber, B_HAR11 ) )
			{
				Plyr->Vel += CHFL( 0.1 );
				if( Plyr->Vel >= CHFL( 0.0 ) ) Plyr->Vel = CHFL( 0.0 );
			}

			Plyr->TurnX = CHFL( 0.0 );
			Plyr->SideSlip = CHFL( 0.0 );
			break;

		case STY_ENDBRAKE:
			if( Anim_Status( PlayerNumber ) == AS_STATIC )
			{
				Plyr->AnimStage--;
				if( Plyr->AnimStage > 1000 ) // < 0 for uint number
				{
					LerpToNormalStance(Plyr, PlayerNumber);
				}
				else
				{
					if( Plyr->Fakie == 0 )
						SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, ENDBRAKE_List[ Plyr->AnimStage ] ), ENDBRAKE_Time[ Plyr->AnimStage ] );
					else
						SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, FENDBRAKE_List[ Plyr->AnimStage ] ), ENDBRAKE_Time[ Plyr->AnimStage ] );
				}
			}

			if( Config_BRAKE( Plyr->PadNumber ) )
			{
				Plyr->Style = STY_BRAKE;
			}
			break;

		case STY_BRAKE:
			if( Anim_Status( PlayerNumber ) == AS_STATIC )
			{
				if( BRAKE_Time[ Plyr->AnimStage ] != -128 )
				{
					if( Plyr->Fakie == 0 )
						SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, BRAKE_List[ Plyr->AnimStage ] ), BRAKE_Time[ Plyr->AnimStage ] );
					else
						SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, FBRAKE_List[ Plyr->AnimStage ] ), BRAKE_Time[ Plyr->AnimStage ] );
					Plyr->AnimStage++;
				}
			}

			if( !Config_BRAKE( Plyr->PadNumber ) )
			{
				Plyr->Style = STY_ENDBRAKE;

				if( Plyr->AnimStage <= 2 )
				{
					LerpToNormalStance(Plyr, PlayerNumber);
				}
			}

			Plyr->Vel -= BRAKE_Power[ Plyr->AnimStage ];
			if( Plyr->Vel < CHFL( 0.0 ) ) Plyr->Vel = CHFL( 0.0 );
			break;

/*
			if( Anim_Status( PlayerNumber ) == AS_STATIC )
			{
				if( Config_BRAKE( Plyr->PadNumber ) )
				{
					// Braking
					Plyr->Style = STY_BRAKE;

					if( BRAKE_Time[ Plyr->AnimStage ] != -128 )
					{
						SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, BRAKE_List[ Plyr->AnimStage ] ), BRAKE_Time[ Plyr->AnimStage ] );

//						TVec.X = Plyr->SpinRot.X + BRAKE_Rots[ Plyr->AnimStage ].X;
//						TVec.Y = Plyr->SpinRot.Y + BRAKE_Rots[ Plyr->AnimStage ].Y;
//						TVec.Z = Plyr->SpinRot.Z + BRAKE_Rots[ Plyr->AnimStage ].Z;
//						SetLerpRots( PlayerNumber, &Plyr->SpinRot, &TVec, BRAKE_Time[ Plyr->AnimStage ], &Plyr->SpinRot );

						Plyr->AnimStage++;
					}
				}
				else
				{
					// Not braking
					Plyr->Style = STY_ENDBRAKE;

					Plyr->AnimStage--;
					if( Plyr->AnimStage > 1000 )
					{
						LerpToNormalStance(Plyr, PlayerNumber);
						break;
					}
					else
					{
						SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, ENDBRAKE_List[ Plyr->AnimStage ] ), ENDBRAKE_Time[ Plyr->AnimStage ] );

//						TVec.X = Plyr->SpinRot.X + ENDBRAKE_Rots[ Plyr->AnimStage ].X;
//						TVec.Y = Plyr->SpinRot.Y + ENDBRAKE_Rots[ Plyr->AnimStage ].Y;
//						TVec.Z = Plyr->SpinRot.Z + ENDBRAKE_Rots[ Plyr->AnimStage ].Z;
//						SetLerpRots( PlayerNumber, &Plyr->SpinRot, &TVec, ENDBRAKE_Time[ Plyr->AnimStage ], &Plyr->SpinRot );
					}
				}
			}


			if( Plyr->Style == STY_BRAKE )
			{
				Plyr->Vel -= BRAKE_Power[ Plyr->AnimStage ];
				if( Plyr->Vel < CHFL( 0.0 ) ) Plyr->Vel = CHFL( 0.0 );
			}

*/
//			break;


		case STY_LATEGRAB:
			if( Anim_Status( PlayerNumber ) == AS_STATIC )
			{
				if( LATEGRAB_Time[ Plyr->AnimStage ] == -128 )
				{
					LerpToNormalStance(Plyr, PlayerNumber);
				}
				else
				{
					SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, LATEGRAB_List[ Plyr->AnimStage ] ), LATEGRAB_Time[ Plyr->AnimStage ] );
					Plyr->AnimStage++;
				}
			}

			Plyr->Vel -= CHFL( 0.10 );
			break;

		case STY_TWISTR:
		case STY_TWISTL:
			// If grab is held then we need to do something else
			if( Plyr->GrabHeld != 0 )
			{
				Grab(Plyr, PlayerNumber);
				if( Plyr->JumpY < CHFL( 0.0 ) )
				{
					printf("Perhaps one should put a late grab release crash in here\n");
				}
			}
			else
			{
				if( Plyr->HaveGrabbed != 0 )
				{
					Plyr->AnimStage = 3;
					SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, TWISTR_List[ Plyr->AnimStage ] ), TWISTR_Time[ Plyr->AnimStage ] );
					Plyr->HaveGrabbed = 0;
				}

				if( Anim_Status( PlayerNumber ) == AS_STATIC )
				{
					SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, TWISTR_List[ Plyr->AnimStage ] ), TWISTR_Time[ Plyr->AnimStage ] );
					Plyr->AnimStage++;
					if( TWISTR_Time[ Plyr->AnimStage ] == -128 ) Plyr->AnimStage = 3;
				}
			}

			Plyr->SpinRot.Y += Plyr->TwistSpeed;
			Plyr->TwistDist += Plyr->TwistSpeed;

			if( Plyr->SpinRot.Y < CHFL( 0.0 ) ) Plyr->SpinRot.Y += CHFL( 4096.0 );	// Stop it getting bigger than 4096
			if( Plyr->SpinRot.Y >= CHFL( 4096.0 ) ) Plyr->SpinRot.Y -= CHFL( 4096.0 );	// Stop it getting bigger than 4096

			if( Plyr->Style == STY_TWISTR )
			{
				if( !Config_RIGHTTWIST( Plyr->PadNumber ) ) Plyr->TwistHeld = 0;

				if( Plyr->TwistHeld == 0 )
				{
					Plyr->TwistSpeed -= CHFL( 2.5 );
					if( Plyr->TwistSpeed < CHFL( 50.0 ) ) Plyr->TwistSpeed = CHFL( 50.0 );
				}
			}

			if( Plyr->Style == STY_TWISTL )
			{
				if( !Config_LEFTTWIST( Plyr->PadNumber ) ) Plyr->TwistHeld = 0;

				if( Plyr->TwistHeld == 0 )
				{
					Plyr->TwistSpeed += CHFL( 2.5 );
					if( Plyr->TwistSpeed > CHFL( -50.0 ) ) Plyr->TwistSpeed = CHFL( -50.0 );
				}
			}

//			Plyr->SpinRot.Y = Plyr->Rot.Y;

			// Near the ground and ready to land!
			if( Plyr->JumpY <= CHFL( 0.0 ) && Plyr->JumpD != 1 )
			{
				chfloat TwistDist2 = CHSHIFTR( chabs( Plyr->TwistDist ) );
				uint16 Maker = TwistDist2 / 2048;
				uint16 Mult = 0;

				if( Maker != 0 )
				{
					if( Maker > 8 )
					{
						Mult = Maker - 8;
						Maker = 8;
					}

					if( Plyr->TwistDist < 0 )
					{
						if( Plyr->Fakie == 0 )
							AwardTrick( PlayerNumber, T_FS180 + Maker - 1, Mult );
						else
							AwardTrick( PlayerNumber, T_BS180 + Maker - 1, Mult );
					}
					else
					{
						if( Plyr->Fakie == 0 )
							AwardTrick( PlayerNumber, T_BS180 + Maker - 1, Mult );
						else
							AwardTrick( PlayerNumber, T_FS180 + Maker - 1, Mult );
					}
				}

				TwistLandOrientation(Plyr, PlayerNumber);
			}
			break;

		case STY_TWISTRLAND:
			if( Anim_Status( PlayerNumber ) == AS_STATIC )
			{
				if( DestLerpFrame( PlayerNumber ) == FRAMEEQU( PlayerNumber, B_RTW8 ) )
				{
					LerpToNormalStance(Plyr, PlayerNumber);
				}
				else
				{
					SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_RTW8 ), 5 );
				}
			}
			break;

		case STY_TWISTANDSPIN:
			// If grab is held then we need to do something else
			if( Plyr->GrabHeld != 0 )
			{
				Grab(Plyr, PlayerNumber);
				if( Plyr->JumpY < CHFL( 0.0 ) )
				{
					printf("Perhaps one should put a late grab release crash in here\n");
				}
			}
			else
			{
				if( Plyr->HaveGrabbed != 0 )
				{
					Plyr->AnimStage = 3;
					SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, TWISTSPIN_List[ Plyr->AnimStage ] ), TWISTSPIN_Time[ Plyr->AnimStage ] );
					Plyr->HaveGrabbed = 0;
				}

				if( Anim_Status( PlayerNumber ) == AS_STATIC )
				{
					SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, TWISTSPIN_List[ Plyr->AnimStage ] ), TWISTSPIN_Time[ Plyr->AnimStage ] );
					Plyr->AnimStage++;
					if( TWISTSPIN_Time[ Plyr->AnimStage ] == -128 ) Plyr->AnimStage = 3;
				}
			}

			// Twisting calculations
			Plyr->SpinRot.Y += Plyr->TwistSpeed;
			Plyr->TwistDist += Plyr->TwistSpeed;
			if( Plyr->SpinRot.Y < CHFL( 0.0 ) ) Plyr->SpinRot.Y += CHFL( 4096.0 );
			if( Plyr->SpinRot.Y >= CHFL( 4096.0 ) ) Plyr->SpinRot.Y -= CHFL( 4096.0 );


			// Spinning calculations
			Plyr->SpinRot.X += Plyr->SpinSpeed;
			Plyr->SpinDist += Plyr->SpinSpeed;
			if( Plyr->SpinRot.X < CHFL( 0.0 ) ) Plyr->SpinRot.X += CHFL( 4096.0 );
			if( Plyr->SpinRot.X >= CHFL( 4096.0 ) ) Plyr->SpinRot.X -= CHFL( 4096.0 );


			// Near the ground and ready to land!
			if( Plyr->JumpY < CHFL( 40.0 ) && Plyr->JumpD != 1 )
			{
				if( OrientedWithGround(Plyr) )
				{
					chfloat TwistDist2 = CHSHIFTR( chabs( Plyr->TwistDist ) );
					chfloat SpinDist2 = CHSHIFTR( chabs( Plyr->SpinDist ) );
					uint16 Mult = ( TwistDist2 / 2048 ) + ( SpinDist2 / 2048 ) / 2;

					TwistLandOrientation(Plyr, PlayerNumber);
//					LerpToNormalStance(Plyr, PlayerNumber);

					if( Mult != 0 )
					{
						if( Mult > 10 ) Mult = 10;
						AwardTrick( PlayerNumber, T_MC_TWIST, Mult - 1 );
					}
				}
				else
				{
					Plyr->Style = STY_ST_ROLL;
					SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_ST_ROLL1 ), 5 );
					Plyr->AnimStage = 0;
					Plyr->SpinRot.X = Plyr->BoardElevation;
					Plyr->SpinRot.Y = Plyr->BoardAzimuth - SPONG( 90.0 );
					Plyr->SpinRot.Z = Plyr->BoardRoll;
					GoToCrashView( PlayerNumber );
				}
			}
			break;

		case STY_STUMPTRIP:
			if( Anim_Status( PlayerNumber ) == AS_STATIC )
			{
				// Trip over the stump
				if( ROLL_Time[ Plyr->AnimStage ] == -128 )
				{
					LerpToNormalStance(Plyr, PlayerNumber);
				}
				else
				{
					if( Plyr->AnimStage == 3 && Plyr->JumpY > CHFL( 32.0 ) )
					{
						// Hold him on this frame
					}
					else
					{
						if( Plyr->AnimStage == 3 && Plyr->JumpY <= CHFL( 32.0 ) )
						{
							Plyr->JumpD = 1;
							Plyr->JumpPower = CHFL( 8.0 );
						}
						SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, ROLL_List[ Plyr->AnimStage ] ), ROLL_Time[ Plyr->AnimStage ] );
//						TVec.X = Plyr->SpinRot.X + ROLL_Rots[ Plyr->AnimStage ].X;
//						TVec.Y = Plyr->SpinRot.Y + ROLL_Rots[ Plyr->AnimStage ].Y;
//						TVec.Z = Plyr->SpinRot.Z + ROLL_Rots[ Plyr->AnimStage ].Z;
//						SetLerpRots( PlayerNumber, &Plyr->SpinRot, &TVec, ROLL_Time[ Plyr->AnimStage ], &Plyr->SpinRot );
						Plyr->AnimStage++;
					}
				}
			}
			break;


		case STY_BONK:
			// Hold on this frame and then continue with the other rubbish
// 			if( Anim_Status( PlayerNumber ) == AS_STATIC )
//			{
				Plyr->Style = STY_BONK1;
//				printf("Leap\n");

				Plyr->JumpD = 1;
				Plyr->JumpPower = CHFL( 10.0 );
//				Plyr->Vel = CHFL( 6.0 );
				Plyr->JumpY = CHFL( 0.1 );

				Plyr->AnimStage = 0;

				AwardTrick( PlayerNumber, T_BONK, 0 );

//			}
			break;


		case STY_BONK1:

			if( Anim_Status( PlayerNumber ) == AS_STATIC )
			{
				switch( Plyr->AnimStage )
				{
					case 0:
						if( Plyr->JumpD != 1 )
						{
//							Plyr->Vel = CHFL( 9.0 );
//							printf("Apex\n");
							SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_BONK2 ), 16 );
							Plyr->AnimStage = 1;
						}
						break;

					case 1:
						if( Plyr->JumpY <= CHFL( 0.0 ) )
						{
//							printf("Landed\n");
							SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_BONK3 ), 5 );
							Plyr->AnimStage = 2;
						}
						break;

					case 2:
						SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_BONK4 ), 5 );
						Plyr->AnimStage = 3;
						break;

					case 3:
						SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_BONK5 ), 5 );
						Plyr->AnimStage = 4;
						break;

					case 4:
					default:
						LerpToNormalStance(Plyr, PlayerNumber);
//						printf("EndBonk\n");
						break;
				}
			}

//			if( Plyr->JumpY <= CHFL( 0.0 ) )
//			{
//				printf("Landed\n");
//				LerpToNormalStance(Plyr, PlayerNumber);
//			}
			break;

/*
			if( Anim_Status( PlayerNumber ) == AS_STATIC )
			{
				if( DestLerpFrame( PlayerNumber ) == FRAMEEQU( PlayerNumber, B_BONK5 ) )
				{
					LerpToNormalStance(Plyr, PlayerNumber);
				}
				else
				{
					switch( Plyr->AnimStage )
					{
						case 0:
							break;

						case 1:
							if( Plyr->JumpD != 1 )
							{
								SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, BONK_List[ Plyr->AnimStage ] ), BONK_Time[ Plyr->AnimStage ] );
								Plyr->AnimStage++;
							}
							break;

						case 3:
						case 4:
						case 5:
							SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, BONK_List[ Plyr->AnimStage - 1 ] ), BONK_Time[ Plyr->AnimStage - 1 ] );
							Plyr->AnimStage++;
							break;
					}
				}
			}




// B BL B T BL
			if( Plyr->JumpY == CHFL( 1.1 ) )
			{
				printf("Takeoff!\n");
				Plyr->JumpPower = CHFL( 10.0 );
				Plyr->JumpD = 1;

				Plyr->Vel = CHFL( 10.0 );
				Plyr->JumpY = CHFL( 1.2 );
			}

			if( ( Plyr->JumpY <= CHFL( 0.0 ) ) && ( Plyr->JumpD != 1 ) && ( Plyr->AnimStage < 3 ) )
			{
				printf("Bonk landing\n");
				Plyr->JumpY = CHFL( 0.0 );
				Plyr->AnimStage = 3;
				SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, BONK_List[ Plyr->AnimStage - 1 ] ), BONK_Time[ Plyr->AnimStage - 1 ] );
				Plyr->AnimStage++;
			}
			break;
*/
		case STY_TWISTLB:
			LerpHomeSpinRotY( Plyr,PlayerNumber );

			Plyr->Vel -= CHFL( 0.25 );
			if( Plyr->Vel < CHFL( 1.0 ) ) Plyr->Vel = CHFL( 1.0 );

			if( Anim_Status( PlayerNumber ) == AS_STATIC )
			{
				if( TWISTLB_Time[ Plyr->AnimStage ] == -128 )
				{
					LerpToNormalStance(Plyr, PlayerNumber);
				}
				else
				{
					SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, TWISTLB_List[ Plyr->AnimStage ] ), TWISTLB_Time[ Plyr->AnimStage ] );
					Plyr->AnimStage++;
				}
			}
			break;

		case STY_TWISTLF:
			LerpHomeSpinRotY( Plyr,PlayerNumber );

			Plyr->Vel -= CHFL( 0.25 );
			if( Plyr->Vel < CHFL( 1.0 ) ) Plyr->Vel = CHFL( 1.0 );

			if( Anim_Status( PlayerNumber ) == AS_STATIC )
			{
				if( TWISTLF_Time[ Plyr->AnimStage ] == -128 )
				{
					LerpToNormalStance(Plyr, PlayerNumber);
				}
				else
				{
					SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, TWISTLF_List[ Plyr->AnimStage ] ), TWISTLF_Time[ Plyr->AnimStage ] );
					Plyr->AnimStage++;
				}
			}
			break;

		case STY_ST_ROLL:
			Plyr->JumpY = CHFL( 16.0 );	// To make him look as if he is rolling ON and not below the ground

			Plyr->SpinRot.Z -= SPONG( 22.5 );

			if( Plyr->BoardElevation < 0 )
			{
//				printf("Uphill roll! ");
				Plyr->Vel -= CHFL( 0.5 );
				if( Plyr->Vel <= CHFL( 0.0 ) ) Plyr->Vel = CHFL( 0.0 );
			}
			else
			{
				Plyr->Vel -= CHFL( 0.5 );
				if( Plyr->Vel < CHFL( 2.5 ) ) Plyr->Vel = CHFL( 2.5 );
			}

			Plyr->AnimStage++;
			Plyr->AnimStage &= 15;

			if( Plyr->AnimStage == 0  && Plyr->Vel <= CHFL( 2.5 ) )
			{
				Plyr->AnimStage = 0;
				SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_ST_ROLL2 ), 8 );
				Plyr->Style = STY_END_ST_ROLL;
				Plyr->JumpY = CHFL( 0.0 );
				InitLerpHomeSpinRotY( Plyr, PlayerNumber, 15, Plyr->BoardAzimuth );
			}
			break;

		case STY_END_ST_ROLL:
			LerpHomeSpinRotY( Plyr, PlayerNumber );

			if( Anim_Status( PlayerNumber ) == AS_STATIC )
			{
				switch( DestLerpFrame( PlayerNumber ) )
				{
					case B_ST_ROLL2:
						SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_ST_ROLL3 ), 8 );
						break;

					case B_ST_ROLL3:
						SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_ST_ROLL4 ), 8 );
						break;

					case B_ST_ROLL4:
						LerpToNormalStance(Plyr, PlayerNumber);
						break;
				}
			}
			break;

		case STY_ST_RECOVERY:
			{
				chfloat PhaseY;

				if( Anim_Status( PlayerNumber ) == AS_STATIC )
				{
					if( DestLerpFrame( PlayerNumber ) != FRAMEEQU( PlayerNumber, B_TRC11 ) )
					{
						SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, TRC_List[ Plyr->AnimStage ] ), TRC_Time[ Plyr->AnimStage ] );
						Plyr->AnimStage++;
					}
				}

				if( Config_JUMP( Plyr->PadNumber ) )
				{
					if( Plyr->TwistSpeed != CHFL( 0.0 ) )
					{
						if( Plyr->TwistSpeed < CHFL( 0.0 ) )
						{
							if( Plyr->TwistSpeed < CHFL( -50.0 ) )
							{
								Plyr->TwistSpeed += CHFL( 5.0 );
								if( Plyr->TwistSpeed > CHFL( -50.0 ) ) Plyr->TwistSpeed = CHFL( -50.0 );
							}

							if( Plyr->TwistSpeed > CHFL( -50.0 ) )
							{
								Plyr->TwistSpeed -= CHFL( 5.0 );
								if( Plyr->TwistSpeed < CHFL( -50.0 ) ) Plyr->TwistSpeed = CHFL( -50.0 );
							}
						}

						if( Plyr->TwistSpeed > CHFL( 0.0 ) )
						{
							if( Plyr->TwistSpeed < CHFL( 50.0 ) )
							{
								Plyr->TwistSpeed += CHFL( 5.0 );
								if( Plyr->TwistSpeed > CHFL( 50.0 ) ) Plyr->TwistSpeed = CHFL( 50.0 );
							}

							if( Plyr->TwistSpeed > CHFL( 50.0 ) )
							{
								Plyr->TwistSpeed -= CHFL( 5.0 );
								if( Plyr->TwistSpeed < CHFL( 50.0 ) ) Plyr->TwistSpeed = CHFL( 50.0 );
							}
						}
					}

					if( Plyr->SpinSpeed != CHFL( 0.0 ) )
					{
						if( Plyr->SpinSpeed < CHFL( 0.0 ) )
						{
							if( Plyr->SpinSpeed < CHFL( -50.0 ) )
							{
								Plyr->SpinSpeed += CHFL( 5.0 );
								if( Plyr->SpinSpeed > CHFL( -50.0 ) ) Plyr->SpinSpeed = CHFL( -50.0 );
							}

							if( Plyr->SpinSpeed > CHFL( -50.0 ) )
							{
								Plyr->SpinSpeed -= CHFL( 5.0 );
								if( Plyr->SpinSpeed < CHFL( -50.0 ) ) Plyr->SpinSpeed = CHFL( -50.0 );
							}
						}

						if( Plyr->SpinSpeed > CHFL( 0.0 ) )
						{
							if( Plyr->SpinSpeed < CHFL( 50.0 ) )
							{
								Plyr->SpinSpeed += CHFL( 5.0 );
								if( Plyr->SpinSpeed > CHFL( 50.0 ) ) Plyr->SpinSpeed = CHFL( 50.0 );
							}

							if( Plyr->SpinSpeed > CHFL( 50.0 ) )
							{
								Plyr->SpinSpeed -= CHFL( 5.0 );
								if( Plyr->SpinSpeed < CHFL( 50.0 ) ) Plyr->SpinSpeed = CHFL( 50.0 );
							}
						}
					}
				}

				// Twisting calculations
				Plyr->SpinRot.Y += Plyr->TwistSpeed;
				if( Plyr->SpinRot.Y < CHFL( 0.0 ) ) Plyr->SpinRot.Y += CHFL( 4096.0 );
				if( Plyr->SpinRot.Y >= CHFL( 4096.0 ) ) Plyr->SpinRot.Y -= CHFL( 4096.0 );

				// Spinning calculations
				Plyr->SpinRot.X += Plyr->SpinSpeed;
				if( Plyr->SpinRot.X < CHFL( 0.0 ) ) Plyr->SpinRot.X += CHFL( 4096.0 );
				if( Plyr->SpinRot.X >= CHFL( 4096.0 ) ) Plyr->SpinRot.X -= CHFL( 4096.0 );


//				Plyr->SpinRot.Y = Plyr->MoveAzimuth;
//				Plyr->SpinRot.X = Plyr->BoardElevation;
//				Plyr->SpinSpeed = CHFL( 0.0 );
//				Plyr->TwistSpeed = CHFL( 0.0 );


				PhaseY = ( ( CHSHIFTR( Plyr->BoardAzimuth ) & 4095 ) - CHSHIFTR( Plyr->SpinRot.Y ) ) & 4095;
				if( CHSHIFTR( Plyr->SpinRot.Y ) < 256 || CHSHIFTR( Plyr->SpinRot.Y ) > 3840 )
				{
					if( Plyr->TwistSpeed <= CHFL( 50.0 ) || Plyr->TwistSpeed >= CHFL( -50.0 ) )
						Plyr->TwistSpeed = CHFL( 0.0 );
				}

				if( Plyr->SpinRot.X < CHFL( 512.0 ) || Plyr->SpinRot.X > CHFL( 3968.0 ) )
				{
					if( Plyr->SpinSpeed <= CHFL( 50.0 ) || Plyr->SpinSpeed >= CHFL( -50.0 ) )
						Plyr->SpinSpeed = CHFL( 0.0 );
				}

				if( Plyr->JumpY <= CHFL( 32.0 ) )
				{
					// Go into that spin crash
				}

				if( Plyr->SpinSpeed == CHFL( 0.0 ) && Plyr->TwistSpeed == CHFL( 0.0 ) )
				{
					// He has recovered and is steady in the air
					InitLerpHomeSpinRotY( Plyr, PlayerNumber, 15, Plyr->BoardAzimuth );
//					Plyr->AnimStage = 0;
					Plyr->Style = STY_ST_RECOVERY2;
				}
			}
			break;

		case STY_ST_RECOVERY2:
			if( Anim_Status( PlayerNumber ) == AS_STATIC )
			{
				if( Plyr->JumpY <= CHFL( 0.0 ) )
				{
					if( TRC_Time[ Plyr->AnimStage ] == -128 )
					{
						chfloat TwistDist2 = CHSHIFTR( chabs( Plyr->TwistDist ) );
						chfloat SpinDist2 = CHSHIFTR( chabs( Plyr->SpinDist ) );
						uint16 Mult = ( TwistDist2 / 2048 ) + ( SpinDist2 / 2048 ) / 2;

						LerpToNormalStance(Plyr, PlayerNumber);

						if( Mult != 0 )
						{
							if( Mult > 10 ) Mult = 10;
							AwardTrick( PlayerNumber, T_MC_TWIST, Mult - 1 );
						}
					}
					else
					{
						SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, TRC_List[ Plyr->AnimStage ] ), TRC_Time[ Plyr->AnimStage ] );

						Plyr->AnimStage++;
					}
				}
			}
			break;

		case STY_RAILSLIDE:
			LerpHomeSpinRotY( Plyr, PlayerNumber );

//			Plyr->Vel -= CHFL( 1.0 );
//			if( Plyr->Vel < CHFL( 3.0 ) ) Plyr->Vel = CHFL( 3.0 );

			Plyr->Vel = CHFL( 5.0 );


			if( Anim_Status( PlayerNumber ) == AS_STATIC )
			{
				// (3) Balance in that state
				chfloat Phase = ( ( CHSHIFTR( Plyr->RailSlip ) * 360 ) / 4096 ) + 30;
//				printf("%ld\n", Phase );

				switch( Plyr->RailType )
				{
					case RAIL_FRONT:
					case RAIL_BACK:
						Plyr->SpinRot.X = Plyr->BoardElevation;
						Plyr->SpinRot.Z = Plyr->RailSlip;
						if( Phase >= 45 ) SetTween( PlayerNumber, FRAMEEQU( PlayerNumber, B_RAILF1L ), FRAMEEQU( PlayerNumber, B_RAILF2L ), 15, Phase - 45 );
						if( Phase < 45 && Phase >= 30 ) SetTween( PlayerNumber, FRAMEEQU( PlayerNumber, B_RAILF0 ), FRAMEEQU( PlayerNumber, B_RAILF1L ), 15, Phase - 30 );
						if( Phase < 30 && Phase >= 15 ) SetTween( PlayerNumber, FRAMEEQU( PlayerNumber, B_RAILF1R ), FRAMEEQU( PlayerNumber, B_RAILF0 ), 15, Phase - 15 );
						if( Phase < 15 ) SetTween( PlayerNumber, FRAMEEQU( PlayerNumber, B_RAILF2R ), FRAMEEQU( PlayerNumber, B_RAILF1R ), 15, Phase );
						break;

					case RAIL_LEFT:
					case RAIL_RIGHT:
						Plyr->SpinRot.X = Plyr->RailSlip;
						Plyr->SpinRot.Z = Plyr->BoardElevation;
						if( Phase >= 45 ) SetTween( PlayerNumber, FRAMEEQU( PlayerNumber, B_RAILB1R ), FRAMEEQU( PlayerNumber, B_RAILB2R ), 15, Phase - 45 );
						if( Phase < 45 && Phase >= 30 ) SetTween( PlayerNumber, FRAMEEQU( PlayerNumber, B_RAILB0 ), FRAMEEQU( PlayerNumber, B_RAILB1R ), 15, Phase - 30 );
						if( Phase < 30 && Phase >= 15 ) SetTween( PlayerNumber, FRAMEEQU( PlayerNumber, B_RAILB1L ), FRAMEEQU( PlayerNumber, B_RAILB0 ), 15, Phase - 15 );
						if( Phase < 15 ) SetTween( PlayerNumber, FRAMEEQU( PlayerNumber, B_RAILB2L ), FRAMEEQU( PlayerNumber, B_RAILB1L ), 15, Phase );
						break;
				}
			}


			// (4) Move between the states ?
			if( ( Config_LEFTTWIST( Plyr->PadNumber ) || Config_RIGHTTWIST( Plyr->PadNumber ) ) && ( NotYLerping( PlayerNumber ) ) )
			{
				if( Config_LEFTTWIST( Plyr->PadNumber ) )
				{
					Plyr->RailType++;
					Plyr->RailType &= 3;

				}

				if( Config_RIGHTTWIST( Plyr->PadNumber ) )
				{
					Plyr->RailType--;
					Plyr->RailType &= 3;
				}

				switch( Plyr->RailType )
				{
					case RAIL_FRONT:
						Plyr->RailAzimuth = Plyr->RailY;
						break;
					case RAIL_LEFT:
						Plyr->RailAzimuth = Plyr->RailY - CHFL( 1024.0 );
						break;
					case RAIL_BACK:
						Plyr->RailAzimuth = Plyr->RailY + CHFL( 2048.0 );
						break;
					case RAIL_RIGHT:
						Plyr->RailAzimuth = Plyr->RailY + CHFL( 1024.0 );
						break;
				}

				InitLerpHomeSpinRotY( Plyr, PlayerNumber, 20, Plyr->RailAzimuth );
			}

			// Wobble control
			if( NotYLerping( PlayerNumber ) )
			{
				Plyr->RailSlip += Plyr->RailSlipVel;

				if( Pad & PAD_LEFT )
				{
					if( Plyr->RailType == RAIL_FRONT || Plyr->RailType == RAIL_LEFT )
						Plyr->RailSlipVel += SPONG( 0.25 );
					else
						Plyr->RailSlipVel -= SPONG( 0.25 );
				}

				if( Pad & PAD_RIGHT )
				{
					if( Plyr->RailType == RAIL_FRONT || Plyr->RailType == RAIL_LEFT )
						Plyr->RailSlipVel -= SPONG( 0.25 );
					else
						Plyr->RailSlipVel += SPONG( 0.25 );
				}
			}

			// stop the wobble
			Plyr->RailSlipVel = SPONG( 0.0 );

			if( Plyr->RailSlipVel < SPONG( 0.0 ) )
			{
//				Plyr->RailSlipVel -= SPONG( 0.1 );
				if( Plyr->RailSlip < SPONG( -30.0 ) )
				{
//					Plyr->RailSlip = SPONG( -30.0 );
//					Plyr->RailSlipVel = SPONG( 1.0 );

					FallOff_RailSlide(Plyr, PlayerNumber);
// (6) Fall off!

// Sort of like a halfAzimuth routine
// Just jumps him off in one of the directions for now (very easy to do)
				}
			}

			if( Plyr->RailSlipVel > SPONG( 0.0 ) )
			{
//				Plyr->RailSlipVel += SPONG( 0.1 );
				if( Plyr->RailSlip > SPONG( 30.0 ) )
				{
//					Plyr->RailSlip = SPONG( 30.0 );
//					Plyr->RailSlipVel = SPONG( -1.0 );

					FallOff_RailSlide(Plyr, PlayerNumber);
// (6) Fall off!
				}
			}

			if( Collision( PlayerNumber ) != RAILSLIDE )
			{
				// (5) Come off the rails from any state
				printf("Gone off the rails\n");
				InitLerpHomeSpinRotY( Plyr, PlayerNumber, 5, Plyr->BoardAzimuth );
				Plyr->Style = STY_ENDRAILSLIDE;
				Plyr->AnimStage = 0;

			}
			break;

		case STY_ENDRAILSLIDE:
			// (7) Lerp to going forwards
			LerpHomeSpinRotY( Plyr, PlayerNumber );
			Plyr->AnimStage++;

			if( Plyr->AnimStage == 10 )
			{
				printf("Mofo\n");
				Plyr->JumpD = 2;
				Plyr->JumpPower = CHFL( 0.0 );
			}

			// (8) Lerp to normal stance when landed
			if( Plyr->JumpY <= CHFL( 0.0 ) ) LerpToNormalStance(Plyr, PlayerNumber);
			break;

		case STY_OFFRAIL:
			LerpHomeSpinRotY( Plyr, PlayerNumber );

			if( Anim_Status( PlayerNumber ) == AS_STATIC )
			{
				if( OFFRAIL_Time[ Plyr->AnimStage ] != -128 )
				{
					SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, OFFRAIL_List[ Plyr->AnimStage ] ), OFFRAIL_Time[ Plyr->AnimStage ] );
					Plyr->AnimStage++;
				}
			}

			if( Plyr->JumpY <= CHFL( 0.0 ) ) LerpToNormalStance(Plyr, PlayerNumber);
			break;

		case STY_CHILLTRICK:
			if( Anim_Status( PlayerNumber ) == AS_STATIC )
			{
				if( CHILLTRICK_Time[ Plyr->AnimStage ] != -128 )
				{
					SetLerp( PlayerNumber, CHILLTRICK_List[ Plyr->AnimStage ], CHILLTRICK_Time[ Plyr->AnimStage ] );
					Plyr->AnimStage++;
				}
			}

			if( Plyr->JumpY <= CHFL( 0.0 ) )
			{
				if( Plyr->AnimStage <= 3 )
				{
					// Generic rolls
					Plyr->Style = STY_ST_ROLL;
					SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_ST_ROLL1 ), 5 );
					Plyr->AnimStage = 0;
					Plyr->SpinRot.X = Plyr->BoardElevation;
					Plyr->SpinRot.Y = Plyr->BoardAzimuth - SPONG( 90.0 );
					Plyr->SpinRot.Z = Plyr->BoardRoll;
					GoToCrashView( PlayerNumber );
				}
				else
				{
					if( Plyr->AnimStage <= 6 )
					{
						// Wobbly landing
						SetLerp( PlayerNumber, FRAMEEQU( PlayerNumber, B_LATE1 ), 3 );
						Plyr->Style = STY_LATEGRAB;
						Plyr->AnimStage = 0;
					}
					else
					{
						LerpToNormalStance(Plyr, PlayerNumber);
						AwardTrick( PlayerNumber, T_CHILLTRICK, 0 );
					}
				}
			}
			break;

		case STY_OLLIE:
			if( Anim_Status( PlayerNumber ) == AS_STATIC )
			{
				if( OLLIE_Time[ Plyr->AnimStage ] != -128 )
				{
					if( OLLIE_List[ Plyr->AnimStage ] == B_OLL2 || OLLIE_List[ Plyr->AnimStage ] == B_FOLL2 )
					{
						Plyr->JumpD = 1;
						Plyr->JumpPower = CHFL( 7.5 );
						Plyr->Vel = CHFL( 5.0 );
					}

					if( Plyr->Fakie == 0 )
						SetLerp( PlayerNumber, OLLIE_List[ Plyr->AnimStage ], OLLIE_Time[ Plyr->AnimStage ] );
					else
						SetLerp( PlayerNumber, FOLLIE_List[ Plyr->AnimStage ], OLLIE_Time[ Plyr->AnimStage ] );

					Plyr->AnimStage++;
				}
				else
					LerpToNormalStance(Plyr, PlayerNumber);
			}
			break;

	}

#if JEBS_COLLISION
	if( Plyr->ConsoleRemoteGhosted == CONSOLE_PLAYER )
		InsertXYZPacket( PlayerNumber );
	if( Plyr->ConsoleRemoteGhosted == GHOSTED_PLAYER || Plyr->ConsoleRemoteGhosted == REMOTE_PLAYER )
		RetrieveXYZPacket( PlayerNumber );
#endif
}


// *************************************************
// Standard animation routines used by various modes
// *************************************************

void CalcRotXYZ( player *LPlyr )
{
	if( ( Modes[ LPlyr->Style ] & MD_NOGROUNDORIENTBOARD ) == 0 )
	{
		if( LPlyr->JumpY <= CHFL( 0.0 ) )
			LPlyr->Rot.X = KILLFRAC( LPlyr->BoardElevation );

		LPlyr->Rot.Y = LPlyr->BoardAzimuth;

		if( LPlyr->Fakie == 1 )
		{
			LPlyr->Rot.X = -KILLFRAC( LPlyr->BoardElevation );
			LPlyr->Rot.Y = LPlyr->BoardAzimuth + CHFL( 2048.0 );
		}

		if( LPlyr->JumpY <= CHFL( 0.0 ) ) // On the ground
		{
			// Lean as you turn
			if( LPlyr->TurnX == 0 )
			{
				LPlyr->Rot.Z = 0;
			}
			else
			{
				LPlyr->Rot.Z = -LPlyr->TurnX * 5;

				if( LPlyr->Fakie == 1 ) LPlyr->Rot.Z = LPlyr->TurnX * 5;
			}
		}
	}
	else
	{
		LPlyr->Rot.X = LPlyr->SpinRot.X;
		LPlyr->Rot.Y = LPlyr->SpinRot.Y;
		LPlyr->Rot.Z = LPlyr->SpinRot.Z;
	}
}

void FallOff_RailSlide( player *LPlyr, int32 LPlayerNumber )
{
	switch( LPlyr->RailType )
	{
		case RAIL_FRONT:
			if( LPlyr->RailSlip < CHFL( 0.0 ) )
				LPlyr->MoveAzimuth += SPONG( 22.5 );
			else
				LPlyr->MoveAzimuth -= SPONG( 22.5 );
			break;
		case RAIL_LEFT:
			if( LPlyr->RailSlip > CHFL( 0.0 ) )
				LPlyr->MoveAzimuth -= SPONG( 45.0 );
			else
				LPlyr->MoveAzimuth += SPONG( 45.0 );
			break;
		case RAIL_BACK:
			if( LPlyr->RailSlip < CHFL( 0.0 ) )
				LPlyr->MoveAzimuth -= SPONG( 67.5 );
			else
				LPlyr->MoveAzimuth += SPONG( 67.5 );
			break;
		case RAIL_RIGHT:
			if( LPlyr->RailSlip < CHFL( 0.0 ) )
				LPlyr->MoveAzimuth -= SPONG( 45.0 );
			else
				LPlyr->MoveAzimuth += SPONG( 45.0 );
			break;
	}

	InitLerpHomeSpinRotY( LPlyr, LPlayerNumber, 10, LPlyr->MoveAzimuth );


//	LPlyr->Vel = CHFL( 10.0 );

	LPlyr->Style = STY_OFFRAIL;
	LPlyr->JumpD = 1;
	LPlyr->JumpPower = CHFL( 4.0 );

	LPlyr->AnimStage = 0;
	SetLerp( LPlayerNumber, FRAMEEQU( LPlayerNumber, OFFRAIL_List[ LPlyr->AnimStage ] ), OFFRAIL_Time[ LPlyr->AnimStage ] );
	LPlyr->AnimStage++;
}

void Initialise_Railslide( player *LPlyr, int32 LPlayerNumber)
{
	chfloat Phase;

	printf("Railslide\n");
	LPlyr->Style = STY_RAILSLIDE;
	LPlyr->JumpY = CHFL( 0.1 );
	LPlyr->RailY = RailAngle( LPlayerNumber );

//	LPlyr->Vel = CHFL( 8.0 );

	// (1) Decide which state to ride in
	Phase = ( ( CHSHIFTR( LPlyr->BoardAzimuth ) & 4095 ) - CHSHIFTR( LPlyr->RailY ) ) & 4095;

	if( Phase < 512 || Phase > 3584 )
	{
		// Rail Front (Frontside)
		printf("Rail Front\n");
		LPlyr->RailType = RAIL_FRONT;
		LPlyr->RailAzimuth = LPlyr->RailY;
		SetLerp( LPlayerNumber, FRAMEEQU( LPlayerNumber, B_RAILF0 ), 3 );
	}

	if( Phase <= 3584 && Phase > 2560 )
	{
		// Rail Left (Toeside)
		printf("Rail Left\n");
		LPlyr->RailType = RAIL_LEFT;
		LPlyr->RailAzimuth = LPlyr->RailY - CHFL( 1024.0 );
		if( LPlyr->RailAzimuth < CHFL( 0.0 ) ) LPlyr->RailAzimuth += CHFL( 4096.0 );
		SetLerp( LPlayerNumber, FRAMEEQU( LPlayerNumber, B_RAILB0 ), 3 );
	}

	if( Phase <= 2560 && Phase > 1536 )
	{
		// Rail Back (Backside)
		printf("Rail Back\n");
		LPlyr->RailType = RAIL_BACK;
		LPlyr->RailAzimuth = LPlyr->RailY + CHFL( 2048.0 );
		if( LPlyr->RailAzimuth >= CHFL( 4096.0 ) ) LPlyr->RailAzimuth -= CHFL( 4096.0 );
		SetLerp( LPlayerNumber, FRAMEEQU( LPlayerNumber, B_RAILF0 ), 3 );
	}

	if( Phase <= 1536 && Phase >= 512 )
	{
		// Rail Right (heelside)
		printf("Rail Right\n");
		LPlyr->RailType = RAIL_RIGHT;
		LPlyr->RailAzimuth = LPlyr->RailY + CHFL( 1024.0 );
		if( LPlyr->RailAzimuth >= CHFL( 4096.0 ) ) LPlyr->RailAzimuth -= CHFL( 4096.0 );
		SetLerp( LPlayerNumber, FRAMEEQU( LPlayerNumber, B_RAILB0 ), 3 );
	}

	LPlyr->MoveAzimuth = LPlyr->BoardAzimuth = LPlyr->RailY;

	LPlyr->SpinRot.X = LPlyr->Rot.X;
	LPlyr->SpinRot.Y = LPlyr->Rot.Y; //LPlyr->RailAzimuth;
	LPlyr->SpinRot.Z = LPlyr->Rot.Z;
/*
#if USE_RAND
	switch( rand() % 4 )
	{
		case 0:
			LPlyr->RailSlipVel = SPONG( 1.0 );
			break;
		case 1:
			LPlyr->RailSlipVel = SPONG( -1.0 );
			break;
		case 2:
			LPlyr->RailSlipVel = SPONG( 2.0 );
			break;
		case 3:
		default:
			LPlyr->RailSlipVel = SPONG( -2.0 );
			break;
	}
#else
			LPlyr->RailSlipVel = SPONG( -2.0 );
#endif
*/

	LPlyr->RailSlipVel = SPONG( -2.0 );
	LPlyr->RailSlip = CHFL( 0.0 );

	// (2) Set interpolation to that state
	InitLerpHomeSpinRotY( LPlyr, LPlayerNumber, 7, LPlyr->RailAzimuth );
}



void TwistLandOrientation( player *LPlyr, int32 LPlayerNumber )
{
	chfloat Phase;

	if( LPlyr->Fakie == 0 )
		Phase = ( ( CHSHIFTR( LPlyr->BoardAzimuth ) & 4095 ) - CHSHIFTR( LPlyr->SpinRot.Y ) ) & 4095;
	else
		Phase = ( ( CHSHIFTR( LPlyr->BoardAzimuth ) & 4095 ) - CHSHIFTR( LPlyr->SpinRot.Y ) - 2048 ) & 4095;

//	printf( "%ld\n", Phase );

	// Normal landing
	// < 512 || > 3584
	if( Phase < 512 || Phase > 3584 )
	{
//		#if I_AM == JEZ
//		printf("Normal\n");
//		#endif
		LPlyr->Style = STY_TWISTRLAND;
		SetLerp( LPlayerNumber, FRAMEEQU( LPlayerNumber, B_RTW7 ), 4 );
		LPlyr->AnimStage = 0;
		HalfAzimuth(LPlyr);
	}

	// Heelside landing board B_TWLB - this is the one my friend
	// <= 3584 && > 2560
	if( Phase <= 3584 && Phase > 2560 )
	{
//		#if I_AM == JEZ
//		printf("Heelside\n");
//		#endif
		LPlyr->Style = STY_TWISTLB;
		LPlyr->AnimStage = 0;
		HalfAzimuth(LPlyr);
		InitLerpHomeSpinRotY( LPlyr, LPlayerNumber, 26, LPlyr->BoardAzimuth );
	}

	// Landing fakie
	// <= 2560 && > 1536
	if( Phase <= 2560 && Phase > 1536 )
	{
//		#if I_AM == JEZ
//		printf("Fakie\n");
//		#endif
//		LerpToNormalStance(LPlyr, LPlayerNumber);


		if( LPlyr->Fakie == 0 )
		{
			// Go to fakie
			LPlyr->Fakie = 1;
			LerpToFakieStance(LPlyr, LPlayerNumber);
			GoToNormalView( LPlayerNumber );
		}
		else
		{
			// Go to normal
			LPlyr->Fakie = 0;
			LerpToNormalStance(LPlyr, LPlayerNumber);
		}

	}

	// Toeside landing
	// <= 1536 && >= 512
	if( Phase <= 1536 && Phase >= 512 )
	{
//		#if I_AM == JEZ
//		printf("Toeside\n");
//		#endif
		LPlyr->Style = STY_TWISTLF;
		LPlyr->AnimStage = 0;
		HalfAzimuth(LPlyr);
		InitLerpHomeSpinRotY( LPlyr, LPlayerNumber, 26, LPlyr->BoardAzimuth );
	}
}



uint8 OrientedWithGround( player *LPlyr )
{
	chfloat Phase = ( ( CHSHIFTR( LPlyr->BoardElevation ) & 4095 ) - CHSHIFTR( LPlyr->SpinRot.X ) ) & 4095;

//	printf( "Phase = %ld\n", Phase );

	// Normal landing
	// < 512 || > 3584
	if( Phase < 512 || Phase > 3584 )
	{
//		#if I_AM == JEZ
//		printf("Landed oriented with ground\n");
//		#endif
		return 1;
	}

	return 0;
}



void HalfAzimuth( player *LPlyr )
{
	chfloat Dest = CHSHIFTR( LPlyr->BoardAzimuth ) & 4095;
	chfloat Src = CHSHIFTR( LPlyr->SpinRot.Y ) & 4095;

	if( LPlyr->Fakie == 1 )
	{
		Src = ( Src - 2048 ) & 4095;
	}

	Src -= Dest;
	Src &= 4095;

	if( Src >= 2048 )
	{
		LPlyr->BoardAzimuth -= CHFL( ( 4096 - Src ) / 2 );
	}
	else
	{
		LPlyr->BoardAzimuth += CHFL( Src / 2 );
	}

//	printf("Dest = %ld, Src = %ld\n", Dest, Src );
}

chfloat YLerp[ MAX_PLAYERS ];
chfloat YDest[ MAX_PLAYERS ];
uint16 TotLerps[ MAX_PLAYERS ];
uint16 LerpP[ MAX_PLAYERS ];

void InitLerpHomeSpinRotY( player *LPlyr, uint8 PlayerNumber, uint16 Lerps, chfloat DestA )
{
	chfloat Src = CHSHIFTR( LPlyr->SpinRot.Y ) & 4095;
	YDest[ PlayerNumber ] = DestA;

	if( LPlyr->Fakie == 1 )
	{
		Src = ( Src - 2048 ) & 4095;
	}


	Src -= CHSHIFTR( YDest[ PlayerNumber ] ) & 4095;
	Src &= 4095;

	if( Src >= 2048 )
	{
		YLerp[ PlayerNumber ] = ( 4096 - Src ) / Lerps;
		if( YLerp[ PlayerNumber ] <= 0 ) YLerp[ PlayerNumber ] = 1;
		YLerp[ PlayerNumber ] = CHFL( YLerp[ PlayerNumber ] );
	}
	else
	{
		YLerp[ PlayerNumber ] = Src / Lerps;
		if( YLerp[ PlayerNumber ] <= 0 ) YLerp[ PlayerNumber ] = 1;
		YLerp[ PlayerNumber ] = -CHFL( YLerp[ PlayerNumber ] );
	}

	TotLerps[ PlayerNumber ] = Lerps;
	LerpP[ PlayerNumber ] = 0;

//	printf("Dest = %ld, Src = %ld\n", Dest, Src );
}

uint8 NotYLerping( uint8 PlayerNumber )
{
	if( LerpP[ PlayerNumber ] < TotLerps[ PlayerNumber ] )
	{
		return 0;
	}
	else
	{
		return 1;
	}
}

void LerpHomeSpinRotY( player *LPlyr, uint8 PlayerNumber )
{
	if( LerpP[ PlayerNumber ] < TotLerps[ PlayerNumber ] )
	{
		LPlyr->SpinRot.Y += YLerp[ PlayerNumber ];
		LerpP[ PlayerNumber ]++;
	}
	else
	{
		LPlyr->SpinRot.Y = YDest[ PlayerNumber ];//LPlyr->BoardAzimuth;
	}
}

void Grab( player *LPlyr, int32 LPlayerNumber )
{
	// GRAB 1
	if( LPlyr->GrabHeld == 1 )
 	{
 		if( Anim_Status( LPlayerNumber ) == AS_STATIC || LPlyr->GrabStage == 0 )
 		{
			if( LPlyr->GrabStage != 3 )
			{
				SetLerp( LPlayerNumber, FRAMEEQU( LPlayerNumber, BONEGRAB1_List[ LPlyr->GrabStage ] ), BONEGRAB1_Time[ LPlyr->GrabStage ] );
				LPlyr->GrabStage++;
			}
			else
				LPlyr->GrabDuration++;

			// BONEGRAB1
//			if( LPlyr->GrabStage == 6 || LPlyr->GrabStage == 7 )
//			{
//				if( LPlyr->GrabRotX > CHFL( -660.0 ) )
//					LPlyr->GrabRotX -= CHFL( 60.0 );
//				else
//					LPlyr->GrabRotX = CHFL( -660.0 );
//			}

//			if( LPlyr->GrabRotZ < CHFL( 100.0 ) )
//				LPlyr->GrabRotZ += CHFL( 5.0 );
//			else
//				LPlyr->GrabRotZ = CHFL( 100.0 );

		}
	}


	// GRAB 2
	if( LPlyr->GrabHeld == 2 )
	{
		if( Anim_Status( LPlayerNumber ) == AS_STATIC || LPlyr->GrabStage == 0 )
		{
			if( LPlyr->GrabStage != 3 )
			{
				SetLerp( LPlayerNumber, FRAMEEQU( LPlayerNumber, BONEGRAB2_List[ LPlyr->GrabStage ] ), BONEGRAB2_Time[ LPlyr->GrabStage ] );
				LPlyr->GrabStage++;
			}
			else
				LPlyr->GrabDuration++;
		}
	}




	// END GRAB 1
	if( LPlyr->GrabHeld == 3 )
	{
		if( Anim_Status( LPlayerNumber ) == AS_STATIC )
		{
			if( LPlyr->GrabStage <= 3 )
			{
				LPlyr->GrabHeld = 0;
			}
			else
			{
				LPlyr->GrabStage--;
				SetLerp( LPlayerNumber, FRAMEEQU( LPlayerNumber, BONEGRAB1_List[ LPlyr->GrabStage ] ), BONEGRAB1_Time[ LPlyr->GrabStage ] );
			}
		}

//		// BONEGRAB1
//		if( LPlyr->GrabRotX < CHFL( 0.0 ) )
//			LPlyr->GrabRotX += CHFL( 51.20 );
//
//		if( LPlyr->GrabRotZ > CHFL( 0.0 ) )
//			LPlyr->GrabRotZ += CHFL( 5.0 );
//		else
//			LPlyr->GrabRotZ = CHFL( 0.0 );
	}


	// END GRAB 2
	if( LPlyr->GrabHeld == 4 )
	{
		if( Anim_Status( LPlayerNumber ) == AS_STATIC )
		{
			if( LPlyr->GrabStage <= 3 )
			{
				LPlyr->GrabHeld = 0;
			}
			else
			{
				LPlyr->GrabStage--;
				SetLerp( LPlayerNumber, FRAMEEQU( LPlayerNumber, BONEGRAB1_List[ LPlyr->GrabStage ] ), BONEGRAB1_Time[ LPlyr->GrabStage ] );
			}
		}
	}

}

void LerpToNormalStance( player *LPlyr, int32 LPlayerNumber )
{
	uint16 Iy;
	int16 Temp;

	if( LPlyr->Fakie == 1 )
	{
		LerpToFakieStance(LPlyr, LPlayerNumber);
	}
	else
	{
		CalculateCrouch(LPlyr);
		LPlyr->Style = STY_NORMAL;

		LPlyr->AnimStage = CENTRE_STAGE;
		LPlyr->AnimTween = 25;
		LPlyr->AnimFrame = 0;
		LPlyr->TurnX = 0;

		Temp = CENTRE_STAGE + RockAround[ 0 ];
		Iy = LPlyr->TurnY / 20;

		SetLerpToTweenOfTwoTweens( LPlayerNumber,
			FRAMEEQU( LPlayerNumber, NormalTable[ Iy ][ 1 ] ),
			FRAMEEQU( LPlayerNumber, NormalTable[ Iy ][ 2 ] ),
			FRAMEEQU( LPlayerNumber, NormalTable[ Iy + 1 ][ 1 ] ),
			FRAMEEQU( LPlayerNumber, NormalTable[ Iy + 1 ][ 2 ] ),
			50, Temp - 25,
			50, Temp - 25,
			20, LPlyr->TurnY % 20 );
	}

	GoToNormalView( LPlayerNumber );
}

void LerpToFakieStance( player *LPlyr, int32 LPlayerNumber )
{
	uint16 Iy;
	int16 Temp;

	CalculateCrouch(LPlyr);
	LPlyr->Style = STY_NORMAL;

	LPlyr->AnimStage = CENTRE_STAGE;
	LPlyr->AnimTween = 25;
	LPlyr->AnimFrame = 0;
	LPlyr->TurnX = 0;

	Temp = CENTRE_STAGE + RockAround[ 0 ];
	Iy = LPlyr->TurnY / 20;

	SetLerpToTweenOfTwoTweens( LPlayerNumber,
		FRAMEEQU( LPlayerNumber, FakieNormalTable[ Iy ][ 1 ] ),
		FRAMEEQU( LPlayerNumber, FakieNormalTable[Iy ][ 2 ] ),
		FRAMEEQU( LPlayerNumber, FakieNormalTable[ Iy + 1 ][ 1 ] ),
		FRAMEEQU( LPlayerNumber, FakieNormalTable[ Iy + 1 ][ 2 ] ),
		50, Temp - 25,
		50, Temp - 25,
		20, LPlyr->TurnY % 20 );
}






int16 YCushion[] = { 0,1,2,2,3,3,3,4,4,4,4,3,3,3,2,2,1,0,-1,-2,-2,-3,-3,-3,-4,-4,-4,-4,-3,-3,-3,-2,-2,-1, -128 };

void CalculateCrouch( player *Plyr )
{
	Plyr->TurnY = ( Plyr->Vel / 2507 ) + 5;

	Plyr->TurnY += YCushion[ Plyr->TurnYStage ];
	Plyr->TurnYStage++;
	if( YCushion[ Plyr->TurnYStage ] == -128 ) Plyr->TurnYStage = 0;
}

void Analog_Swing( player *LPlyr, chfloat Swing, chfloat Desired )
{
	if( Desired < LPlyr->TurnX )
	{
		LPlyr->TurnX -= Swing;
		if( LPlyr->TurnX < Desired ) LPlyr->TurnX = Desired;
	}

	if( Desired > LPlyr->TurnX )
	{
		LPlyr->TurnX += Swing;
		if( LPlyr->TurnX > Desired ) LPlyr->TurnX = Desired;
	}
}


void GrabScoring( player *LPlyr, int32 LPlayerNumber )
{
	uint32 Mp;

	chfloat TwistDist2 = CHSHIFTR( chabs( LPlyr->TwistDist ) );
	chfloat SpinDist2 = CHSHIFTR( chabs( LPlyr->SpinDist ) );

//	printf("TwistDist2 = %ld\n", TwistDist2 );
//	printf("SpinDist2 = %ld\n", SpinDist2 );
//	printf("GrabDuration = %ld\n", LPlyr->GrabDuration );

	Mp = LPlyr->GrabDuration / 10;
	if( Mp > 10 ) Mp = 10;

	if( LPlyr->GrabHeld == 3 )
	{
		switch( LPlyr->Style )
		{
			case STY_FRONTSPIN:
				if( SpinDist2 > 2048 )
					AwardTrick( LPlayerNumber, T_FSMUTE, Mp );
				else
					AwardTrick( LPlayerNumber, T_MUTE, Mp );
				break;
			case STY_BACKSPIN:
				if( SpinDist2 > 2048 )
					AwardTrick( LPlayerNumber, T_BSMUTE, Mp );
				else
					AwardTrick( LPlayerNumber, T_MUTE, Mp );
				break;
			case STY_TWISTL:
				if( TwistDist2 > 2048 )
					AwardTrick( LPlayerNumber, T_TFMUTE, Mp );
				else
					AwardTrick( LPlayerNumber, T_MUTE, Mp );
				break;
			case STY_TWISTR:
				if( TwistDist2 > 2048 )
					AwardTrick( LPlayerNumber, T_TBMUTE, Mp );
				else
					AwardTrick( LPlayerNumber, T_MUTE, Mp );
				break;
			case STY_TWISTANDSPIN:
				if( SpinDist2 > 2048 && TwistDist2 > 2048 )
					AwardTrick( LPlayerNumber, T_MC_TWIST_MUTE, Mp );
				else
					AwardTrick( LPlayerNumber, T_MUTE, Mp );
				break;
			default:
				AwardTrick( LPlayerNumber, T_MUTE, Mp );
				break;
		}
	}

	if( LPlyr->GrabHeld == 4 )
	{
		switch( LPlyr->Style )
		{
			case STY_FRONTSPIN:
				if( SpinDist2 > 2048 )
					AwardTrick( LPlayerNumber, T_FSMETHOD, Mp );
				else
					AwardTrick( LPlayerNumber, T_METHOD, Mp );
				break;
			case STY_BACKSPIN:
				if( SpinDist2 > 2048 )
					AwardTrick( LPlayerNumber, T_BSMETHOD, Mp );
				else
					AwardTrick( LPlayerNumber, T_METHOD, Mp );
				break;
			case STY_TWISTL:
				if( TwistDist2 > 2048 )
					AwardTrick( LPlayerNumber, T_TFMETHOD, Mp );
				else
					AwardTrick( LPlayerNumber, T_METHOD, Mp );
				break;
			case STY_TWISTR:
				if( TwistDist2 > 2048 )
					AwardTrick( LPlayerNumber, T_TBMETHOD, Mp );
				else
					AwardTrick( LPlayerNumber, T_METHOD, Mp );
				break;
			case STY_TWISTANDSPIN:
				if( SpinDist2 > 2048 && TwistDist2 > 2048 )
					AwardTrick( LPlayerNumber, T_MC_TWIST_METHOD, Mp );
				else
					AwardTrick( LPlayerNumber, T_METHOD, Mp );
				break;
			default:
				AwardTrick( LPlayerNumber, T_METHOD, Mp );
				break;
		}
	}
}




void Spin_Scoring( player *LPlyr, int32 LPlayerNumber )
{
	chfloat SpinDist2 = CHSHIFTR( chabs( LPlyr->SpinDist ) );
	uint16 Maker = SpinDist2 / 2048;
	uint16 Mult = 0;

//	if( Maker != 0 )
//	{
		if( Maker > 8 )
		{
			Mult = Maker - 8;
			Maker  = 8;
		}

//		Maker--;
		Maker /= 2;

		if( LPlyr->SpinDist < 0 )
			AwardTrick( LPlayerNumber, T_BSpin360 + Maker, Mult );
		else
			AwardTrick( LPlayerNumber, T_FSpin360 + Maker, Mult );
//	}
}
